﻿#
# Power Shell script
# Removing and recreating Performance counters for PlateReader
# 2016-10-18 Olivier Gauthier, creation
# 2016-11-23 Guy Brousseau, last modified
#
# To run this script, you must first issue the following command in a Power Shell window:
# PS P:\> set-executionpolicy unrestricted
#

# Rebuild the perfcounters to be sure they are correct otherwise the Delete and Create methods will fail miserably
iex 'C:\windows\SysWOW64\lodctr /R'
Restart-Service winmgmt -Force
Remove-Item -Path 'HKLM:\SYSTEM\CurrentControlSet\Services\AutoVu.LprCamera\Performance' -Recurse

# Variables for Performance Counter Category
$CatName = "AutoVu.LprCamera"
$CatHelp = "LprCameraHelp"
$CatType = [System.Diagnostics.PerformanceCounterCategoryType]::MultiInstance
$TotalInstance = "Total"
$LprInstance = "Lpr Camera"
$ContextInstance = "Context Camera"

$categoryexists = [System.Diagnostics.PerformanceCounterCategory]::Exists($CatName)
if ($categoryexists)
{
    # Delete the category
	[System.Diagnostics.PerformanceCounterCategory]::Delete($CatName) | Out-Null
}

 # All counters will be added in this collection
$counterCollection = New-Object System.Diagnostics.CounterCreationDataCollection

# Creation and initialization of CounterCreationData added to $counterCollection
$bestPlatePerSecondCounter = New-Object System.Diagnostics.CounterCreationData
$bestPlatePerSecondCounter.CounterName = "BestPlatePerSecond"
$bestPlatePerSecondCounter.CounterType = [System.Diagnostics.PerformanceCounterType]::RateOfCountsPerSecond32
$bestPlatePerSecondCounter.CounterHelp = "BestPlatePerSecondHelp"
$counterCollection.Add($bestPlatePerSecondCounter)

$processingStartedPerSecondCounter = New-Object System.Diagnostics.CounterCreationData
$processingStartedPerSecondCounter.CounterName = "ProcessingStartedPerSecond"
$processingStartedPerSecondCounter.CounterType = [System.Diagnostics.PerformanceCounterType]::RateOfCountsPerSecond32
$processingStartedPerSecondCounter.CounterHelp = "ProcessingStartedPerSecondHelp"
$counterCollection.Add($processingStartedPerSecondCounter)

$fpsCounter = New-Object System.Diagnostics.CounterCreationData
$fpsCounter.CounterName = "FPS"
$fpsCounter.CounterType = [System.Diagnostics.PerformanceCounterType]::RateOfCountsPerSecond32
$fpsCounter.CounterHelp = "FPSHelp"
$counterCollection.Add($fpsCounter)

$liveReadPerSecondCounter = New-Object System.Diagnostics.CounterCreationData
$liveReadPerSecondCounter.CounterName = "LiveReadPerSecond"
$liveReadPerSecondCounter.CounterType = [System.Diagnostics.PerformanceCounterType]::RateOfCountsPerSecond32
$liveReadPerSecondCounter.CounterHelp = "LiveReadPerSecondHelp"
$counterCollection.Add($liveReadPerSecondCounter)

$grabbedImagePerSecondCounter = New-Object System.Diagnostics.CounterCreationData
$grabbedImagePerSecondCounter.CounterName = "GrabbedImagePerSecond"
$grabbedImagePerSecondCounter.CounterType = [System.Diagnostics.PerformanceCounterType]::RateOfCountsPerSecond32
$grabbedImagePerSecondCounter.CounterHelp = "GrabbedImagePerSecondHelp"
$counterCollection.Add($grabbedImagePerSecondCounter)

$exposureSetsPerSecondCounter = New-Object System.Diagnostics.CounterCreationData
$exposureSetsPerSecondCounter.CounterName = "ExposureSetPerSecond"
$exposureSetsPerSecondCounter.CounterType = [System.Diagnostics.PerformanceCounterType]::RateOfCountsPerSecond32
$exposureSetsPerSecondCounter.CounterHelp = "ExposureSetPerSecondHelp"
$counterCollection.Add($exposureSetsPerSecondCounter)

$synchronizationAttemptsCounter = New-Object System.Diagnostics.CounterCreationData
$synchronizationAttemptsCounter.CounterName = "SyncsPerSecond"
$synchronizationAttemptsCounter.CounterType = [System.Diagnostics.PerformanceCounterType]::RateOfCountsPerSecond32
$synchronizationAttemptsCounter.CounterHelp = "SyncsPerSecondHelp"
$counterCollection.Add($synchronizationAttemptsCounter)

# Create performance counter category linked to $counterCollection
[System.Diagnostics.PerformanceCounterCategory]::Create($CatName, $CatHelp, $CatType, $counterCollection)

New-Object System.Diagnostics.PerformanceCounter($CatName, $bestPlatePerSecondCounter.CounterName, $TotalInstance, $false)
New-Object System.Diagnostics.PerformanceCounter($CatName, $bestPlatePerSecondCounter.CounterName, $LprInstance, $false)
New-Object System.Diagnostics.PerformanceCounter($CatName, $bestPlatePerSecondCounter.CounterName, $ContextInstance, $false)

New-Object System.Diagnostics.PerformanceCounter($CatName, $processingStartedPerSecondCounter.CounterName, $TotalInstance, $false)
New-Object System.Diagnostics.PerformanceCounter($CatName, $processingStartedPerSecondCounter.CounterName, $LprInstance, $false)
New-Object System.Diagnostics.PerformanceCounter($CatName, $processingStartedPerSecondCounter.CounterName, $ContextInstance, $false)

New-Object System.Diagnostics.PerformanceCounter($CatName, $fpsCounter.CounterName, $TotalInstance, $false)
New-Object System.Diagnostics.PerformanceCounter($CatName, $fpsCounter.CounterName, $LprInstance, $false)
New-Object System.Diagnostics.PerformanceCounter($CatName, $fpsCounter.CounterName, $ContextInstance, $false)

New-Object System.Diagnostics.PerformanceCounter($CatName, $liveReadPerSecondCounter.CounterName, $TotalInstance, $false)
New-Object System.Diagnostics.PerformanceCounter($CatName, $liveReadPerSecondCounter.CounterName, $LprInstance, $false)
New-Object System.Diagnostics.PerformanceCounter($CatName, $liveReadPerSecondCounter.CounterName, $ContextInstance, $false)

New-Object System.Diagnostics.PerformanceCounter($CatName, $grabbedImagePerSecondCounter.CounterName, $LprInstance, $false)
New-Object System.Diagnostics.PerformanceCounter($CatName, $grabbedImagePerSecondCounter.CounterName, $ContextInstance, $false)

New-Object System.Diagnostics.PerformanceCounter($CatName, $exposureSetsPerSecondCounter.CounterName, $TotalInstance, $false)

New-Object System.Diagnostics.PerformanceCounter($CatName, $synchronizationAttemptsCounter.CounterName, $TotalInstance, $false)
