﻿//
// Controller for dynamically changing the page culture (language).
// Based on Angular-translate.
//

(function () {
    "use strict";

    angular
        .module("portalApp.controllers")
        .controller("LanguageSwitchController", LanguageSwitchController);

    LanguageSwitchController.$inject = ["$scope", "$rootScope", "$translate", "localizationFactory", "moment", "$window"];

    function LanguageSwitchController($scope, $rootScope, $translate, localizationFactory, moment, $window) {

        // Register callback for 'changeLanguage'
        $scope.changeLanguage = function (langKey) {
            $window.sessionStorage.currentLanguage = langKey;
            $translate.use(langKey);
            moment.locale(langKey);
        };

        $scope.currentLanguageText = function () {
            return localizationFactory.getData().text;
        }

        $scope.supportedLanguages = function() {
            return localizationFactory.getSupportedLanguages();
        }

        // Register callback for $translate.use()
        $rootScope.$on("$translateChangeSuccess", function(event, data) {
            var language = data.language;            
            localizationFactory.updateData(language);
        });
    }
})();