﻿//
// Handles login request to the back-end and UI interaction
//

(function() {
    "use strict";

    angular
        .module("portalApp.controllers")
        .controller("LoginController", LoginController);

    LoginController.$inject = ["$state", "portalService", "focusFactory", "$rootScope", "$timeout", "$window"];

    function LoginController($state, portalService, focusFactory, $rootScope, $timeout, $window) {
        var vm = this;
        vm.title = "LoginController";
        vm.username = "";
        vm.password = "";
        vm.isLoginPending = false;
        vm.canReset = false;
        vm.login = login;        
        vm.loginClass = 'show-opacity';
        $rootScope.bodyClass = "login";

        ///////////////////////////////////////////////

        portalService.canReset()
            .then(function(response) {
                vm.canReset = response.data;
            });

        // Callback for requesting a login
        function login() {                                    

            vm.error = false; // clear previous error if any

            // username cannot be empty
            if (vm.username === "")
            {
                vm.error = true; // !!!should be more descriptive about the error
                vm.username = "";
                vm.password = "";
                return;
            }

            // Start remote query
            vm.isLoginPending = true;
            portalService.login(vm.username, vm.password)
                .then(function(response) {
                    // success, redirect to properties page
                    $window.sessionStorage.accessToken = response.data.accessToken;
                    $window.sessionStorage.refreshToken = response.data.refreshToken;
                    vm.error = false;

                    if (response.data.isOutOfBox) {
                        $state.go("outofbox.welcome");
                    } else {
                        $state.go("main.dashboard.overview");
                        $timeout(function () {
                            $rootScope.bodyClass = "";
                        }, 2000);
                    }
                    
                }, function() {
                    //failure
                    // Erase the token if the user fails to login
                    delete $window.sessionStorage.accessToken;
                    delete $window.sessionStorage.refreshToken;
                    focusFactory.doFocus("password");
                    vm.error = true;
                }).finally(function () {
                    if (vm.error) {
                        vm.isLoginPending = false;
                        vm.password = ""; //reset password box
                    }                                                            
            });
        }        
    }
})();

