﻿(function () {
    'use strict';

    angular
        .module('portalApp.controllers')
        .controller('MainController', MainController);

    MainController.$inject = ["localizationFactory", "$window", "$state", "$rootScope", "$translate", "portalService", "$timeout", "$scope", "$q"];

    var lastOpenedHelpWindow = null;
    var lastOpened3RdpartyWindow = null;

    function MainController(localizationFactory, $window, $state, $rootScope, $translate, portalService, $timeout, $scope, $q) {
        var vm = this;
        vm.title = "MainController";
        vm.setActiveMainMenu = setActiveMainMenu;
        vm.topMenuClasses = {};
        vm.topMenuClasses['main.dashboard'] = '';
        vm.topMenuClasses['main.config'] = '';
        vm.topMenuClasses['main.diagnostics'] = '';
        vm.topMenuClasses['main.help'] = '';

        vm.sideMenuClasses = {};
        vm.sideMenuClasses['main.dashboard.overview'] = '';
        vm.sideMenuClasses['main.dashboard.cameraFeeds'] = '';
        vm.sideMenuClasses['main.config.network'] = '';        
        vm.sideMenuClasses['main.config.security'] = '';
        vm.sideMenuClasses['main.config.zoomAndFocus'] = '';
        vm.sideMenuClasses['main.config.cameras'] = '';
        vm.sideMenuClasses['main.config.lpr.analytics'] = '';
        vm.sideMenuClasses['main.config.lpr.vehicleDetection'] = '';
        vm.sideMenuClasses['main.config.lpr.speedEstimation'] = '';
        vm.sideMenuClasses['main.config.extension'] = '';
        vm.sideMenuClasses['main.config.general.time'] = '';
        vm.sideMenuClasses['main.config.general.power'] = '';
        vm.sideMenuClasses['main.config.maintenance'] = '';

        vm.collapsableSideMenuClasses = {};
        vm.collapsableSideMenuClasses['main.config.general'] = 'collapse';
        vm.collapsableSideMenuClasses['main.config.lpr'] = 'collapse';

        vm.currentLanguage = getCurrentLanguage;
        vm.help = getHelp;
        vm.get3rdParty = get3RdPartyDescription;
        vm.stickyAlertClass = '';
        vm.stickyAlertLabel = '';
        vm.watchIsLoading = undefined;
        vm.watchConnectionLost = undefined;
        vm.watchConnectionRetry = undefined;
        vm.unregisterConnected = undefined;
        vm.unregisterConnectionLost = undefined;
        vm.unregisterReconnecting = undefined;
        vm.unregisterLoading = undefined;
        vm.showLoading = showLoading;
        vm.showConnectionLost = showConnectionLost;
        vm.showReconnecting = showReconnecting;
        var stickyAlertDangerShow = 'alert alert-danger alert-sticky fade in';
        var stickyAlertDangerHide = 'alert alert-danger alert-sticky fade';
        var stickyAlertInfoShow = 'alert alert-info alert-sticky fade in';
        var stickyAlertInfoHide = 'alert alert-info alert-sticky fade';
        var stickyAlertWarningShow = 'alert alert-warning alert-sticky fade in';
        var stickyAlertWarningHide = 'alert alert-warning alert-sticky fade';
        var stickyAlertHidden = 'hidden';
        vm.stickyAlertClass = stickyAlertHidden;
        vm.showSpinner = false;        
        var defer = $q.defer();

        $scope.$on('$destroy', function () {
            vm.watchIsLoading();
            vm.watchConnectionLost();
            vm.watchConnectionRetry();
            vm.unregisterConnectionLost();
            vm.unregisterConnected();
            vm.unregisterReconnecting();
            vm.unregisterLoading();
            vm.unregisterConfigChanged();
            vm.unregisterStateChangeSuccess();
        });

        vm.unregisterConfigChanged = $rootScope.$on("userConfigChanged", function (event, userConfig) {
            evaluateUserConfigChange(userConfig);
        });

        activate()
            .then(function (response) {
                var userConfig = response.data;
                evaluateUserConfigChange(userConfig);
            });

        function activate() {
            vm.setActiveMainMenu($state.current);
            var request = function () {
                if ($rootScope.reconnecting)
                    $rootScope.activated = true;

                portalService.getUserConfig()
                    .then(function success(response) {
                        $rootScope.isLoading = false;
                        $rootScope.connectionLost = false;
                        $rootScope.reconnecting = false;
                        $rootScope.activated = true;

                        defer.resolve(response);
                    }, function error() {
                        $rootScope.connectionLost = true;
                        $rootScope.isLoading = false;
                        // wait 5 seconds then retry
                        vm.timeout = $timeout(function () {
                            $rootScope.reconnecting = true;
                            request();
                        }, 5000);
                    });
            }

            request();

            return defer.promise;
        }

        function evaluateUserConfigChange(userConfig) {
            //vm.virtualLoopEnabled = (userConfig.ReadModeConfiguration.ReadingMode === 3);
            //vm.speedEstimationEnabled = userConfig.EstimateVehicleSpeed;
        }

        function showLoading(delay) {
            $timeout(function () {
                if ($rootScope.isLoading && !$rootScope.connectionLost && !$rootScope.reconnecting) {
                    vm.showSpinner = false;
                    vm.stickyAlertClass = stickyAlertInfoShow;
                    vm.stickyAlertLabel = 'STE_LABEL_Loading';
                }
            }, delay);            
        }

        function showConnectionLost(delay) {
            $timeout(function () {
                if ($rootScope.connectionLost && !$rootScope.reconnecting) {
                    vm.showSpinner = false;
                    vm.stickyAlertClass = stickyAlertDangerShow;
                    vm.stickyAlertLabel = 'STE_SENTENCE_ConnectionLost';
                }
            }, delay);
            
        }

        function showReconnecting(delay) {
            $timeout(function () {
                if ($rootScope.reconnecting) {
                    vm.stickyAlertClass = stickyAlertWarningShow;
                    vm.showSpinner = true;
                    vm.stickyAlertLabel = 'STE_SENTENCE_ConnectionRetry';
                }
            }, delay);
        }

        function handleConnectionLost() {
            if ($rootScope.connectionLost) {
                if (!$rootScope.reconnecting && !$rootScope.isLoading) {
                    if (vm.stickyAlertClass === stickyAlertInfoShow) {
                        // before switching class, we fade previous
                        vm.stickyAlertClass = stickyAlertInfoHide;
                    }

                    vm.showConnectionLost(500);
                }

            } else if (vm.stickyAlertClass === stickyAlertDangerShow) {
                vm.stickyAlertClass = stickyAlertDangerHide;
            }
        }

        function handleReconnecting() {
            if ($rootScope.reconnecting) {
                if (vm.stickyAlertClass === stickyAlertDangerShow) {
                    // before switching class, we fade previous
                    vm.stickyAlertClass = stickyAlertDangerHide;
                }

                vm.showReconnecting(500);

            } else if (vm.stickyAlertClass === stickyAlertWarningShow) {
                vm.stickyAlertClass = stickyAlertWarningHide;
                vm.showSpinner = false;
            }
        }

        vm.watchIsLoading = $rootScope.$watch('isLoading', function () {
            if ($rootScope.isLoading) {
                var delay = 500;
                if (!$rootScope.connectionLost && !$rootScope.reconnecting) {
                    vm.showLoading(delay);
                }
            } else if (vm.stickyAlertClass !== stickyAlertDangerHide || vm.stickyAlertClass !== stickyAlertHidden) {
                vm.stickyAlertClass = stickyAlertDangerHide;
            }
        });

        vm.watchConnectionLost = $rootScope.$watch('connectionLost', function () {
            handleConnectionLost();
        });

        vm.watchConnectionRetry = $rootScope.$watch('reconnecting', function () {
            handleReconnecting();
        });

        vm.unregisterLoading = $rootScope.$on("Loading", function (e, message) {
            $rootScope.$apply(function () {
                $rootScope.isLoading = true;
                $rootScope.connectionLost = false;
                $rootScope.reconnecting = false;
            });
        });

        vm.unregisterConnectionLost = $rootScope.$on("LostConnection", function (e, message) {
            if ($rootScope.activated) {
                $rootScope.connectionLost = true;
                handleConnectionLost();
            }
        });

        vm.unregisterConnected = $rootScope.$on("Connected", function (e, message) {
            $rootScope.$apply(function () {
                $rootScope.isLoading = false;
                $rootScope.connectionLost = false;
                $rootScope.reconnecting = false;                
            });
        });

        vm.unregisterReconnecting = $rootScope.$on("Reconnecting", function (e, message) {
            if ($rootScope.activated) {
                $rootScope.reconnecting = true;
                handleReconnecting();
            }            
        });

        vm.unregisterStateChangeSuccess = $rootScope.$on('$stateChangeSuccess', function (event, toState, toParams, fromState, fromParams) {
            vm.setActiveMainMenu(toState);            
        });

        function setActiveMainMenu(state) {
            // the following states need partial match to set active top menu
            angular.forEach(vm.topMenuClasses, function (value, key) {
                if (state.name.indexOf(key) > -1)
                    vm.topMenuClasses[key] = 'active';
                else
                    vm.topMenuClasses[key] = '';
            });

            // the following states need exact match to set active side menu
            angular.forEach(vm.sideMenuClasses, function (value, key) {
                if (state.name === key)
                    vm.sideMenuClasses[key] = 'active';
                else
                    vm.sideMenuClasses[key] = '';
            });            

            // the following states need partial match to set collapse in collapsable side menu
            angular.forEach(vm.collapsableSideMenuClasses, function (value, key) {
                if (state.name.indexOf(key) > -1)
                    vm.collapsableSideMenuClasses[key] = 'collapse in';
                else
                    vm.collapsableSideMenuClasses[key] = 'collapse';
            });            
        }

        // Displaying help file in a secondary window.
        // The Browser may block pop-up window (in which case it has to be disabled)
        function getHelp() {
            // Pick the proper language for help
            var prefix = localizationFactory.getData().langKey;

            // Note: there is no version number in the help file name.
            // This has to be removed from the TechDoc naming scheme.
            var filename = prefix.toUpperCase() + ".Sharp Admin Guide.pdf";

            // Make sure there is always only one instance of the help.
            // Works only for window spawned from current session.
            if (lastOpenedHelpWindow && !lastOpenedHelpWindow.closed)
                lastOpenedHelpWindow.close();

            // Open new page in the desired language (this is a customization of the original pdf.js package)
            var encodedUri = "web/viewer.html?" + "lang=" + prefix + "&file=" + encodeURIComponent("../help/" + filename) + "#pagemode=bookmarks";
            var win = $window.open(encodedUri, "_blank");
            lastOpenedHelpWindow = win;
            
            if (!lastOpenedHelpWindow || lastOpenedHelpWindow.closed)
                $window.alert($translate.instant("STE_SENTENCE_CouldNotOpenHelpFile"));
        };

        // Displaying third party software descriptions in a secondary window.
        // The Browser may block pop-up window (in which case it has to be disabled)
        function get3RdPartyDescription() {
            // Pick the proper language for help
            var prefix = localizationFactory.getData().langKey;

            // Note: there is no version number in the help file name.
            // This has to be removed from the TechDoc naming scheme.
            var filename = prefix.toUpperCase() + "-Genetec-Third-Party-Computer-Software-AutoVu ALPR cameras.pdf";

            // Make sure there is always only one instance of the help.
            // Works only for window spawned from current session.
            if (lastOpened3RdpartyWindow && !lastOpened3RdpartyWindow.closed)
                lastOpened3RdpartyWindow.close();

            // Open new page in the desired language (this is a customization of the original pdf.js package)
            var encodedUri = "web/viewer.html?" + "lang=" + prefix + "&file=" + encodeURIComponent("../help/" + filename);
            var win = $window.open(encodedUri, "_blank");
            lastOpened3RdpartyWindow = win;

            if (!lastOpened3RdpartyWindow || lastOpened3RdpartyWindow.closed)
                $window.alert($translate.instant("STE_SENTENCE_CouldNotOpenPage"));
        };

        function getCurrentLanguage() {
              return localizationFactory.getData().langKey.toLowerCase();
        }
    }
})();
