﻿(function () {
    "use strict";

    angular
        .module("portalApp.controllers")
        .controller("OutOfBoxController", OutOfBoxController);

    OutOfBoxController.$inject = ["portalService", "userConfig", "signalRService", "$scope", "$timeout", "$rootScope", "$window", "$state", "$translate"];

    function OutOfBoxController(portalService, userConfig, signalRService, $scope, $timeout, $rootScope, $window, $state, $translate) {
        var vm = this;
        vm.title = "OutOfBoxController";
        vm.passwordStrength = { score: 0 };
        vm.newPassword1 = undefined;
        vm.newPassword2 = undefined;
        vm.show = false;
        vm.isSaving = false;
        vm.alert = {};
        vm.save = save;
        vm.showAlert = showAlert;
        vm.userConfig = userConfig.data;
        vm.logoPath = "https://" + location.hostname + "/images/logo-sharpv-autovu.png";
        vm.nextState = nextState;
        vm.previousState = previousState;

        $rootScope.bodyClass = "login";

        $scope.$on('$destroy', function () {
        });

        // type can be: success, info, warning or danger
        // msg is a string
        function showAlert(type, msg, useTimeout) {
            vm.showMessage = true;
            vm.message = { type: type, text: msg }
            if (useTimeout) {
                $timeout(function () {
                    vm.showMessage = false;
                }, 5000);
            }
        }

        function save() {
            var result = zxcvbn(vm.newPassword2);

            // Make sure the password meets the minimum criteria before submitting as the server can't do much on a hashed password.
            if (result.score < 3) {
                vm.showAlert('danger', $translate.instant("STE_SENTENCE_CannotSetNewPassword"));
                return;
            }

            vm.isSaving = true;

            portalService.setUserConfig(vm.userConfig)
                .then(function () {
                    // password will not be set if config change failed
                    portalService.setPassword(vm.newPassword2)
                        .then(function () {
                            // success                 
                            $state.go("outofbox.completed");
                        }, function () {
                            // failed
                            vm.showAlert("text-danger", "STE_SENTENCE_CouldNotSetNewPassword", false);
                            vm.isSaving = false;
                        });
                }, function () {
                    //failed
                    showProperErrorMessage();
                }).finally(function () {
                    vm.isSaving = false;
                });
        }

        function showProperErrorMessage() {
            portalService.getPoePlusStatus()
                .then(function (response) {
                    if (response.data === "False") {
                        vm.showAlert("text-danger", $translate.instant("STE_SENTENCE_WrongPowerSupply"), false);                       
                    } else {
                        vm.showAlert("text-danger", $translate.instant("STE_SENTENCE_CouldNotSaveConfiguration"), false);
                    }
                });
        }


        function nextState() {
            if ($state.current.name === "outofbox.welcome")
                $state.go("outofbox.power");
            else if ($state.current.name === "outofbox.power") {
                $state.go("outofbox.password");
            } else if ($state.current.name === "outofbox.completed") {
                delete $window.sessionStorage.accessToken;
                delete $window.sessionStorage.refreshToken;
                $state.go("login");
            }
        }

        function previousState() {
            if ($state.current.name === "outofbox.password") {
                vm.newPassword1 = '';
                vm.newPassword2 = '';
                $state.go("outofbox.power");
            }
            else if ($state.current.name === "outofbox.power") {
                $state.go("outofbox.welcome");
            }
        }
    }
})();