﻿(function () {
    "use strict";

    angular
        .module("portalApp.controllers")
        .controller("AnalyticsController", AnalyticsController);

    AnalyticsController.$inject = ["portalService", "$rootScope", "$scope", "$timeout", "$uibModal", "$translate", "checkChangeStateFactory", "$q", "$state"];

    function AnalyticsController(portalService, $rootScope, $scope, $timeout, $uibModal, $translate, checkChangeStateFactory, $q, $state) {
        var vm = this;
        vm.title = "AnalyticsController";        
        vm.save = save;
        vm.cancel = cancel;
        vm.isSaving = false;
        vm.show = false;
        vm.alert = {};
        vm.show2 = false;
        vm.alert2 = {};
        vm.closeAlert = closeAlert;
        vm.closeAlert2 = closeAlert2;
        vm.closeModal = closeModal;
        vm.updateContext = updateContext;
        vm.getStateFromBoolean = getStateFromBoolean;
        vm.readingModes = [
            { id: 0, name: "STE_LABEL_ContinuousReadingMode" },
            // available only if not ITS { id: 3, name: $translate.instant("STE_LABEL_ContinuousWithVehicleDetectionReadingMode") },
            { id: 1, name: "STE_LABEL_ConditionalReadingMode" },
            { id: 2, name: "STE_LABEL_SingleReadOnTrigger" }
        ];
        vm.readStrategies = [
            { id: 0, name: "STE_LABEL_FastMovingPlate" },
            { id: 1, name: "STE_LABEL_GateControl" },
            { id: 3, name: "STE_LABEL_SlowMovingPlate" }
        ];
        var imagesRoot = "../../images/";
        vm.readCameraOrientation = [
            { id: 0, approachingFlow: "STE_LABEL_CameraOrientation_Unknown", movingAwayFlow: "", image: imagesRoot + "CameraOrientationUnknown.png"},
            { id: 1, approachingFlow: "STE_LABEL_CameraOrientation_FlowRightToLeft", movingAwayFlow: "STE_LABEL_CameraOrientation_FlowLeftToRight", image: imagesRoot + "CameraOrientationLeft.png"},
            { id: 2, approachingFlow: "STE_LABEL_CameraOrientation_FlowTopToBottom", movingAwayFlow: "STE_LABEL_CameraOrientation_FlowBottomToTop", image: imagesRoot + "CameraOrientationStraight.png"},
            { id: 3, approachingFlow: "STE_LABEL_CameraOrientation_FlowLeftToRight", movingAwayFlow: "STE_LABEL_CameraOrientation_FlowRightToLeft", image: imagesRoot + "CameraOrientationRight.png"}
        ];
        vm.inputNames = []; 
        vm.timelines = [
            { id: 0, name: "STE_CAPTIVE_SENTENCE_FRAGMENT_Before" },
            { id: 1, name: "STE_CAPTIVE_SENTENCE_FRAGMENT_After" }
        ];
        vm.getReadingModeName = getReadingModeName;
        vm.getTriggerSourceName = getTriggerSourceName;
        vm.getSourceTypeName = getSourceTypeName;
        vm.updateReadingMode = updateReadingMode;
        vm.getReadStrategyName = getReadStrategyName;
        vm.getCameraOrientationApproachingFlow = getCameraOrientationApproachingFlow;
        vm.getCameraOrientationMovingAwayFlow = getCameraOrientationMovingAwayFlow;
        vm.updateStrategy = updateStrategy;
        vm.updateCameraOrientation = updateCameraOrientation;
        vm.setInputs = setInputs;
        vm.updateReadingModeInput = updateReadingModeInput; // for conditional trigger
        vm.updateReadingModeState = updateReadingModeState; // for conditional trigger
        vm.addReadTrigger = addReadTrigger; // for single read trigger
        vm.deleteReadTrigger = deleteReadTrigger; // for single read trigger
        vm.editReadTrigger = editReadTrigger; // for single read trigger
        vm.updateReadTriggerSource = updateReadTriggerSource; // for single read trigger dialog
        vm.updateReadTriggerState = updateReadTriggerState; // for single read trigger dialog
        vm.updateReadTriggerStartOffset = updateReadTriggerStartOffset; // for single read trigger dialog
        vm.updateReadTriggerStartingTimeline = updateReadTriggerStartingTimeline; // for single read trigger dialog
        vm.captureWindowDescription = captureWindowDescription;
        vm.noPlateCaptureDescription = noPlateCaptureDescription;
        vm.applyReadTriggerChanges = applyReadTriggerChanges;

        vm.timeout = undefined;

        $rootScope.isLoading = true;
        $rootScope.connectionLost = false;
        $rootScope.reconnecting = false;
        $rootScope.activated = false;

        var defer = $q.defer();
        // Clean up to perform when the view is closed
        $scope.$on("$destroy", function () {
            $timeout.cancel(vm.timeout);
            defer.reject();
        });

        activate()
            .then(function (response) {
                vm.lprContexts = response.data;
                
                // Add internal information to the existing user config values in case we edit them
                vm.userConfig.ReadModeConfiguration.SingleReadTriggers.forEach(function (rt) {
                    prepareReadTrigger(rt);
            });
        });

        function activate() {
            var request = function () {
                if ($rootScope.reconnecting)
                    $rootScope.activated = true;

                portalService.getUserConfig()
                    .then(function success(response) {
                        vm.userConfig = response.data;
                        if (vm.userConfig.LprContextName == null)
                            vm.userConfig.LprContextName = ""; // make sure not to display 'null' in the UI
                        // create a copy for canceling
                        vm.backup = angular.copy(vm.userConfig);

                        portalService.getProperties()
                            .then(function (response) {
                                vm.dto = response.data;
                                //if (!vm.dto.IsIts) {
                                //    vm.readingModes.splice(1, 0, { id: 3, name: $translate.instant("STE_LABEL_ContinuousWithVehicleDetectionReadingMode") }) // allowed only if not a Sharp ITS
                                //}
                                vm.setInputs();
                                portalService.getLprContexts()
                                    .then(function success(response) {
                                        $rootScope.isLoading = false;
                                        $rootScope.connectionLost = false;
                                        $rootScope.reconnecting = false;
                                        $rootScope.activated = true;
                                        // we resolve only when the second call is completed
                                        defer.resolve(response);
                                    }, function error() {
                                        $rootScope.connectionLost = true;
                                        $rootScope.isLoading = false;
                                        // wait 5 seconds then retry
                                        vm.timeout = $timeout(function () {
                                            $rootScope.reconnecting = true;
                                            request();
                                        }, 5000);
                                    });
                            }, function error() {
                                $rootScope.connectionLost = true;
                                $rootScope.isLoading = false;
                                // wait 5 seconds then retry
                                vm.timeout = $timeout(function () {
                                    $rootScope.reconnecting = true;
                                    request();
                                }, 5000);
                            });                        
                    }, function error() {
                        $rootScope.connectionLost = true;
                        $rootScope.isLoading = false;
                        // wait 5 seconds then retry
                        vm.timeout = $timeout(function () {
                            $rootScope.reconnecting = true;
                            request();
                        }, 5000);
                    });
            }

            request();

            return defer.promise;
        }

        checkChangeStateFactory.checkFormOnStateChange($scope);

        function setInputs() {
            angular.forEach(vm.dto.Inputs, function (value, name) {
                this.push(name);
            }, vm.inputNames);
            vm.sourceTypes = [{ id: 0, type: 1, name: $translate.instant("STE_LABEL_ExternalSource") }, { id: 1, type: 0, name: vm.inputNames[0] }, { id: 2, type: 0, name: vm.inputNames[1] }];
        }

        function getStateFromBoolean(value) {
            if (value)
                return $translate.instant("STE_LABEL_InputStateOn");
            else
                return $translate.instant("STE_LABEL_InputStateOff");
        }

        function getTimeline(id) {
            for (var i = 0; i < vm.timelines.length; i++)
                if (vm.timelines[i].id === id)
                    return vm.timelines[i];

            return null;
        }

        function getTriggerSourceName(readTrigger) {
            return getSourceTypeName(readTrigger.SourceType, readTrigger.SourceName);
        }

        function getSourceTypeName(type, name) {
            if (type === 0) { // input
                return "{0} {1}".format($translate.instant("STE_LABEL_Input"), name);
            }

            return $translate.instant("STE_LABEL_ExternalSource");
        }

        function getReadingModeName(index) {
            if (index === 3) {
                return "STE_LABEL_ContinuousWithVehicleDetectionReadingMode"; // Ugly but necessary, index 0 is continuous whereas index 3 is continuous with virtual loop
            }
            for (var i = 0; i < vm.readingModes.length; i++) {
                if (vm.readingModes[i].id === index)
                    return vm.readingModes[i].name;
            }
            return "???"; // not found
        }

        function getReadStrategyName(index) {
            for (var i = 0; i < vm.readStrategies.length; i++) {
                if (vm.readStrategies[i].id === index)
                    return vm.readStrategies[i].name;
            }
            if (index === 2)
                return "Plate Spinner";
            return "???"; // not found
        }

        function getCameraOrientationApproachingFlow(index) {
            // Start at 1 so we don't concatenate direction of travel when direction is unknown
            for (var i = 1; i < vm.readCameraOrientation.length; i++) {
                if (vm.readCameraOrientation[i].id === index)
                    return $translate.instant("STE_LABEL_CameraOrientation_Approaching") + ": " + $translate.instant(vm.readCameraOrientation[i].approachingFlow);
            }
            return $translate.instant(vm.readCameraOrientation[0].approachingFlow);
        }

        function getCameraOrientationMovingAwayFlow(index) {
            // Start at 1 so we don't concatenate direction of travel when direction is unknown
            for (var i = 1; i < vm.readCameraOrientation.length; i++) {
                if (vm.readCameraOrientation[i].id === index)
                    return $translate.instant("STE_LABEL_CameraOrientation_MovingAway") + ": " + $translate.instant(vm.readCameraOrientation[i].movingAwayFlow);
            }
            return $translate.instant(vm.readCameraOrientation[0].movingAwayFlow);
        }

        function updateReadingMode(readingMode) {
            vm.userConfig.ReadModeConfiguration.ReadingMode = readingMode.id;
            vm.form.$setDirty(); // set dirty flag
        }

        function updateStrategy(strategy) {
            vm.userConfig.ReadStrategy = strategy.id;
            vm.form.$setDirty(); // set dirty flag
        }

        function updateCameraOrientation(cameraOrientation) {
            for (var i = 0; i < vm.readCameraOrientation.length; i++) {
                if (vm.readCameraOrientation[i].id === cameraOrientation.id) {
                    vm.userConfig.CameraOrientation = cameraOrientation.id;
                    vm.form.$setDirty();
                }
            }
        }

        function updateReadingModeInput(name) {
            vm.userConfig.ReadModeConfiguration.ConditionalReadConfiguration.InputName = name;
            vm.form.$setDirty(); // set dirty flag
        }

        function updateReadingModeState(state) {
            vm.userConfig.ReadModeConfiguration.ConditionalReadConfiguration.InputState = state;
            vm.form.$setDirty(); // set dirty flag
        }

        function addReadTrigger() {
            var newReadTrigger = {

                SourceType: 1, // TriggerSourceType.Input: input == 0, external == 1
                SourceName: "External", // input name if sourcetype is 'input' (0) or "External" if sourcetype  is 'external' (1)

                LprCameraName: "Lpr Camera", // applies only if sourcetype is 'input' (0)
                InputState: false, // applies only if sourcetype is 'input' (0)

                CaptureStartOffset: -500, // negative is 'before', positive is 'after'
                CaptureDuration: 1000,
                NoPlateFrameTime: 0,
                UseLprImageAsContextImage: vm.dto.IsIts
            };
            prepareReadTrigger(newReadTrigger);

            // Pop a dialog with new data and add to the list when completed
            openReadTrigger(newReadTrigger, true);
        }

        function prepareReadTrigger(rt) {
            
            rt._unsignedCaptureStartOffset = Math.abs(rt.CaptureStartOffset); // added property which contrary to the CaptureStartOffset will always be positive
            
            if (rt.CaptureStartOffset < 0)
                rt._timeline = getTimeline(0); // before
            else
                rt._timeline = getTimeline(1); // after
        }

        function editReadTrigger(index) {
            // Create a temporary object copied from current array
            var rt = angular.copy(vm.userConfig.ReadModeConfiguration.SingleReadTriggers[index]);
            prepareReadTrigger(rt);

            // Pop a dialog with temporary data and update the list when completed (in applyReadTriggerChanges)
            openReadTrigger(rt, false);
        }

        function deleteReadTrigger(index) {
            //!!!should we prompt the user for confirmation?
            if (index >= 0 && index < vm.userConfig.ReadModeConfiguration.SingleReadTriggers.length) {
                vm.userConfig.ReadModeConfiguration.SingleReadTriggers.splice(index, 1);
                vm.form.$setDirty(); // set dirty flag
            }
        }

        function updateContext(context, form) {
            vm.userConfig.LprContextName = context;
            vm.form.$setDirty(); // set dirty flag
        }

        function closeModal() {
            vm.modalInstance.close();
        }

        function openReadTrigger(readTrigger, isNew) {
            closeAlert2(); // clear any previous error
            vm.modalInstance = $uibModal.open({
                templateUrl: "readTrigger.html",
                size: "md",
                scope: $scope,
                resolve: {
                    readTrigger: function () {
                        $scope.readTrigger = readTrigger;
                    },
                    isNew: function () {
                        $scope.isNew = isNew;
                    }
                }
            });
        }

        function updateReadTriggerSource(source) {
            $scope.readTrigger.SourceType = source.type;
            $scope.readTrigger.SourceName = source.name;
        }

        function updateReadTriggerState(state) {
            if ($scope.readTrigger.SourceType === 0) // input
                $scope.readTrigger.InputState = state;
        }

        function updateReadTriggerStartingTimeline(timeline) {
            vm.form.readTriggerForm.$setDirty();
            $scope.readTrigger._timeline = timeline;
            switch (timeline.id) {
                case 0: // before
                    $scope.readTrigger.CaptureStartOffset = -$scope.readTrigger._unsignedCaptureStartOffset;
                    // should set the readTriggerForm dirty flag here, but since it is not available, do it in the HTML file
                    break;

                case 1: // after
                    $scope.readTrigger.CaptureStartOffset = $scope.readTrigger._unsignedCaptureStartOffset;
                    // should set the readTriggerForm dirty flag here, but since it is not available, do it in the HTML file
                    break;

                default:
                    break;
            }
        }

        function updateReadTriggerStartOffset() {
            // Keep CaptureStartOffet aligned with _unsignedCaptureStartOffset
            if ($scope.readTrigger._timeline.id === 0) // before
                $scope.readTrigger.CaptureStartOffset = -$scope.readTrigger._unsignedCaptureStartOffset;
            else //after
                $scope.readTrigger.CaptureStartOffset = $scope.readTrigger._unsignedCaptureStartOffset;
        }

        function captureWindowDescription(trigger) {
            return $translate.instant("STE_LABEL_StartXmsYTriggerAndLastforZms").format(trigger._unsignedCaptureStartOffset,
                $translate.instant(trigger._timeline.name), trigger.CaptureDuration);
        }

        function noPlateCaptureDescription(trigger) {
            return $translate.instant("STE_LABEL_XmsAfterTrigger").format(trigger.NoPlateFrameTime);
        }     

        function showStickyAlert(type, msg) {
            vm.alert = { type: type, msg: msg }
            vm.show = true;
            $timeout(function () {
                vm.show = false;
            }, 5000);
        }

        // Apply the temporary object in the list of triggers after validating it.
        // If invalid, return false, else return true.
        function applyReadTriggerChanges(readTrigger) {

            closeAlert2();

            var index = -1;
            for (var i = 0; i < vm.userConfig.ReadModeConfiguration.SingleReadTriggers.length; i++) {
                var rt = vm.userConfig.ReadModeConfiguration.SingleReadTriggers[i];
                if (rt.SourceType === readTrigger.SourceType && rt.SourceName === readTrigger.SourceName && rt.InputState === readTrigger.InputState) {
                    index = i;
                    break;
                }
            }

            // If the readTrigger is new, add it to the list (if valid), otherwise update it in the array.
            if ($scope.isNew) {
                // Make sure the object can be saved (no conflict with existing entries)
                if (index >= 0) {
                    vm.alert2 = { type: 'warning', msg: $translate.instant("STE_SENTENCE_DuplicateTrigger") };
                    vm.show2 = true;
                    return false;  // an object with the same type and state already exist, cannot have two
                }
                
                // Add new item in the list
                vm.userConfig.ReadModeConfiguration.SingleReadTriggers.push(readTrigger);
            }
            else {
                // Replace existing one ith 'readTrigger'
                for (var i = 0; i < vm.userConfig.ReadModeConfiguration.SingleReadTriggers.length; i++) {
                    var rt = vm.userConfig.ReadModeConfiguration.SingleReadTriggers[i];
                    if (rt.SourceType === readTrigger.SourceType && rt.SourceName === readTrigger.SourceName && rt.InputState === readTrigger.InputState) {
                        // Replace the object at index i
                        vm.userConfig.ReadModeConfiguration.SingleReadTriggers[i] = readTrigger;
                        break; // done
                    }
                    //!!!TODO: if we don't find the object in the array, ignore it?
                }
            }

            vm.form.$setDirty(); // set dirty flag
            vm.modalInstance.close();

            return true;
        }

        function save() {
            vm.isSaving = true;

            portalService.setUserConfig(vm.userConfig)
                .then(function () {
                    vm.show = true;
                    vm.alert = { type: 'success', msg: $translate.instant("STE_SENTENCE_ConfigurationSavedSuccessfully") }
                    $timeout(function () {
                        vm.show = false;
                    }, 5000);

                    // Advise other views that user config changed with data possibly affecting them (here: vm.userConfig.ReadModeConfiguration.ReadingMode)
                    $rootScope.$emit("userConfigChanged", vm.userConfig);

                    vm.backup = angular.copy(vm.userConfig); // refresh the backup snapshot
                    if (vm.form)
                        vm.form.$setPristine();
                }, function () {
                    //error
                    vm.show = true;
                    vm.alert = { type: 'danger', msg: $translate.instant("STE_SENTENCE_CouldNotSaveConfiguration") }
                    $timeout(function () {
                        vm.show = false;
                    }, 5000);
                }).finally(function () {
                    vm.isSaving = false;
                });
        }

        function cancel() {
            vm.userConfig = angular.copy(vm.backup); // revert to backup values
            // Add internal information to the existing user config values in case we edit them
            vm.userConfig.ReadModeConfiguration.SingleReadTriggers.forEach(function(rt) {
                prepareReadTrigger(rt);
            });
            vm.form.$setPristine();
        }

        function closeAlert() {
            vm.show = false;
        }
        function closeAlert2() {
            vm.show2 = false;
        }
    }
})();
