﻿(function () {
    "use strict";

    var firstClick;

    angular
        .module("portalApp.controllers")
        .controller("CamerasController", CamerasController);

    CamerasController.$inject = ["portalService", "$scope", "$rootScope", "$timeout", "signalRService", "$window",  "$log", "checkChangeStateFactory", "$q", "$translate", "$sce"];

    function CamerasController(portalService, $scope, $rootScope, $timeout, signalRService, $window, $log, checkChangeStateFactory, $q, $translate, $sce) {
        firstClick = true;        
        var vm = this;
        vm.title = "CamerasController";
        
        // camera selection
        vm.cameras = [];
        vm.setCameras = setCameras;
        vm.selectedCamera = undefined;
        vm.selectCamera = selectCamera;
        vm.updateSuggestion = updateSuggestion;

        // Exposure type
        vm.exposureTypes = [];

        vm.selectExposureType = selectExposureType;
        vm.getExposureTypeName = getExposureTypeName;

        // exposure settings
        vm.updateExposureOption = updateExposureOption;
        vm.updateExposureSettings = updateExposureSettings;
        vm.updateExposureSettingsFromScope = updateExposureSettingsFromScope;
        vm.resetFixedGainAndShutterTime = resetFixedGainAndShutterTime;
        vm.resetRangeGainAndShutterTime = resetRangeGainAndShutterTime;
        vm.updateUiFromUserConfig = updateUiFromUserConfig;
        vm.appliedSettingsMin = "";
        vm.appliedSettingsMax = "";

        // Lighting
        vm.lightings = ["Sun", "Cloud", "Fluorescent", "Incandescent", "YellowSpot", "WhiteSpot"]; // all choosable lightings
        vm.lighting = ""; // will be updated in function below
        vm.selectLighting = selectLighting;
        vm.getLightingName = getLightingName;

        // Region of interest
        vm.clearRegion = clearRegion;

        // Alert
        vm.show = false;
        vm.alert = {};

        // Save/cancel
        vm.isSaving = false;
        vm.save = save;
        vm.cancel = cancel;
        vm.saveCompleted = saveCompleted;
        vm.saveUserConfig = saveUserConfig;
        vm.translate = translate;
        vm.userConfig = {}; // Data binding for the video feed directive

        // Extract camera capabilities
        vm.minGain = 0; // will be updated in function extractCapabilities
        vm.maxGain = 0; // will be updated in function extractCapabilities
        vm.minShutterTime = 0; // will be updated in function extractCapabilities
        vm.maxShutterTime = 0; // will be updated in function extractCapabilities

        // Extract camera defaults
        vm.gainDef = 0; // will be updated in function extractDefaultValues
        vm.shutterTimeDef = 0; // will be updated in function extractDefaultValues
        vm.minShutterTimeDef = 0; // will be updated in function extractDefaultValues
        vm.maxShutterTimeDef = 0; // will be updated in function extractDefaultValues
        vm.minGainDef = 0; // will be updated in function extractDefaultValues
        vm.maxGainDef = 0; // will be updated in function extractDefaultValues

        // Image feed buffer and counter
        vm.chunkCount = 0;
        vm.chunks = [];
        vm.ImageCount = 0;

        $rootScope.isLoading = true;
        $rootScope.connectionLost = false;
        $rootScope.reconnecting = false;
        $rootScope.activated = false;

        vm.registeredTranslate = $rootScope.$on('$translateChangeSuccess', function () {
            vm.translate();
        });

        if ($translate.isReady())
            vm.translate();

        var defer = $q.defer();

        activate().then(function() {
            $rootScope.isLoading = false;
            $rootScope.connectionLost = false;
            $rootScope.reconnecting = false;
            $rootScope.activated = true;
        });

        function activate() {
            var request = function () {
                var promises = [
                    portalService.getProperties(),
                    portalService.getUserConfig(),
                    portalService.getRegionOfInterest("Lpr Camera")
                ];

                return $q.all(promises).then(
                    function success(responses) {
                        vm.dto = responses[0].data;
                        vm.setCameras();

                        vm.userConfig = responses[1].data;

                        // create a copy for canceling
                        vm.backup = angular.copy(responses[1].data);

                        // Append the region of interest to the userConfig, in order to pass it to the directive. The ROI will still be saved in its own file
                        vm.userConfig.regionOfInterest = responses[2].data;
                        vm.backupRegion = angular.copy(responses[2].data);

                        // Select the current camera and update UI accordingly without regards to previously selected camera
                        vm.selectCamera(vm.selectedCamera, true);

                        defer.resolve();
                    },
                    function error() {
                        $rootScope.connectionLost = true;
                        $rootScope.isLoading = false;
                        // wait 5 seconds then retry
                        vm.timeout = $timeout(function () {
                            $rootScope.reconnecting = true;
                            request();
                        }, 5000);
                    }
                );
            }

            request();

            return defer.promise;
        }

        function translate() {
            vm.exposureTypes = [
            { id: "Automatic", name: $translate.instant("STE_LABEL_ExpositionDefault") },
            { id: "Manual", name: $translate.instant("STE_LABEL_ExpositionCustomFixed") },
            { id: "Custom", name: $translate.instant("STE_LABEL_ExpositionCustomRange") }];
            vm.registeredTranslate(); // unregister
            vm.registeredTranslate = null;
        }

        function setCameras() {
            // create lpr camera object
            var lprCamera = new CameraObject("Lpr Camera", "STE_LABEL_LprCamera", true);
            vm.cameras.push(lprCamera);

            vm.selectedCamera = lprCamera; // default one

            // create context camera only if SharpSubType is not Its
            if (!vm.dto.IsIts) {
                var contextCamera = new CameraObject("Context Camera", "STE_LABEL_ContextCamera", false);
                vm.cameras.push(contextCamera);
            }
        }

        function CameraObject(id, name, isLpr) {
            this.id = id;
            this.name = name;
            this.isLpr = isLpr;
        }

        checkChangeStateFactory.checkFormOnStateChange($scope);

        checkChangeStateFactory.checkPageUnloadEvent($scope);

        // Clean up to perform when the view is closed
        $scope.$on("$destroy", function () {
            $timeout.cancel(vm.timeout);
            defer.reject();
            if (vm.form !== undefined && !vm.form.$pristine) {
                // Make sure we push the last saved exposure settings to the camera.
                angular.forEach(vm.cameras, function (camera) {
                    var currentExposureSettings = {};
                    if (camera.isLpr) 
                        currentExposureSettings = vm.backup.ExposureSettings;
                    else
                        currentExposureSettings = vm.backup.ExposureSettingsContext;

                    if (currentExposureSettings !== undefined && currentExposureSettings.Iris !== undefined) {
                        currentExposureSettings.IrisDefault = (currentExposureSettings.Iris < 0);
                        currentExposureSettings.CurrentIris = currentExposureSettings.Iris * 100;
                        vm.updateExposureSettings(currentExposureSettings, camera.id);
                    }
                });
            }

            // Stop the video info feed.
            signalRService.stopFeed();
        });

        function updateUiFromUserConfig() {
            vm.gainSliderValue = { from: vm.currentExposureSettings.GainMin, to: vm.currentExposureSettings.GainMax };
            vm.shutterSliderValue = { from: vm.currentExposureSettings.ShutterTimeMin, to: vm.currentExposureSettings.ShutterTimeMax };
            if (vm.currentExposureSettings.Iris < 0)
                vm.currentExposureSettings.IrisDefault = true;
            else
                vm.currentExposureSettings.IrisDefault = false;
        }

        function extractCapabilities() {
            return $q(function (resolve, reject) {
                portalService.getExposureCapabilities(vm.selectedCamera.id)
                    .then(function (value) {
                        if (!isSuccess(value.status)) { // OK
                            var msg = $translate.instant("STE_SENTENCE_CannotExtractExposureCapabilitiesForX").format(vm.selectedCamera.name);
                            $log.debug(msg);
                            reject(msg); // failed
                        }
                        else {
                            $log.debug("Extracted exposure capabilities for '{0}': Gain: [{1}-{2}], ShutterTime: [{3}-{4}]".format(vm.selectedCamera.name,
                                value.data.MinGainCapability, value.data.MaxGainCapability,
                                value.data.MinExposureCapability, value.data.MaxExposureCapability
                                ));

                            var capabilities = value.data;
                            vm.minGain = capabilities.MinGainCapability.toFixed(0);
                            vm.maxGain = (Math.floor(capabilities.MaxGainCapability * 100) / 100).toFixed(0);
                            vm.minShutterTime = capabilities.MinExposureCapability.toFixed(2);
                            vm.maxShutterTime = (Math.floor(capabilities.MaxExposureCapability * 100) / 100).toFixed(2);

                            resolve(""); // success
                        };
                    });
            });
        }

        function extractDefaultValues() {
            return $q(function (resolve, reject) {
                portalService.getExposureDefaults(vm.selectedCamera.id)
                    .then(function (value) {
                        if (!isSuccess(value.status)) { // OK
                            var msg = $translate.instant("STE_SENTENCE_CannotExtractExposureDefaultsForX").format(vm.selectedCamera.name);
                            $log.debug(msg);
                            reject(msg); // failed
                        }
                        else
                            $log.debug("Extracted exposure defaults for '{0}': Gain: {1};[{2}-{3}], ShutterTime: {4};[{5}-{6}]".format(vm.selectedCamera.name,
                                value.data.Gain,
                                value.data.GainMin, value.data.GainMax,
                                value.data.ShutterTime,
                                value.data.ShutterTimeMin, value.data.ShutterTimeMax
                                ));

                        var defaultValues = value.data;
                        vm.gainDef = defaultValues.Gain;
                        vm.minGainDef = defaultValues.GainMin;
                        vm.maxGainDef = defaultValues.GainMax;
                        vm.shutterTimeDef = defaultValues.ShutterTime;
                        vm.minShutterTimeDef = defaultValues.ShutterTimeMin;
                        vm.maxShutterTimeDef = defaultValues.ShutterTimeMax;

                        resolve(""); // success
                    });
            });
        }

        function saveRegionOfInterest() {
            // Save it only if it changed
            if (vm.selectedCamera && vm.selectedCamera.isLpr && !angular.equals(vm.userConfig.regionOfInterest, vm.backupRegion)) {
                var imageSize = vm.userConfig.VehicleDetectionConfiguration.ContextImageSize;
                portalService.postRegionOfInterest("Lpr Camera", vm.userConfig.regionOfInterest.Coordinates, imageSize.Width, imageSize.Height)
                .then(
                    function success() {
                        saveUserConfig();
                    },
                    function error() {
                        showAlert('danger', $translate.instant("STE_SENTENCE_InvalidRegionOfInterest"));
                        vm.isSaving = false;
                    }
                );
            }
            else { // save only user config (no region of interest)
                saveUserConfig();
            }
        }

        function save() {
            vm.isSaving = true;

            // Reflect changes to the camera
            vm.updateExposureSettings(vm.currentExposureSettings);

            // Save region of interest and user config if camera is LPR
            saveRegionOfInterest();
        }

        function saveUserConfig() {
            // only save userconfig if it has changed
            var tmp = angular.copy(vm.userConfig);
            tmp.regionOfInterest = vm.backupRegion; // ignore the roi when comparing because it is saved outside

            if (!angular.equals(tmp, vm.backup)) {
                portalService.setUserConfig(vm.userConfig)
                .then(function() {
                    vm.saveCompleted();
                    }, function() {
                        //error
                        showAlert('danger', $translate.instant("STE_SENTENCE_CouldNotSaveConfiguration"));
                    }).finally(function() {
                        vm.isSaving = false;
                });
            }
            else {
                vm.saveCompleted();
                vm.isSaving = false;
            }
        }

        // Start video straming based on currently selected camera
        function startStreaming() {
            var isLpr = vm.selectedCamera.isLpr;
            var request = new signalRService.FeedRequestOptions(1, isLpr, !isLpr); // request only video, no data
            signalRService.initializeFeedHub(request);
        }

        function saveCompleted() {
            showAlert('success', $translate.instant("STE_SENTENCE_ConfigurationSavedSuccessfully"));

            // Take back the backup snapshot
            vm.backup = angular.copy(vm.userConfig);
            vm.backupRegion = angular.copy(vm.userConfig.regionOfInterest);
            firstClick = true;

            // Reset form
            if (vm.form !== undefined)
                vm.form.$setPristine();
        }
        
        // Return the localized name for the enum (string) value
        function getLightingName(name) {

            var n = name ? name : vm.currentExposureSettings ? vm.currentExposureSettings.Lighting: "";
            switch (n) {
                case "Sun":
                case "Cloud":
                case "Fluorescent":
                case "Incandescent":
                case "YellowSpot":
                case "WhiteSpot":
                    return $translate.instant("STE_LABEL_Lighting" + n);
                default: return "??? - " + n;
            }
        }

        function selectLighting(name) {
            vm.currentExposureSettings.Lighting = name;
            vm.updateExposureSettings(vm.currentExposureSettings);
            vm.form.$setDirty();
        }

        function selectExposureType(type) {
            vm.currentExposureSettings.ExposureMode = type.id;
            vm.form.$setDirty();
            vm.updateSuggestion();
            vm.updateExposureSettings(vm.currentExposureSettings);
        }

        function getExposureTypeName(id) {
            for (var i = 0; i < vm.exposureTypes.length; i++) {
                if (vm.exposureTypes[i].id === id)
                    return vm.exposureTypes[i].name;
            }

            return "";
        }

        function updateExposureOption() {
            if (vm.currentExposureSettings.IrisDefault) {
                vm.currentExposureSettings.BackupCurrentIris = vm.currentExposureSettings.CurrentIris;
            } else if (!vm.currentExposureSettings.IrisDefault && vm.currentExposureSettings.CurrentIris < 0) {
                if (vm.currentExposureSettings.BackupCurrentIris !== -1)
                    vm.currentExposureSettings.CurrentIris = vm.currentExposureSettings.BackupCurrentIris;
                else
                    vm.currentExposureSettings.CurrentIris = 0; // force minimum value to 0
            }

            vm.updateSuggestion();
            vm.updateExposureSettings(vm.currentExposureSettings);
        }

        function updateExposureSettingsFromScope() {
            if (vm.form)
                vm.form.$setDirty();
            vm.updateExposureOption();
        }

        // This is the function that calls the Portal backend to set the exposure settings
        function updateExposureSettings(exposureSettings, targetCamera) {
            
            // this is a valid check when value is either undefined or null
            if (targetCamera == null) { 
                targetCamera = vm.selectedCamera.id;
            }

            if (exposureSettings !== undefined && angular.isDefined(exposureSettings.ExposureMode) && angular.isDefined(exposureSettings.Gain) && angular.isDefined(exposureSettings.ShutterTime)) {

                if (exposureSettings.IrisDefault) { // this is when default checkbox is checked
                    exposureSettings.Iris = -1;
                } else {
                    exposureSettings.Iris = exposureSettings.CurrentIris / 100;
                }
                // Log what is happening
                if (exposureSettings.ExposureMode === "Automatic") { // no value for neither gain nor shutter
                    $log.debug("{0}: setExposureSettings({1}) : Iris: {2}".format(targetCamera, exposureSettings.ExposureMode, exposureSettings.Iris));
                }
                if (exposureSettings.ExposureMode === "Manual") { // one value for gain, one value for shutter
                    $log.debug("{0}: setExposureSettings({1}): Gain: {2}, shutter: {3}, Iris: {4}".format(targetCamera, exposureSettings.ExposureMode, exposureSettings.Gain, exposureSettings.ShutterTime, exposureSettings.Iris));
                }
                else if (exposureSettings.ExposureMode === "Custom") { // two value for gain, two values for shutter
                    $log.debug("{0}: setExposureSettings({1}): Gain: [{2}-{3}], shutter: [{4}-{5}], Iris: {6}".format(targetCamera, exposureSettings.ExposureMode,
                        exposureSettings.GainMin, exposureSettings.GainMax,
                        exposureSettings.ShutterTimeMin, exposureSettings.ShutterTimeMax, exposureSettings.Iris
                        ));
                }

                // Set values in the server
                portalService.setExposureSettings(targetCamera, exposureSettings)
                    .then(function success(result) {
                        if (!isSuccess(result.status))
                            showAlert('danger', $translate.instant("STE_SENTENCE_ErrorSavingExposure"));
                    },
                    function error() {
                        showAlert('danger', $translate.instant("STE_SENTENCE_ErrorSavingExposure"));
                    });
            }
        }

        function resetFixedGainAndShutterTime(skipDirty) {
            if (vm.currentExposureSettings.Gain !== vm.gainDef) {
                vm.currentExposureSettings.Gain = vm.gainDef;
                if (vm.form && !skipDirty)
                    vm.form.$setDirty();
            }
            if (vm.currentExposureSettings.ShutterTime !== vm.shutterTimeDef) {
                vm.currentExposureSettings.ShutterTime = vm.shutterTimeDef;
                if (vm.form && !skipDirty)
                    vm.form.$setDirty();
            }
            vm.updateUiFromUserConfig();
            vm.updateExposureOption();
        }

        function resetRangeGainAndShutterTime(skipDirty) {
            if (vm.currentExposureSettings.GainMin !== vm.minGainDef) {
                vm.currentExposureSettings.GainMin = vm.minGainDef;
                if (vm.form && !skipDirty)
                    vm.form.$setDirty();
            }

            if (vm.currentExposureSettings.GainMax !== vm.maxGainDef) {
                vm.currentExposureSettings.GainMax = vm.maxGainDef;
                if (vm.form && !skipDirty)
                    vm.form.$setDirty();
            }

            if (vm.currentExposureSettings.ShutterTimeMin !== vm.minShutterTimeDef) {
                vm.currentExposureSettings.ShutterTimeMin = vm.minShutterTimeDef;
                if (vm.form && !skipDirty)
                    vm.form.$setDirty();
            }

            if (vm.currentExposureSettings.ShutterTimeMax !== vm.maxShutterTimeDef) {
                vm.currentExposureSettings.ShutterTimeMax = vm.maxShutterTimeDef;
                if (vm.form && !skipDirty)
                    vm.form.$setDirty();
            }

            vm.updateUiFromUserConfig();
            vm.updateExposureOption();
        }

        function updateSuggestion() {

            if (vm.currentExposureSettings.ExposureMode !== 'Custom') {
                vm.appliedSettingsMin = "";
                vm.appliedSettingsMax = "";
                return;
            }

            if (vm.selectCamera.isLpr) {
                vm.appliedSettingsMin = $translate.instant("STE_SENTENCE_AdjustmentHintLprMin");
                vm.appliedSettingsMax = $translate.instant("STE_SENTENCE_AdjustmentHintLprMax");
            }
            else {
                vm.appliedSettingsMin = $translate.instant("STE_SENTENCE_AdjustmentHintContextMin");
                vm.appliedSettingsMax = $translate.instant("STE_SENTENCE_AdjustmentHintContextMax");
            }

            // popover html template
            $scope.htmlPopover = $sce.trustAsHtml('<span class="text-normal"><i class="fa fa-info-circle"></i>&nbsp;' + vm.appliedSettingsMin + '</span><span class="text-normal"><i class="fa fa-info-circle"></i>&nbsp;' + vm.appliedSettingsMax + '</span>');
        }

        function cancel() {
            // Restore backed-up data
            var currentExposureSettings = (vm.selectedCamera.isLpr) ? angular.copy(vm.backup.ExposureSettings) : angular.copy(vm.backup.ExposureSettingsContext);
            vm.currentExposureSettings.ExposureMode = currentExposureSettings.ExposureMode;
            vm.currentExposureSettings.Lighting = currentExposureSettings.Lighting;
            vm.currentExposureSettings.FlashEnabled = currentExposureSettings.FlashEnabled;

            if (vm.currentExposureSettings.ExposureMode === "Manual") { // one value for gain, one value for shutter
                vm.currentExposureSettings.Gain = currentExposureSettings.Gain;
                vm.currentExposureSettings.ShutterTime = currentExposureSettings.ShutterTime;
            }
            else if (vm.currentExposureSettings.ExposureMode === "Custom") { // two value for gain, two values for shutter
                vm.currentExposureSettings.GainMin = currentExposureSettings.GainMin;
                vm.currentExposureSettings.GainMax = currentExposureSettings.GainMax;
                vm.currentExposureSettings.ShutterTimeMin = currentExposureSettings.ShutterTimeMin;
                vm.currentExposureSettings.ShutterTimeMax = currentExposureSettings.ShutterTimeMax;
            }

            if (currentExposureSettings.Iris < 0)
                vm.currentExposureSettings.IrisDefault = true;
            else
                vm.currentExposureSettings.IrisDefault = false;

            vm.currentExposureSettings.Iris = currentExposureSettings.Iris;
            vm.currentExposureSettings.CurrentIris = vm.currentExposureSettings.Iris * 100;

            vm.userConfig.regionOfInterest = angular.copy(vm.backupRegion);            

            // make sure we reset the exposure settings to the camera
            vm.updateExposureSettings(vm.currentExposureSettings);
            vm.updateUiFromUserConfig();
            vm.updateSuggestion();

            vm.form.$setPristine();
        }

        function clearRegion() {
            if(vm.userConfig.regionOfInterest.Coordinates.length > 0) {
                vm.userConfig.regionOfInterest.Coordinates = [];
                vm.form.$setDirty();
            }

            firstClick = true;
        }

        function selectCamera(selectedCamera, bypassValidation) {
            // By default bypassValidation will be set only at startup
            if (!bypassValidation) {
                if (selectedCamera === vm.selectedCamera)
                    return; // reselected the same camera

                // Alert user, if any modification done to form
                if (vm.form.$dirty) {
                    if (promptUserForTerminateOrCancel()) {
                        cancel(true);
                    }
                    else
                        return; // cancel the request
                }
            }

            vm.show = false; // clear any pending alert

            vm.selectedCamera = selectedCamera;
            firstClick = true;

            // Accumulate all asynchronous calls and perform dependant tasks only when they are all done.
            // Extract default values as they will initialize the range values when set to "Default"
            // Extract capabilities first as they will limit the values that can be represented in the slider controls
            var promises = [extractCapabilities(), extractDefaultValues()];

            $q.all(promises)
            .then(function () {

                // Value that will be populated by UI controls and set in ExposureSettings or ExposureSettingsContext depending of the selected camera
                // Must be set AFTER the camera capabilites are extracted because the slider values are validated against the min/max capability values
                vm.currentExposureSettings = (vm.selectedCamera.isLpr) ? vm.userConfig.ExposureSettings : vm.userConfig.ExposureSettingsContext;
                
                // Log the exposure settings that are active now with that camera
                if (vm.currentExposureSettings.ExposureMode === "Automatic") { // no value for neither gain nor shutter
                    $log.debug("Current Exposure settings are: '{0}'".format(vm.currentExposureSettings.ExposureMode));
                }
                if (vm.currentExposureSettings.ExposureMode === "Manual") { // one value for gain, one value for shutter
                    $log.debug("Current Exposure settings are: '{0}': Gain: {1}, Shutter: {2}".format(vm.currentExposureSettings.ExposureMode, vm.currentExposureSettings.Gain, vm.currentExposureSettings.ShutterTime));
                }
                else if (vm.currentExposureSettings.ExposureMode === "Custom") { // two value for gain, two values for shutter
                    $log.debug("Current Exposure settings are: '{0}': Gain: [{1}-{2}], Shutter: [{3}-{4}]".format(vm.currentExposureSettings.ExposureMode,
                        vm.currentExposureSettings.GainMin, vm.currentExposureSettings.GainMax,
                        vm.currentExposureSettings.ShutterTimeMin, vm.currentExposureSettings.ShutterTimeMax
                        ));
                }

                vm.currentExposureSettings.CurrentIris = -1; //default value to be overriden by vm.currentExposureSettings.Iris * 100
                vm.currentExposureSettings.BackupCurrentIris = -1;

                if (vm.currentExposureSettings.Iris >= 0) {
                    vm.currentExposureSettings.CurrentIris = vm.currentExposureSettings.Iris * 100;
                    vm.currentExposureSettings.BackupCurrentIris = vm.currentExposureSettings.CurrentIris;
                }

                if (vm.currentExposureSettings.ExposureMode !== 'Manual')
                    vm.resetFixedGainAndShutterTime(true);
                if (vm.currentExposureSettings.ExposureMode !== 'Custom')
                    vm.resetRangeGainAndShutterTime(true);                

                vm.updateSuggestion();

                // start streaming the newly picked camera
                startStreaming();

            }, function (failure) {
                //!!!TODO: output a message to the user? restore the camera?
            });
        }

        function promptUserForTerminateOrCancel() {
            var ok = $window.confirm($translate.instant("STE_SENTENCE_SelectOtherCameraAndRevertChanges"));
            return ok; // allow selection of another camera and loose your changes
        }

        // type can be: success, info, warning or danger
        // msg is a string
        function showAlert(type, msg) {
            vm.show = true;
            vm.alert = { type: type, msg: msg }
            $timeout(function () {
                vm.show = false;
            }, 5000);
        }
        
        // Success calls return status 200 to 299
        function isSuccess(status) {
            return 200 <= status && status < 300;
        }

        if (vm.backup) {
            // Reflect changes to the camera
            updateExposureSettings(vm.backup.ExposureSettings, vm.cameras[0].id); // index 0 == LPR
            updateExposureSettings(vm.backup.ExposureSettingsContext, vm.cameras[1].id); // index 1 == Context
        }
    }
})();
