﻿(function () {
    "use strict";

    angular
        .module("portalApp.controllers")
        .controller("ExtensionController", ExtensionController);

    ExtensionController.$inject = ["portalService", "$rootScope", "$scope", "$timeout", "$state", "checkChangeStateFactory", "$translate", "$q"];

    function ExtensionController(portalService,$rootScope, $scope, $timeout, $state, checkChangeStateFactory, $translate, $q) {
        var vm = this;
        vm.title = "ExtensionController";
        vm.save = save;
        vm.cancel = cancel;
        vm.updateExtension = updateExtension;
        vm.show = false;
        vm.isSaving = false;
        vm.alert = {};
        vm.setSelectedExtension = setSelectedExtension;
        vm.switchStateOnExtension = switchStateOnExtension;
        vm.getExtensionName = getExtensionName;
        vm.showStickyAlert = showStickyAlert;
        vm.resetForm = resetForm;
        vm.testFtp = testFtp;
        vm.sendFtpSample = sendFtpSample;
        vm.sendHttpSample = sendHttpSample;
        vm.formats = ["JSON", "XML"];
        vm.selectedExtension = null;
        vm.updateOutputFormat = updateOutputFormat;
        vm.testSecurityCenter = testSecurityCenter;
        vm.timeout = undefined;
        vm.ftpRegex = /^ftp\:\/\/[^\/\s]+(\/\S*)?$/;
        vm.httpRegex = /^https?\:\/\/[^\/\s]+(\/\S*)?$/;

        // regex that supports IPv4, IPv6 and hostname taken from here: http://jsfiddle.net/DanielD/8S4nq/
        vm.hostNameIpRegex = /((^\s*((([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5]))\s*$)|(^\s*((([0-9A-Fa-f]{1,4}:){7}([0-9A-Fa-f]{1,4}|:))|(([0-9A-Fa-f]{1,4}:){6}(:[0-9A-Fa-f]{1,4}|((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(([0-9A-Fa-f]{1,4}:){5}(((:[0-9A-Fa-f]{1,4}){1,2})|:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3})|:))|(([0-9A-Fa-f]{1,4}:){4}(((:[0-9A-Fa-f]{1,4}){1,3})|((:[0-9A-Fa-f]{1,4})?:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){3}(((:[0-9A-Fa-f]{1,4}){1,4})|((:[0-9A-Fa-f]{1,4}){0,2}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){2}(((:[0-9A-Fa-f]{1,4}){1,5})|((:[0-9A-Fa-f]{1,4}){0,3}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(([0-9A-Fa-f]{1,4}:){1}(((:[0-9A-Fa-f]{1,4}){1,6})|((:[0-9A-Fa-f]{1,4}){0,4}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:))|(:(((:[0-9A-Fa-f]{1,4}){1,7})|((:[0-9A-Fa-f]{1,4}){0,5}:((25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)(\.(25[0-5]|2[0-4]\d|1\d\d|[1-9]?\d)){3}))|:)))(%.+)?\s*$))|(^\s*((?=.{1,255}$)(?=.*[A-Za-z].*)[0-9A-Za-z](?:(?:[0-9A-Za-z]|\b-){0,61}[0-9A-Za-z])?(?:\.[0-9A-Za-z](?:(?:[0-9A-Za-z]|\b-){0,61}[0-9A-Za-z])?)*)\s*$)/;

        $rootScope.isLoading = true;
        $rootScope.connectionLost = false;
        $rootScope.reconnecting = false;
        $rootScope.activated = false;

        var defer = $q.defer();
        // Clean up to perform when the view is closed
        $scope.$on("$destroy", function () {
            $timeout.cancel(vm.timeout);
            defer.reject();
        });

        activate()
            .then(function (response) {
                vm.extensions = response.data;
                vm.setSelectedExtension();
            });

        function activate() {
            var request = function () {
                if ($rootScope.reconnecting)
                    $rootScope.activated = true;

                portalService.getUserConfig()
                    .then(function success(response) {
                        vm.userConfig = response.data;
                        // create a copy for canceling
                        vm.backup = angular.copy(vm.userConfig);

                        portalService.getExtensions()
                            .then(function success(response) {
                                $rootScope.isLoading = false;
                                $rootScope.connectionLost = false;
                                $rootScope.reconnecting = false;
                                $rootScope.activated = true;
                                // we resolve only when the second call is completed
                                defer.resolve(response);
                            }, function error() {
                                $rootScope.connectionLost = true;
                                $rootScope.isLoading = false;
                                // wait 5 seconds then retry
                                vm.timeout = $timeout(function () {
                                    $rootScope.reconnecting = true;
                                    request();
                                }, 5000);
                            });

                    }, function error() {
                        $rootScope.connectionLost = true;
                        $rootScope.isLoading = false;
                        // wait 5 seconds then retry
                        vm.timeout = $timeout(function () {
                            $rootScope.reconnecting = true;
                            request();
                        }, 5000);
                    });
            }

            request();

            return defer.promise;
        }                

        checkChangeStateFactory.checkFormOnStateChange($scope);        

        function save() {
            vm.isSaving = true;

            if (vm.selectedExtension.Name === "None") {
                vm.userConfig.CurrentExtension = vm.selectedExtension.Guid;
                portalService.setUserConfig(vm.userConfig)
                .then(function () {
                        vm.isSaving = false;
                        vm.showStickyAlert('success', $translate.instant("STE_SENTENCE_ConfigurationSavedSuccessfully"));
                        vm.resetForm();
                }, function() {
                        vm.isSaving = false;
                        vm.showStickyAlert('danger', $translate.instant("STE_SENTENCE_CouldNotSaveConfiguration"));
                });
            }
            else if (vm.selectedExtension.Name === "Ftp") {
                portalService.setFtpExtensionSettings(vm.ftp, vm.selectedExtension.Guid)
                .then(function () {
                        vm.isSaving = false;
                        vm.showStickyAlert('success', $translate.instant("STE_SENTENCE_ConfigurationSavedSuccessfully"));
                        vm.resetForm();
                }, function() {
                        vm.isSaving = false;
                        vm.showStickyAlert('danger', $translate.instant("STE_SENTENCE_CouldNotSaveConfiguration"));
                });
            }
            else if (vm.selectedExtension.Name === "Http") {
                portalService.setHttpExtensionSettings(vm.http, vm.selectedExtension.Guid)
                .then(function () {
                        vm.isSaving = false;
                        vm.showStickyAlert('success', $translate.instant("STE_SENTENCE_ConfigurationSavedSuccessfully"));
                        vm.resetForm();
                }, function() {
                        vm.isSaving = false;
                        vm.showStickyAlert('danger', $translate.instant("STE_SENTENCE_CouldNotSaveConfiguration"));
                });
            }
            else if (vm.selectedExtension.Name === "Patroller") {
                portalService.setPatrollerExtensionSettings(vm.patroller, vm.selectedExtension.Guid)
                .then(function () {
                        vm.isSaving = false;
                        vm.showStickyAlert('success', $translate.instant("STE_SENTENCE_ConfigurationSavedSuccessfully"));
                        vm.resetForm();
                }, function() {
                        vm.isSaving = false;
                        vm.showStickyAlert('danger', $translate.instant("STE_SENTENCE_CouldNotSaveConfiguration"));
                });
            }
            else if (vm.selectedExtension.Name === "SecurityCenter") {
                portalService.setSecurityCenterExtensionSettings(vm.sc, vm.selectedExtension.Guid)
                .then(function () {
                        vm.isSaving = false;
                        vm.showStickyAlert('success', $translate.instant("STE_SENTENCE_ConfigurationSavedSuccessfully"));
                        vm.resetForm();
                }, function() {
                        vm.isSaving = false;
                        vm.showStickyAlert('danger', $translate.instant("STE_SENTENCE_CouldNotSaveConfiguration"));
                });
            }
        }

        function cancel() {
            vm.userConfig = angular.copy(vm.backup); // revert to backup values

            vm.setSelectedExtension();
            vm.updateExtension(vm.selectedExtension);
            vm.form.$setPristine();
        }

        function setSelectedExtension() {
            // select proper extension
            for (var i = 0; i < vm.extensions.length; i++) {
                if (vm.extensions[i].Guid === vm.userConfig.CurrentExtension) {
                    vm.selectedExtension = vm.extensions[i];
                    break;
                }
            }

            vm.switchStateOnExtension();
        }

        function resetForm() {
            // reset form state to pristine
            vm.form.$setPristine();

            // refresh the userconfig object
            portalService.getUserConfig()
                .then(function(response) {
                    vm.userConfig = response.data;
                    vm.backup = angular.copy(vm.userConfig); // refresh the backup snapshot
                });
        }

        function showStickyAlert(type, msg) {
            vm.alert = { type: type, msg: msg }
            vm.show = true;
            $timeout(function () {
                vm.show = false;
            }, 5000);
        }

        function switchStateOnExtension() {
            if (vm.selectedExtension.Name === "Ftp") {
                portalService.getFtpExtensionSettings(vm.selectedExtension.Guid)
                    .then(function(response) {
                        vm.ftp = response.data;
                        $state.go("main.config.extension.ftp", { SkipCheck: 1 });
                    });

            }
            else if (vm.selectedExtension.Name === "Http") {
                portalService.getHttpExtensionSettings(vm.selectedExtension.Guid)
                    .then(function(response) {
                        vm.http = response.data;
                        $state.go("main.config.extension.http", { SkipCheck: 1 });
                    });

            }
            else if (vm.selectedExtension.Name === "Patroller") {
                portalService.getPatrollerExtensionSettings(vm.selectedExtension.Guid)
                    .then(function(response) {
                        vm.patroller = response.data;
                        $state.go("main.config.extension.patroller", { SkipCheck: 1 });
                    });

            }
            else if (vm.selectedExtension.Name === "SecurityCenter") {
                portalService.getSecurityCenterExtensionSettings(vm.selectedExtension.Guid)
                    .then(function(response) {
                        vm.sc = response.data;
                        $state.go("main.config.extension.securitycenter", { SkipCheck: 1 });
                    });

            }
            else
                $state.go("main.config.extension", { SkipCheck: 1 });
        }

        function getExtensionName(extension) {
            if (!extension) {
                return $translate.instant("STE_LABEL_ExtensionNone");
            }
            else if (extension.Name === "Ftp") {
                return "FTP";
            }
            else if (extension.Name === "Http") {
                return "HTTP";
            }
            else if (extension.Name === "Patroller") {
                return extension.Name;
            }
            else if (extension.Name === "SecurityCenter") {
                return "Security Center";
            }
            else
                return extension.Name;
        }

        function updateExtension(extension) {
            vm.selectedExtension = extension;
            vm.form.$setDirty(); // set dirty flag
            vm.switchStateOnExtension();
        }

        function updateOutputFormat(format) {
            vm.http.Format = format;
            vm.form.$setDirty(); // set dirty flag
        }

        function testFtp() {
            portalService.testFtp(vm.ftp)
                .then(function(response) {
                    if (response.data === true) {
                        // success
                        showStickyAlert('success', $translate.instant("STE_SENTENCE_ConnectionTestSucceeded"));
                    } else {
                        // failure
                        showStickyAlert('danger', $translate.instant("STE_SENTENCE_ConnectionTestFailed"));
                    }
                });
        }

        function sendFtpSample() {
            portalService.sendFtpSample(vm.ftp)
                .then(function(response) {
                    if (response.status === 200 && response.data === true) {
                        // success
                        showStickyAlert('success', $translate.instant("STE_SENTENCE_TransmissionSuccess"));
                    } else {
                        // failure                        
                        showStickyAlert('danger', $translate.instant("STE_SENTENCE_TransmissionFailure"));
                    }
                },
                function (response) {
                    // failure
                    showStickyAlert('danger', $translate.instant("STE_SENTENCE_TransmissionFailure"));
                });
        }

        function sendHttpSample() {
            portalService.sendHttpSample(vm.http)
                .then(function(response) {
                    // success
                    if (response.status === 200 && response.data === true) {
                        // success
                        showStickyAlert('success', $translate.instant("STE_SENTENCE_TransmissionSuccess"));
                    } else {
                        // failure                        
                        showStickyAlert('danger', $translate.instant("STE_SENTENCE_TransmissionFailure"));
                    }
                }, function(response) {
                    // failure
                    showStickyAlert('danger', $translate.instant("STE_SENTENCE_TransmissionFailure"));
                });
        }

        function testSecurityCenter() {
            portalService.getTryConnectionToPort(vm.sc.Server, vm.sc.Port)
                .then(function (response) {
                    if (response.data === "OK") {
                        // success
                        showStickyAlert('success', $translate.instant("STE_SENTENCE_ConnectionTestSucceeded"));
                    } else {
                        // failure
                        showStickyAlert('danger', $translate.instant("STE_SENTENCE_ConnectionTestFailed"));
                    }
                }, function (response) {
                    // failure
                    showStickyAlert('danger', $translate.instant("STE_SENTENCE_TransmissionFailure"));
                });
        }
    }
})();
