﻿(function() {
    "use strict";

    angular
        .module("portalApp.controllers")
        .controller("MaintenanceController", MaintenanceController);

    MaintenanceController.$inject = ["portalService", "signalRService", "$rootScope", "$scope", "$uibModal", "backendServerUrl", "$timeout", "$window", "$q", "$filter", "$interval", "$translate", "$state"];

    function MaintenanceController(portalService, signalRService, $rootScope, $scope, $uibModal, backendServerUrl, $timeout, $window, $q, $filter, $interval, $translate, $state) {
        var vm = this;
        vm.class = "MaintenanceController";
        vm.ipv6Regex = /(([0-9a-fA-F]{1,4}:){7,7}[0-9a-fA-F]{1,4}|([0-9a-fA-F]{1,4}:){1,7}:|([0-9a-fA-F]{1,4}:){1,6}:[0-9a-fA-F]{1,4}|([0-9a-fA-F]{1,4}:){1,5}(:[0-9a-fA-F]{1,4}){1,2}|([0-9a-fA-F]{1,4}:){1,4}(:[0-9a-fA-F]{1,4}){1,3}|([0-9a-fA-F]{1,4}:){1,3}(:[0-9a-fA-F]{1,4}){1,4}|([0-9a-fA-F]{1,4}:){1,2}(:[0-9a-fA-F]{1,4}){1,5}|[0-9a-fA-F]{1,4}:((:[0-9a-fA-F]{1,4}){1,6})|:((:[0-9a-fA-F]{1,4}){1,7}|:)|fe80:(:[0-9a-fA-F]{0,4}){0,4}%[0-9a-zA-Z]{1,}|::(ffff(:0{1,4}){0,1}:){0,1}((25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])\.){3,3}(25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])|([0-9a-fA-F]{1,4}:){1,4}:((25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])\.){3,3}(25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9]))/;
        vm.downloadSettings = downloadSettings;
        vm.openImportModal = openImportModal;
        vm.openResetDefaultModal = openResetDefaultModal;
        vm.freeSpace = freeSpace;
        vm.getProgressBarClassOnPercentage = getProgressBarClassOnPercentage;
        vm.reboot = reboot;
        vm.importTarget = importTarget;
        vm.updateTarget = updateTarget;
        vm.isAlertShown = false;
        vm.isImporting = false;
        vm.isUpdating = false;
        vm.isReseting = false;
        vm.testServer = testServer;
        vm.updateProtocol = updateProtocol;
        vm.cancel = cancel;
        vm.save = save;
        vm.openUpdateModal = openUpdateModal;
        vm.validateSyslogServer = validateSyslogServer;
        vm.syslogSettings = undefined;
        vm.backup = {};
        vm.ping = null;
        vm.resetNetwork = false;
        vm.resetValues = resetValues;
        vm.timeout = undefined;
        var defer = $q.defer();

        $scope.$on('$destroy', function () {
            $timeout.cancel(vm.timeout);
            // cancel all promises
            defer.reject();
        });

        $rootScope.isLoading = true;
        $rootScope.connectionLost = false;
        $rootScope.reconnecting = false;
        $rootScope.activated = false;

        activate()
            .then(function (response) {
                $rootScope.isLoading = false;
                $rootScope.connectionLost = false;
                $rootScope.reconnecting = false;
                $rootScope.activated = true;
            })
            .catch(function () {
                //console.log("OMG FAIL!!!");
            });

        function activate() {
            var request = function () {
                var promises = [];
                promises.push(portalService.getProperties());
                promises.push(portalService.getReclaimableSpace());
                promises.push(portalService.getLoggerConfiguration());
                promises.push(portalService.getUserConfig());
                $q.all(promises).then(function (data) {
                    vm.dto = data[0].data;
                    vm.reclaimableSpace = data[1].data;
                    vm.resetValues(data[2].data);
                    vm.userConfig = data[3].data;
                    vm.backup = angular.copy(vm.syslogSettings);
                    defer.resolve();
                }, function error() {
                    $rootScope.connectionLost = true;
                    $rootScope.isLoading = false;
                    // wait 5 seconds then retry
                    vm.timeout = $timeout(function () {
                        $rootScope.reconnecting = true;
                        request();
                    }, 5000);
                });
            }

            request();

            return defer.promise;
        }

        function resetValues(data) {
            vm.syslogSettings = data;

            // create a copy for cancelling
            vm.backup = angular.copy(vm.syslogSettings);

            if (data && data.hasOwnProperty("Server")) {
                var lastIndexOfSemicolon = data.Server.lastIndexOf(":");
                vm.syslogServername = data.Server.substring(0, lastIndexOfSemicolon);
                vm.syslogServerPort = data.Server.substring(lastIndexOfSemicolon + 1, data.Server.length);
            }
            // Set default port number based on protocol, if no port is provided
            if (!vm.syslogServerPort) {
                vm.syslogServerPort = (vm.syslogSettings.NetworkProtocol === "UDP") ? 514 : 601;
            }
        }

        function getProgressBarClassOnPercentage(value) {            
            if (value < 50) {
                return "progress-bar-success";
            } else if (value < 85) {
                return "progress-bar-warning";
            } else {
                return "progress-bar-danger";
            }
        }

        $scope.$on('$destroy', function () {
            // Make sure that the interval is destroyed too
            if (vm.ping) {
                $interval.cancel(vm.ping);
            }
        });

        // Cancel the pending transfers and exit the windos
        $scope.closeModal = function () {
            $scope.isCanceled = true;
            vm.isUpdating = false;

            // unblock the one waiting for all promises completion
            angular.forEach($scope.deferred, function (cancelDefer) {
                cancelDefer.promise.isGloballyCancelled = true;
                cancelDefer.resolve();
            });

            $scope.modalInstance.dismiss("cancel");
        };

        $scope.import = function() {
            vm.isImporting = true;
            portalService.postSettings(vm.file)
                .then(function(result) {
                    $scope.modalInstance.close();
                    if (result && result.data)
                        showStickyAlert("success", $translate.instant("STE_SENTENCE_ImportDone"), true, rebootAfterReset);
                    else
                        showStickyAlert("danger", $translate.instant("STE_SENTENCE_ImportFailed"), true);
                    vm.isImporting = false;
                }, function() {
                    //error
                    $scope.modalInstance.dismiss("cancel");
                    showStickyAlert("danger", $translate.instant("STE_SENTENCE_ImportFailed"), true);
                    vm.isImporting = false;
            });
        };

        $scope.reset = function () {
            vm.isReseting = true;
            portalService.resetConfiguration(vm.resetNetwork)
                .then(function(result) {
                    $scope.modalInstance.close();
                    // Resetting to factory defaults also clears the password, so redirect to the login page.
                    showStickyAlert("success", $translate.instant("STE_SENTENCE_ResetDone"), true, rebootAfterReset);
                }, function() {
                    //error
                    $scope.modalInstance.dismiss("cancel");
                    showStickyAlert("danger", $translate.instant("STE_SENTENCE_ResetFailed"), true);
                }).finally(function() {
                    vm.isReseting = false;
            });
        };

        $scope.update = function () {
            vm.isUpdating = true;            
            
            // load zip content
            JSZip.loadAsync(vm.zipfile)
                .then(function (zip) {
                    if ($scope.isCanceled)
                        return;
                    var productVersionPattern = /^SharpOS-(\d{2})\.(\d{1})-(\d{4}\d{2}\d{2})/;
                    var folder = zip.folder(/SharpOS-/);

                    if (folder.length === 0) {
                        $scope.modalInstance.dismiss("cancel");
                        vm.isUpdating = false;
                        showStickyAlert("danger", $translate.instant("STE_SENTENCE_ZipFileIsNotValid"), true);
                        return;
                    }

                    var match = folder[0].name.match(productVersionPattern);
                    if (match === null || match[1] !== '12' || match.length !== 4) { // accept all 12.x files
                        $scope.modalInstance.dismiss("cancel");
                        vm.isUpdating = false;
                        showStickyAlert("danger", $translate.instant("STE_SENTENCE_ZipFileIsNotValid"), true);
                        return;
                    }

                    portalService.putProductVersion(match[2])
                        .then(function () {
                            if ($scope.isCanceled)
                                return;
                            var plateReaderZip = zip.file(/Sharp_/);
                            var plateReaderName = plateReaderZip[0].name.replace(folder[0].name, '');
                            var separator = plateReaderName.indexOf('_');
                            var start = separator + 1;
                            var end = plateReaderName.lastIndexOf(".");
                            var plateReaderVersionName = plateReaderName.substring(start, end);
                            var updateInfo = { Application: 'SharpOS', Version: plateReaderVersionName };

                            $scope.deferred = [];
                            var promises = [];

                            portalService.initializeApplicationUpload(updateInfo)
                            .then(function () {

                                if ($scope.isCanceled)
                                    return;

                                zip.forEach(function (relativePath, file) {

                                    if ($scope.isCanceled)
                                        return;

                                    $q.when()
                                        .then(function() {
                                            var deferred = $q.defer();
                                            if (!file.dir) {
                                                // upload each files
                                                file.async("arraybuffer")
                                                    .then(function (data) {

                                                        if ($scope.isCanceled)
                                                            return;

                                                        portalService.webupdateUpload(updateInfo, relativePath, data)
                                                            .then(function () {
                                                                deferred.resolve();
                                                            }, function error(err) {
                                                                console.log("File transfer failed. " + err);
                                                                deferred.reject();
                                                            });
                                                    }, function error(err) {
                                                        console.log("Reading file in memory failed. " + err);
                                                        deferred.reject();
                                                    });
                                            } else {
                                                deferred.resolve(); //nothing todo
                                            }
                                            $scope.deferred.push(deferred);
                                            promises.push(deferred.promise);
                                        });
                                });
                                
                            }, function () {
                                // error
                                $scope.modalInstance.dismiss("cancel");
                                vm.isUpdating = false;
                                showStickyAlert("danger", $translate.instant("STE_SENTENCE_FailedToUploadFile"), true);
                                $scope.isCanceled = true;
                            }).finally(function () {
                                $q.all(promises).then(function () {
                                        vm.isUpdating = false;
                                        $scope.modalInstance.close();

                                        if ($scope.isCanceled)
                                            return;

                                        showStickyAlert("success", $translate.instant("STE_SENTENCE_UpdateFileTransferedUpdateWillBegin"), false);
                                        portalService.completeApplicationUpload(updateInfo)
                                            .then(function () {
                                                var isSecure = false;
                                                if (window.location.protocol === "https:") {
                                                    isSecure = true;
                                                }

                                                if ($scope.isCanceled)
                                                    return;

                                                portalService.startWebUpdaterServer(isSecure)
                                                    .then(function (response) {
                                                        var origin;
                                                        if (!window.location.origin) {
                                                            origin = window.location.protocol + "//" + window.location.hostname + ":" + window.location.port;
                                                        }
                                                        else {
                                                            origin = window.location.origin;
                                                        }

                                                        // Redirect to the new WebUpdater page
                                                        window.location = origin + response.data; // add location to the WebUpdater
                                                    }, function () {
                                                        //error
                                                        $scope.modalInstance.dismiss("cancel");
                                                        vm.isUpdating = false;
                                                        showStickyAlert("danger", $translate.instant("STE_SENTENCE_UpdateFailed"), true);
                                                    });
                                            });
                                    }, function(response) {
                                        $scope.modalInstance.dismiss("cancel");
                                        vm.isUpdating = false;
                                        showStickyAlert("danger", $translate.instant("STE_SENTENCE_FailedToUploadFile"), true);
                                    });

                                });
                        }, function() {
                            $scope.modalInstance.dismiss("cancel");
                            vm.isUpdating = false;
                            showStickyAlert("danger", $translate.instant("STE_SENTENCE_FailedToUploadFile"), true);
                        });

                }, function() {
                    //error
                    $scope.modalInstance.dismiss("cancel");
                    vm.isUpdating = false;
                    showStickyAlert("danger", $translate.instant("STE_SENTENCE_UpdateFailed"), true);
                });

        }; 

        function openImportModal() {
            $scope.modalInstance = $uibModal.open({
                windowClass: "modal-danger",
                templateUrl: "import.html",
                scope: $scope,
                size: "md",
                backdrop: 'static'
            });
        }

        function openResetDefaultModal() {
            $scope.modalInstance = $uibModal.open({
                windowClass: "modal-danger",
                templateUrl: "reset.html",
                scope: $scope,
                size: "md",
                backdrop: 'static'
            });
        }

        function downloadSettings() {

            showStickyAlert('spin', $translate.instant("STE_LABEL_Downloading"), false);
            var success = true;

            // the zip is downloaded here, then loaded in a Blob and linked through an ObjectURL
            portalService.getDiagnostics()
                .then(function (result) {
                    // the date is added to the filename so that every download will be different
                    var date = moment().format('YYYY-MM-DD_HH:mm:ss');
                    var fName = "Diagnostics-" + date +".zip";
                    var file = new Blob([result.data], { type: 'application/*' });
                    var fileUrl = (window.URL || window.webkitURL).createObjectURL(file);

                    if (navigator.msSaveBlob) {
                        navigator.msSaveBlob(file, fName);
                    }
                    else {
                        // try to get the invisble element
                        var downloadLink = document.getElementById("diagnosticsDownload");
                        if (!downloadLink) {
                            // we create an invisible a element to do the download in the browser
                            downloadLink = document.createElement("a");
                            downloadLink.setAttribute("id", "diagnosticsDownload");
                            document.body.appendChild(downloadLink);
                            downloadLink.style = "display: none";
                        }

                        downloadLink.href = fileUrl;
                        downloadLink.download = fName;
                        downloadLink.click();
                    }
            }, function() {               
                success = false;
            })
                .finally(function ()
                {
                    if (!success) {
                        showStickyAlert('danger', $translate.instant("STE_SENTENCE_DownloadFailed"), true);
                    } else {
                        showStickyAlert('success',  $translate.instant("STE_SENTENCE_DownloadSucceeded"), true);
                    }
            });
        }

        function freeSpace() {
            vm.isFreeingSpace = true;
            portalService.getFreeSpace()
                .then(function (result) {
                    vm.reclaimableSpace.reclaimableSpace = result.data.Size;
                    showStickyAlert("success", $translate.instant("STE_SENTENCE_FreeingUpSpaceDone").format($filter('bytes')(result.data.ReclaimedSize)));
                }, function () {
                    //error
                    showStickyAlert("danger", $translate.instant("STE_SENTENCE_FreeingUpSpaceFailed"));
                }).finally(function () {
                    vm.isFreeingSpace = false;
            });
        }

        // This function reboots the unit when the user clicks the Reboot Unit button.
        // Since the HTTP/HTTPS selection does not change, we can safely ping the portal service.
        function reboot() {
            vm.isRebooting = true;
            signalRService.stopConnections();
            portalService.rebootUnit();
            showStickyAlert("info", $translate.instant("STE_SENTENCE_TryingToConnectToServer"), false);

            vm.ping = $interval(function() {
                portalService.getPing(backendServerUrl)
                    .then(function (result) {
                        showStickyAlert("success", $translate.instant("STE_SENTENCE_RebootCompletedConnectionRestored"), true);
                        vm.isRebooting = false;
                        $window.location.href = "/";
                        $interval.cancel(vm.ping); // connection restored, stop pinging
                    }, function () {
                        
                    });
            }, 10000);
        }
        
        // This function reboots the unit once settings have been reset to factory default.
        function rebootAfterReset() {
            vm.isRebooting = true;
            signalRService.stopConnections();
            portalService.rebootUnit();
            showStickyAlert("info", $translate.instant("STE_SENTENCE_RebootingPleaseWait"), false);            

            var wasUsingHttpBeforeReset = !vm.userConfig.PortalSecurityConfiguration.UseHttps;

            if (wasUsingHttpBeforeReset) {
                vm.ping = $interval(function () {
                    portalService.getPortalPing()
                    .then(function (result) {
                        showStickyAlert("success", $translate.instant("STE_SENTENCE_RebootCompletedConnectionRestored"), true);
                        vm.isRebooting = false;
                        vm.needReboot = false;
                        
                        $interval.cancel(vm.ping); // connection restored, stop pinging

                        delete $window.sessionStorage.accessToken;
                        delete $window.sessionStorage.refreshToken;
                        $window.location.href = "/";

                    }, function () { });
                }, 10000);
            } else {
                // going from https to http avoids us to use ping because of mixed site content blocked by browsers
                $timeout(function () {
                    vm.isRebooting = false;

                    delete $window.sessionStorage.accessToken;
                    delete $window.sessionStorage.refreshToken;
                    $window.location = "http://" + location.hostname;
                }, 75000);
            }
        }

        function importTarget(event) {
            $scope.vm.file = event.target.files[0];
            $scope.$apply(function() {
                vm.form.importForm.$setDirty();
            });
        }

        function updateTarget(event) {
            $scope.vm.zipfile = event.target.files[0];
            $scope.$apply(function() {
                vm.form.updateForm.$setDirty();
            });            
        }        

        function testServer() {
            
            if (vm.syslogServername.match(vm.ipv6Regex) && vm.syslogSettings.NetworkProtocol === "UDP") {
                showStickyAlert('danger', $translate.instant("STE_SENTENCE_SyslogUDPIPv6NotSupported"), true);
                return;
            }

            // Make sure the serveur answers
            portalService.testSyslog(vm.syslogServername, vm.syslogServerPort, vm.syslogSettings.NetworkProtocol)
                .then(function(response) {
                    if (response.data === true) {
                        // success
                        showStickyAlert('success', $translate.instant("STE_SENTENCE_ConnectionTestSucceeded"), true);
                    } else {
                        // failure                        
                        showStickyAlert('danger', $translate.instant("STE_SENTENCE_ConnectionTestFailed"), true);
                    }
                });
        }

        function updateProtocol(protocol) {
            vm.syslogSettings.NetworkProtocol = protocol;
            vm.validateSyslogServer();
        }

        function cancel() {
            vm.syslogSettings = angular.copy(vm.backup); // revert to backup values
            vm.resetValues(vm.backup);

            vm.form.$setPristine();
        }

        function save() {
            vm.isSaving = true;

            // Recompose server value from its two parts
            vm.syslogSettings.Server = (vm.syslogServername ? vm.syslogServername : "")
                + (vm.syslogServerPort ? ":" + vm.syslogServerPort : "");

            // Save result
            portalService.setLoggerConfiguration(vm.syslogSettings)
                .then(function() {
                    showStickyAlert('success', $translate.instant("STE_SENTENCE_ConfigurationSavedSuccessfully"), true);
                    vm.resetValues(vm.syslogSettings);
                    vm.form.$setPristine();
                }, function() {
                    //error
                    showStickyAlert('danger', $translate.instant("STE_SENTENCE_CouldNotSaveConfiguration"), true);
                }).finally(function() {
                    vm.isSaving = false;
                });
        }

        function openUpdateModal() {
            // check before if an update is already in progress
            portalService.getWebUpdaterIsRunning()
                .then(function (result) {
                    if (result.data) {

                        // An upgrade is pending, display a notification to the user, wait 5 seconds and switch to WebUpdater page
                        showStickyAlert('danger', $translate.instant("STE_SENTENCE_UpdateAlreadyInProgress"), false);
                        
                        $timeout(function () {
                            // web updater already running, change location to webupdater page
                            var origin;
                            if (!window.location.origin) {
                                origin = window.location.protocol + "//" + window.location.hostname + ":" + window.location.port;
                            }
                            else {                                
                                origin = window.location.origin;
                            }

                            // Redirect to the new page
                            window.location = origin + result.data; // add the port and location to the WebUpdater
                            vm.isAlertShown = false;
                        }, 5000);
                        
                    } else {

                        $scope.isCanceled = false;

                        // No update is pending, ask for file to upload                        
                        $scope.modalInstance = $uibModal.open({
                            templateUrl: "update.html",
                            scope: $scope,
                            size: "md",
                            backdrop: 'static'
                        });
                    }
                }, function () {
                    showStickyAlert('danger', $translate.instant("STE_SENTENCE_UpdateFailed"), true);
                });            
        }

        function validateSyslogServer() {
            if (vm.syslogServername && vm.syslogServername.match(vm.ipv6Regex)) {
                if (vm.syslogSettings.NetworkProtocol === "UDP") {
                    vm.form.servername.$setValidity('servername', false);
                }
                else if (vm.syslogSettings.NetworkProtocol === "TCP") {
                    vm.form.servername.$setValidity('servername', true);
                }
            }           
        }
        
        function showStickyAlert(type, msg, timeout, continueWith) {
            var spin = false;
            if (type === 'spin') {
                spin = true;
                type = 'info';
            }
            vm.alert = { type: type, msg: msg, spin: spin }
            vm.isAlertShown = true;
            if (timeout) {
                $timeout(function() {
                    vm.isAlertShown = false;

                    if (!!continueWith && typeof continueWith === "function") {
                        continueWith();
                    }

                }, 5000);
            }
        }
    }
})();