﻿(function () {
    "use strict";

    angular
        .module("portalApp.controllers")
        .controller("NetworkController", NetworkController);

    NetworkController.$inject = ["portalService", "$scope", "$rootScope", "$timeout", "checkChangeStateFactory", "$translate", "$q"];

    function NetworkController(portalService, $scope, $rootScope, $timeout, checkChangeStateFactory, $translate, $q) {
        var vm = this;
        vm.title = "NetworkController";
        vm.save = save;
        vm.cancel = cancel;
        vm.show = false;
        vm.isSaving = false;
        vm.alert = {};
        vm.IPregex = /^(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$/;
        vm.IPV6Regex = /(([0-9a-fA-F]{1,4}:){7,7}[0-9a-fA-F]{1,4}|([0-9a-fA-F]{1,4}:){1,7}:|([0-9a-fA-F]{1,4}:){1,6}:[0-9a-fA-F]{1,4}|([0-9a-fA-F]{1,4}:){1,5}(:[0-9a-fA-F]{1,4}){1,2}|([0-9a-fA-F]{1,4}:){1,4}(:[0-9a-fA-F]{1,4}){1,3}|([0-9a-fA-F]{1,4}:){1,3}(:[0-9a-fA-F]{1,4}){1,4}|([0-9a-fA-F]{1,4}:){1,2}(:[0-9a-fA-F]{1,4}){1,5}|[0-9a-fA-F]{1,4}:((:[0-9a-fA-F]{1,4}){1,6})|:((:[0-9a-fA-F]{1,4}){1,7}|:)|fe80:(:[0-9a-fA-F]{0,4}){0,4}%[0-9a-zA-Z]{1,}|::(ffff(:0{1,4}){0,1}:){0,1}((25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])\.){3,3}(25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])|([0-9a-fA-F]{1,4}:){1,4}:((25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])\.){3,3}(25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9]))/;
        vm.timeout = undefined;
        vm.isDhcp = true;
        vm.setDhcp = setDhcp;
        vm.showStickyAlert = showStickyAlert;

        $rootScope.isLoading = true;
        $rootScope.connectionLost = false;
        $rootScope.reconnecting = false;
        $rootScope.activated = false;

        var defer = $q.defer();
        // Clean up to perform when the view is closed
        $scope.$on("$destroy", function () {
            $timeout.cancel(vm.timeout);
            defer.reject();
        });

        activate()
            .then(function (response) {
                vm.userConfig = response.data;
                vm.isDhcp = determineIfDhcp();
                // create a copy for canceling
                vm.backup = angular.copy(vm.userConfig);
            });

        function determineIfDhcp() {
            return vm.userConfig.NetworkConfiguration.NetworkType === 0 || vm.userConfig.NetworkConfiguration.NetworkType === 2; // DHCP or None
        }

        function setDhcp(value) {
            if (value)
                vm.userConfig.NetworkConfiguration.NetworkType = 0; // DHCP
            else
                vm.userConfig.NetworkConfiguration.NetworkType = 1; // Static
        }

        function activate() {

            var request = function () {
                if ($rootScope.reconnecting)
                    $rootScope.activated = true;

                portalService.getUserConfig()
                    .then(function success(response) {
                        $rootScope.isLoading = false;
                        $rootScope.connectionLost = false;
                        $rootScope.reconnecting = false;
                        $rootScope.activated = true;

                        defer.resolve(response);
                    }, function error() {
                        $rootScope.connectionLost = true;
                        $rootScope.isLoading = false;
                        // wait 5 seconds then retry
                        vm.timeout = $timeout(function () {
                            $rootScope.reconnecting = true;
                            request();
                        }, 5000);
                    });
            }

            request();

            return defer.promise;
        }

        checkChangeStateFactory.checkFormOnStateChange($scope);

        function save() {
            vm.isSaving = true;

            portalService.setUserConfig(vm.userConfig)
                .then(function () {
                    showStickyAlert('success', $translate.instant("STE_SENTENCE_ConfigurationSavedSuccessfully"));
                    vm.backup = angular.copy(vm.userConfig); // refresh the backup snapshot
                    vm.form.$setPristine();
                }, function () {
                    //error
                    showStickyAlert('danger', $translate.instant("STE_SENTENCE_CouldNotSaveConfiguration"));
                }).finally(function () {
                    vm.isSaving = false;
                });
        }

        function cancel() {
            vm.userConfig = angular.copy(vm.backup); // revert to backup values
            vm.isDhcp = determineIfDhcp();
            vm.form.$setPristine();
        }

        function showStickyAlert(type, msg) {
            vm.alert = {
                type: type, msg: msg
            }
            vm.show = true;
            $timeout(function () {
                vm.show = false;
            }, 5000);
        }
    }
})();
