﻿(function () {
    "use strict";

    angular
        .module("portalApp.controllers")
        .controller("PowerController", PowerController);

    PowerController.$inject = ["portalService", "$rootScope", "$scope", "checkChangeStateFactory", "$q", "$translate", "$timeout"];

    function PowerController(portalService, $rootScope, $scope, checkChangeStateFactory, $q, $translate, $timeout) {
        var vm = this;
        vm.class = "PowerController";
        vm.save = save;
        vm.cancel = cancel;
        vm.show = false;
        vm.isSaving = false;
        var defer = $q.defer();

        $scope.$on("$destroy", function () {            
            defer.reject();
        });

        $rootScope.isLoading = true;
        $rootScope.connectionLost = false;
        $rootScope.reconnecting = false;
        $rootScope.activated = false;

        activate()
            .then(function (response) {
                vm.userConfig = response.data;

                // Check for power line default value
                if (vm.userConfig.PowerLineFrequency === undefined) {
                    vm.userConfig.PowerLineFrequency = 60; // this must match the C# UserConfiguration default value
                }

                // create a copy for canceling
                vm.backup = angular.copy(vm.userConfig);
            });

        function activate() {

            var request = function () {
                if ($rootScope.reconnecting)
                    $rootScope.activated = true;

                portalService.getUserConfig()
                    .then(function success(response) {
                        $rootScope.isLoading = false;
                        $rootScope.connectionLost = false;
                        $rootScope.reconnecting = false;
                        $rootScope.activated = true;

                        defer.resolve(response);
                    }, function error() {
                        $rootScope.connectionLost = true;
                        $rootScope.isLoading = false;
                        // wait 5 seconds then retry
                        vm.timeout = $timeout(function () {
                            $rootScope.reconnecting = true;
                            request();
                        }, 5000);
                    });
            }

            request();

            return defer.promise;
        }

        checkChangeStateFactory.checkFormOnStateChange($scope);

        function save() {
            vm.isSaving = true;
            portalService.setUserConfig(vm.userConfig)
                .then(function () {
                    showAlert('success', $translate.instant("STE_SENTENCE_ConfigurationSavedSuccessfully"));
                    vm.backup = angular.copy(vm.userConfig); // refresh the backup snapshot
                    vm.form.$setPristine();
                }, function () {
                    //error
                    showAlert('danger', $translate.instant("STE_SENTENCE_CouldNotSaveConfiguration"));
                }).finally(function () {
                    vm.isSaving = false;
            });
        }        

        function cancel() {
            vm.userConfig = angular.copy(vm.backup); // revert to backup values
            vm.form.$setPristine();
        }

        // type can be: success, info, warning or danger
        // msg is a string
        function showAlert(type, msg) {
            vm.show = true;
            vm.alert = { type: type, msg: msg }
            $timeout(function () {
                vm.show = false;
            }, 5000);
        }       
    }
})();
