﻿(function () {
    "use strict";

    angular
        .module("portalApp.controllers")
        .controller("SecurityController", SecurityController);

    SecurityController.$inject = ["portalService", "$rootScope", "$scope", "$uibModal", "$timeout", "backendServerUrl", "checkChangeStateFactory", "$window", "signalRService", "$interval", "$translate", "$q", "ngClipboard"];

    function SecurityController(portalService, $rootScope, $scope, $uibModal, $timeout, backendServerUrl, checkChangeStateFactory, $window, signalRService, $interval, $translate, $q, ngClipboard) {
        var vm = this;
        vm.title = "SecurityController";        
        vm.cancel = cancel;
        vm.openPasswordModal = openPasswordModal;
        vm.passwordStrength = { score: 0 };
        vm.oldPassword = undefined;
        vm.newPassword1 = undefined;
        vm.newPassword2 = undefined;
        vm.show = false;
        vm.isSaving = false;
        vm.isBusy = false;
        vm.copied = false;
        vm.alert = {};
        vm.save = save;
        vm.saveAndReboot = saveAndReboot;
        vm.downloadCertificateBaseUrl = backendServerUrl + "/portal/certificate.cer";
        vm.openUploadCertDialog = openUploadCertDialog;
        vm.openSelfSignedModal = openSelfSignedModal;
        vm.openCreateCsrModal = openCreateCsrModal;
        vm.openCertificateModal = openCertificateModal;
        vm.uploadFile = uploadFile;
        vm.deleteCertificate = deleteCertificate;
        vm.updateCertificates = updateCertificates;
        vm.setActive = setActive;
        vm.setRemoteAssistance = setRemoteAssistance;
        vm.toClipboard = toClipboard;
        vm.isAlreadyHttps = location.protocol === "https:";
        vm.httpsStatusClass = '';
        vm.httpsStatus = false;
        vm.needReboot = false;
        vm.certificateFile = null;
        vm.certificatesd = null;
        vm.SSValidity = 5; //default to 5 years                
        vm.ping = null;
        vm.timeout = undefined;
        vm.remoteAssistanceDate = undefined;
        var defer = $q.defer();

        $scope.$on('$destroy', function () {
            // Make sure that the interval is destroyed too
            if (vm.ping) {
                $interval.cancel(vm.ping);
            }
            $timeout.cancel(vm.timeout);
            // cancel all promises
            defer.reject();
        });

        $rootScope.isLoading = true;
        $rootScope.connectionLost = false;
        $rootScope.reconnecting = false;
        $rootScope.activated = false;        

        activate()
            .then(function(response) {
                $rootScope.isLoading = false;
                $rootScope.connectionLost = false;
                $rootScope.reconnecting = false;
                $rootScope.activated = true;
            })
            .catch(function() {
                //console.log("OMG FAIL!!!");
            });

        function activate() {
            // this calls the WebPortal and does not need to be put in promise chain
            portalService.getRemoteAssistanceConfig()
                .then(function(response) {
                    var rac = response.data;
                    vm.isRemoteAssistanceEnabled = rac.enabled;
                    vm.remoteAssistanceDate = rac.enabledUntilISO8601Formatted; // set for use in calculateRemoteAssistanceExpirationDate()
                    calculateRemoteAssistanceExpirationDate();
                });

            var request = function() {
                var promises = [];
                promises.push(portalService.getUserConfig());
                promises.push(portalService.getProperties());
                promises.push(portalService.getCertificates());
                $q.all(promises).then(function(data) {
                    vm.userConfig = data[0].data;
                    calculateRemoteAssistanceExpirationDate();
                    vm.backup = angular.copy(vm.userConfig);                        
                    vm.dto = data[1].data;
                    vm.certificates = data[2].data;
                    vm.backupCertificates = angular.copy(vm.certificates);
                    vm.updateCertificates();
                    vm.SSIPv4Address = vm.dto.IPv4Address;
                    vm.SSIPv6Addresses = vm.dto.IPv6Addresses;
                    vm.CsrIPv4Address = vm.dto.IPv4Address;
                    vm.CsrIPv6Addresses = vm.dto.IPv6Addresses;
                    vm.SSServerName = vm.dto.Name;
                    vm.CsrServerName = vm.dto.Name;
                    vm.httpsStatusClass = vm.userConfig.PortalSecurityConfiguration.UseHttps ? "text-success" : "text-danger";
                    vm.httpsStatus = vm.userConfig.PortalSecurityConfiguration.UseHttps ? "STE_LABEL_Active" : "STE_SENTENCE_MustHaveCertificateToEnableHttps" ;
                    defer.resolve();
                }, function error() {
                    $rootScope.connectionLost = true;
                    $rootScope.isLoading = false;
                    // wait 5 seconds then retry
                    vm.timeout = $timeout(function () {
                        $rootScope.reconnecting = true;
                        request();
                    }, 5000);
                });
            }

            request();

            return defer.promise;
        }

        $scope.closeModal = function () {
            $scope.modalInstance.dismiss("cancel");
        };

        function updateCertificates() {
            if (vm.certificates) {
                angular.forEach(vm.certificates, function (value, key) {
                    var to = moment(value.ValidTo);
                    if (vm.userConfig.DateTimeFormat === 1) // imperial format
                        value.ValidTo = to.format('MM/DD/YYYY h:mm:ss A');
                    else // international format
                        value.ValidTo = to.format('YYYY-MM-DD HH:mm:ss');

                    // update IsExpired field, set to true if ValidTo is passed
                    var now = moment();
                    value.IsExpired = to.isBefore(now);

                    var from = moment(value.ValidFrom);
                    if (vm.userConfig.DateTimeFormat === 1) // imperial format
                        value.ValidFrom = from.format('MM/DD/YYYY h:mm:ss A');
                    else // international format
                        value.ValidFrom = from.format('YYYY-MM-DD HH:mm:ss');

                    value.DownloadLink = vm.downloadCertificateBaseUrl + "?thumbprint=" + value.Thumbprint;
                });                
            }
        }

        // 'vm.remoteAssistanceDate' (and optionnaly 'vm.UserConfig') must be set prior to calling this method.
        // If user config is not set, then default to international format (this will happen when the PlateReader is not responding)
        function calculateRemoteAssistanceExpirationDate() {
            if (!vm.userConfig || vm.userConfig.DateTimeFormat !== 1)
                vm.remoteAssistanceEnabledUntil = moment(vm.remoteAssistanceDate).format("YYYY-MM-DD HH:mm:ss"); // international format
            else
                vm.remoteAssistanceEnabledUntil = moment(vm.remoteAssistanceDate).format("MM/DD/YYYY h:mm:ss A"); // imperial format
        }

        $scope.createSelfSigned = function () {
            vm.isBusy = true;
            var request = {
                ServerName: vm.SSServerName,
                IPv4Address: vm.SSIPv4Address,
                IPv6Addresses: vm.SSIPv6Addresses,
                CountryCode: vm.SSCountry,
                State: vm.SSState,
                Locality: vm.SSLocality,
                Organization: vm.SSOrg,
                OrganizationUnit: vm.SSOrgUnit,
                Validity: vm.SSValidity
            }

            portalService.createSelfSigned(request)
            .then(function (response) {
                portalService.getCertificates()
                    .then(function (result) {
                        vm.certificates = result.data;
                        vm.backupCertificates = angular.copy(vm.certificates);
                        vm.updateCertificates();
                        // when creating a certificate and no active certificate is already configured, set it to active
                        if (!vm.userConfig.PortalSecurityConfiguration.ActiveCertificateThumbprint) {
                            vm.userConfig.PortalSecurityConfiguration.ActiveCertificateThumbprint = response.data;
                            vm.userConfig.PortalSecurityConfiguration.UseHttps = true;
                            vm.needReboot = true;
                            vm.httpsStatus = "STE_LABEL_ActivedAfterReboot";
                            vm.httpsStatusClass = "text-warning";
                            // set new certificate as active
                            angular.forEach(vm.certificates, function (value, key) {
                                if (value.Thumbprint === response.data)
                                    value.Active = true;
                            });
                            vm.form.$setDirty();
                        }
                        else { 
                            // need to keep previous certificate selection
                            angular.forEach(vm.certificates, function (value, key) {
                                if (value.Thumbprint === vm.userConfig.PortalSecurityConfiguration.ActiveCertificateThumbprint && !value.Active)
                                    value.Active = true;
                            });
                        }
                    });
                showAlert('success', $translate.instant("STE_SENTENCE_OperationSucceeded"), true);
                $scope.modalInstance.close();
            }, function () {
                failed($translate.instant("STE_SENTENCE_OperationFailed"));
            }).finally(function() {
                vm.isBusy = false;
            });
        };

        $scope.createCsr = function () {

            var request = {
                ServerName: vm.CsrServerName,
                IPv4Address: vm.CsrIPv4Address,
                IPv6Addresses: vm.CsrIPv6Addresses,
                CountryCode: vm.CsrCountry,
                State: vm.CsrState,
                Locality: vm.CsrLocality,
                Organization: vm.CsrOrg,
                OrganizationUnit: vm.CsrOrgUnit
            }

            portalService.createCsr(request)
            .then(function (response) {
                portalService.getCertificates()
                    .then(function (result) {
                        vm.certificates = result.data;
                        vm.updateCertificates();
                        if (vm.userConfig.PortalSecurityConfiguration.ActiveCertificateThumbprint) {
                            // need to keep previous certificate selection
                            angular.forEach(vm.certificates, function (value, key) {
                                if (value.Thumbprint === vm.userConfig.PortalSecurityConfiguration.ActiveCertificateThumbprint && !value.Active)
                                    value.Active = true;
                            });
                        }
                    });
                showAlert('success', $translate.instant("STE_SENTENCE_OperationSucceeded"), true);
                $scope.modalInstance.close();
            }, function () {
                failed($translate.instant("STE_SENTENCE_OperationFailed"));
            });
        }

        checkChangeStateFactory.checkFormOnStateChange($scope);
        
        $scope.savePassword = function () {
            var result = zxcvbn(vm.newPassword2);
            $scope.modalInstance.close();

            // Make sure the password meets the minimum criteria before submitting as the server can't do much on a hashed password.
            if (result.score < 3) {
                showAlert('danger', $translate.instant("STE_SENTENCE_CannotSetNewPassword"));
                return;
            }

            // first validate old password
            portalService.login("@", vm.oldPassword)
                .then(function (response) {
                    //// delete previous token
                    //delete $window.sessionStorage.token;

                    // then save if all if Ok
                    portalService.setPassword(vm.newPassword2)
                        .then(function (response) {
                            // success
                            showAlert('success', $translate.instant("STE_SENTENCE_PasswordChanged"), true);
                            $window.sessionStorage.token = response.data;
                            vm.oldPassword = "";
                            vm.newPassword1 = "";
                            vm.newPassword2 = "";
                        }, function () {
                            // failed
                            //!!!TODO: indicate the setPassword call did not succeed (why?)
                            failed($translate.instant("STE_SENTENCE_CannotSetNewPassword"));
                        });                    
                }, function () {
                    //!!!TODO: indicate the login call did not succeed (why?)
                    failed($translate.instant("STE_SENTENCE_OldPasswordInvalid"));
                });
        }

        function openUploadCertDialog() {
            if (vm.selectedCertificate) {
                setTimeout(function() {
                    document.getElementById('file').click();
                    $scope.clicked = true;
                }, 0);
            }
        }

        function openSelfSignedModal() {
            $scope.modalInstance = $uibModal.open({
                templateUrl: "selfSigned.html",
                scope: $scope,
                size: "md",
                backdrop: 'static'
            });
        }

        function openCreateCsrModal() {
            $scope.modalInstance = $uibModal.open({
                templateUrl: "createCsr.html",
                scope: $scope,
                size: "md",
                backdrop: 'static'
            });
        }

        function toClipboard(element) {
            vm.copied = true;
            ngClipboard.toClipboard(element);
        }

        function uploadFile(event) {
            var file = event.target.files[0];

            if (file && file.type === "application/x-x509-ca-cert") {
                vm.certificateFile = file;
                $scope.closeModal();

                // the user selected a valid cer file, let's install it
                portalService.setCertificate(vm.certificateFile)
                    .then(function (response) {
                        //update certificates
                        portalService.getCertificates()
                            .then(function (result) {
                                vm.certificates = result.data;
                                vm.backupCertificates = angular.copy(vm.certificates);
                                vm.updateCertificates();

                                // when installing a certificate and no active certificate is already configured, set it to active
                                if (!vm.userConfig.PortalSecurityConfiguration.ActiveCertificateThumbprint) {
                                    vm.userConfig.PortalSecurityConfiguration.ActiveCertificateThumbprint = response.data;
                                    vm.userConfig.PortalSecurityConfiguration.UseHttps = true;
                                    vm.needReboot = true;
                                    vm.httpsStatus = "STE_LABEL_ActivedAfterReboot";
                                    vm.httpsStatusClass = "text-warning";
                                    // set new certificate as active
                                    angular.forEach(vm.certificates, function (value, key) {
                                        if (value.Thumbprint === response.data)
                                            value.Active = true;
                                    });
                                    vm.form.$setDirty();
                                }
                            });
                        showAlert('success', "Install signed certificate... Successful", true);
                    }, function(response) {
                        showAlert('danger', response.data, true);
                        vm.isSaving = false;
                    });
                
            }
            else {
                $timeout(function () {
                    showAlert('danger', $translate.instant("STE_SENTENCE_WrongCertificateFileSelected"), true);
                }, 0);
            }
        }

        function setActive(certificate) {
            vm.httpsStatusClass = "text-warning";            

            if (!certificate.Active) {
                vm.userConfig.PortalSecurityConfiguration.ActiveCertificateThumbprint = "";
                vm.userConfig.PortalSecurityConfiguration.UseHttps = false;
                if (vm.isAlreadyHttps) {
                    vm.needReboot = true;
                    vm.httpsStatus = "STE_LABEL_DeactivedAfterReboot";
                }
                else {
                    vm.httpsStatusClass = "text-danger";  
                    vm.httpsStatus = "STE_SENTENCE_MustHaveCertificateToEnableHttps";
                    vm.needReboot = false;
                }
            }
            else {
                vm.userConfig.PortalSecurityConfiguration.ActiveCertificateThumbprint = certificate.Thumbprint;
                vm.userConfig.PortalSecurityConfiguration.UseHttps = true;
                vm.needReboot = true;
                vm.httpsStatus = "STE_LABEL_ActivedAfterReboot";
            }
            angular.forEach(vm.certificates, function (value, key) {
                // only one certificate can be active
                if (value.Active && certificate.Thumbprint !== value.Thumbprint)
                    value.Active = false;
            });
        }

        function setRemoteAssistance(param) {
            portalService.setRemoteAssistance(param)
                .then(function (response) {
                    vm.isRemoteAssistanceEnabled = response.data.enabled;
                    vm.remoteAssistanceDate = new Date(new Date(response.data.enabledUntilISO8601Formatted));
                    calculateRemoteAssistanceExpirationDate();
                    showAlert('success', $translate.instant("STE_SENTENCE_OperationSucceeded"), true);
                }, function () {
                    showAlert('danger', $translate.instant("STE_SENTENCE_OperationFailed"), true);
                });
        }

        function cancel() {
            vm.userConfig = angular.copy(vm.backup); // revert to backup values
            vm.certificates = angular.copy(vm.backupCertificates);
            vm.updateCertificates();
            vm.httpsStatusClass = vm.userConfig.PortalSecurityConfiguration.UseHttps ? "text-success" : "text-danger";
            vm.httpsStatus = vm.userConfig.PortalSecurityConfiguration.UseHttps ? "STE_LABEL_Active" : "STE_SENTENCE_MustHaveCertificateToEnableHttps";
            vm.needReboot = false;
            vm.form.$setPristine();
        }

        function failed(reason) {
            showAlert('danger', reason);
            $scope.closeModal();
        }

        function openPasswordModal() {
            $scope.modalInstance = $uibModal.open({
                templateUrl: "password.html",
                scope: $scope,
                size: "lg",
                backdrop: 'static'
            });
        }

        function openCertificateModal(certificate) {
            vm.selectedCertificate = certificate;
            $scope.modalInstance = $uibModal.open({
                templateUrl: "certificateDetails.html",
                scope: $scope,
                size: "md",
                backdrop: 'static'
            });
        }        

        function deleteCertificate(certificate) {
            portalService.deleteCertificate(certificate)
                .then(function () {                    
                    var needSaveAndReboot = certificate.Active || certificate.Thumbprint === vm.backup.PortalSecurityConfiguration.ActiveCertificateThumbprint;
                    showAlert('success', $translate.instant("STE_SENTENCE_CertificateDeleted"), !needSaveAndReboot);

                    var index = vm.certificates.indexOf(certificate);
                    vm.certificates.splice(index, 1);  
                    vm.backupCertificates = angular.copy(vm.certificates);
                    vm.updateCertificates();

                    // when deleted certificate is the active one, we need to change the config that will disable https and reboot
                    if (needSaveAndReboot) {  
                        // only when replacing the deleted certificate with another, don't change https settings
                        if (!vm.userConfig.PortalSecurityConfiguration.ActiveCertificateThumbprint ||
                            certificate.Thumbprint === vm.backup.PortalSecurityConfiguration.ActiveCertificateThumbprint ||
                            certificate.Thumbprint === vm.userConfig.PortalSecurityConfiguration.ActiveCertificateThumbprint) {
                            // active certificate has been deleted, remove https config
                            vm.userConfig.PortalSecurityConfiguration.UseHttps = false;
                            vm.userConfig.PortalSecurityConfiguration.ActiveCertificateThumbprint = "";
                        }

                        // need to reboot when certificate already used has been deleted
                        if (certificate.Thumbprint === vm.backup.PortalSecurityConfiguration.ActiveCertificateThumbprint) {                                    
                            vm.saveAndReboot();
                        }
                        else {
                            if (vm.isAlreadyHttps) {
                                vm.needReboot = true;
                                vm.httpsStatus = "STE_LABEL_DeactivedAfterReboot";
                            }
                            else {
                                vm.httpsStatusClass = "text-danger";
                                vm.httpsStatus = "STE_SENTENCE_MustHaveCertificateToEnableHttps";
                                vm.needReboot = false;
                            }

                            // nothing changed, reset the form
                            if (angular.equals(vm.userConfig, vm.backup))
                                vm.form.$setPristine();
                        }                                
                    }
                    
                }, function(response) {
                    showAlert('danger', $translate.instant("STE_SENTENCE_CertificateCouldNotBeDeleted"), true);
            });
        }


        function saveAndReboot() {
            vm.isSaving = true;
            vm.isRebooting = true;
            signalRService.stopConnections();
            portalService.setUserConfig(vm.userConfig)
            showAlert("info", $translate.instant("STE_SENTENCE_RebootingPleaseWait"), false);
            
            if (!vm.userConfig.PortalSecurityConfiguration.UseHttps && vm.backup.PortalSecurityConfiguration.UseHttps) {
                // going from https to http avoids us to use ping because of mixed site content blocked by browsers
                $timeout(function() {
                    vm.isRebooting = false;
                    $window.location = "http://" + location.hostname;
                }, 75000);
            } else {
                var counter = 0;                
                vm.ping = $interval(function () {
                    if (counter === 7) {
                        vm.isRebooting = false;
                        $window.location = "http://" + location.hostname; // even when https setting is active, the browser will be redirected
                    }
                    counter++;
                    portalService.getPortalPing()
                        .then(function(result) {
                            showAlert("success", $translate.instant("STE_SENTENCE_RebootCompletedConnectionRestored"), true);
                            vm.isRebooting = false;
                            $window.location.href = "/";
                            $interval.cancel(vm.ping); // connection restored, stop pinging
                        }, function() {

                        });
                }, 10000);
            }
        }

        function save() {
            vm.isSaving = true;
            if (vm.needReboot) {
                return vm.saveAndReboot();
            }

            portalService.setUserConfig(vm.userConfig)
                .then(function () {
                    showAlert('success', $translate.instant("STE_SENTENCE_ConfigurationSavedSuccessfully"), true);
                    vm.backup = angular.copy(vm.userConfig); // refresh the backup snapshot
                    vm.form.$setPristine();
                }, function () {
                    //error
                    showAlert('danger', $translate.instant("STE_SENTENCE_CouldNotSaveConfiguration"), true);
                }).finally(function () {
                    vm.isSaving = false;
            });
        }

        // type can be: success, info, warning or danger
        // msg is a string
        function showAlert(type, msg, useTimeout) {
            vm.show = true;
            vm.alert = { type: type, msg: msg }
            if (useTimeout) {
                $timeout(function() {
                    vm.show = false;                    
                }, 5000);
            }
        }
    }
})();
