﻿(function () {
    "use strict";

    angular
        .module("portalApp.controllers")
        .controller("SpeedEstimationController", SpeedEstimationController);

    SpeedEstimationController.$inject = ["portalService", "$scope", "$rootScope", "$timeout", "checkChangeStateFactory", "$translate", "$q", "$uibModal", "$state", "signalRService"];

    function SpeedEstimationController(portalService, $scope, $rootScope, $timeout, checkChangeStateFactory, $translate, $q, $uibModal, $state, signalRService) {
        var vm = this;
        vm.title = "SpeedEstimationController";
        vm.save = save;
        vm.cancel = cancel;
        vm.show = false;
        vm.show = false;
        vm.isSaving = false;
        vm.alert = {};
        vm.showStickyAlert = showStickyAlert;
        vm.start = start;
        vm.cancelCalibration = cancelCalibration;
        vm.edit = edit;
        vm.clearCalibration = clearCalibration;
        vm.closeModal = closeModal;
        vm.updateCalibrationStateClass = updateCalibrationStateClass;
        vm.calibrationStateClass = 'text-danger';
        vm.calibrationState = undefined;
        vm.calibrationStateSummary = undefined;
        vm.calibrationStateRatio = 0;
        vm.calibrationStateProgressClass = 'progress-bar-info';
        vm.isAdvanced = false;
        vm.calibrationDataHasAwaitedPlates = false;
        vm.updateTranslation = updateTranslation;

        // Edition page stuff
        vm.plateToAdd = '';
        vm.calibrationData = undefined;
        vm.cancelEdit = cancelEdit;
        vm.doneEdit = doneEdit;
        vm.addPlate = addPlate;
        vm.deletePlateToWaitFor = deletePlateToWaitFor;
        vm.updateSummary = updateSummary;

        $rootScope.isLoading = true;
        $rootScope.connectionLost = false;
        $rootScope.reconnecting = false;
        $rootScope.activated = false;

        var defer = $q.defer();

        vm.unregisterTranslateChangeSuccess = $rootScope.$on('$translateChangeSuccess', function () {
            vm.updateTranslation();
        });

        if ($translate.isReady())
            vm.updateTranslation();
        
        function updateTranslation() {
            vm.calibrationState = $translate.instant('STE_LABEL_Uncalibrated');
        }
    
        // Clean up to perform when the view is closed
        $scope.$on("$destroy", function () {
            $timeout.cancel(vm.timeout);
            defer.reject();
            vm.unregisterHub();
            vm.unregisterSpeedEstimationError();
        });

        activate()
            .then(function (response) {
                $rootScope.isLoading = false;
                $rootScope.connectionLost = false;
                $rootScope.reconnecting = false;
                $rootScope.activated = true;
            })
            .catch(function () {
                //console.log("OMG FAIL!!!");
            });

        function activate() {

            var request = function () {
                var promises = [];
                promises.push(portalService.getUserConfig());
                promises.push(portalService.getSpeedEstimationSummary());
                if ($state.current.name === 'main.config.lpr.speedEstimationEdit')
                    promises.push(portalService.getSpeedEstimationCalibrationData());
                $q.all(promises).then(function (response) {
                    vm.userConfig = response[0].data;
                    vm.backup = angular.copy(vm.userConfig);
                    vm.calibrationStateSummary = response[1].data;
                    if (vm.calibrationStateSummary !== null)
                        vm.updateCalibrationStateClass();

                    if ($state.current.name === 'main.config.lpr.speedEstimationEdit') {
                        vm.calibrationData = response[2].data;
                        vm.calibrationDataHasAwaitedPlates = Object.keys(vm.calibrationData.AwaitedPlates).length > 0 ? true: false;
                    }
                    
                    defer.resolve();
                    }, function error() {
                        $rootScope.connectionLost = true;
                        $rootScope.isLoading = false;
                        // wait 5 seconds then retry
                        vm.timeout = $timeout(function () {
                            $rootScope.reconnecting = true;
                            request();
                        }, 5000);
                    });
            }

            request();

            return defer.promise;
        }

        // Performance hub
        signalRService.initializeSpeedEstimationHub();

        vm.unregisterHub = $rootScope.$on("CalibrationReceived", function (e, message) {
            $rootScope.$apply(function () {
                vm.doneEdit();
                vm.updateSummary();
            });
        });

        // Genetec error message handling
        vm.unregisterSpeedEstimationError = $rootScope.$on("SpeedEstimationError", function (e, message) {
            vm.showStickyAlert('danger', $translate.instant("STE_SENTENCE_OperationFailed"));
        });

        function save() {
            vm.isSaving = true;            

            portalService.setUserConfig(vm.userConfig)
                .then(function () {
                    vm.showStickyAlert('success', $translate.instant("STE_SENTENCE_ConfigurationSavedSuccessfully"));
                    vm.backup = angular.copy(vm.userConfig); // refresh the backup snapshot
                    vm.form.$setPristine();
                }, function () {
                    //error
                    vm.showStickyAlert('danger', $translate.instant("STE_SENTENCE_CouldNotSaveConfiguration"));
                }).finally(function () {
                    vm.isSaving = false;
                    vm.updateSummary();
                });

        }

        function showStickyAlert(type, msg) {
            vm.alert = { type: type, msg: msg }
            vm.show = true;
            $timeout(function () {
                vm.show = false;
            }, 5000);
        }

        function cancel() {
            vm.userConfig = angular.copy(vm.backup); // revert to backup values
            vm.form.$setPristine();
        }

        function clearCalibration() {
            vm.isClearingSpeedEstimationCalibration = true;
            portalService.speedEstimationClearCalibration()
                .then(function (result) {
                    $scope.modalInstance.close();
                }, function () {
                    //error
                    $scope.modalInstance.dismiss("cancel");
                    showStickyAlert("danger", $translate.instant("STE_SENTENCE_OperationFailed"), true);
                }).finally(function () {
                    vm.isClearingSpeedEstimationCalibration = false;
                    vm.updateSummary();
                });
        };

        function closeModal(){
            $scope.modalInstance.close();
        }

        function start() {
            portalService.postSpeedEstimationStartCalibration()
                .then(function() {
                    vm.showStickyAlert('success', $translate.instant("STE_SENTENCE_CalibrationStartedSuccessfully"));
                    // update summary state object
                    vm.updateSummary();
                }, function() {
                    //error
                    vm.showStickyAlert('danger', $translate.instant("STE_SENTENCE_CouldNotStartCalibration"));
                });
        }

        function updateCalibrationStateClass() {
            if (vm.calibrationStateSummary.ManualActionRequired !== 'None') {
                switch(vm.calibrationStateSummary.ManualActionRequired) {
                    case 'EditDataOrRestartCalibration':
                        vm.calibrationState = $translate.instant('STE_SENTENCE_EditDataOrRestartCalibration');
                        break;
                    case 'EnterAPlateToWaitFor':
                        vm.calibrationState = $translate.instant('STE_SENTENCE_EnterAPlateToAwait');
                        break;
                    case 'StartCalibration':
                        vm.calibrationState = $translate.instant("STE_SENTENCE_StartCalibration");
                        break;
                    case 'ValidateOneOfTheReceivedPlates':
                        vm.calibrationState = $translate.instant('STE_SENTENCE_ValidateOneOfTheReceivedPlates');
                        break;
                }
                
                vm.calibrationStateClass = 'text-danger';
            }

            if (vm.calibrationStateSummary.State === 'Calibrated') {
                vm.calibrationStateClass = 'text-success';
                vm.calibrationState = $translate.instant('STE_LABEL_Ready');
            }
            else if (vm.calibrationStateSummary.State === 'FinalizingCalibration') {
                vm.calibrationState = $translate.instant('STE_LABEL_FinalizingCalibration');
            }

            vm.calibrationStateRatio = (vm.calibrationStateSummary.NotWaitedForVehicleCount / vm.calibrationStateSummary.NotWaitedForVehicleRequiredCount) * 100;

            if (vm.calibrationStateRatio === 100)
                vm.calibrationStateProgressClass = 'progress-bar-success';
        }

        function deletePlateToWaitFor(id) {
            var defer = $q.defer();
            portalService.deleteSpeedEstimationPlateToWaitFor(id)
                .then(function () {
                    // recover updated calibration data
                    vm.doneEdit()
                        .then(function() {
                            defer.resolve();
                        }, function () {
                            //error                    
                            defer.reject();
                        });
                }, function () {
                    //error                    
                    defer.reject();
                });

            return defer.promise;
        } 

        function cancelCalibration() {
            // open dialog to confirm data will be cleared
            $scope.modalInstance = $uibModal.open({
                windowClass: "modal-danger",
                templateUrl: "speedEstimationClearCalibration.html",
                scope: $scope,
                size: "md",
                backdrop: 'static'
            });            
        }

        function edit() {
            $state.go("main.config.lpr.speedEstimationEdit");
        }

        function cancelEdit() {
            vm.updateSummary()
                .then(function() {
                    $state.go("main.config.lpr.speedEstimation");
                });            
        }

        function doneEdit() {
            var defer = $q.defer();
            portalService.getSpeedEstimationCalibrationData()
                .then(function (response) {
                    if (vm.calibrationData == undefined) {
                        vm.calibrationData = response.data;
                    } else {
                        vm.calibrationData.AwaitedPlates = response.data.AwaitedPlates;
                        vm.calibrationData.OtherPlates = response.data.OtherPlates; 
                    }

                    vm.calibrationDataHasAwaitedPlates = Object.keys(vm.calibrationData.AwaitedPlates).length > 0 ? true : false;
                    defer.resolve();
                });

            return defer.promise;
        }

        function updateSummary() {
            var defer = $q.defer();
            portalService.getSpeedEstimationSummary()
                .then(function (response) {
                    vm.calibrationStateSummary = response.data;
                    if (vm.calibrationStateSummary !== null)
                        vm.updateCalibrationStateClass();
                    defer.resolve();
                });
            return defer.promise;
        }

        function addPlate() {
            portalService.postSpeedEstimationAddPlate(vm.plateToAdd.toUpperCase())
                .then(function() {
                    vm.plateToAdd = ''; // clear input
                    vm.form.$setPristine();
                    // recover updated calibration data
                    vm.doneEdit();
                }, function () {
                    //error
                    vm.showStickyAlert('danger', $translate.instant("STE_SENTENCE_CouldNotSaveConfiguration"));
                });
        }
    }
})();
