﻿(function () {
    "use strict";

    angular
        .module("portalApp.controllers")
        .controller("MapController", MapController);

    MapController.$inject = ["$scope", "uiGmapGoogleMapApi", "uiGmapIsReady", "coords", "$uibModalInstance"];

    function MapController($scope, uiGmapGoogleMapApi, uiGmapIsReady, coords, $uibModalInstance) {
        var vm = this;
        vm.coords = coords;
        $scope.render = true;

        $scope.close = function () {
            $uibModalInstance.close();
        };

        if (!!!$scope.map) {
            $scope.map = {
                center: {
                    // center to Montréal (Genetec) if coords not defined
                    latitude: vm.coords ? vm.coords.Y : 45.479722,
                    longitude: vm.coords ? vm.coords.X : -73.763611
                },
                zoom: 14
            };
        }

        uiGmapGoogleMapApi.then(function (maps) {
            // workaround a change done in lodash version 4 from version 3 see: https://github.com/angular-ui/angular-google-maps/issues/1682
            if (typeof _.contains === 'undefined') {
                _.contains = _.includes;
            }
            if (typeof _.object === 'undefined') {
                _.object = _.zipObject;
            }            
        });

        uiGmapIsReady.promise(1).then(function (instances) {
            instances.forEach(function (inst) {
                var map = inst.map;
                // call resize so that map render takes the whole space
                google.maps.event.trigger(map, 'resize');

                if (vm.coords) {
                    var coords = new google.maps.LatLng(vm.coords.Y, vm.coords.X);
                    var marker = new google.maps.Marker({
                        position: coords,
                        title: "SharpEVO"
                    });

                    // To add the marker to the map, call setMap();
                    marker.setMap(map);

                    // set center after resize so that it stays centered
                    map.setCenter(coords);
                }
            });
        });
    }
})();