﻿(function() {
    "use strict";

    angular
        .module("portalApp.controllers")
        .controller("OverviewController", OverviewController);

    OverviewController.$inject = ["$scope", "$rootScope", "$timeout", "$translate", "portalService", "signalRService", "$filter", "$uibModal", "$q", "$ocLazyLoad"];

    function OverviewController($scope, $rootScope, $timeout, $translate, portalService, signalRService, $filter, $uibModal, $q, $ocLazyLoad) {
        var vm = this;
        vm.error = "";
        vm.title = "OverviewController";    
        vm.totalCpuUsage = 0;
        vm.cpusUsage = null;
        vm.storageRatio = undefined;
        vm.storageDbSize = $filter("bytes")(0);
        vm.storageMaxDbSize = $filter("bytes")(0);
        vm.storageNbOfReads = undefined;
        vm.ramUsage = undefined;
        vm.ramTotal = undefined;
        vm.ramRatio = undefined;
        vm.driveCUsage = undefined;
        vm.driveCTotal = undefined;
        vm.driveCRatio = undefined;
        vm.driveDUsage = undefined;
        vm.driveDTotal = undefined;
        vm.driveDRatio = undefined;
        vm.driveEUsage = undefined;
        vm.driveETotal = undefined;
        vm.driveERatio = undefined;
        vm.elapsedLogin = "-";
        vm.elapsedReboot = "-";
        vm.elapsedAppRestart = "-";
        vm.elapsedTimeSync = "-";
        vm.extensionName = "";
        vm.isExtensionConnected = false;
        vm.gatewayConnection = false;
        vm.isInternetConnected = false;
        vm.isUploadToCloudConnected = null;
        vm.videoStreams = undefined;
        vm.readsStored = 60;
        vm.getProgressBarClassOnPercentage = getProgressBarClassOnPercentage;
        vm.getTextClassOnBoolean = getTextClassOnBoolean;
        vm.getStateFromBoolean = getStateFromBoolean;
        vm.textClassOnLicense = getTextClassOnLicense(undefined);
        vm.licenseStatus = "STE_LABEL_LicenseStatusUnknown";
        vm.openVersionsModal = openVersionsModal;
        vm.openMapModal = openMapModal;
        vm.openUploadToCloudDetailsModal = openUploadToCloudDetailsModal;
        vm.blinkLed = blinkLed;
        vm.isLoadingFileVersions = false;
        vm.isLoadingComponentsVersions = false;
        vm.isLoadingUploadToCloudDetails = false;
        vm.openTestRelaysModal = openTestRelaysModal;
        vm.toggleOutputRelay = toggleOutputRelay;
        vm.inputStates = null;
        vm.outputStates = null;
        vm.timeout = undefined;
        vm.displayType = null;
        vm.files = [];
        vm.components = [];
        vm.uploadToCloudDetails = [];
        $rootScope.isLoading = true;
        $rootScope.connectionLost = false;
        $rootScope.reconnecting = false;
        $rootScope.activated = false;

        var defer = $q.defer();
        // Clean up to perform when the view is closed
        $scope.$on("$destroy", function () {
            $timeout.cancel(vm.timeout);
            defer.reject();
        });

        // Initialize the 2 ui-grids
        vm.gridFiles = {
            enableSorting: false,
            enableColumnResizing: false,
            enableColumnMenus: false,
            enableHorizontalScrollbar : 0, // hidden
            enableVerticalScrollbar: 2, // when needed
            columnDefs: [
                { displayName: "STE_LABEL_Name", field: "Name", headerCellFilter: "translate", width: "80%" },
                { displayName: "STE_LABEL_Version", field: "Version", headerCellFilter: "translate", width: "20%" }
            ],
            onRegisterApi: function (gridApi) {
                vm.gridApiFiles = gridApi;
            }
        };

        vm.gridComponents = {
            enableSorting: false,
            enableColumnResizing: false,
            enableColumnMenus: false,
            enableHorizontalScrollbar: 0, // hidden
            enableVerticalScrollbar: 2, // when needed
            columnDefs: [
                { displayName: "STE_LABEL_Components", field: "getComponentName()", headerCellFilter: "translate", width: "60%" },
                { displayName: "STE_LABEL_Version", field: "Version", headerCellFilter: "translate", width: "20%" },
                { displayName: "STE_LABEL_MeetsRequirements", field: "getRequirements()", headerCellFilter: "translate", width: "20%" }
            ],
            onRegisterApi: function (gridApi) {
                vm.gridApiComponents = gridApi;
            }
        };

        activate()
            .then(function(response) {
                vm.dto = response.data;
                vm.inputStates = vm.dto.Inputs;
                vm.outputStates = vm.dto.Relay;
                vm.textClassOnLicense = getTextClassOnLicense(vm.dto.License);
                vm.licenseStatus = "STE_LABEL_LicenseStatus" + vm.dto.License;
                vm.displayType = (vm.dto.IsIts) ? vm.dto.Type + " ITS" : vm.dto.Type;
            });

        function activate() {            
            var request = function() {
                if ($rootScope.reconnecting)
                    $rootScope.activated = true;

                portalService.getProperties()
                    .then(function success(response) {
                        $rootScope.isLoading = false;
                        $rootScope.connectionLost = false;
                        $rootScope.reconnecting = false;
                        $rootScope.activated = true;

                        defer.resolve(response);
                    }, function error() {
                        $rootScope.connectionLost = true;
                        $rootScope.isLoading = false;
                        // wait 5 seconds then retry
                        vm.timeout = $timeout(function () {
                            $rootScope.reconnecting = true;
                            request();
                        }, 5000);
                    });
            }

            request();

            return defer.promise;
        }

        function getProgressBarClassOnPercentage(value, warningPercent, dangerPercent) {
            if (value < warningPercent)
                return "progress-bar-success";
            else if (value < dangerPercent)
                return "progress-bar-warning";
            else
                return "progress-bar-danger";
        }

        function getTextClassOnBoolean(value) {
            if (value)
                return "text-success";
            else
                return "text-danger";
        }

        function getTextClassOnLicense(value) {
            if (value === "Valid")
                return "text-success";
            else if (value === "Unknown")
                return "text-info";
            else
                return "text-danger";
        }

        function getStateFromBoolean(value) {
            if (value)
                return $translate.instant("STE_LABEL_InputStateOn");
            else
                return $translate.instant("STE_LABEL_InputStateOff");
        }

        $scope.closeModal = function () {
            $scope.modalInstance.close();
        };

        function openVersionsModal() {
            vm.isLoadingFileVersions = true;
            vm.isLoadingComponentsVersions = true;
            $scope.modalInstance = $uibModal.open({
                templateUrl: "versions.html",
                size: "lg",
                scope: $scope,
                resolve: {
                    files: function () {
                        portalService
                            .getInstalledFiles()
                            .then(
                                function success(response) {
                                    vm.files = response.data;
                                    vm.gridFiles.data = vm.files;
                                },
                                function failure(response) {
                                    vm.error = response;
                                })
                            .finally(function () {
                                vm.isLoadingFileVersions = false;
                            });
                    },
                    components: function () {
                        portalService
                            .getComponents()
                            .then(
                                function success(response) {
                                    vm.components = response.data;

                                    // Add helper methods for the ui-grid infinite scroll
                                    angular.forEach(vm.components, function (row) {
                                        row.getComponentName = function () {
                                            return this.ComponentType.concat(this.IsKnownComponent ? "" : " *");
                                        }

                                        row.getRequirements = function () {
                                            return this.IsVersionAcceptable ? "ok" : this.MinimumVersion;
                                        }
                                    });

                                    vm.gridComponents.data = vm.components;
                                },
                                function failure(response) {
                                    vm.error = response;
                                })
                            .finally(function () {
                                vm.isLoadingComponentsVersions = false;
                            });
                    }
                }
            });
        }

        function openUploadToCloudDetailsModal() {
            vm.isLoadingUploadToCloudDetails = true;
            $scope.modalInstance = $uibModal.open({
                templateUrl: "uploadToCloudDetails.html",
                size: "lg",
                scope: $scope,
                resolve: {
                    details: function () {
                        portalService
                            .getUploadToCloudDetails()
                            .then(
                            function success(response) {
                                vm.uploadToCloudDetails = response.data;
                            },
                            function failure(response) {
                                vm.error = response;
                            })
                            .finally(function () {
                                vm.isLoadingUploadToCloudDetails = false;
                            });
                    }
                }
            });
        }

        function openMapModal() {
            // this will load the js files async to reduce the initial portal page load
            $ocLazyLoad.load('js/bower/angular-google-maps.min.js')
                .then(function (result) {
                    $scope.modalInstance = $uibModal.open({
                        templateUrl: "map.html",
                        controller: "MapController",
                        size: "lg",
                        scope: $scope,
                        resolve: {
                            coords: function() {
                                return vm.coords;
                            }
                        }
                    });
                });

            // uncomment if you want to see the modules load events
            //$scope.$on('ocLazyLoad.moduleLoaded', function (e, module) {
            //    console.log('module name : ', module);                
            //});                 
        }

        function openTestRelaysModal() {
            $scope.modalInstance = $uibModal.open({
                templateUrl: "testRelays.html",
                size: "sm",
                scope: $scope
            });        
        }

        function blinkLed() {
            portalService.startIdentify();
        }
        
        function toggleOutputRelay(id) {
            portalService.toggleOutputRelay(id)
            .then(function () {
                // eventually the output state will be refreshed in the UI by a SignalR pushed message
            }
            , function (result) {
                // failure: shake the status
                var x = document.getElementById(id);
                if (x) {
                    x.classList.add("shake");
                    $timeout(function() {
                        x.classList.remove("shake");
                    }, 1500);
                }
            });
        }
        
        // Performance hub
        signalRService.initializePerformanceHub();

        $rootScope.$on("CpuUsage", function (e, message) {
            $rootScope.$apply(function () {
                vm.totalCpuUsage = $filter("setDecimal")(message, 0);
            });
        });

        $rootScope.$on("CpusUsage", function (e, message) {
            $rootScope.$apply(function () {
                vm.cpusUsage = message;
                angular.forEach(vm.cpusUsage, function(cpu) {
                    cpu.CpuUsage = $filter("setDecimal")(cpu.CpuUsage, 0);
                });
            });
        });

        $rootScope.$on("DriveCSpace", function (e, message) {
            $rootScope.$apply(function () {
                vm.driveCUsage = $filter("bytes")(message.TotalSpace - message.FreeSpace);
                vm.driveCTotal = $filter("bytes")(message.TotalSpace);
                vm.driveCRatio = message.RatioUsed;
            });
        });
        
        $rootScope.$on("DriveDSpace", function (e, message) {
            $rootScope.$apply(function () {
                vm.driveDUsage = $filter("bytes")(message.TotalSpace - message.FreeSpace);
                vm.driveDTotal = $filter("bytes")(message.TotalSpace);
                vm.driveDRatio = message.RatioUsed;
            });
        });

        $rootScope.$on("DriveESpace", function (e, message) {
            $rootScope.$apply(function () {
                vm.driveEUsage = $filter("bytes")(message.TotalSpace - message.FreeSpace);
                vm.driveETotal = $filter("bytes")(message.TotalSpace);
                vm.driveERatio = message.RatioUsed;
            });
        });

        $rootScope.$on("RamUsage", function (e, message) {
            $rootScope.$apply(function () {
                vm.ramUsage = $filter("bytes")(message.TotalSpace - message.FreeSpace);
                vm.ramTotal = $filter("bytes")(message.TotalSpace);
                vm.ramRatio = message.RatioUsed;
            });
        });

        $rootScope.$on("StorageInfo", function (e, message) {
            $rootScope.$apply(function () {
                vm.storageNbOfReads = message.NbOfReads;
                vm.storageRatio = (message.DbSize / message.MaxDbSize) * 100;
                vm.storageDbSize = $filter("bytes")(message.DbSize);
                vm.storageMaxDbSize = $filter("bytes")(message.MaxDbSize);
            });
        });

        // IO hub
        signalRService.initializeIoHub();

        $rootScope.$on("IoStates", function (e, message) {
            $rootScope.$apply(function () {
                if (!angular.equals(message.Inputs, vm.inputStates)) {
                    vm.inputStates = message.Inputs;
                }

                if (!angular.equals(message.Outputs, vm.outputStates)) {
                    vm.outputStates = message.Outputs;
                }
            });
        });

        // Last activities hub
        signalRService.initializeLastActivitiesHub();
        
        $rootScope.$on("LastActivities", function (e, message) {
            $rootScope.$apply(function () {
                vm.elapsedReboot = $filter("elapsedTime")(message.LastReboot, message.ReferenceTime);
                vm.elapsedAppRestart = $filter("elapsedTime")(message.LastAppRestart, message.ReferenceTime);
                vm.elapsedTimeSync = $filter("elapsedTime")(message.LastTimeSync, message.ReferenceTime);
                vm.elapsedLogin = $filter("elapsedTime")(message.LastLogin, message.ReferenceTime);
                vm.extensionName = message.ExtensionName;
                vm.isExtensionConnected = message.IsExtensionConnected;
                vm.gatewayConnection = message.GatewayConnection;
                vm.isUploadToCloudConnected = message.IsUploadToCloudConnected;
                vm.videoStreams = message.VideoStreams;
                vm.coords = message.Coords;
            });
        });

        $rootScope.$on("UpdateInternetConnectivityStatus", function(e, message) {
            $rootScope.$apply(function() {
                vm.isInternetConnected = message;
            });
        });
    }
})();