﻿(function () {
    "use strict";

    angular.module('portalApp.directives')
        .directive('awaitedPlate', awaitedPlate);


    function awaitedPlate() {
        var controller = ['$scope', '$uibModal', '$window', 'portalService', '$rootScope',
            function ($scope, $uibModal, $window, portalService, $rootScope) {

                var vm = this;
                vm.isNumber = angular.isNumber;
                vm.loading = false;
                vm.notReceivedPlate = undefined;

                function init() {
                    vm.busy = false;

                    // substitue the utc time stamp for local
                    angular.forEach(vm.plate.ReceivedPlates, function (plate, key) {
                        var m = moment.utc(plate.TimeStamp).local();
                        plate.TimeStamp = m.format('YYYY-MM-DDTHH:mm:ssZ'); // required format for 'date' filter input
                    });

                    $scope.$watchCollection(function () {
                        return vm.plate.ReceivedPlates;
                    }, function (value) {
                        if (Object.keys(vm.plate.ReceivedPlates).length > 0)
                            vm.notReceivedPlate = false;
                        else
                            vm.notReceivedPlate = true;
                    });
                }

                init();

                vm.handleError = function () {
                    vm.busy = false;
                    $rootScope.$emit("SpeedEstimationError", {});
                }

                vm.delete = function () {
                    if (vm.busy)
                        return;

                    vm.busy = true;

                    var id = vm.plate.PlateToWaitForId;
                    // calls the del method on the SpeedEstimationController with id as argument
                    vm.del()(id)
                        .then(function successCallback(response) {
                            vm.busy = false;
                        }, function errorCallback(response) {
                            vm.handleError();
                        });
                }

                vm.removePlate = function () {

                    vm.deletePlate(vm.editedPlate, true);
                }

                vm.deletePlate = function (plate, goToNext) {
                    if (plate.busy)
                        return;

                    plate.busy = true;
                    portalService.deleteSpeedEstimationCalibrationPlate(plate.PlateId)
                        .then(function successCallback(response) {
                            // recover updated calibration data                            
                            vm.done()
                                .then(function successCallback(response) {
                                    plate.busy = false;
                                    if (goToNext) {
                                        if (vm.nextPlate !== undefined)
                                            vm.next();
                                        else if (vm.previousPlate !== undefined)
                                            vm.previous();
                                        else
                                            vm.closeModal();
                                    }
                                }, function errorCallback(response) {
                                    vm.handleError();
                                });
                        }, function errorCallback(response) {
                            vm.handleError();
                        });
                }

                vm.imageCanvasImageLoaded = function () {
                    var canvas = this.canvas;

                    // Take 100% width and preserve aspect ratio for height
                    var parent = canvas.parentElement;
                    var aspectRatio = this.height / this.width;
                    canvas.width = parent.clientWidth;
                    canvas.height = parent.clientWidth * aspectRatio;
                    vm.drawImage(canvas);

                    // listen for mouse events
                    canvas.onmousedown = myDown;
                    canvas.onmouseup = myUp;
                    canvas.onmousemove = myMove;
                }

                vm.drawImage = function (canvas) {
                    if (canvas === null || canvas === undefined)
                        return;
                    var ctx = canvas.getContext("2d");
                    if (ctx === null || ctx === undefined)
                        return;

                    var image = $window.document.getElementById(canvas.imageId);
                    if (image === null || image === undefined)
                        return;

                    var tnCanvas = document.createElement('canvas');
                    var tnCanvasContext = tnCanvas.getContext('2d');

                    tnCanvas.width = image.width * 2;
                    tnCanvas.height = image.height * 2;

                    tnCanvasContext.drawImage(image, 0, 0);

                    // draw lines under plate number
                    tnCanvasContext.strokeStyle = "#D27E00";
                    tnCanvasContext.lineWidth = 2;
                    tnCanvasContext.beginPath();
                    tnCanvasContext.moveTo(canvas.shapes[0].x, canvas.shapes[0].y);
                    tnCanvasContext.lineTo(canvas.shapes[1].x, canvas.shapes[1].y);
                    tnCanvasContext.stroke();

                    // draw end of lines
                    tnCanvasContext.moveTo(canvas.shapes[0].x, canvas.shapes[0].y);
                    tnCanvasContext.arc(canvas.shapes[0].x, canvas.shapes[0].y, 4, 0, 2 * Math.PI, false);
                    tnCanvasContext.fillStyle = '#D27E00';
                    tnCanvasContext.fill();
                    tnCanvasContext.stroke();
                    tnCanvasContext.moveTo(canvas.shapes[1].x, canvas.shapes[1].y);
                    tnCanvasContext.arc(canvas.shapes[1].x, canvas.shapes[1].y, 4, 0, 2 * Math.PI, false);
                    tnCanvasContext.fillStyle = '#D27E00';
                    tnCanvasContext.fill();
                    tnCanvasContext.stroke();

                    ctx.clearRect(0, 0, canvas.width, canvas.height);
                    ctx.drawImage(tnCanvas, 0, 0);
                }

                vm.closeModal = function () {
                    vm.modalInstance.close();
                }

                vm.validate = function () {
                    vm.busy = true;
                    var canvas1 = $window.document.getElementById("image1Canvas");
                    if (canvas1 === null || canvas1 === undefined)
                        return;

                    // recover points coordinates from the shapes array
                    vm.editedPlate.PlateCorners.CornersInImageA.Left.x = canvas1.shapes[0].x + vm.editedPlate.Image1OffsetX;
                    vm.editedPlate.PlateCorners.CornersInImageA.Left.y = canvas1.shapes[0].y + vm.editedPlate.Image1OffsetY;
                    vm.editedPlate.PlateCorners.CornersInImageA.Right.x = canvas1.shapes[1].x + vm.editedPlate.Image1OffsetX;
                    vm.editedPlate.PlateCorners.CornersInImageA.Right.y = canvas1.shapes[1].y + vm.editedPlate.Image1OffsetY;

                    var canvas2 = $window.document.getElementById("image2Canvas");
                    if (canvas2 === null || canvas2 === undefined)
                        return;

                    vm.editedPlate.PlateCorners.CornersInImageB.Left.x = canvas2.shapes[0].x + vm.editedPlate.Image2OffsetX;
                    vm.editedPlate.PlateCorners.CornersInImageB.Left.y = canvas2.shapes[0].y + vm.editedPlate.Image2OffsetY;
                    vm.editedPlate.PlateCorners.CornersInImageB.Right.x = canvas2.shapes[1].x + vm.editedPlate.Image2OffsetX;
                    vm.editedPlate.PlateCorners.CornersInImageB.Right.y = canvas2.shapes[1].y + vm.editedPlate.Image2OffsetY;

                    portalService.postSpeedEstimationValidatePlate(vm.editedPlate)
                        .then(function successCallback(response) {
                            vm.modalInstance.close();
                            vm.editedPlate.Validated = true;
                            vm.plate.Validated = true;
                            vm.done()
                                .then(function successCallback(response) {
                                    vm.busy = false;
                                    vm.form.$setPristine();
                                }, function errorCallback(response) {
                                    vm.handleError();
                                });
                        }, function errorCallback(response) {
                            vm.handleError();
                        });
                }

                vm.previous = function () {
                    if (vm.previousPlate !== undefined) {
                        var keys = Object.keys(vm.receivedPlates);
                        var i = keys.indexOf(vm.previousPlate.PlateId.toString());
                        vm.index = parseInt(keys[i]);
                        vm.loading = true;
                        vm.getPlateCalibrationInfoAndSetup(i);
                    }
                }

                vm.next = function () {
                    if (vm.nextPlate !== undefined) {
                        var keys = Object.keys(vm.receivedPlates);
                        var i = keys.indexOf(vm.nextPlate.PlateId.toString());
                        vm.index = parseInt(keys[i]);
                        vm.loading = true;
                        vm.getPlateCalibrationInfoAndSetup(i);
                    }
                }

                vm.getPlateCalibrationInfoAndSetup = function (index) {
                    portalService.getSpeedEstimationAwaitedCalibrationPlateInfo(vm.receivedPlates[vm.index].PlateToWaitForId, vm.receivedPlates[vm.index].PlateId)
                            .then(function successCallback(response) {
                                var keys = Object.keys(vm.receivedPlates);
                                vm.editedPlate = response.data;
                                vm.nextPlate = vm.receivedPlates[keys[index + 1]];
                                vm.previousPlate = vm.receivedPlates[keys[index - 1]];
                                if (vm.editedPlate.Speed === null)
                                    vm.editedPlate.Speed = 0;
                                vm.editedPlate.Speed = parseInt(vm.editedPlate.Speed);
                                vm.setupCanvasImageAndShapes();
                                vm.loading = false;
                            }, function errorCallback(response) {
                                vm.handleError();
                            });
                }

                // handle mousedown events
                function myDown(e) {

                    // tell the browser we're handling this mouse event
                    e.preventDefault();
                    e.stopPropagation();

                    var canvas = e.currentTarget;
                    var bb = canvas.getBoundingClientRect();

                    // get the current mouse position
                    var mx = parseInt(e.clientX - bb.left);
                    var my = parseInt(e.clientY - bb.top);

                    // test each shape to see if mouse is inside
                    canvas.dragok = false;
                    for (var i = 0; i < canvas.shapes.length; i++) {
                        var s = canvas.shapes[i];
                        var dx = s.x - mx;
                        var dy = s.y - my;
                        // test if the mouse is inside this circle
                        if (dx * dx + dy * dy < s.r * s.r) {
                            canvas.dragok = true;
                            s.isDragging = true;
                            $rootScope.$apply(function () {
                                vm.form.$setDirty();
                            });
                        }
                    }
                    // save the current mouse position
                    canvas.startX = mx;
                    canvas.startY = my;
                }

                // handle mouseup events
                function myUp(e) {
                    // tell the browser we're handling this mouse event
                    e.preventDefault();
                    e.stopPropagation();

                    var canvas = e.currentTarget;

                    // clear all the dragging flags
                    canvas.dragok = false;
                    for (var i = 0; i < canvas.shapes.length; i++) {
                        canvas.shapes[i].isDragging = false;
                    }
                }

                // handle mouse moves
                function myMove(e) {
                    var canvas = e.currentTarget;
                    var bb = canvas.getBoundingClientRect();

                    // if we're dragging anything...
                    if (canvas.dragok) {

                        // tell the browser we're handling this mouse event
                        e.preventDefault();
                        e.stopPropagation();

                        // get the current mouse position
                        var mx = parseInt(e.clientX - bb.left);
                        var my = parseInt(e.clientY - bb.top);

                        // calculate the distance the mouse has moved
                        // since the last mousemove
                        var dx = mx - canvas.startX;
                        var dy = my - canvas.startY;

                        // move each rect that isDragging 
                        // by the distance the mouse has moved
                        // since the last mousemove
                        for (var i = 0; i < canvas.shapes.length; i++) {
                            var s = canvas.shapes[i];
                            if (s.isDragging) {
                                s.x += dx;
                                s.y += dy;
                            }
                        }

                        // redraw the scene with the new rect positions
                        vm.drawImage(canvas);

                        // reset the starting mouse position for the next mousemove
                        canvas.startX = mx;
                        canvas.startY = my;

                    }
                }

                vm.setupCanvasImageAndShapes = function () {
                    var img1 = $window.document.getElementById("image1");
                    var canvas1 = $window.document.getElementById("image1Canvas");

                    if (img1 === null || img1 === undefined || canvas1 === null || canvas1 === undefined)
                        return;

                    canvas1.shapes = [];
                    canvas1.dragok = false;
                    canvas1.startX = undefined;
                    canvas1.startY = undefined;
                    canvas1.imageId = img1.id;
                    canvas1.shapes.push({
                        x: vm.editedPlate.PlateCorners.CornersInImageA.Left.x - vm.editedPlate.Image1OffsetX,
                        y: vm.editedPlate.PlateCorners.CornersInImageA.Left.y - vm.editedPlate.Image1OffsetY,
                        r: 4,
                        fill: "#D27E00",
                        isDragging: false
                    });
                    canvas1.shapes.push({
                        x: vm.editedPlate.PlateCorners.CornersInImageA.Right.x - vm.editedPlate.Image1OffsetX,
                        y: vm.editedPlate.PlateCorners.CornersInImageA.Right.y - vm.editedPlate.Image1OffsetY,
                        r: 4,
                        fill: "#D27E00",
                        isDragging: false
                    });
                    img1.canvas = canvas1;
                    img1.onload = vm.imageCanvasImageLoaded;
                    img1.src = "data:image/png;base64," + vm.editedPlate.Image1;

                    var img2 = $window.document.getElementById("image2");
                    var canvas2 = $window.document.getElementById("image2Canvas");

                    if (img2 === null || img2 === undefined || canvas2 === null || canvas2 === undefined)
                        return;

                    canvas2.shapes = [];
                    canvas2.dragok = false;
                    canvas2.startX = undefined;
                    canvas2.startY = undefined;
                    canvas2.imageId = img2.id;
                    canvas2.shapes.push({
                        x: vm.editedPlate.PlateCorners.CornersInImageB.Left.x - vm.editedPlate.Image2OffsetX,
                        y: vm.editedPlate.PlateCorners.CornersInImageB.Left.y - vm.editedPlate.Image2OffsetY,
                        r: 4,
                        fill: "#D27E00",
                        isDragging: false
                    });
                    canvas2.shapes.push({
                        x: vm.editedPlate.PlateCorners.CornersInImageB.Right.x - vm.editedPlate.Image2OffsetX,
                        y: vm.editedPlate.PlateCorners.CornersInImageB.Right.y - vm.editedPlate.Image2OffsetY,
                        r: 4, fill: "#D27E00",
                        isDragging: false
                    });
                    img2.canvas = canvas2;
                    img2.onload = vm.imageCanvasImageLoaded;
                    img2.src = "data:image/png;base64," + vm.editedPlate.Image2;

                    vm.form.$setPristine();
                    vm.form.$setUntouched();
                }

                vm.openValidationModal = function (index) {
                    vm.receivedPlates = angular.copy(vm.plate.ReceivedPlates);
                    vm.loading = true;

                    vm.buttonLabel = 'STE_BUTTON_Validate';
                    vm.minSpeed = 1;
                    vm.notWaitedFor = false;

                    // open dialog to validate data
                    vm.modalInstance = $uibModal.open({
                        templateUrl: "speedEstimationValidation.html",
                        scope: $scope,
                        size: "lg",
                        backdrop: 'static'
                    });

                    portalService.getSpeedEstimationAwaitedCalibrationPlateInfo(vm.receivedPlates[index].PlateToWaitForId, vm.receivedPlates[index].PlateId)
                        .then(function successCallback(response) {

                            var keys = Object.keys(vm.receivedPlates);
                            var i = keys.indexOf(index.toString());
                            vm.nextPlate = vm.receivedPlates[keys[i + 1]];
                            vm.previousPlate = vm.receivedPlates[keys[i - 1]];
                            vm.editedPlate = response.data;
                            if (vm.editedPlate.Speed === null)
                                vm.editedPlate.Speed = 0;
                            vm.backup = angular.copy(vm.editedPlate);
                            vm.editedPlate.Speed = parseInt(vm.editedPlate.Speed);
                            vm.loading = false;
                            vm.index = index;

                            vm.modalInstance.opened.then(function () {
                                setTimeout(function successCallback(response) {
                                    vm.setupCanvasImageAndShapes();
                                }, 0);
                            });
                        }, function errorCallback(response) {
                            vm.handleError();
                        });
                }
            }
        ];

        return {
            restrict: 'E',
            scope: {
                plate: "=",
                del: "&",
                done: "&",
                timeformat: "<"
            },
            controller: controller,
            controllerAs: 'vm',
            bindToController: true,
            template: '<div class="panel panel-default panel-plate" id="PlateToWaitFor{{ vm.plate.PlateToWaitForId }}">' +
                            '<div class="panel-heading">' +
                                '<i class="fa fa-spinner fa-spin ng-animate-disabled" ng-show="vm.plate.IsWaiting" aria-hidden="true"></i>&nbsp;{{ vm.plate.PlateNumber }}' +
                                '<button type="button" class="close" ng-disabled="vm.busy" ng-really-message="{{\'STE_SENTENCE_DeletePlateConfirmation\'}}" ng-really-click="vm.delete()">' +
                                    '<i class="fa fa-close text-default"></i>' +
                                '</button>' +
                            '</div>' +
                            '<div class="panel-body">' +
                                '<div class="">' +
                                    '<div ng-if="vm.notReceivedPlate"><p class="text-center" translate>STE_SENTENCE_SpeedEstimationAwaitingPlate</P></div>' +
                                    '<div class="thumbnails with-description" ng-repeat="plateInfo in vm.plate.ReceivedPlates" ng-click="plateInfo.busy || vm.openValidationModal(plateInfo.PlateId)">' +
                                        '<div class="thumbnails-img">' +
                                            '<img ng-src="data:image/png;base64,{{ plateInfo.Thumbnail }}">' +
                                            '<div class="thumbnails-hover show-opacity change-icon">' +
                                                '<i ng-if="plateInfo.Validated && !vm.busy" class="faswitch fa fa-check-circle fa-4x text-success" style="position: absolute;left: 40%;top: 30%"></i>' +
                                                '<i ng-if="plateInfo.Validated && !vm.busy" class="faswitch fa fa-pencil-square-o fa-4x text-primary" style="position: absolute;left: 40%;top: 30%"></i>' +
                                                '<i ng-show="plateInfo.Validated && vm.busy" class="fa fa-refresh fa-spin fa-3x fa-fw text-warning ng-animate-disabled" style="position: absolute;left: 40%;top: 30%"></i>' +
                                                '<button type="button" class="close" ng-disabled="plateInfo.busy" ng-really-message="{{\'STE_SENTENCE_DeleteImageConfirmation\'}}" ng-really-click="vm.deletePlate(plateInfo)">' +
                                                    '<i class="fa fa-close text-default"></i>' +
                                                '</button>' +
                                            '</div>' +
                                        '</div>' +
                                        '<div class="thumbnails-text">' +
                                            '<div ng-hide="vm.plate.Validated"><a translate>STE_LABEL_Validate</a></div>' +
                                            '<div ng-show="vm.plate.Validated">' +
                                                '<span>{{ vm.plate.PlateNumber }}&nbsp;</span>' +
                                                '<span class="pull-right">{{ plateInfo.Speed }} {{ plateInfo.SpeedUnitLabel }}</span>' +
                                            '</div>' +
                                            '<div style="padding-top: 5px;">' +
                                                '<span ng-if="vm.timeformat != 1">{{ plateInfo.TimeStamp | date: \'yyyy-MM-dd HH:mm:ss\' }}</span>' +
                                                '<span ng-if="vm.timeformat == 1">{{ plateInfo.TimeStamp | date: \'MM/dd/yyyy h:mm:ss a\' }}</span>' +
                                            '</div>' +
                                        '</div>' +
                                    '</div>' +
                                '</div>' +
                            '</div>' +
                        '</div>'
        }
    }
})();