﻿(function () {
    "use strict";

    angular.module("portalApp.directives")
        .directive("gnWithFloatingLabel", gnWithFloatingLabel);

    gnWithFloatingLabel.$inject = ["$compile", "$translate"];
    
    //
    // Used as an attribute on an input element, it will add a 'div' class for 
    // a label taking its text from the 'placeHolder' attribute and an error indicator. 
    // The label text is passed to the 'translate' filter (for localization).
    // It then add or remove the 'empty' attribute depending if the input is empty or filled.
    // Validation field name is composed of the gnWithFloatingLabel directive attribute value plus the input id.
    //
    function gnWithFloatingLabel($compile, $translate) {
        return {
            restrict: "A",
            link: function ($scope, $element, attrs) {

                //!!!TODO: Find a reliable way to do that:
                //if ($element.tagName != "INPUT")
                //    return; // apply only to <input> elements

                var id = attrs.id;
                var form = attrs.gnWithFloatingLabel; // form to which to attach the 
                var flagDirtyWhenPristine = attrs.flagDirtyWhenPristine ? '' : '&& ' + form + '.' + id + '.$dirty'; // when flag is not set, error will not be flaged when user have not changed the value
                var ngclass = form ? ' ng-class="{ \'has-error\' : ' + form + '.' + id + '.$invalid ' + flagDirtyWhenPristine + ' }"' : '';

                var template = '<div class="floating-label"' + ngclass + ' translate=' + attrs.placeholder + '></div>';

                //append floating label (and validation) template
                $element.after($compile(template)($scope));

                //remove placeholder  
                $element.removeAttr("placeholder");

                // add/remove 'empty' attribute when value changes
                $scope.$watch(function () {
                    if ($element.val().toString().length < 1) {
                        $element.addClass("empty");
                    } else {
                        $element.removeClass("empty");
                    }
                });             
            }
        };
    }
})();