﻿(function () {
    "use strict";

    angular.module("portalApp.directives")
        .directive("gnFloatingLabelWithIcon", gnFloatingLabelWithIcon);

    gnFloatingLabelWithIcon.$inject = ["$compile", "$translate"];
    
    //
    // Used as an attribute on an input element, it will add a 'div' class for 
    // a label taking its text from the 'placeHolder' attribute and an error indicator. 
    // The label text is passed to the 'translate' filter (for localization).
    // It then add or remove the 'empty' attribute depending if the input is empty or filled.
    // Validation field name is composed of the gnFloatingLabelWithIcon directive attribute value plus the input id.
    // The difference with gnWithFloatingLabel lies in the fact that if an "icon" attribute is specified, it will preceed the lavel value
    //
    function gnFloatingLabelWithIcon($compile, $translate) {
        return {
            restrict: "A",
            link: function ($scope, $element, attrs) {

                //!!!TODO: Find a reliable way to do that:
                //if ($element.tagName != "INPUT")
                //    return; // apply only to <input> elements

                var icon = "";
                var id = attrs.id;
                var form = attrs.gnWithFloatingLabel;
                var ngclass = "";
                if (form)
                    ngclass = 'ng-class="{ \'has-error\' : ' + form + '.' + id + '.$invalid && ' + form + '.' + id + '.$dirty }"';

                if (angular.isDefined(attrs.icon))
                    icon = attrs.icon;

                $translate(attrs.placeholder).then(function (value) {
                    var template = '<div class="floating-label"' + ngclass + '>' + icon + value + "</div>";

                    //append floating label template
                    $element.after($compile(template)($scope));

                    //remove placeholder  
                    $element.removeAttr("placeholder");

                    // add/remove 'empty' attribute when value changes
                    $scope.$watch(function () {
                        if ($element.val().toString().length < 1) {
                            $element.addClass("empty");
                        } else {
                            $element.removeClass("empty");
                        }
                    });
                });                
            }
        };
    }
})();