﻿/**
 * @Author: Geoffrey Bauduin <bauduin.geo@gmail.com>
    https://github.com/lght/ion-range-slider-angularjs
    (I could not import that package using Bower, so I just copied it here)

    The MIT License (MIT)

    Copyright (c) 2016 Geoffrey Bauduin

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in all
    copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
    SOFTWARE.
 */

(function () {
    "use strict";

    angular.module("ion.rangeslider", []);

    angular.module("ion.rangeslider")
        .directive("ionRangeSlider", ionRangeSliderDefinition);

    ionRangeSliderDefinition.$inject = [];

    function ionRangeSliderDefinition() {
        return {
            restrict: "E",
            scope: {
                type: "@",
                min: "@",
                max: "@",
                from: "=",
                to: "=",
                step: "@",
                minInterval: "@",
                maxInterval: "@",
                dragInterval: "@",
                fromFixed: "=",
                fromMin: "@",
                fromMax: "@",
                fromShadow: "=",
                toFixed: "@",
                toMin: "@",
                toMax: "@",
                toShadow: "=",
                prettifyEnabled: "=",
                prettifySeparator: "=",
                prettify: "@",
                forceEdges: "=",
                keyboard: "=",
                keyboardStep: "=",
                grid: "@",
                gridMargin: "@",
                gridNum: "=",
                gridSnap: "=",
                hideMinMax: "=",
                hideFromTo: "=",
                prefix: "=",
                postfix: "=",
                maxPostfix: "=",
                decorateBoth: "=",
                valuesSeparator: "=",
                inputValuesSeparator: "=",
                disable: "=",
                onChange: "&",
                onFinish: "&",
                onStart: "&",
                onUpdate: "&"
            },
            replace: true,
            link: function (scope, element, attributes) {
                element.ionRangeSlider({
                    // GBro, 2016-04-20
                    // Updated 2.x attribute names using: http://ionden.com/a/plugins/ion.rangeSlider/en.html
                    // Added support for model binding for 'from' and 'to' values (single-type and double-type slider)
                    type: scope.type,
                    min: scope.min,
                    max: scope.max,
                    from: scope.from,
                    to: scope.to,
                    step: scope.step,

                    min_interval: scope.minInterval,
                    max_interval: scope.maxInterval,
                    drag_interval: scope.dragInterval,

                    from_fixed: scope.fromFixed,
                    from_min: scope.fromMin,
                    from_max: scope.fromMax,
                    from_shadow: scope.fromShadow,
                    to_fixed: scope.toFixed,
                    to_min: scope.toMin,
                    to_max: scope.toMax,
                    to_shadow: scope.toShadow,

                    prettify_enabled: scope.prettifyEnabled,
                    prettify_separator: scope.prettifySeparator,
                    prettify: scope.prettify,

                    force_edges: scope.forceEdges,
                    keyboard: scope.keyboard,
                    keyboard_step: scope.keyboardStep,

                    grid: scope.grid,
                    grid_margin: scope.gridMargin,
                    grid_num: scope.gridNum,
                    grid_snap: scope.gridSnap,

                    hide_min_max: scope.hideMinMax,
                    hide_from_to: scope.hideFromTo,

                    prefix: scope.prefix,
                    postfix: scope.postfix,
                    max_postfix: scope.maxPostfix,
                    decorate_both: scope.decorateBoth,
                    values_separator: scope.valuesSeparator,
                    input_values_separator: scope.inputValuesSeparator,

                    disable: scope.disable,

                    onChange: function (arg) {                                                
                        scope.$apply(function () {
                            if (typeof scope.to !== 'undefined')
                                scope.to = arg.to;
                            scope.from = arg.from;
                        });

                        if (scope.onChange)
                            scope.onChange();
                    },
                    onFinish: function() {
                        if (scope.onFinish)
                            scope.onFinish();
                    },
                    onStart: scope.onStart,
                    onUpdate: scope.onUpdate
                });

                // Changes to the model values are updated in the DOM object by $watch
                //var watchers = [];

               scope.$watch("min", function (value) {
                    element.data("ionRangeSlider").update({
                        min: value
                    });
               });
                scope.$watch('max', function (value) {
                    element.data("ionRangeSlider").update({
                        max: value
                    });
                });
                scope.$watch('to', function (value) {
                    // As this value may be altered by the user action as well as programmatically, we must find a way to 
                    // allow the latter, while skipping the former (to prevent loop)
                    if (value !== element.data("to"))
                        element.data("ionRangeSlider").update({
                            to: value
                        });
                });
                scope.$watch('from', function (value) {
                    // As this value may be altered by the user action as well as programmatically, we must find a way to 
                    // allow the latter, while skipping the former (to prevent loop)
                    if (value !== element.data("from"))
                        element.data("ionRangeSlider").update({
                            from: value
                        });
                });
               scope.$watch('disable', function (value) {
                    element.data("ionRangeSlider").update({
                        disable: value
                    });
                });
            }
        }
    }
})();