﻿(function () {
    "use strict";

    angular.module('portalApp.directives')
        .directive('ngReallyClick', ngReallyClick);

    ngReallyClick.$inject = ["$uibModal"];

    function ngReallyClick($uibModal) {
        
        var modalInstanceCtrl = function ($scope, $uibModalInstance) {
            $scope.ok = function () {
                $uibModalInstance.close();
            };

            $scope.cancel = function () {
                $uibModalInstance.dismiss('cancel');
            };
        };

        return {
            restrict: 'A',
            require: '?ngModel', // optional, but make sure to check for null!
            link: function (scope, element, attrs, ngModel) {
                var _element = element;

                element.bind('click', function (e) {
                    e.stopPropagation(); // avoids that click event triggers for control under the directive

                    var clickAction = attrs.ngReallyClick;

                    if (scope.$eval(attrs.ngReallySkip)) {
                        scope.$eval(clickAction);
                        return;
                    }

                    // Does it look like a switch ?
                    var isaSwitch = ngModel !== null && _element.parent().hasClass('switch') && _element[0] !== undefined && _element[0].checked !== undefined;

                    if (isaSwitch) {
                        scope.$apply(function () {
                            // Leave the previous value until the user confirms the change
                            var previousValue = !_element[0].checked;
                            _element[0].checked = previousValue;
                            ngModel.$setViewValue(previousValue);
                            ngModel.$$parentForm.$setPristine(); // Remove the dirty flag, because no change has been confirmed yet
                        });
                    }

                    var message = attrs.ngReallyMessage || "Are you sure ?"; // hard coded default message in case it's missing, better than nothing
                    var cancelButtonLabel = attrs.ngReallyCancelLabel || "STE_BUTTON_Cancel"; // default cancel button label
                    var okButtonLabel = attrs.ngReallyOkLabel || "STE_BUTTON_OK"; // default ok button label
                    var windowClass = attrs.ngReallyWindowClass || "modal-danger"; // default to danger

                    var modalHtml = '<div class="modal-header">' +
                                        '<button type="button" class="close" ng-click="cancel()" aria-label="Close"><i class="fa fa-times"></i></button>' +
                                        '<div class="modal-title" translate>STE_TITLE_Confirmation</div>' +
                                    '</div>' +
                                    '<div class="modal-body" translate>' + message +
                                    '</div>' +
                                    '<div class="modal-footer">' +
                                        '<button class="btn btn-default" ng-click="cancel()" translate>' + cancelButtonLabel + '</button>' +
                                        '<button class="btn btn-danger" ng-click="ok()" translate>' + okButtonLabel + '</button>' +                                        
                                    '</div>';

                    var modalInstance =  $uibModal.open({
                        windowClass: windowClass,
                        template: modalHtml,
                        controller: modalInstanceCtrl,
                        size: "md",
                        backdrop: 'static'
                    });

                    modalInstance.result.then(function () {
                        if (isaSwitch) {
                            // Set the new value now that the user has confirmed the change
                            var newValue = !_element[0].checked;
                            _element[0].checked = newValue;
                            ngModel.$setViewValue(newValue);
                        }
                        scope.$eval(clickAction);
                    }, function() {
                    });
                });

            }
        }
    }
})();