﻿(function () {
    "use strict";

    angular.module('portalApp.directives')
        .directive('otherPlates', otherPlates);

    function otherPlates() {
        var controller = ['$scope', '$uibModal', '$window', 'portalService', '$rootScope', '$timeout', '$q',
            function ($scope, $uibModal, $window, portalService, $rootScope, $timeout, $q) {

                var vm = this;
                vm.isNumber = angular.isNumber;
                vm.loading = false;

                function init() {
                    vm.busy = false;
                    vm.plates = angular.copy(vm.plates);
                }

                init();

                vm.handleError = function () {
                    vm.busy = false;
                    vm.loading = false;
                    $rootScope.$emit("SpeedEstimationError", {});
                }

                vm.deletePlate = function (plateId, goToNext) {
                    var promises = [];
                    vm.busy = true;
                    portalService.deleteSpeedEstimationCalibrationPlate(plateId)
                        .then(function successCallback(response) {
                            // recover updated calibration data
                            promises.push(vm.update());
                            promises.push(vm.done());
                            $q.all(promises).then(function successCallback() {
                                vm.busy = false;
                                if (goToNext) {
                                    if (vm.nextPlate !== undefined)
                                        vm.next();
                                    else if (vm.previousPlate !== undefined)
                                        vm.previous();
                                    else
                                        vm.closeModal();
                                }
                            }, function errorCallback(response) {
                                vm.handleError();
                            });
                        }, function errorCallback(response) {
                            vm.handleError();
                        });
                }

                vm.removePlate = function () {
                    vm.deletePlate(vm.editedPlate.PlateId, true);
                }

                // handle mousedown events
                function myDown(e) {

                    // tell the browser we're handling this mouse event
                    e.preventDefault();
                    e.stopPropagation();

                    var canvas = e.currentTarget;
                    var bb = canvas.getBoundingClientRect();

                    // get the current mouse position
                    var mx = parseInt(e.clientX - bb.left);
                    var my = parseInt(e.clientY - bb.top);

                    // test each shape to see if mouse is inside
                    canvas.dragok = false;
                    for (var i = 0; i < canvas.shapes.length; i++) {
                        var s = canvas.shapes[i];
                        var dx = s.x - mx;
                        var dy = s.y - my;
                        // test if the mouse is inside this circle
                        if (dx * dx + dy * dy < s.r * s.r) {
                            canvas.dragok = true;
                            s.isDragging = true;
                            $rootScope.$apply(function () {
                                vm.form.$setDirty();
                            });
                        }
                    }
                    // save the current mouse position
                    canvas.startX = mx;
                    canvas.startY = my;
                }

                // handle mouseup events
                function myUp(e) {
                    // tell the browser we're handling this mouse event
                    e.preventDefault();
                    e.stopPropagation();

                    var canvas = e.currentTarget;

                    // clear all the dragging flags
                    canvas.dragok = false;
                    for (var i = 0; i < canvas.shapes.length; i++) {
                        canvas.shapes[i].isDragging = false;
                    }
                }

                // handle mouse moves
                function myMove(e) {
                    var canvas = e.currentTarget;
                    var bb = canvas.getBoundingClientRect();

                    // if we're dragging anything...
                    if (canvas.dragok) {

                        // tell the browser we're handling this mouse event
                        e.preventDefault();
                        e.stopPropagation();

                        // get the current mouse position
                        var mx = parseInt(e.clientX - bb.left);
                        var my = parseInt(e.clientY - bb.top);

                        // calculate the distance the mouse has moved
                        // since the last mousemove
                        var dx = mx - canvas.startX;
                        var dy = my - canvas.startY;

                        // move each rect that isDragging 
                        // by the distance the mouse has moved
                        // since the last mousemove
                        for (var i = 0; i < canvas.shapes.length; i++) {
                            var s = canvas.shapes[i];
                            if (s.isDragging) {
                                s.x += dx;
                                s.y += dy;
                            }
                        }

                        // redraw the scene with the new rect positions
                        vm.drawImage(canvas);

                        // reset the starting mouse position for the next mousemove
                        canvas.startX = mx;
                        canvas.startY = my;

                    }
                }

                vm.openValidationModal = function (index) {
                    vm.receivedPlates = vm.plates;
                    vm.loading = true;

                    // open dialog to validate data
                    vm.buttonLabel = 'STE_BUTTON_Save';
                    vm.notWaitedFor = true;
                    vm.minSpeed = 0;
                    vm.modalInstance = $uibModal.open({
                        templateUrl: "speedEstimationValidation.html",
                        scope: $scope,
                        size: "lg",
                        backdrop: 'static'
                    });

                    portalService.getSpeedEstimationOtherCalibrationPlateInfo(vm.receivedPlates[index].PlateId)
                        .then(function successCallback(response) {
                            vm.editedPlate = response.data;
                            var keys = Object.keys(vm.receivedPlates);
                            var i = keys.indexOf(index.toString());
                            vm.nextPlate = vm.receivedPlates[keys[i + 1]];
                            vm.previousPlate = vm.receivedPlates[keys[i - 1]];
                            vm.backup = angular.copy(vm.editedPlate);
                            vm.index = index;
                            vm.editedPlate.Speed = parseInt(vm.editedPlate.Speed);
                            vm.loading = false;

                            vm.modalInstance.opened.then(function () {
                                setTimeout(function successCallback(response) {
                                    vm.setupCanvasImageAndShapes();
                                }, 0);
                            });
                        }, function errorCallback(response) {
                            vm.handleError();
                        });

                }

                vm.imageCanvasImageLoaded = function () {
                    var canvas = this.canvas;

                    // Take 100% width and preserve aspect ratio for height
                    var parent = canvas.parentElement;
                    var aspectRatio = this.height / this.width;
                    canvas.width = parent.clientWidth;
                    canvas.height = parent.clientWidth * aspectRatio;
                    vm.drawImage(canvas);

                    // listen for mouse events
                    canvas.onmousedown = myDown;
                    canvas.onmouseup = myUp;
                    canvas.onmousemove = myMove;
                }

                vm.drawImage = function (canvas) {
                    if (canvas === null || canvas === undefined)
                        return;

                    var ctx = canvas.getContext("2d");
                    if (ctx === null || ctx === undefined)
                        return;

                    var image = $window.document.getElementById(canvas.imageId);
                    if (image === null || image === undefined)
                        return;

                    var tnCanvas = document.createElement('canvas');
                    var tnCanvasContext = tnCanvas.getContext('2d');

                    // Take 100% width and preserve aspect ratio for height
                    var parent = canvas.parentElement;
                    var aspectRatio = image.height / image.width;
                    canvas.width = parent.clientWidth;
                    canvas.height = parent.clientWidth * aspectRatio;

                    tnCanvas.width = image.width;
                    tnCanvas.height = image.height;

                    tnCanvasContext.drawImage(image, 0, 0);

                    // draw lines under plate number
                    tnCanvasContext.strokeStyle = "#D27E00";
                    tnCanvasContext.lineWidth = 2;
                    tnCanvasContext.beginPath();
                    tnCanvasContext.moveTo(canvas.shapes[0].x, canvas.shapes[0].y);
                    tnCanvasContext.lineTo(canvas.shapes[1].x, canvas.shapes[1].y);
                    tnCanvasContext.stroke();

                    // draw end of lines
                    tnCanvasContext.moveTo(canvas.shapes[0].x, canvas.shapes[0].y);
                    tnCanvasContext.arc(canvas.shapes[0].x, canvas.shapes[0].y, 4, 0, 2 * Math.PI, false);
                    tnCanvasContext.fillStyle = '#D27E00';
                    tnCanvasContext.fill();
                    tnCanvasContext.stroke();
                    tnCanvasContext.moveTo(canvas.shapes[1].x, canvas.shapes[1].y);
                    tnCanvasContext.arc(canvas.shapes[1].x, canvas.shapes[1].y, 4, 0, 2 * Math.PI, false);
                    tnCanvasContext.fillStyle = '#D27E00';
                    tnCanvasContext.fill();
                    tnCanvasContext.stroke();

                    ctx.clearRect(0, 0, canvas.width, canvas.height);
                    ctx.drawImage(tnCanvas, 0, 0);
                }

                vm.closeModal = function () {
                    vm.modalInstance.close();
                }

                vm.validate = function () {
                    vm.busy = true;
                    var canvas1 = $window.document.getElementById("image1Canvas");
                    if (canvas1 === null || canvas1 === undefined)
                        return;

                    // recover points coordinates from the shapes array
                    vm.editedPlate.PlateCorners.CornersInImageA.Left.x = canvas1.shapes[0].x + vm.editedPlate.Image1OffsetX;
                    vm.editedPlate.PlateCorners.CornersInImageA.Left.y = canvas1.shapes[0].y + vm.editedPlate.Image1OffsetY;
                    vm.editedPlate.PlateCorners.CornersInImageA.Right.x = canvas1.shapes[1].x + vm.editedPlate.Image1OffsetX;
                    vm.editedPlate.PlateCorners.CornersInImageA.Right.y = canvas1.shapes[1].y + vm.editedPlate.Image1OffsetY;

                    var canvas2 = $window.document.getElementById("image2Canvas");
                    if (canvas2 === null || canvas2 === undefined)
                        return;

                    vm.editedPlate.PlateCorners.CornersInImageB.Left.x = canvas2.shapes[0].x + vm.editedPlate.Image2OffsetX;
                    vm.editedPlate.PlateCorners.CornersInImageB.Left.y = canvas2.shapes[0].y + vm.editedPlate.Image2OffsetY;
                    vm.editedPlate.PlateCorners.CornersInImageB.Right.x = canvas2.shapes[1].x + vm.editedPlate.Image2OffsetX;
                    vm.editedPlate.PlateCorners.CornersInImageB.Right.y = canvas2.shapes[1].y + vm.editedPlate.Image2OffsetY;

                    portalService.postSpeedEstimationValidatePlate(vm.editedPlate)
                        .then(function successCallback(response) {
                            vm.done()
                                .then(function () {
                                    vm.busy = false;
                                    vm.form.$setPristine();
                                }, function errorCallback(response) {
                                    vm.handleError();
                                });
                        }, function errorCallback(response) {
                            vm.handleError();
                        });

                    vm.modalInstance.close();
                }

                vm.setupCanvasImageAndShapes = function () {
                    var img1 = $window.document.getElementById("image1");
                    var canvas1 = $window.document.getElementById("image1Canvas");

                    if (img1 === null || img1 === undefined || canvas1 === null || canvas1 === undefined)
                        return;

                    canvas1.shapes = [];
                    canvas1.dragok = false;
                    canvas1.startX = undefined;
                    canvas1.startY = undefined;
                    canvas1.imageId = img1.id;
                    canvas1.shapes.push({
                        x: vm.editedPlate.PlateCorners.CornersInImageA.Left.x - vm.editedPlate.Image1OffsetX,
                        y: vm.editedPlate.PlateCorners.CornersInImageA.Left.y - vm.editedPlate.Image1OffsetY,
                        r: 4,
                        fill: "#D27E00",
                        isDragging: false
                    });
                    canvas1.shapes.push({
                        x: vm.editedPlate.PlateCorners.CornersInImageA.Right.x - vm.editedPlate.Image1OffsetX,
                        y: vm.editedPlate.PlateCorners.CornersInImageA.Right.y - vm.editedPlate.Image1OffsetY,
                        r: 4,
                        fill: "#D27E00",
                        isDragging: false
                    });
                    img1.canvas = canvas1;
                    img1.onload = vm.imageCanvasImageLoaded;
                    img1.src = "data:image/png;base64," + vm.editedPlate.Image1;

                    var img2 = $window.document.getElementById("image2");
                    var canvas2 = $window.document.getElementById("image2Canvas");

                    if (img2 === null || img2 === undefined || canvas2 === null || canvas2 === undefined)
                        return;

                    canvas2.shapes = [];
                    canvas2.dragok = false;
                    canvas2.startX = undefined;
                    canvas2.startY = undefined;
                    canvas2.imageId = img2.id;
                    canvas2.shapes.push({
                        x: vm.editedPlate.PlateCorners.CornersInImageB.Left.x - vm.editedPlate.Image2OffsetX,
                        y: vm.editedPlate.PlateCorners.CornersInImageB.Left.y - vm.editedPlate.Image2OffsetY,
                        r: 4,
                        fill: "#D27E00",
                        isDragging: false
                    });
                    canvas2.shapes.push({
                        x: vm.editedPlate.PlateCorners.CornersInImageB.Right.x - vm.editedPlate.Image2OffsetX,
                        y: vm.editedPlate.PlateCorners.CornersInImageB.Right.y - vm.editedPlate.Image2OffsetY,
                        r: 4, fill: "#D27E00",
                        isDragging: false
                    });
                    img2.canvas = canvas2;
                    img2.onload = vm.imageCanvasImageLoaded;
                    img2.src = "data:image/png;base64," + vm.editedPlate.Image2;

                    vm.form.$setPristine();
                    vm.form.$setUntouched();
                }

                vm.previous = function () {
                    if (vm.previousPlate !== undefined) {
                        var keys = Object.keys(vm.receivedPlates);
                        var i = keys.indexOf(vm.previousPlate.PlateId.toString());
                        vm.index = parseInt(keys[i]);
                        vm.loading = true;
                        vm.getPlateCalibrationInfoAndSetup(i);
                    }
                }

                vm.next = function () {
                    if (vm.nextPlate !== undefined) {
                        var keys = Object.keys(vm.receivedPlates);
                        var i = keys.indexOf(vm.nextPlate.PlateId.toString());
                        vm.index = parseInt(keys[i]);
                        vm.loading = true;
                        vm.getPlateCalibrationInfoAndSetup(i);
                    }
                }

                vm.getPlateCalibrationInfoAndSetup = function (index) {
                    portalService.getSpeedEstimationOtherCalibrationPlateInfo(vm.receivedPlates[vm.index].PlateId)
                        .then(function successCallback(response) {
                            var keys = Object.keys(vm.receivedPlates);
                            vm.nextPlate = vm.receivedPlates[keys[index + 1]];
                            vm.previousPlate = vm.receivedPlates[keys[index - 1]];
                            vm.editedPlate = response.data;
                            vm.editedPlate.Speed = parseInt(vm.editedPlate.Speed);
                            vm.setupCanvasImageAndShapes();
                            vm.loading = false;
                        }, function errorCallback(response) {
                            vm.handleError();
                        });
                }
            }
        ];

        return {
            restrict: 'E',
            scope: {
                plates: "=",
                ratio: "=",
                count: "=",
                required: "=",
                done: "&",
                update: "&"
            },
            controller: controller,
            controllerAs: 'vm',
            bindToController: true,
            template: '<div class="panel panel-default panel-plate">' +
                            '<div class="panel-body">' +
                                '<table class="table">' +
                                    '<tbody>' +
                                        '<tr>' +
                                            '<td class="td-xs" ng-show="vm.ratio < 100">' +
                                                '<i class="fa fa-spinner fa-spin" aria-hidden="true"></i>' +
                                            '</td>' +
                                            '<td>' +
                                                '<div class="progress">' +
                                                    '<div class="progress-bar" ng-class="vm.ratio === 100 ? \'progress-bar-success\': \'progress-bar-info\'" role="progressbar" aria-valuemin="0" aria-valuemax="100" ng-style="{\'width\' : vm.ratio + \'%\'}"></div>' +
                                                '</div>' +
                                            '</td>' +
                                            '<td class="td-sm text-right">' +
                                                '<div class="span1"><span>{{ vm.count }}&nbsp;/&nbsp;{{ vm.required }}</span></div>' +
                                           '</td>' +
                                        '</tr>' +
                                    '</tbody>' +
                                '</table>' +
                                '<label class="filled" style="height: auto;" translate>STE_LABEL_SpeedEstimationTwentyPlates</label>' +
                                '<div class="row">' +
                                    '<div class="col-xs-12">' +
                                        '<div class="thumbnails" ng-repeat="plateInfo in vm.plates" ng-click="vm.openValidationModal(plateInfo.PlateId)">' +
                                            '<div class="thumbnails-img">' +
                                                '<img ng-src="data:image/png;base64,{{ plateInfo.Thumbnail }}">' +
                                                '<div class="thumbnails-hover show-opacity">' +
                                                    '<button type="button" ng-disabled="vm.busy" class="close" ng-really-message="{{\'STE_SENTENCE_DeleteImageConfirmation\'}}" ng-really-click="vm.deletePlate(plateInfo.PlateId)">' +
                                                        '<i class="fa fa-close text-default"></i>' +
                                                    '</button>' +
                                                '</div>' +
                                            '</div>' +
                                        '</div>' +
                                    '</div>' +
                                '</div>' +
                            '</div>' +
                        '</div>'
        }
    }
})();