﻿(function () {
    "use strict";

    var noImage = "images/noVideo.png";
    var lprCameraTag = "Lpr Camera";
    var contextCameraTag = "Context Camera";
    var imageSize = { width: 1280, height: 960 };
    var roiStrokeColor = "#CC0000";
    var roiFillColor = "rgba(132, 0, 0, 0.2)";
    var zoneStrokeColor = "rgba(220, 20, 60, 1.0)"; // Crimson
    var zoneFillColor = "rgba(220, 20, 60, 0.2)"; // 20% shade of Crimson (https://www.w3schools.com/colors/colors_picker.asp?colorhex=DC143C)
    var fovStrokeColor = "#FF8C00"; // color DarkOrange
    var fovFillColor = "rgba(255, 140, 0, 0.15)";
    var markerSize = { width: 80, height: 95 };
    var rulerFillColor = "rgba(80, 80, 80, 0.5)";
    var rulerStrokeColor = "#CC0000";
    var rulerTextColor = "white";
    var zoomBoxBackgroundColor = "rgba(80, 80, 80, 0.5)";
    var zoomStrokeColor = "#FFFFFF"; // white
    var zoomFocusStrokeColor = "#CC0000"; // red

    var videoFeedController = ["$scope", "$rootScope", "$window", "signalRService", "portalService", "$translate", "$q", "$timeout",
        function ($scope, $rootScope, $window, signalRService, portalService, $translate, $q, $timeout) {

            var vm = this;
            var defer = $q.defer();

            // Variables
            vm.camera = new CameraObject();
            vm.firstZoneClick = true;
            vm.isFovValid = true; // set to true initially to prevent error message from showing, but keep isZoneValid false so user can't click Next.
            vm.uid = generateUuid();
            vm.zoomFactor = 1; // full scale, 2 == zoomed-in 2:1, 4 == zoomed-in again
            vm.miniZoom = false;
            vm.zoomClicked = []; // when zoomed, where the user clicked (in canvas coordinates, not image)
            vm.clickDelta = { x: 0, y: 0 };
            vm.zoomRatio = 1; // ratio between the view size and the image size (horizontal)
            vm.origin = { x: 0, y: 0 }; // upperleft corner of zoom rectangle
            vm.rulerPosition = { x: 0.5, y: 0.5 }; // position of the outer rectangle in % of canvas (can be moved). -1 means: center of the image
            vm.draggingRuler = false;
            vm.rulerMoved = false;
            vm.canvasResized = false;

            // Methods
            vm.assessReadyToConfigureZone = assessReadyToConfigureZone;
            vm.checkVideoActivity = checkVideoActivity;
            vm.clearZone = clearZone;
            vm.drawFoV = drawFoV;
            vm.drawVideo = drawVideo;
            vm.feedDetected = feedDetected;
            vm.translate = translate;

            vm.setupRoiDrawingArea = setupRoiDrawingArea;
            vm.drawRoiPolygon = drawRoiPolygon;

            // Translation
            vm.registeredTranslate = $rootScope.$on("$translateChangeSuccess", function () {
                vm.translate();
            });

            window.addEventListener('resize', resetResizeFlag, false);

            if ($translate.isReady())
                vm.translate();

            registerStreamingCallbacks();

            // Clean up to perform when the view is closed
            $scope.$on("$destroy", function () {
                $timeout.cancel(vm.timeout);
                defer.reject();
                clearInterval(vm.timer);

                if (vm.registeredTranslate) {
                    vm.registeredTranslate();
                    vm.registeredTranslate = null;
                }

                if (vm.registeredImageStart) {
                    vm.registeredImageStart();
                    vm.registeredImageStart = null;
                }
                if (vm.registeredImageChunk) {
                    vm.registeredImageChunk();
                    vm.registeredImageChunk = null;
                }
                if (vm.registeredImageDone) {
                    vm.registeredImageDone();
                    vm.registeredImageDone = null;
                }
            });

            activate();

            function activate() {
                $timeout(function () { onDocumentReady(); }, 0);
            }

            function resetResizeFlag() {
                vm.canvasResized = false;
            }

            function resizeCanvas() {
                vm.canvasResized = false;
                if (vm.domImage === undefined || vm.domImage === null) // are we unit testing?
                    return;
                if (vm.domCanvas.getContext) {
                    // only raise the flag when both are the same size because of resize animations, the parent could still being resized
                    if (vm.domCanvas.width === vm.domCanvas.parentElement.clientWidth) {
                        vm.canvasResized = true;
                        return;
                    }
                    adjustImage();
                }
            }

            $scope.$watch('vm.videoActivity', function (newValue, oldValue) {
                if (newValue === true) {
                    vm.domCanvas.width = 0;
                    vm.canvasResized = false;
                }
            });

            function adjustImage() {
                // Take 100% width and preserve aspect ratio for height
                var aspectRatio = vm.domImage.height / vm.domImage.width;
                var parent = vm.domCanvas.parentElement;
                vm.domCanvas.width = parent.clientWidth;
                vm.domCanvas.height = parent.clientWidth * aspectRatio;
                var ctx = vm.domCanvas.getContext("2d");

                if (vm.editZoom !== true && ctx !== undefined) {
                    vm.camera.scaleX =
                        vm.domCanvas.width / vm.domImage.width; // now time to calculate canvas scaling globally
                    vm.camera.scaleY = vm.domCanvas.height / vm.domImage.height;
                    ctx.scale(vm.camera.scaleX, vm.camera.scaleY);
                }
            }

            //
            function assessReadyToConfigureZone() {
                // Check that the slope of points are close enough to let user proceed

                var config = vm.userConfig.VehicleDetectionConfiguration;

                var contextDistance = config.ContextImagePointB.X - config.ContextImagePointA.X;
                var slopeContext = contextDistance !== 0 ? (config.ContextImagePointB.Y - config.ContextImagePointA.Y) / contextDistance : 0.0;

                var lprDistance = config.LprImagePointB.X - config.LprImagePointA.X;
                var slopeLpr = lprDistance !== 0 ? (config.LprImagePointB.Y - config.LprImagePointA.Y) / lprDistance : 0.0;

                vm.isFovValid = Math.abs(slopeContext - slopeLpr) < 0.05 && (contextDistance > 0 && lprDistance > 0 || contextDistance < 0 && lprDistance < 0);
            }

            // Constructor for a CameraObject
            function CameraObject() {
                if (vm.cameraType === undefined || vm.cameraType === null || vm.cameraType === "") {
                    console.log("Unable to initialize the camera video feed. The directive code is missing mandatory parameters.");
                    return;
                }

                this.cameraType = vm.cameraType.toLowerCase();
                this.isLprCamera = function () {
                    return this.cameraType === "lpr";
                };
                this.cameraName = this.isLprCamera() ? lprCameraTag : contextCameraTag;
                this.src = noImage; // image that will be allocated when received and freed when displayed
                this.chunks = []; // temporary buffer space for receiving partial image chunks
                this.imageCount = 0;
                this.chunksCount = function () {
                    return this.chunks.length;
                };
                this.scaleX = 1.0;
                this.scaleY = 1.0;
                this.width = 1024;
                this.height = 768;
            }

            //
            function checkVideoActivity() {
                if (vm.skipCheckVideoActivity)
                    return; // do not check video activity

                if (!vm.videoActivityChecked) {
                    vm.drawVideo();
                    $rootScope.$apply(function () {
                        vm.videoActivity = false;
                    });
                }

                $rootScope.$apply(function () {
                    vm.videoActivityChecked = false;
                });
            }

            // Delete all points defining the zone
            function clearZone() {
                if (vm.userConfig.VehicleDetectionConfiguration.ValidationRegion) {
                    vm.userConfig.VehicleDetectionConfiguration.ValidationRegion.VertexList = [];
                    vm.form.$setDirty();
                    vm.firstZoneClick = true;
                }
            }

            // Draw LPR FoV on Context image
            function drawFoV(ctx) {

                // Determine if the FoV seems good (to allow user to continue to next step)
                vm.assessReadyToConfigureZone();

                // If slopes of point pairs is not equal, don't draw the FoV
                if (!vm.isFovValid)
                    return false;

                var config = vm.userConfig.VehicleDetectionConfiguration;
                var cDx = config.ContextImagePointB.X - config.ContextImagePointA.X;
                var cDy = config.ContextImagePointB.Y - config.ContextImagePointA.Y;
                var lDx = config.LprImagePointB.X - config.LprImagePointA.X;
                var lDy = config.LprImagePointB.Y - config.LprImagePointA.Y;

                var cDistance = Math.sqrt((cDx * cDx) + (cDy * cDy));
                var lDistance = Math.sqrt((lDx * lDx) + (lDy * lDy));

                if (lDistance === 0.0)
                    return false; // can't represent the rectangle if the two points are together

                var ratio = cDistance / lDistance;

                // Expects that point rectangle lpr image will be smaller than context image
                if (ratio > 1.0)
                    return false;

                var fov = {
                    X: Math.round(config.ContextImagePointA.X - config.LprImagePointA.X * ratio),
                    Y: Math.round(config.ContextImagePointA.Y - config.LprImagePointA.Y * ratio),
                    Width: Math.round(imageSize.width * ratio),
                    Height: Math.round(imageSize.height * ratio)
                };

                // Draw the Fov rectangle (origin == upper left corner)
                ctx.strokeStyle = fovStrokeColor;
                ctx.fillStyle = fovFillColor;
                ctx.lineWidth = 2;
                ctx.setLineDash([5, 3]); // 5px dashes and 3xp spaces

                ctx.beginPath();
                ctx.moveTo(fov.X, fov.Y);
                ctx.lineTo(fov.X + fov.Width, fov.Y);
                ctx.lineTo(fov.X + fov.Width, fov.Y + fov.Height);
                ctx.lineTo(fov.X, fov.Y + fov.Height);
                ctx.closePath();
                ctx.fill();
                ctx.stroke();

                // Update the user config with the new FoV
                vm.userConfig.VehicleDetectionConfiguration.LprFovInContextImage = fov;

                return true; // fov is valid
            }

            // Draw a given marker on a given canvas (LPR or context)
            function drawMarker(name, x, y, ctx) {
                var fillColor = (name === "A") ? "Red" : "Blue";
                var ratio = vm.domImage.width / vm.domCanvas.width;
                var markerDiameter = 54 * ratio;
                ctx.fillStyle = fillColor;

                // triangle under
                var height = 37 * ratio;
                ctx.beginPath();
                ctx.moveTo(x, y);
                ctx.lineTo(x - markerDiameter / 2, y - height);
                ctx.lineTo(x + markerDiameter / 2, y - height);
                ctx.closePath();
                ctx.fill();

                // circle over
                ctx.arc(x, y - height, markerDiameter / 2, 0, 2 * Math.PI);
                ctx.fill();

                // caption
                ctx.fillStyle = "White";
                ctx.font = Math.floor(markerDiameter / 2) + "px Verdana";
                ctx.fillText(name, x - markerDiameter / 5 + 1, y - height + markerDiameter / 5 - 1);
            }

            //
            function drawVideo() {
                if (vm.domImage === undefined || vm.domImage === null) // are we unit testing?
                    return;

                if (vm.domCanvas.getContext) {

                    // Set the image to display (and if it needs to be freed)
                    vm.domImage.toFree = (vm.camera.src !== noImage) ? vm.camera.src : null;
                    vm.domImage.src = vm.camera.src; // will trigger 'onload'

                    vm.domImage.onload = function () {

                        if (!vm.canvasResized)
                            resizeCanvas();

                        if (vm.cameraResolution !== undefined)
                            vm.cameraResolution = imageSize.width + "x" + imageSize.height;

                        vm.userConfig.VehicleDetectionConfiguration.ContextImageSize.Width = vm.domImage.width;
                        vm.userConfig.VehicleDetectionConfiguration.ContextImageSize.Height = vm.domImage.height;

                        // use getContext to use the canvas for drawing
                        var ctx = vm.domCanvas.getContext("2d");
                        ctx.clearRect(0, 0, vm.domCanvas.width, vm.domCanvas.height);

                        if (vm.editZoom === true) {
                            var zoomWidth = vm.domCanvas.width * vm.zoomFactor; // zoomFactor is 1, 2 or 4
                            var zoomHeight = vm.domCanvas.height * vm.zoomFactor;

                            vm.zoomRatio = vm.domCanvas.width / vm.domImage.width;

                            ctx.drawImage(vm.domImage, 0, 0, imageSize.width, imageSize.height, vm.origin.x, vm.origin.y, zoomWidth, zoomHeight);
                        } else {
                            

                            ctx.drawImage(vm.domImage, 0, 0);
                        }

                        if (this.toFree) {
                            window.URL.revokeObjectURL(this.toFree); // free allocated blob
                        }

                        // Draw the zone polygon
                        if (vm.showZone === true)
                            drawZone(ctx);

                        // Draw two points A and B
                        if (vm.showMarkers === true) {
                            var config = vm.userConfig.VehicleDetectionConfiguration;
                            if (vm.camera.isLprCamera()) {
                                drawMarker("A", config.LprImagePointA.X, config.LprImagePointA.Y, ctx);
                                drawMarker("B", config.LprImagePointB.X, config.LprImagePointB.Y, ctx);
                            }
                            else {
                                drawMarker("A", config.ContextImagePointA.X, config.ContextImagePointA.Y, ctx);
                                drawMarker("B", config.ContextImagePointB.X, config.ContextImagePointB.Y, ctx);
                            }
                        }

                        // Draw the field of view
                        if (vm.showFov === true) {
                            vm.drawFoV(ctx);
                        }

                        // Draw the region of interest
                        if (vm.showRoi === true) {
                            vm.drawRoiPolygon(ctx);
                        }

                        // Draw the candidate plate bounding box
                        if (vm.showBoundingBox === true && vm.camera.ThumbnailPoints && vm.camera.ThumbnailPoints.length === 4) {
                            ctx.lineWidth = 7;
                            ctx.strokeStyle = "yellow";
                            ctx.beginPath();
                            ctx.moveTo(vm.camera.ThumbnailPoints[0].X, vm.camera.ThumbnailPoints[0].Y);
                            ctx.lineTo(vm.camera.ThumbnailPoints[1].X, vm.camera.ThumbnailPoints[1].Y);
                            ctx.lineTo(vm.camera.ThumbnailPoints[2].X, vm.camera.ThumbnailPoints[2].Y);
                            ctx.lineTo(vm.camera.ThumbnailPoints[3].X, vm.camera.ThumbnailPoints[3].Y);
                            ctx.lineTo(vm.camera.ThumbnailPoints[0].X, vm.camera.ThumbnailPoints[0].Y);
                            ctx.stroke();
                        }

                        // Draw the crosshair
                        if (vm.showCrosshair === true) {
                            var crosshairSegmentSize = 60;
                            var center = { x: vm.domImage.width / 2, y: vm.domImage.height / 2 };
                            ctx.lineWidth = 5;
                            ctx.strokeStyle = "brown";

                            ctx.beginPath();
                            ctx.moveTo(center.x - crosshairSegmentSize, center.y);
                            ctx.lineTo(center.x, center.y);
                            ctx.moveTo(center.x, center.y - crosshairSegmentSize);
                            ctx.lineTo(center.x, center.y);
                            ctx.moveTo(center.x + crosshairSegmentSize, center.y);
                            ctx.lineTo(center.x, center.y);
                            ctx.moveTo(center.x, center.y + crosshairSegmentSize);
                            ctx.lineTo(center.x, center.y);
                            ctx.stroke();
                        }

                        // Draw the zoom and focus
                        if (vm.editZoom === true) {
                            drawImageOnImage(ctx);

                            if (vm.showRuler === true)
                                drawRuler(ctx);
                        }
                    }
                }
                else {
                    alert($translate.instant("STE_SENTENCE_SafariOrFirefoxRequired"));
                }

                
            }

            // Draw the context image and the polygon zone on top
            // As the canvas can be resized without being redrawn, use parentElement client width and height instead of canvas's.
            function drawZone(ctx) {
                var zone = vm.userConfig.VehicleDetectionConfiguration.ValidationRegion;
                if (zone && zone.VertexList.length > 0) {
                    ctx.strokeStyle = zoneStrokeColor;
                    ctx.lineWidth = 2;
                    ctx.beginPath();

                    var point = zone.VertexList[0];

                    if (zone.VertexList.length > 1) { // need at least three points to draw a polygon
                        ctx.moveTo(point.X, point.Y);

                        for (var i = 1; i < zone.VertexList.length; i++) {
                            point = zone.VertexList[i];
                            ctx.lineTo(point.X, point.Y);
                        }
                    }
                    else if (zone.VertexList.length === 1) { // list is a single point (occurs only while editing the list)
                        ctx.moveTo(point.X, point.Y);
                        ctx.lineTo(point.X + 2, point.Y);
                        ctx.lineTo(point.X + 2, point.Y + 2);
                        ctx.lineTo(point.X, point.Y + 2);
                    }

                    ctx.closePath();

                    ctx.stroke();
                    ctx.fillStyle = zoneFillColor;
                    ctx.fill();
                }
            }

            //
            function feedDetected() {
                vm.videoActivity = true;
                vm.videoActivityChecked = true;
            }

            // Generate a unique Guid
            function generateUuid() {
                var d = new Date().getTime();
                var uuid = "xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx".replace(/[xy]/g, function (c) {
                    var r = (d + Math.random() * 16) % 16 | 0;
                    d = Math.floor(d / 16);
                    return (c === "x" ? r : (r & 0x3 | 0x8)).toString(16);
                });
                return uuid;
            };

            // Perform once all operations that depend on the DOM elements to be completely initialized (including evaluating Angular ng-if)
            function onDocumentReady() {
                // Retrieve both DOM elements that we will work with. We append a unique uid so that we can put many directives in a single page.
                vm.domImage = window.document.getElementById("CameraImage-" + vm.uid);
                vm.domCanvas = window.document.getElementById("CameraCanvas-" + vm.uid);

                // Wait for the userConfig to get loaded before doing our thing
                var unwatch = $scope.$watch("vm.userConfig", function () {
                    if (vm.userConfig && vm.userConfig.VehicleDetectionConfiguration) {
                        setupMarkersDefaultPositions();

                        setupMarkerDrawingArea();

                        setupZoomAndFocusDrawingArea();

                        vm.assessReadyToConfigureZone();

                        unwatch(); // remove the watcher once we receive it
                    }
                }, true);

                setupZoneDrawingArea();

                setupRoiDrawingArea();

                // Start monitoring the image
                vm.timer = setInterval(checkVideoActivity, 2000);
            }

            // Message received when a new image is started to be transferred
            function onReceivedImageStart(e, message) {
                $rootScope.$apply(function () {
                    if (message === null || message === undefined)
                        return;

                    vm.videoErrorConnection = false;

                    var imageInfo = message;

                    if (imageInfo.IsLprCamera === vm.camera.isLprCamera()) {
                        vm.camera.chunks.length = 0; // same as array.clear that doesn't exist in javascript
                        vm.camera.imageCount = imageInfo.ImageCount;
                        vm.camera.name = imageInfo.CameraName;

                        feedDetected();
                    }
                });
            }

            // Message received when a new image chunk is received
            function onReceivedImageChunk(e, message) {
                $rootScope.$apply(function () {
                    if (message === null || message === undefined)
                        return;

                    vm.videoErrorConnection = false;

                    var imageInfo = message;

                    if (imageInfo.IsLprCamera === vm.camera.isLprCamera()) {
                        vm.camera.chunks[imageInfo.Index] = imageInfo.Chunk; // add the chunk at proper index in the array
                    }

                });
            }

            // Message received when animage has been fully transferred
            function onReceivedImageDone(e, message) {
                $rootScope.$apply(function () {
                    if (message === null || message === undefined)
                        return;

                    vm.videoErrorConnection = false;

                    var imageInfo = message;

                    if (imageInfo.IsLprCamera === vm.camera.isLprCamera()) {
                        if (vm.camera.chunksCount() < imageInfo.ImageChunks) {
                            //console.log('Missing some chunk(s) expecting ' + imageInfo.ImageChunks + ' got ' + vm.chunkCountContext);
                            return; // image is missing chunks
                        }

                        if (imageInfo.ImageCount !== vm.camera.imageCount)
                            return;

                        var imageString = vm.camera.chunks.join("");
                        var data = signalRService.base64ToArrayBuffer(imageString);
                        var blob = new Blob([data], { type: imageInfo.ImageFormat });

                        vm.camera.src = window.URL.createObjectURL(blob); // will be freed when drawn

                        vm.camera.ThumbnailPoints = imageInfo.ThumbnailPoints;

                        // Display context image
                        vm.drawVideo();
                    }

                });
            }

            // Subscribe to video feed events (images and chunks)
            function registerStreamingCallbacks() {
                vm.registeredImageStart = $rootScope.$on("NewImageFeedInfo", onReceivedImageStart);
                vm.registeredImageChunk = $rootScope.$on("NewImageChunk", onReceivedImageChunk);
                vm.registeredImageDone = $rootScope.$on("ImageFeedInfoDone", onReceivedImageDone);
            }

            function translate() {

                // Add translations here...

                vm.registeredTranslate(); // unregister
                vm.registeredTranslate = null;
            }

            // Initiate the markers positions with default values if not already defined
            function setupMarkersDefaultPositions() {
                var config = vm.userConfig.VehicleDetectionConfiguration;

                // Check if the configuration has ever been done. If not, poke points at some valid place
                if (config.ContextImagePointA.X === 0 && config.ContextImagePointA.Y === 0 && config.ContextImagePointB.X === 0 && config.ContextImagePointB.Y === 0) {
                    config.ContextImagePointA.X = 620; //!!!determine better default points that fit in view
                    config.ContextImagePointA.Y = 233;
                    config.ContextImagePointB.X = 1059;
                    config.ContextImagePointB.Y = 572;
                }

                // Determine the position of markers in LPR image
                if (config.LprImagePointA.X === 0 && config.LprImagePointA.Y === 0 && config.LprImagePointB.X === 0 && config.LprImagePointB.Y === 0) {
                    config.LprImagePointA.X = 477; //!!!determine better default points that fit in view
                    config.LprImagePointA.Y = 108;
                    config.LprImagePointB.X = 1131;
                    config.LprImagePointB.Y = 638;
                }
            }

            // Setup the drawing area for markers
            function setupMarkerDrawingArea() {
                if (vm.domCanvas === null || vm.domCanvas === undefined) // we are unit testing?
                    return;

                // Make sure we don't execute when canvas isn't supported
                if (!vm.domCanvas.getContext || vm.editMarkers !== true)
                    return;

                var dragOffset = { dx: 0, dy: 0 };
                var draggingPointA = false;
                var draggingPointB = false;

                vm.domCanvas.addEventListener('mousedown', mouseDown, false);
                vm.domCanvas.addEventListener('mouseup', mouseUp, false);
                vm.domCanvas.addEventListener('mousemove', mouseMove, false);

                function mouseDown(e) {
                    // Tell the browser we're handling this event
                    e.preventDefault();
                    e.stopPropagation();

                    var pointA, pointB, left, right, top, bottom;

                    if (vm.camera.isLprCamera()) {
                        pointA = vm.userConfig.VehicleDetectionConfiguration.LprImagePointA;
                        pointB = vm.userConfig.VehicleDetectionConfiguration.LprImagePointB;
                    }
                    else {
                        pointA = vm.userConfig.VehicleDetectionConfiguration.ContextImagePointA;
                        pointB = vm.userConfig.VehicleDetectionConfiguration.ContextImagePointB;
                    }

                    // Check if we clicked one of the points
                    // check B first as it is drawn on top of A, thus user will click on that one if both are superimposed
                    left = (pointB.X - markerSize.width / 2) * vm.camera.scaleX;
                    right = left + markerSize.width * vm.camera.scaleX;
                    top = (pointB.Y - markerSize.height) * vm.camera.scaleY;
                    bottom = pointB.Y * vm.camera.scaleY;
                    if (e.offsetX >= left && e.offsetX <= right && e.offsetY >= top && e.offsetY <= bottom && !e.shiftKey && !e.altKey && !e.ctrlKey) {
                        dragOffset.dx = e.offsetX - (pointB.X * vm.camera.scaleX);
                        dragOffset.dy = e.offsetY - (pointB.Y * vm.camera.scaleY);
                        draggingPointB = true;
                        vm.videoCanvasCursor = "move";
                        return;
                    }

                    left = (pointA.X - markerSize.width / 2) * vm.camera.scaleX;
                    right = left + (markerSize.width * vm.camera.scaleX);
                    top = (pointA.Y - markerSize.height) * vm.camera.scaleX;
                    bottom = pointA.Y * vm.camera.scaleY;
                    if (e.offsetX >= left && e.offsetX <= right && e.offsetY >= top && e.offsetY <= bottom && !e.shiftKey && !e.altKey && !e.ctrlKey) {
                        dragOffset.dx = e.offsetX - (pointA.X * vm.camera.scaleX);
                        dragOffset.dy = e.offsetY - (pointA.Y * vm.camera.scaleY);
                        draggingPointA = true;
                        vm.videoCanvasCursor = "move";
                        return; // can drag only one point at the time
                    }

                    vm.videoCanvasCursor = "default";
                }

                function mouseMove(e) {
                    var pointA, pointB;

                    if (vm.camera.isLprCamera()) {
                        pointA = vm.userConfig.VehicleDetectionConfiguration.LprImagePointA;
                        pointB = vm.userConfig.VehicleDetectionConfiguration.LprImagePointB;
                    }
                    else {
                        pointA = vm.userConfig.VehicleDetectionConfiguration.ContextImagePointA;
                        pointB = vm.userConfig.VehicleDetectionConfiguration.ContextImagePointB;
                    }

                    // Check mouse is over one of the points
                    var leftB = (pointB.X - markerSize.width / 2) * vm.camera.scaleX;
                    var rightB = leftB + markerSize.width * vm.camera.scaleX;
                    var topB = (pointB.Y - markerSize.height) * vm.camera.scaleY;
                    var bottomB = pointB.Y * vm.camera.scaleY;

                    var leftA = (pointA.X - markerSize.width / 2) * vm.camera.scaleX;
                    var rightA = leftA + (markerSize.width * vm.camera.scaleX);
                    var topA = (pointA.Y - markerSize.height) * vm.camera.scaleX;
                    var bottomA = pointA.Y * vm.camera.scaleY;

                    if (e.offsetX >= leftB && e.offsetX <= rightB && e.offsetY >= topB && e.offsetY <= bottomB && !e.shiftKey && !e.altKey && !e.ctrlKey) {
                        vm.videoCanvasCursor = "move";
                    }
                    else if (e.offsetX >= leftA && e.offsetX <= rightA && e.offsetY >= topA && e.offsetY <= bottomA && !e.shiftKey && !e.altKey && !e.ctrlKey) {
                        vm.videoCanvasCursor = "move";
                    }
                    else {
                        vm.videoCanvasCursor = "default";
                    }


                    if (draggingPointA) {
                        // Moved point A
                        pointA.X = Math.round((e.offsetX - dragOffset.dx) / vm.camera.scaleX);
                        pointA.Y = Math.round((e.offsetY - dragOffset.dy) / vm.camera.scaleY);

                        // Update the user config with the new marker positions
                        if (vm.camera.isLprCamera())
                            vm.userConfig.VehicleDetectionConfiguration.LprImagePointA = pointA;
                        else
                            vm.userConfig.VehicleDetectionConfiguration.ContextImagePointA = pointA;

                        vm.form.$setDirty(); // set dirty flag
                    }
                    else if (draggingPointB) {
                        // Moved point B
                        pointB.X = Math.round((e.offsetX - dragOffset.dx) / vm.camera.scaleX);
                        pointB.Y = Math.round((e.offsetY - dragOffset.dy) / vm.camera.scaleY);

                        // Update the user config with the new marker positions
                        if (vm.camera.isLprCamera())
                            vm.userConfig.VehicleDetectionConfiguration.LprImagePointB = pointB;
                        else
                            vm.userConfig.VehicleDetectionConfiguration.ContextImagePointB = pointB;

                        vm.form.$setDirty(); // set dirty flag
                    }
                }

                function mouseUp(e) {
                    // Tell the browser we're handling this event
                    e.preventDefault();
                    e.stopPropagation();

                    // No more dragging
                    draggingPointA = false;
                    draggingPointB = false;
                }
            }

            // Setup the drawing area for the zone
            function setupZoneDrawingArea() {
                if (vm.domCanvas === null || vm.domCanvas === undefined) // we are unit testing?
                    return;

                // Make sure we don't execute when canvas isn't supported
                if (!vm.domCanvas.getContext || vm.editZone !== true)
                    return;

                vm.domCanvas.addEventListener('mousedown', mouseDown, false);

                function mouseDown(e) {
                    // Tell the browser we're handling this event
                    e.preventDefault();
                    e.stopPropagation();

                    var zone = vm.userConfig.VehicleDetectionConfiguration.ValidationRegion;
                    if (zone.VertexList.length > 30)
                        return; // no more than 30 point in a zone

                    vm.form.$setDirty();

                    // Clear existing zone on first click
                    if (vm.firstZoneClick) {
                        vm.clearZone(false);
                        vm.firstZoneClick = false;
                    }

                    // toElement being the canvas, ONLY defined in Chrome
                    var xRatio = imageSize.width / (e.toElement ? e.toElement.clientWidth : e.currentTarget.clientWidth);
                    var yRatio = imageSize.height / (e.toElement ? e.toElement.clientHeight : e.currentTarget.clientHeight);

                    // Add newly clicked point, converting from screen coordinates to scaled image coordinates
                    // to match original context image size
                    var x = Number((e.offsetX * xRatio).toFixed(0));
                    var y = Number((e.offsetY * yRatio).toFixed(0));
                    zone.VertexList.push({ X: x, Y: y });
                }
            }

            // Setup the drawing area for the region of interest
            function setupRoiDrawingArea() {
                if (vm.domCanvas === null || vm.domCanvas === undefined) // we are unit testing?
                    return;

                // Make sure we don't execute when ROI isn't supported nor enabled
                if (!vm.domCanvas.getContext || vm.editRoi !== true)
                    return;

                vm.domCanvas.addEventListener('dblclick', mouseDblClick); // double-click should not interfere with ROI configuration
                vm.domCanvas.addEventListener('mousedown', mouseDown, false);

                function mouseDblClick(e) {
                    if (!vm.camera.isLprCamera() || vm.editRoi !== true) // only applies to Context camera when feature (edit-roi) is enabled
                        return;

                    // go full-screen
                    if (vm.domCanvas.requestFullscreen) {
                        vm.domCanvas.requestFullscreen();
                    } else if (vm.domCanvas.webkitRequestFullscreen) {
                        vm.domCanvas.webkitRequestFullscreen();
                    } else if (vm.domCanvas.mozRequestFullScreen) {
                        vm.domCanvas.mozRequestFullScreen();
                    } else if (vm.domCanvas.msRequestFullscreen) {
                        vm.domCanvas.msRequestFullscreen();
                    }
                }

                function mouseDown(e) {

                    if (!vm.camera.isLprCamera() || vm.editRoi !== true)// only applies to LPR camera when feature (edit-roi) is enabled
                        return;

                    if (vm.userConfig.regionOfInterest.Coordinates.length > 30)
                        return; // no more than 30 point in a ROI

                    vm.form.$setDirty();

                    //tell the browser we're handling this event
                    e.preventDefault();
                    e.stopPropagation();

                    if (vm.firstClick) {
                        clearRegion();
                        vm.firstClick = false;
                    }

                    // toElement being the canvas, ONLY defined in Chrome
                    var xRatio = imageSize.width / (e.toElement ? e.toElement.clientWidth : e.currentTarget.clientWidth);
                    var yRatio = imageSize.height / (e.toElement ? e.toElement.clientHeight : e.currentTarget.clientHeight);

                    var x = Number((e.offsetX * xRatio).toFixed(0));
                    var y = Number((e.offsetY * yRatio).toFixed(0));
                    vm.userConfig.regionOfInterest.Coordinates.push({ X: x, Y: y });

                    var ctx = vm.domCanvas.getContext("2d");
                    ctx.mozImageSmoothingEnabled = false;
                    ctx.msImageSmoothingEnabled = false;
                    ctx.imageSmoothingEnabled = false;

                    drawRoiPolygon(ctx);
                }
            }

            function setupZoomAndFocusDrawingArea() {
                if (vm.domCanvas === undefined || vm.domCanvas === null || vm.domCanvas.getContext == null) // we are unit testing
                    return;

                vm.videoCanvasCursor = "default";

                vm.domCanvas.addEventListener('mousedown', mouseDown, false);
                vm.domCanvas.addEventListener('mouseup', mouseUp, false);
                vm.domCanvas.addEventListener('mousemove', mouseMove, false);
                vm.domCanvas.addEventListener('mouseleave', mouseLeave, false);

                function mouseLeave(e) {
                    vm.zoomMouseDown = false;
                }

                function mouseDown(e) {
                    // Make sure we don't execute when zoom isn't enabled
                    if (vm.editZoom !== true)
                        return;

                    // Tell the browser we're handling this event
                    e.preventDefault();
                    e.stopPropagation();
                    vm.zoomMouseDown = true;

                    if (vm.showRuler === true) {
                        vm.rulerMoved = false;

                        // Check if we clicked in the ruler area (rulerPosition is the center of the box)
                        var width = 40;
                        var left = vm.domCanvas.width * vm.rulerPosition.x - width / 2;
                        var right = left + width;
                        var height = vm.rulerHeight ? vm.rulerHeight : 35;
                        var top = vm.domCanvas.height * vm.rulerPosition.y - height / 2 - 5;
                        var bottom = top + (height * vm.zoomFactor * vm.zoomRatio) + 5;
                        if (e.offsetX >= left && e.offsetX <= right
                            && e.offsetY >= top && e.offsetY <= bottom
                            && !e.shiftKey && !e.altKey && !e.ctrlKey) {
                            vm.draggingRuler = true;
                            vm.clickDelta.x = e.offsetX - vm.domCanvas.width * vm.rulerPosition.x;
                            vm.clickDelta.y = e.offsetY - vm.domCanvas.height * vm.rulerPosition.y;
                            vm.videoCanvasCursor = "pointer";
                            return;
                        }
                    }

                    // Check if we clicked in the preview zoom rectangle (to prevent zooming)
                    vm.miniZoom = false;
                    if (vm.zoomFactor > 1
                        && e.offsetX >= vm.domCanvas.width * 4 / 5
                        && e.offsetY <= vm.domCanvas.height * 1 / 5) {
                        vm.miniZoom = true;
                        vm.videoCanvasCursor = "default";
                        return;
                    }
                }

                function mouseMove(e) {
                    // Make sure we don't execute when zoom isn't enabled
                    if (vm.editZoom !== true)
                        return;

                    if (vm.showRuler === true && vm.draggingRuler) {
                        // Move the ruler
                        vm.rulerPosition.x = (e.offsetX - vm.clickDelta.x) / vm.domCanvas.width;
                        vm.rulerPosition.y = (e.offsetY - vm.clickDelta.y) / vm.domCanvas.height;
                        vm.rulerMoved = true;
                    }

                    var width = 40;
                    var left = vm.domCanvas.width * vm.rulerPosition.x - width / 2;
                    var right = left + width;
                    var height = vm.rulerHeight ? vm.rulerHeight : 35;
                    var top = vm.domCanvas.height * vm.rulerPosition.y - height / 2 - 5;
                    var bottom = top + (height * vm.zoomFactor * vm.zoomRatio) + 5;
                    if (vm.showRuler === true && e.offsetX >= left && e.offsetX <= right && e.offsetY >= top && e.offsetY <= bottom && !e.shiftKey && !e.altKey && !e.ctrlKey)
                        vm.videoCanvasCursor = "pointer";
                    else if (vm.zoomFactor > 1 && e.offsetX >= vm.domCanvas.width * 4 / 5 && e.offsetY <= vm.domCanvas.height * 1 / 5)
                        vm.videoCanvasCursor = "default";
                    else if (vm.zoomFactor === 4)
                        vm.videoCanvasCursor = "zoomout";
                    else
                        vm.videoCanvasCursor = "zoomin";
                }

                function mouseUp(e) {
                    // Make sure we don't execute when zoom isn't enabled
                    if (vm.editZoom !== true || vm.zoomMouseDown !== true)
                        return;

                    // Check for moving the ruler
                    if (vm.showRuler === true && vm.draggingRuler) {
                        vm.draggingRuler = false;
                        if (vm.rulerMoved) {
                            vm.rulerMoved = false;
                            return;
                        }
                    }

                    if (vm.miniZoom) {
                        vm.miniZoom = false;
                        return; // don't zoom the image if clicked in the zoom preview zone
                    }


                    // Toggle full-scale / scaled-in
                    switch (vm.zoomFactor) {
                        case 1:
                            goScaledIn(e); // provide the coordinates, so we can center the zoom on it
                            vm.videoCanvasCursor = "zoomin";
                            break;
                        case 2:
                            goScaledIn(e); // provide the coordinates, so we can center the zoom on it
                            vm.videoCanvasCursor = "zoomout";
                            break;
                        case 4:
                            goFullScale();
                            vm.videoCanvasCursor = "zoomin";
                            break;
                    }

                    // Raise event to redraw the focus graph
                    $rootScope.$emit('onZoomClicked');
                }
            }

            function goFullScale() {
                vm.zoomFactor = 1;
                vm.origin = { x: 0, y: 0 };
                vm.zoomClicked = [];
                sendRectangle();
            }

            function goScaledIn(point) {
                // Save each zoom click for later use
                vm.zoomClicked.push({ x: Math.max(0, point.layerX), y: Math.max(0, point.layerY) });

                // Calculate what region should be zoomed-in
                vm.origin.x = -vm.zoomClicked[0].x * vm.zoomFactor;
                vm.origin.y = -vm.zoomClicked[0].y * vm.zoomFactor;

                if (vm.zoomClicked.length === 2) {
                    vm.origin.x -= vm.zoomClicked[1].x;
                    vm.origin.y -= vm.zoomClicked[1].y;
                }

                vm.zoomFactor *= 2;

                sendRectangle();
            }

            function sendRectangle() {
                if (vm.domCanvas == null) // we are unit testing
                    return;

                var x = Math.max(0, -vm.origin.x * vm.domImage.width / (vm.domCanvas.width * vm.zoomFactor));
                var y = Math.max(0, -vm.origin.y * vm.domImage.height / (vm.domCanvas.height * vm.zoomFactor));
                var width1 = vm.domImage.width / vm.zoomFactor;
                var height1 = vm.domImage.height / vm.zoomFactor;

                // Should be integer
                x = Math.round(x);
                y = Math.round(y);
                width1 = Math.round(width1);
                height1 = Math.round(height1);

                // Crop to image dimension.
                x = Math.max(0, x);
                y = Math.max(0, y);
                var right = Math.min(vm.domImage.width, x + width1);
                var bottom = Math.min(vm.domImage.height, y + height1);
                var width = right - x;
                var height = bottom - y;

                portalService.setFocusRectangle(vm.camera.cameraName, x, y, width, height);
            }

            // Draw the ruler according to scale and ruler size
            function drawRuler(ctx) {

                var height = vm.rulerHeight ? vm.rulerHeight : 35; // default value in case value is undefined (invalid)
                var boxSize = { width: 40, height: height }; // size of the rectangle without margins of 5 pixels

                // Initial position will be center of the image
                var pos = {
                    x: vm.domCanvas.width * vm.rulerPosition.x - boxSize.width / 2,
                    y: vm.domCanvas.height * vm.rulerPosition.y - boxSize.height / 2 - 5
                }

                // Background box
                ctx.beginPath();
                ctx.fillStyle = rulerFillColor;
                ctx.fillRect(pos.x, pos.y, boxSize.width, boxSize.height * vm.zoomFactor * vm.zoomRatio + 5);
                ctx.stroke();

                // Bracket
                ctx.beginPath();
                ctx.lineWidth = 3;
                ctx.strokeStyle = rulerStrokeColor;
                ctx.moveTo(pos.x + 5 + 13, pos.y + 5);
                ctx.lineTo(pos.x + 5, pos.y + 5); // top horizontal bar
                ctx.lineTo(pos.x + 5, pos.y + (boxSize.height * vm.zoomFactor * vm.zoomRatio)); // left vertical bar
                ctx.lineTo(pos.x + 5 + 13, pos.y + (boxSize.height * vm.zoomFactor * vm.zoomRatio)); // bottom horizontal bar
                ctx.stroke();

                // Text
                ctx.fillStyle = rulerTextColor;
                ctx.font = "12px Arial";
                ctx.fillText(boxSize.height + "px", pos.x + 5 + ctx.lineWidth + 2, pos.y + 5 + 12 + 2);
            }

            function drawImageOnImage(ctx) {
                if (vm.zoomFactor === 1)
                    return; // don't display if at full scale

                // Background box (always 1/5 of view)
                var fifthWidth = vm.domCanvas.width / 5;
                var fifthHeight = vm.domCanvas.height / 5;

                ctx.beginPath();
                ctx.fillStyle = zoomBoxBackgroundColor;
                ctx.fillRect(fifthWidth * 4, 0, fifthWidth, fifthHeight);
                ctx.lineWidth = 3;
                ctx.strokeStyle = zoomStrokeColor;
                ctx.rect(fifthWidth * 4, 0, fifthWidth, fifthHeight);
                ctx.stroke();

                // Rectangle indicating the position on the image
                var zoomWidth = fifthWidth / vm.zoomFactor;
                var zoomHeight = fifthHeight / vm.zoomFactor;
                var newX = (fifthWidth * 4) + ((-vm.origin.x / 5) / vm.zoomFactor);
                var newY = 0 + ((-vm.origin.y / 5) / vm.zoomFactor);
                
                ctx.beginPath();
                ctx.lineWidth = 3;
                ctx.strokeStyle = zoomFocusStrokeColor;
                ctx.rect(newX, newY, zoomWidth, zoomHeight);
                ctx.stroke();
            }

            // Draw the Region of interest polygon if the feature is enabled in the directive
            function drawRoiPolygon(ctx) {
                var roi = vm.userConfig.regionOfInterest;
                if (!roi || roi.Coordinates.length === 0)
                    return;

                ctx.strokeStyle = roiStrokeColor;
                ctx.lineWidth = 2;

                ctx.beginPath();
                ctx.moveTo(roi.Coordinates[0].X, roi.Coordinates[0].Y);

                // Special case to display something when it is the first click
                if (roi.Coordinates.length === 1) {
                    ctx.lineTo(roi.Coordinates[0].X + 2, roi.Coordinates[0].Y);
                    ctx.lineTo(roi.Coordinates[0].X + 2, roi.Coordinates[0].Y + 2);
                    ctx.lineTo(roi.Coordinates[0].X, roi.Coordinates[0].Y + 2);
                    ctx.lineTo(roi.Coordinates[0].X, roi.Coordinates[0].Y);
                }

                for (var index = 1; index < roi.Coordinates.length; index++) {
                    ctx.lineTo(roi.Coordinates[index].X, roi.Coordinates[index].Y);
                }

                ctx.closePath();
                ctx.stroke();

                ctx.fillStyle = roiFillColor;
                ctx.fill();
            }

        }];


    var videoFeedDirective = function () {
        return {
            bindToController: true,
            controller: videoFeedController,
            controllerAs: "vm",
            restrict: "E",
            scope: {
                cameraType: "@",        // type of camera: 'context' or 'lpr'
                cameraResolution: "=",  // resolution of the camera set when loading an image
                form: "=",              // instance of the form object
                skipCheckVideoActivity: "=",
                userConfig: "=",        // instance of the current/unsaved user configuration (client has to manage the persistence and backup of it)
                // BOUNDING BOX       
                showBoundingBox: "=",   // true or false
                // CROSSHAIR          
                showCrosshair: "=",     // true or false
                // FOV                
                showFov: "=",           // true or false
                // MARKERS            
                editMarkers: "=",       // true or false
                showMarkers: "=",       // true or false
                // REGION OF INTEREST 
                editRoi: "=",           // true or false
                showRoi: "=",           // true or false
                // ZONE               
                editZone: "=",          // true or false
                showZone: "=",          // true or false
                // ZOOM AND FOCUS
                showRuler: "=",         // true or false
                editZoom: "=",          // true or false
                rulerHeight: "="        // int
            },

            template: '<div class="video-feed-directive">' +
                      '    <img id="CameraImage-{{vm.uid}}" style="display: none;" />' +
                      '    <canvas id="CameraCanvas-{{vm.uid}}" class="img-responsive" ng-class="{ \'move_cursor\': vm.videoCanvasCursor===\'move\', \'zoomin_cursor\': vm.videoCanvasCursor===\'zoomin\', \'zoomout_cursor\': vm.videoCanvasCursor===\'zoomout\', \'pointer_cursor\': vm.videoCanvasCursor===\'pointer\', \'default_cursor\': vm.videoCanvasCursor===\'default\' }"></canvas>' +
                      '    <div class="canvas-status" ng-show="vm.videoActivity===false"><h2 class="canvas-status-text disconnected"><i class="fa fa-exclamation-triangle"></i>&nbsp;<translate>STE_LABEL_FeedLost</translate></h2></div>' +
                      '    <div class="canvas-status" ng-show="vm.videoActivity===undefined"><h2 class="canvas-status-text connecting"><i class="fa fa-spinner fa-spin"></i>&nbsp;<translate>STE_LABEL_Connecting</translate>...</h2></div>' +
                      "</div>"
        };
    }


    angular
        .module("portalApp.directives")
        .directive("videoFeed", videoFeedDirective);

})();