﻿(function () {
    "use strict";

    angular
        .module("portalApp.factories")
        .factory("authInterceptor", authInterceptor);

    authInterceptor.$inject = ["$rootScope", "$q", "$window", "$location", "$injector"];

    function authInterceptor($rootScope, $q, $window, $location, $injector) {
        return {
            request: function (config) {
                config.headers = config.headers || {};
                if ($window.sessionStorage.accessToken) {
                    config.headers.Authorization = 'Bearer ' + $window.sessionStorage.accessToken;
                }
                else if ($window.sessionStorage.recoveryToken) {
                    config.headers.Authorization = 'Bearer ' + $window.sessionStorage.recoveryToken;
                }
                return config;
            },
            responseError: function (response) {
                if (response.status === 401) {
                    if ($window.sessionStorage.accessToken) {
                        if (response.config.url === "/login") {
                            // this means the user is trying to change his password and has failed typing the old one correctly
                            return $q.reject(response);
                        }
                    } else {
                        $location.path('/login');
                    }
                }
                if (response.status === 907) {
                    // this means token signature is invalid, i.e. generated with another secret key
                    if ($window.sessionStorage.accessToken) {
                        delete $window.sessionStorage.accessToken;
                    } 
                    if ($window.sessionStorage.refreshToken) {
                        delete $window.sessionStorage.refreshToken;
                    }

                    $location.path('/login');
                }
                // error 906 means the session token expired, we need to ask for a new one
                else if (response.status === 906) {
                    if ($window.sessionStorage.accessToken) {
                        var deferred = $q.defer();

                        var inflightAuthRequest = $injector.get("$http").post("/refresh", { RefreshToken: $window.sessionStorage.refreshToken });

                        inflightAuthRequest.then(function (r) {
                            inflightAuthRequest = null;
                            if (r.data.accessToken && r.data.refreshToken) {
                                $window.sessionStorage.accessToken = r.data.accessToken;
                                $window.sessionStorage.refreshToken = r.data.refreshToken;
                                $injector.get("$http")(response.config).then(function (resp) {
                                    deferred.resolve(resp);
                                }, function (resp) {
                                    deferred.reject();
                                });
                            } else {
                                deferred.reject();
                            }
                        }, function (response) {
                            inflightAuthRequest = null;
                            deferred.reject();
                            delete $window.sessionStorage.accessToken;
                            delete $window.sessionStorage.refreshToken;
                            $location.path('/login');
                            return;
                        });
                        return deferred.promise;
                    } else {
                        $location.path('/login');
                    }
                }
                return $q.reject(response);
            }
        };
    }

})();