﻿(function () {
    "use strict";

    angular
        .module("portalApp.factories")
        .factory("checkChangeStateFactory", checkChangeStateFactory);

    checkChangeStateFactory.$inject = ["$rootScope", "$translate"];

    function checkChangeStateFactory($rootScope, $translate) {

        return {
            checkFormOnStateChange: addCheck,
            checkPageUnloadEvent: checkUnloadEvent
        };

        function addCheck($scope) {
            var removeListener = $rootScope.$on('$stateChangeStart', function (event, toState, toParams, fromState, fromParams, options) {
                if (!$scope.vm.form)
                    return;

                if ($scope.vm.form.$pristine) {
                    return;
                }

                // navigating to child state will not pop the window
                if (toParams.SkipCheck) {
                    return;
                }
          
                var canContinue = confirm($translate.instant("STE_SENTENCE_SettingsChangedContinueWithoutSaving"));
                if (canContinue) {
                    return;
                }
                event.preventDefault();
            }); 
    
            $scope.$on("$destroy", removeListener);
        };

        function checkUnloadEvent($scope) {
            if (window.addEventListener) {
                window.addEventListener("beforeunload", handleUnloadEvent);
            }
            else {
                //For IE browsers
                window.attachEvent("onbeforeunload", handleUnloadEvent);
            }

            function handleUnloadEvent(event) {
                var message = "";
                if (!$scope.form || $scope.form.$pristine) {
                    return;
                }
                else {
                    message = $translate.instant("STE_SENTENCE_SettingsChangedContinueWithoutSaving");
                }

                event.returnValue = message;

                return message;
            };

            function removeUnloadEvent() {
                if (window.removeEventListener) {
                    window.removeEventListener("beforeunload", handleUnloadEvent);
                }
                else {
                    window.detachEvent("onbeforeunload", handleUnloadEvent);
                }
            }

            $scope.$on("$destroy", removeUnloadEvent);
        };
    }
})();