﻿//
// Holds the global values for the current application language (localization)
// and allow to query/modify them.
//

(function () {
    "use strict";

    angular
        .module("portalApp.factories")
        .factory("localizationFactory", localizationFactory);

    localizationFactory.$inject = ["$rootScope", "$translate", "navigator", "moment", "$window"];

    function localizationFactory($rootScope, $translate, navigator, moment, $window) {
        var supportedLanguages;
        var currentLanguage;
        var currentLanguageText;

        var browserLang = (navigator.language || navigator.userLanguage).substring(0, 2);
        setupLanguages(browserLang);

        // recover session language to reload with the same language
        if ($window.sessionStorage.currentLanguage) {
            currentLanguage = $window.sessionStorage.currentLanguage;
        }
        // Apply initial UI language
        $translate.use(currentLanguage);
        moment.locale(currentLanguage);

        return {
            getData: getData,
            updateData: updateData,
            getSupportedLanguages: getSupportedLanguages
        };

        ///////////////////////////////////////////////

        function getData() {
            return {
                langKey: currentLanguage, // the two-letters language code
                text: currentLanguageText // the human-readable description for the language code in the native language
            };
        }

        function getSupportedLanguages() {
            return supportedLanguages;
        }

        function updateData(langKey) {
            currentLanguage = langKey;
            currentLanguageText = languageToText(supportedLanguages, langKey);
            $rootScope.currentLanguage = langKey; // for <html lang=...>
        }

        function setupLanguages(preferredLanguage)
        {
            supportedLanguages =
            [
                // Add more items here as more languages are supported.
                // First entry is the default language
                { langKey: "en", text: "English" },
                { langKey: "fr", text: "Français" }
            ];

            // Determine and set what is the preferred language (based on navigator preferences)
          currentLanguage = supportedLanguages[0].langKey;
            for (var i = 0; i < supportedLanguages.length; i++)
                if (preferredLanguage === supportedLanguages[i].langKey) {
                    currentLanguage = supportedLanguages[i].langKey;
                    break;
                }

            currentLanguageText = languageToText(supportedLanguages, currentLanguage);

            $rootScope.currentLanguage = currentLanguage; // for <html lang=...>
        }

        function languageToText(languages, langKey) {
            for (var i = 0; i < languages.length; i++)
                if (languages[i].langKey === langKey)
                    return languages[i].text;

            return langKey; // unsupported language
        }
    }
})();