﻿//
// Filter used to display Longitude/Latitude coords
//
// <params name="latLong">the coordinates (X, Y)</param>
//
(function () {
    "use strict";

    angular.module("portalApp.filters")
    .filter("gnLatLong", defineLatLongFilter);

    defineLatLongFilter.$inject = ["$translate"];

    function defineLatLongFilter($translate)
    {
        return function (latLong) {
            if (!latLong) return "-";
            // Latitude is the Y axis, Longitude is the X axis
            return ddToDms(latLong.Y, latLong.X);
        }

        function ddToDms(lat, long) {

            if (isNaN(lat) || isNaN(long))
                return "-";

            // Get latitude in DMS.
            var latResult = getDms(lat);
            latResult += (lat >= 0) ? $translate.instant("STE_LABEL_NorthAbbreviation") : $translate.instant("STE_LABEL_SouthAbbreviation");

            // Get longitude in DMS.
            var longResult = getDms(long);
            longResult += (long >= 0) ? $translate.instant("STE_LABEL_EastAbbreviation") : $translate.instant("STE_LABEL_WestAbbreviation");

            // Separate them with a space.
            var dmsResult = latResult + ' ' + longResult;

            // Return the resultant string
            return dmsResult;
        }

        function getDms(val) {

            var result;

            val = Math.abs(val);

            var valDeg = Math.floor(val);
            var valMin = Math.floor((val - valDeg) * 60);
            var valSec = Math.round((val - valDeg - valMin / 60) * 3600 * 1000) / 1000;

            result = valDeg + "°" + valMin + "'" + valSec + '"';

            return result;
        }
    }
})();