﻿(function () {

    "use strict";

    angular
        .module("portalApp", [
            "oc.lazyLoad",
            "ngAnimate",
            "ui.bootstrap",
            "ion.rangeslider",
            "infinite-scroll",
            "portalApp.factories",
            "portalApp.services",
            "portalApp.routing",
            "portalApp.localization",
            "portalApp.controllers",
            "portalApp.directives",
            "portalApp.filters"
        ])        
        .value("backendServerUrl", getBackendUrl("/api"))
        .value("backendLoggerUrl", getBackendUrl("/logger"))
        .value("maxLogMessages", 5000)
        .value("navigator", navigator)
        .value("moment", moment)
        .config(config);

    config.$inject = ["$httpProvider", "$animateProvider", "$ocLazyLoadProvider"];

    function getBackendUrl(path) {
        var url = location.protocol + "//" + location.hostname;
        if (location.port !== "")
            url += ":" + location.port;
        return url + path;
    }

    function config($httpProvider, $animateProvider, $ocLazyLoadProvider) {

        //initialize get if not there
        if (!$httpProvider.defaults.headers.get) {
            $httpProvider.defaults.headers.get = {};
        }

        // Answer edited to include suggestions from comments
        // because previous version of code introduced browser-related errors

        //disable IE ajax request caching
        $httpProvider.defaults.headers.get['If-Modified-Since'] = 'Mon, 26 Jul 1997 05:00:00 GMT';
        // extra
        $httpProvider.defaults.headers.get['Cache-Control'] = 'no-cache';
        $httpProvider.defaults.headers.get['Pragma'] = 'no-cache';

        $animateProvider.classNameFilter(/^(?:(?!ng-animate-disabled).)*$/);
        $httpProvider.interceptors.push('authInterceptor');

        $ocLazyLoadProvider.config({
            events: true
        });

        // not used anymore, but keep it just in case
        //uiGmapGoogleMapApiProvider.configure({
        //    key: 'AIzaSyAcZ1KJLP8oFP2XklnjsKl_Oq8fq6rFNmw',
        //    v: '3.22', //defaults to latest 3.X anyhow
        //    libraries: 'weather,geometry,visualization'
        //});
    }

        //
        // Global function override to allow using "{0} on {1}".format(3, 4) to obtain: "3 on 4", just like in C#.
        // We support only string, number and boolean arguments
        //
        if (!String.prototype.format) {
            String.prototype.format = function () {
                var str = this.toString();
                if (!arguments.length)
                    return str;
    
                for (var i = 0; i < arguments.length; i++) {
                    var t = typeof arguments[i];
                    if (t === "string" || t === "number" || t === "boolean")
                        str = str.replace(RegExp("\\{" + i + "\\}", "gi"), arguments[i]);
                }
    
                return str;
            }
        }
})();