﻿(function() {
    "use strict";

    angular
        .module("portalApp.routing")
        .config(config)
        .run(run);

    config.$inject = ["$stateProvider", "$urlRouterProvider"];

    function config($stateProvider, $urlRouterProvider) {
        $stateProvider
            .state("login", {
                url: "/",
                controller: "LoginController",
                controllerAs: "vm",
                templateUrl: "login.html",
                onEnter: function ($window, $state, $rootScope) {
                    validateUrl();
                    if ($window.sessionStorage.recoveryToken) {
                        $state.go($window.sessionStorage.currentState);
                    }
                    else if ($window.sessionStorage.accessToken && !$window.sessionStorage.Message) {
                        if (!$window.sessionStorage.currentState) {
                            $state.go("main.dashboard.overview");
                        }
                        else {
                            $state.go($window.sessionStorage.currentState);
                        }
                    }                    
                }
            })
            .state("logout", {
                controller: function (portalService, $state) {
                    portalService.logout();
                    $state.go("login");
                }
            })
            .state("recovery", {
                controller: "RecoveryController",
                controllerAs: "vm",
                templateUrl: "recovery/authentication1.html"
            })
            .state("recoverySetup", {
                controller: "RecoveryController",
                controllerAs: "vm",
                templateUrl: "recovery/authentication2.html"
            })
            .state("recoveryButton", {
                controller: "RecoveryController",
                controllerAs: "vm",
                templateUrl: "recovery/waitButton.html"
            })
            .state("recoveryPlate", {
                controller: "RecoveryController",
                controllerAs: "vm",
                templateUrl: "recovery/waitPlate.html"
            })
            .state("recoveryPassword", {
                controller: "RecoveryController",
                controllerAs: "vm",
                templateUrl: "recovery/setPassword.html"
            })
            .state("main", {
                abstract: true,
                controller: "MainController",
                controllerAs: "vm",
                templateUrl: "main.html"
            })
            .state("outofbox", {
                abstract: true,
                controller: "OutOfBoxController",
                controllerAs: "vm",
                templateUrl: "outofbox/outofbox.html",
                resolve: {
                    certificateInfos: function (portalService, $rootScope) {
                        return portalService.getInstalledCertificateInfos()
                            .then(function (value) {
                                $rootScope.connectionLost = false;
                                return value;
                            })
                            .catch(function (reason) {
                                $rootScope.connectionLost = true;
                                return { data: null };
                            });
                    }, 
                    userConfig: function (portalService, $rootScope) {
                        return portalService.getUserConfig()
                            .then(function (value) {
                                $rootScope.connectionLost = false;
                                return value;
                            })
                            .catch(function (reason) {
                                $rootScope.connectionLost = true;
                                return { data: null };
                            });
                    }                    
                }
            })
            .state("outofbox.welcome", {
                views: {
                    "outofboxView": {
                        templateUrl: "outofbox/welcome.html"
                    }
                }
            })
            .state("outofbox.power", {
                views: {
                    "outofboxView": {
                        templateUrl: "outofbox/power.html"
                    }
                }
            })
            .state("outofbox.password", {                
                views: {
                    "outofboxView": {
                        templateUrl: "outofbox/password.html"
                    }
                }
            })
            .state("outofbox.completed", {
                views: {
                    "outofboxView": {
                        templateUrl: "outofbox/completed.html"
                    }
                }
            })
            .state("main.dashboard", {
                abstract: true,
                views: {
                    "mainView": {                        
                        templateUrl: "dashboard/dashboard.html"
                    }
                }
            })
            .state("main.dashboard.overview", {
                views: {
                    "dashboardView": {
                        controller: "OverviewController",
                        controllerAs: "vm",
                        templateUrl: "dashboard/overview.html" 
                    }
                },
                onExit: function ($log, signalRService) {
                    //$log.debug("Exiting the feeds state");                    
                    signalRService.stopConnections(); // make sure no feed persists when leaving the page
                }
            })
            .state("main.dashboard.cameraFeeds", {
                views: {
                    "dashboardView": {
                        controller: "CameraFeedsController",
                        controllerAs: "vm",
                        templateUrl: "dashboard/cameraFeeds.html"
                    }
                },
                onEnter: function ($log, signalRService) {
                    //$log.debug("Entering the feeds state");
                },
                onExit: function ($log, signalRService) {
                    //$log.debug("Exiting the feeds state");                    
                    signalRService.stopConnections(); // make sure no feed persists when leaving the page
                }
            })
            .state("main.config", {
                abstract: true,
                views: {
                    "mainView": {                      
                        templateUrl: "configuration/configuration.html"                       
                    }
                }
            })
            .state("main.config.network", {
                views: {
                    "configView": {   
                        controller: "NetworkController",
                        controllerAs: "vm",  
                        templateUrl: "configuration/network.html"                                              
                    }
                }
            })
            .state("main.config.security", {
                views: {
                    "configView": {
                        controller: "SecurityController",
                        controllerAs: "vm",
                        templateUrl: "configuration/security.html"
                    }
                }
            })
            .state("main.config.zoomAndFocus", {
                views: {
                    "configView": {
                        controller: "ZoomAndFocusController",
                        controllerAs: "vm",
                        templateUrl: "configuration/zoomAndFocus.html"
                    }
                },
                onEnter: function ($log, signalRService) {
                    //$log.debug("Entering the feeds state");
                },
                onExit: function ($log, signalRService) {
                    //$log.debug("Exiting the feeds state");                    
                    signalRService.stopConnections(); // make sure no feed persists when leaving the page
                }
            })
            .state("main.config.cameras", {
                views: {
                    "configView": {
                        controller: "CamerasController",
                        controllerAs: "vm",
                        templateUrl: "configuration/cameras.html"                   
                    }
                },
                onEnter: function ($log, signalRService) {
                    //$log.debug("Entering the feeds state");
                },
                onExit: function ($log, signalRService) {
                    //$log.debug("Exiting the feeds state");
                    signalRService.stopConnections(); // make sure no feed persists when leaving the page
                }
            })
            .state("main.config.lpr", {
                abstract: true,
                views: {
                    "configView": {                        
                        template: "<div ui-view='lprView' class='withFooter'></div>"
                    }
                }                                
            })
            .state("main.config.lpr.analytics", {
                views: {
                    "lprView": {
                        controller: "AnalyticsController",
                        controllerAs: "vm",
                        templateUrl: "configuration/analytics.html"
                    }
                }
            })
            .state("main.config.lpr.vehicleDetection", {
                views: {
                    "lprView": {
                        controller: "VehicleDetectionController",
                        controllerAs: "vm",
                        templateUrl: "configuration/vehicleDetection/vehicleDetection.html"
                    }
                },
                onExit: function ($log, signalRService) {
                    //$log.debug("Exiting the feeds state");                    
                    signalRService.stopConnections(); // make sure no feed persists when leaving the page
                }
            })
            .state("main.config.lpr.vehicleDetectionEdit", {
                views: {
                    "lprView": {
                        controller: "VehicleDetectionEditController",
                        controllerAs: "vm",
                        templateUrl: "configuration/vehicleDetection/vehicleDetectionEdit.html"
                    }
                },
                onExit: function ($log, signalRService) {
                    //$log.debug("Exiting the feeds state");                    
                    signalRService.stopConnections(); // make sure no feed persists when leaving the page
                }
            })
            .state("main.config.lpr.speedEstimation", {
                views: {
                    "lprView": {
                        controller: "SpeedEstimationController",
                        controllerAs: "vm",
                        templateUrl: "configuration/speedEstimation/speedEstimation.html"
                    }
                },
                onExit: function ($log, signalRService) {
                    //$log.debug("Exiting the feeds state");                    
                    signalRService.stopConnections(); // make sure no feed persists when leaving the page
                }
            })
            .state("main.config.lpr.speedEstimationEdit", {
                views: {
                    "lprView": {
                        controller: "SpeedEstimationController",
                        controllerAs: "vm",
                        templateUrl: "configuration/speedEstimation/speedEstimationEdit.html"
                    }
                },
                onExit: function ($log, signalRService) {
                    //$log.debug("Exiting the feeds state");                    
                    signalRService.stopConnections(); // make sure no feed persists when leaving the page
                }
            })
            .state("main.config.extension", {
                params:{SkipCheck:null},
                views: {
                    "configView": {
                        controller: "ExtensionController",
                        controllerAs: "vm",
                        templateUrl: "configuration/extension.html"
                    }
                }
            })            
            .state("main.config.extension.ftp", {
                views: {
                    "extensionView": {
                        templateUrl: "configuration/extensions/ftp.html"
                    }
                }
            })
            .state("main.config.extension.http", {
                views: {
                    "extensionView": {
                        templateUrl: "configuration/extensions/http.html"
                    }
                }
            })
            .state("main.config.extension.patroller", {
                views: {
                    "extensionView": {
                        templateUrl: "configuration/extensions/patroller.html"
                    }
                }
            })
            .state("main.config.extension.securitycenter", {
                views: {
                    "extensionView": {
                        templateUrl: "configuration/extensions/securitycenter.html"
                    }
                }
            })
            .state("main.config.extension.uploadtocloud", {
                views: {
                    "extensionView": {
                        templateUrl: "configuration/extensions/uploadtocloud.html"
                    }
                }
            })
            .state("main.config.general", {
                abstract: true,
                views: {
                    "configView": {
                        template: "<div ui-view='generalView' class='withFooter'></div>"
                    }
                }
            })
            .state("main.config.general.time", {
                views: {
                    "generalView": {
                        controller: "TimeController",
                        controllerAs: "vm",
                        templateUrl: "configuration/general/time.html"
                    }
                },
                onExit: function ($log, signalRService) {
                    //$log.debug("Exiting the feeds state");                    
                    signalRService.stopConnections(); // make sure no feed persists when leaving the page
                }
            })
            .state("main.config.general.power", {
                views: {
                    "generalView": {
                        controller: "PowerController",
                        controllerAs: "vm",
                        templateUrl: "configuration/general/power.html"
                    }
                }
            })
            .state("main.config.maintenance", {
                views: {
                    "configView": {
                        controller: "MaintenanceController",
                        controllerAs: "vm",
                        templateUrl: "configuration/maintenance.html"
                        } 
                    }
            })
            .state("main.diagnostics", {
                abstract: true,
                views: {
                    "mainView": {                        
                        templateUrl: "diagnostics/diagnostics.html"
                    }
                }
            })
            .state("main.diagnostics.logs", {
                views: {
                    "diagnosticsView": {
                        controller: "LogsController",
                        controllerAs: "vm",
                        templateUrl: "diagnostics/logs.html",
                        resolve: {
                            ngAnimate: ['$$animateJs', '$ocLazyLoad', function ($$animateJs, $ocLazyLoad) {
                                return $ocLazyLoad.load([
                                    'js/bower/ui-grid.min.js'
                                ]);
                            }]
                        }
                    }
                },
                onExit: function ($log, signalRService) {
                    //$log.debug("Exiting the feeds state");                    
                    signalRService.stopConnections(); // make sure no feed persists when leaving the page
                }
            })
            //.state("main.diagnostics.commands", {
            //    views: {
            //        "diagnosticsView": {
            //            template: ""
            //        }
            //    }
            //})
           .state("main.help", {
             abstract: true,
             views: {
                 "mainView": {
                     templateUrl: "help/help.html"
                 }
             },
             onEnter: function ($rootScope) {
                 $rootScope.connectionLost = false;
             }
           })
         .state("main.help.about", {
             views: {
                 "helpView": {
                     templateUrl: "help/about.html"
                 }
             }
         })
         .state("main.help.documentation", {
             views: {
                 "helpView": {
                     templateUrl: "help/documentation.html"
                 }
             }
           }
        )
        ;

        $urlRouterProvider.otherwise("/");        
    }

    run.$inject = ["$rootScope", "$window", "$timeout"];

    function run($rootScope, $window, $timeout) {
        $rootScope.$on('$stateChangeSuccess', function (event, toState, toParams, fromState, fromParams) {
            if (toState.name !== "logout")
                $window.sessionStorage.currentState = toState.name;
        });
    }

    // simplified regex for IPv6 address
    // Note: NOT for validating if an IPv6 has the correct format, but simply to find out if the current window location is using IPv6
    // (other possible square brackets (often found in the query string) are always encoded by the browser (even if they appear as is in the address bar))
    var ipv6Regex = /\[(.)*\]/;
    
    function validateUrl() {
        var url = window.location.href;
        if (!isUrlInvalid(url)) {
            return;
        }
        window.location.href = convertIpv6Url(url);
    }

    function isUrlInvalid(url) {
        // We only have a problem if the address has the IPv6 format and if we are using IE or Edge
        var match = url.match(ipv6Regex);

        if (!match) {
            return false;
        }

        // Internet Explorer 6-11
        var isIE = /*@cc_on!@*/false || !!document.documentMode;

        // Edge 20+
        var isEdge = !isIE && !!window.StyleMedia;

        return isIE || isEdge;
    }

    function convertIpv6Url(url) {
        // Transform url to use ipv6-literal.net
        // examples:
        // [2001:db8:85a3:8d3:1319:8a2e:370:7348] becomes 2001-db8-85a3-8d3-1319-8a2e-370-7348.ipv6-literal.net
        // [2001:db8::1/32] becomes 2001-db8--1s32.ipv6-literal.net
        // [::1] becomes 0--1.ipv6-literal.net
        var match = url.match(ipv6Regex);
        if (!match) return url;

        var ipv6Address = match[0];

        var uncAddress = ipv6Address.replace(/\[|\]/g, "") // remove square brackets
            .replace(/\:/g, "-") // replace colons (:) with dashes (-)
            .replace(/\//g, "s") // replace slash (/) with 's' 
            .concat(".ipv6-literal.net"); // append the domain

        if (uncAddress[0] === "-") { // the UNC path cannot start with a dash (-) so we have to add a leading 0
            uncAddress = "0".concat(uncAddress);
        }

        return url.replace(ipv6Address, uncAddress);
    }

})();