﻿(function () {
    "use strict";

    angular
        .module("portalApp.services")
        .service("portalService", portalService);

    portalService.$inject = ["$http", "$base64", "$state", "backendServerUrl", "$q", "$window", "$log"];

    function portalService($http, $base64, $state, backendServerUrl, $q, $window, $log) {

        var blockLogout = false;  

        var service = {
            login: login,
            refresh: refresh,
            logout: logout,
            getInstalledFiles: getInstalledFiles,
            getProperties: getProperties,
            createHash: createHash,
            createMD5: createMD5,
            getComponents: getComponents,
            getEnvironment: getEnvironment,
            startIdentify: startIdentify,
            resetBestReadCounter: resetBestReadCounter,
            getUserConfig: getUserConfig,
            setUserConfig: setUserConfig,
            setVehDetectionConfig: setVehDetectionConfig,
            setPassword: setPassword,
            getInstalledCertificateInfos: getInstalledCertificateInfos,
            getExtensions: getExtensions,
            getFtpExtensionSettings: getFtpExtensionSettings,
            setFtpExtensionSettings: setFtpExtensionSettings,
            testFtp: testFtp,
            sendFtpSample : sendFtpSample,
            sendHttpSample : sendHttpSample,
            getHttpExtensionSettings: getHttpExtensionSettings,
            setHttpExtensionSettings: setHttpExtensionSettings,
            getPatrollerExtensionSettings: getPatrollerExtensionSettings,
            setPatrollerExtensionSettings: setPatrollerExtensionSettings,
            getSecurityCenterExtensionSettings: getSecurityCenterExtensionSettings,
            setSecurityCenterExtensionSettings: setSecurityCenterExtensionSettings,
            setCertificate: setCertificate,
            getCsr: getCsr,
            deleteCertificate: deleteCertificate,
            getLprContexts: getLprContexts,
            postRegionOfInterest: postRegionOfInterest,
            getRegionOfInterest: getRegionOfInterest,
            getTime: getTime,
            getTryConnectionToPort: getTryConnectionToPort,
            testNtp: testNtp,
            getVideoformats: getVideoformats,
            getExposureModes: getExposureModes,
            setExposureSettings: setExposureSettings,
            getExposureCapabilities: getExposureCapabilities,
            getExposureDefaults : getExposureDefaults,
            setTime: setTime,
            getPing: getPing,
            getPortalPing: getPortalPing,
            registerImageCapture: registerImageCapture,
            stopImageCapture: stopImageCapture,
            postSettings: postSettings,
            getFreeSpace: getFreeSpace,
            getReclaimableSpace: getReclaimableSpace,
            resetConfiguration: resetConfiguration,
            rebootUnit: rebootUnit,
            toggleOutputRelay: toggleOutputRelay,
            beginZoomAndFocus: beginZoomAndFocus,
            endZoomAndFocus: endZoomAndFocus,
            setFocusExposureLevel: setFocusExposureLevel,
            resetFocusExposure: resetFocusExposure,
            setFocusRectangle: setFocusRectangle,
            getDiagnostics: getDiagnostics,
            getLoggerConfiguration: getLoggerConfiguration,
            setLoggerConfiguration: setLoggerConfiguration,
            testSyslog: testSyslog,
            webupdateUpload: webupdateUpload,
            putProductVersion: putProductVersion,
            initializeApplicationUpload: initializeApplicationUpload,
            completeApplicationUpload: completeApplicationUpload,
            startWebUpdaterServer: startWebUpdaterServer,
            getWebUpdaterIsRunning: getWebUpdaterIsRunning,
            setRemoteAssistance: setRemoteAssistance,
            getPoePlusStatus: getPoePlusStatus,
            getRemoteAssistanceConfig: getRemoteAssistanceConfig,
            createCsr: createCsr,
            createSelfSigned: createSelfSigned,
            disableLogout: disableLogout,
            enableLogout: enableLogout,
            speedEstimationClearCalibration: speedEstimationClearCalibration,
            restartVehicleDetectionCalibration: restartVehicleDetectionCalibration,
            getSpeedEstimationSummary: getSpeedEstimationSummary,
            postSpeedEstimationStartCalibration: postSpeedEstimationStartCalibration,
            postSpeedEstimationAddPlate: postSpeedEstimationAddPlate,
            getSpeedEstimationCalibrationData: getSpeedEstimationCalibrationData,
            getSpeedEstimationAwaitedCalibrationPlateInfo: getSpeedEstimationAwaitedCalibrationPlateInfo,
            deleteSpeedEstimationPlateToWaitFor: deleteSpeedEstimationPlateToWaitFor,
            deleteSpeedEstimationCalibrationPlate: deleteSpeedEstimationCalibrationPlate,
            postSpeedEstimationValidatePlate: postSpeedEstimationValidatePlate,
            getSpeedEstimationOtherCalibrationPlateInfo: getSpeedEstimationOtherCalibrationPlateInfo,
            checkAccessCode: checkAccessCode,
            canReset: canReset,
            cancelReset: cancelReset,
            recoveryButton: recoveryButton,
            recoveryPlate: recoveryPlate,
            getUploadToCloudDetails: getUploadToCloudDetails,
            resetNewPassword: resetNewPassword,
            getCertificates: getCertificates
    };

        return service;

        ///////////////////////////////////////////////

        function login(username, pwd) {
            var hash = createHash(pwd);

            var request = { 'username': username, 'password': hash }

            return $http.post("/login", request);
        }

        function refresh() {
            return $http.post("/refresh", { RefreshToken: $window.sessionStorage.refreshToken });
        }

        function logout() {
            if (blockLogout)
                return;
            delete $window.sessionStorage.accessToken;
            delete $window.sessionStorage.refreshToken;
        }

        function getInstalledFiles() {
            return $http.get(backendServerUrl + "/portal/installedfiles");
        }

        function getComponents() {
            return $http.get(backendServerUrl + "/portal/componentfirmwareversions");            
        }

        function getProperties() {
            return $http.get(backendServerUrl + "/portal/properties");
        }

        function getUserConfig() {
            return $http.get(backendServerUrl + "/portal/userconfig?type=JSON");
        }
        
        function setUserConfig(userConfig) {
            return $http.post(backendServerUrl + "/portal/userconfig", userConfig);
        }

        function setVehDetectionConfig(vehDetectionConfig) {
            return $http.post(backendServerUrl + "/portal/vehdetectionconfig", vehDetectionConfig);
        }

        function setPassword(password) {
            var hash = createHash(password);
            var md5 = createMD5(password);
            return $http.post("/password", "'" + hash + "_" + md5 + "'");
        }

        function getEnvironment() {
            return $http.get("/environment");
        }

        function startIdentify() {
            return $http.get(backendServerUrl + "/identify.cgi");
        }

        function resetBestReadCounter() {
            return $http.put(backendServerUrl + "/portal/bestreadcount", "");
        }

        function getInstalledCertificateInfos() {
            return $http.get(backendServerUrl + "/portal/certificateinfos");
        }

        function getExtensions() {
            return $http.get(backendServerUrl + "/portal/extensions");
        }

        function getFtpExtensionSettings(guid) {
            return $http.get(backendServerUrl + "/portal/ftpextensionsettings?guid=" + guid);
        }

        function setFtpExtensionSettings(settings, guid) {
            return $http.post(backendServerUrl + "/portal/ftpextensionsettings?guid=" + guid, settings);
        }

        function testFtp(settings) {
            return $http.post(backendServerUrl + "/portal/ftptest", settings);
        }

        function sendFtpSample(settings) {
            return $http.post(backendServerUrl + "/portal/ftpsample", settings);
        }

        function sendHttpSample(settings) {
            return $http.post(backendServerUrl + "/portal/httpsample", settings);
        }

        function testSyslog(servername, port, protocol) {
            var server = servername + ":" + port;
            return $http.post(backendServerUrl + "/portal/syslogtest", { IsEnabled: true, Server: server, NetworkProtocol: protocol });
        }

        function getHttpExtensionSettings(guid) {
            return $http.get(backendServerUrl + "/portal/httpextensionsettings?guid=" + guid);
        }

        function setHttpExtensionSettings(settings, guid) {
            return $http.post(backendServerUrl + "/portal/httpextensionsettings?guid=" + guid, settings);
        }

        function getPatrollerExtensionSettings(guid) {
            return $http.get(backendServerUrl + "/portal/patrollerextensionsettings?guid=" + guid);
        }

        function setPatrollerExtensionSettings(settings, guid) {
            return $http.post(backendServerUrl + "/portal/patrollerextensionsettings?guid=" + guid, settings);
        }

        function getSecurityCenterExtensionSettings(guid) {
            return $http.get(backendServerUrl + "/portal/securitycenterextensionsettings?guid=" + guid);
        }

        function setSecurityCenterExtensionSettings(settings, guid) {
            return $http.post(backendServerUrl + "/portal/securitycenterextensionsettings?guid=" + guid, settings);
        }

        function setCertificate(certificate) {
            return $http.post(backendServerUrl + "/portal/certificate", certificate);
        }

        function getCsr() {
            return $http.get(backendServerUrl + "/portal/csr");
        }

        function deleteCertificate(certificateInfos) {
            return $http.delete(backendServerUrl + "/portal/certificate?thumbprint=" + certificateInfos.Thumbprint + "&isRequest=" + certificateInfos.IsRequest);
        }

        function getLprContexts() {
            return $http.get(backendServerUrl + "/portal/lprcontexts");
        }

        function postRegionOfInterest(lprCameraName, coordinates, imageWidth, imageHeight) {
            return $http.post(backendServerUrl + "/portal/regionofinterest?lprCameraName=" +
                lprCameraName + "&imageWidth=" +
                imageWidth + "&imageHeight=" + imageHeight,
                coordinates);
        }

        function getRegionOfInterest(lprCameraName) {
            return $http.get(backendServerUrl + "/portal/regionofinterest?lprCameraName=" + lprCameraName);
        }

        function getTime() {
            return $http.get(backendServerUrl + "/portal/time");
        }

        function getTryConnectionToPort(address, port) {
            return $http.get(backendServerUrl + "/portal/tryconnectiontoport?address=" + address + "&port=" + port);
        }

        function testNtp(ntpServer) {
            return $http.post(backendServerUrl + "/portal/ntptest", "'" + ntpServer + "'");
        }

        function getVideoformats() {
            return $http.get(backendServerUrl + "/portal/videoformats");
        }

        function getExposureModes() {
            return $http.get(backendServerUrl + "/portal/exposuremodes");
        }

        function setExposureSettings(cameraName, exposureSettings) {
            if (exposureSettings.ExposureMode !== "Automatic" 
              && (exposureSettings.Gain < 0 || exposureSettings.ShutterTime <= 0)) {
                $log.debug("portalService.setExposureSettings: Invalid gain or shutter time. No change to settings.");
                return null;
            }
            return $http.post(backendServerUrl + "/portal/exposureconfig?cameraName=" + cameraName, exposureSettings);
        }

        function getExposureCapabilities(cameraName) {
            return $http.get(backendServerUrl + "/portal/exposurecapabilities?cameraName=" + cameraName);
        }

        function getExposureDefaults(cameraName) {
            return $http.get(backendServerUrl + "/portal/exposuresettings/defaults?cameraName=" + cameraName);
        }

        function setTime(time) {
            return $http.post(backendServerUrl + "/portal/time?newTime=" + time, "");
        }

        function getPing(url) {
            return $http.get(url + "/portal/ping", { timeout: 5000 });
        }

        function getPortalPing() {
            return $http.get(location.protocol + "//" + location.hostname + "/ping?" + new Date().toString(), { timeout: 5000 });
        }

        function registerImageCapture(cameraNames, recordingType, duration) {

            // convert from JavaScript format to C# enum format
            var imageFeedType = recordingType === "saveAllImages" ? "AllReads" : "BestReads";

            return $http.post(backendServerUrl + "/portal/registerImageCapture?feedType={0}&minutesToRecord={1}".format(imageFeedType, duration), cameraNames);
        }

        function stopImageCapture(token) {
            return $http.post(backendServerUrl + "/portal/stopImageCapture/{0}".format(token), "");
        }

        function postSettings(file) {
            return $http.post(backendServerUrl + "/portal/settings/", file);
        }

        function getFreeSpace() {
            return $http.get(backendServerUrl + "/diskmanagement/freespace");
        }

        function getReclaimableSpace() {
            return $http.get(backendServerUrl + "/diskmanagement/reclaimableSpace");
        }

        function resetConfiguration(resetNetwork) {
            return $http.post(backendServerUrl + "/portal/reset", resetNetwork);
        }

        function rebootUnit() {
            return $http.get(backendServerUrl + "/portal/reboot");
        }

        function toggleOutputRelay(name) {
            return $http.post(backendServerUrl + "/portal/outputpin.{0}/toggle".format(name), "");
        }

        function beginZoomAndFocus(cameraName) {
            // Stop auto-exposure, allow temporarily override of exposure settings (keep a backup for restoring at end of operation)
            var timeout = 30 * 60 * 1000; // 30 minutes timeout (same as session)
            return $http.post(backendServerUrl + "/portal/zoomandfocus/begin?camera={0}&timeout={1}".format(cameraName, timeout), "");
        }

        // index is a floating number betwwen 0.0 and 1.0.
        function setFocusExposureLevel(cameraName, level, flashEnabled) {
            return $http.post(backendServerUrl + "/portal/zoomandfocus/exposure?camera={0}&level={1}&flashEnabled={2}".format(cameraName, level, flashEnabled), "");
        }

        // x, y, width, height: rectangle to consider for focus.
        function setFocusRectangle(cameraName, x, y, width, height) {
            return $http.post(backendServerUrl + "/portal/zoomandfocus/rectangle?camera={0}&x={1}&y={2}&width={3}&height={4}".format(cameraName, x, y, width, height), "");
        }

        function resetFocusExposure(cameraName) {
            return $http.post(backendServerUrl + "/portal/zoomandfocus/exposure/reset?camera={0}".format(cameraName), "");
        }

        function endZoomAndFocus(cameraName) {
            // Restore exposure to previous value (saved in beginZoomAndFocus)
            return $http.post(backendServerUrl + "/portal/zoomandfocus/end?camera={0}".format(cameraName), "");
        }

        function getDiagnostics() {

            // very important to put arraybuffer in the config otherwise the zip will corrupted
            return $http.get(backendServerUrl + "/portal/Diagnostics.zip", { responseType: "arraybuffer" });
        }

        function getLoggerConfiguration() {
            return $http.get(backendServerUrl + "/portal/loggerconfig");
        }

        function setLoggerConfiguration(configuration) {
            return $http.post(backendServerUrl + "/portal/loggerconfig", configuration);
        }

        function putProductVersion(version) {
            return $http.put(backendServerUrl + "/portal/webupdater/productversion", version);
        }

        function webupdateUpload(updateInfo, name, data) {
            var config = { headers: { 'Content-Type': 'application/octet-stream' } };
            var blob = new Blob([data], { type: 'application/octet-binary' });
            var toto = "/portal/webupdater/upload?applicationType={0}&version={1}&filename={2}&filesize={3}&offset={4}&length={5}".format(updateInfo.Application, updateInfo.Version, name, data.byteLength, 0, data.byteLength);
            var promise = $http.put(backendServerUrl + toto, blob, config);
            console.log("WebUpdate upload created for " + name);
            return promise;
        }

        function initializeApplicationUpload(updateInfo) {
            return $http.put(backendServerUrl + "/portal/webupdater/initializeapplicationupload", updateInfo);
        }

        function completeApplicationUpload(updateInfo) {
            return $http.put(backendServerUrl + "/portal/webupdater/completeapplicationupload", updateInfo);
        }

        function startWebUpdaterServer(isSecure) {
            return $http.get(backendServerUrl + "/portal/webupdater/startserver?IsSecure=" + isSecure);
        }

        function getWebUpdaterIsRunning() {
            return $http.get(backendServerUrl + "/portal/webupdater/isrunning");
        }

        function setRemoteAssistance(expireAfterXMinutes) {
            return $http.post("/RemoteAssistance", expireAfterXMinutes);
        }

        function getRemoteAssistanceConfig() {
            return $http.get("/RemoteAssistance");
        }

        function getPoePlusStatus() {
            return $http.get("/PoePlusStatus");
        }

        function createCsr(request) {
            return $http.post(backendServerUrl + "/portal/csr", request);
        }

        function createSelfSigned(request) {
            return $http.post(backendServerUrl + "/portal/selfsigned", request);
        }

        function speedEstimationClearCalibration() {
            return $http.post(backendServerUrl + "/portal/speedestimation/clearcalibration");
        }

        function restartVehicleDetectionCalibration() {
            return $http.post(backendServerUrl + "/portal/vehicleDetection/restartcalibration");
		}

        function getSpeedEstimationSummary() {
            return $http.get(backendServerUrl + "/portal/speedestimation/summary");
        }

        function postSpeedEstimationStartCalibration() {
            return $http.post(backendServerUrl + "/portal/speedestimation/start");
        }

        function postSpeedEstimationAddPlate(plateNumber) {
            return $http.post(backendServerUrl + "/portal/speedestimation/addplate?plateNumber=" + plateNumber, "");
        }

        function getSpeedEstimationCalibrationData() {
            return $http.get(backendServerUrl + "/portal/speedestimation/calibrationdata");
        }

        function deleteSpeedEstimationPlateToWaitFor(plateId) {
            return $http.delete(backendServerUrl + "/portal/speedestimation/deleteplate?plateId=" + plateId, "");
        }

        function deleteSpeedEstimationCalibrationPlate(id) {
            return $http.delete(backendServerUrl + "/portal/speedestimation/deletecalibrationplate?id=" + id, "");
        }

        function postSpeedEstimationValidatePlate(plateInfo) {
            plateInfo.Image1 = null; // flush images to reduce load
            plateInfo.Image2 = null;
            return $http.post(backendServerUrl + "/portal/speedestimation/validate", plateInfo);
        }

        function getSpeedEstimationAwaitedCalibrationPlateInfo(plateToWaitForId, plateId) {
            return $http.get(backendServerUrl + "/portal/speedestimation/awaitedcalibrationplateinfo?plateToWaitForId=" + plateToWaitForId + "&plateId=" + plateId);
        }

        function getSpeedEstimationOtherCalibrationPlateInfo(plateId) {
            return $http.get(backendServerUrl + "/portal/speedestimation/othercalibrationplateinfo?plateId=" + plateId);
        }

        function disableLogout() {
            blockLogout = true;
        }

        function enableLogout() {
            blockLogout = false;
        }

        function checkAccessCode(rdpPassword) {
            return $http.post("/checkAccessCode", "'" + rdpPassword + "'");
        }

        function canReset() {
            return $http.get("/canreset");
        }

        function cancelReset(token) {
            return $http.post(backendServerUrl + "/portal/cancelreset", "'" + token + "'");
        }

        function recoveryButton() {
            return $http.post(backendServerUrl + "/portal/recoveryButton");
        }

        function recoveryPlate(plate) {
            return $http.post(backendServerUrl + "/portal/recoveryPlate", "'" + plate + "'");
        }

        function resetNewPassword(newPassword) {
            var hash = createHash(newPassword);
            var md5 = createMD5(newPassword);
            return $http.post("/resetpassword", "'" + hash + "_" + md5 + "'");
        }

        function getUploadToCloudDetails() {
            return $http.get(backendServerUrl + "/portal/uploadtoclouddetails");
        }

        function getCertificates() {
            return $http.get(backendServerUrl + "/portal/certificates");
        }

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

        function createHash(value) {
            var p = CryptoJS.enc.Utf16LE.parse(value);
            var sha1 = CryptoJS.SHA512(p);
            var hash = sha1.toString().toUpperCase();

            return hash;
        }

        function createMD5(value) {
            return CryptoJS.MD5("admin" + ':RealmOfGenetec:' + value).toString();
        }
    }
})();