﻿(function () {
    "use strict";

    var generalTransport = ["webSockets", "longPolling"];
    var timeFormat = "YYYY-MM-DD h:mm:ss a";

    var _private = {
        perfHub: null,
        feedHub: null,
        logHub: null,
        ioHub: null,
        lastActivitiesHub: null,
        speedEstimationHub: null,
	    vehicleDetectionCalibrationHub: null,
        recoveryHub: null,
        perfConnection: null,
        feedConnection: null,
        logConnection: null,
        ioConnection: null,
        lastActivitiesConnection: null,
        speedEstimationConnection: null,
        vehicleDetectionCalibrationConnection: null,
        recoveryConnection: null,
        stopped: false,
        stopping: false,
        feedRequest: null,
        refreshingToken: false
    };

    _private.refreshTokens = function ($injector, $window) {
        if (_private.refreshingToken)
            return;
        _private.refreshingToken = true;
        var request = $injector.get("$http").post("/refresh", { RefreshToken: $window.sessionStorage.refreshToken });
        var _this = this;
        request.then(function (result) {
            request = null;
            if (result.data.accessToken && result.data.refreshToken) {
                $window.sessionStorage.accessToken = result.data.accessToken;
                $window.sessionStorage.refreshToken = result.data.refreshToken;

                // refresh all hubs query string with new token
                if (_this.perfConnection != null)
                    _this.perfConnection.qs = { Bearer: $window.sessionStorage.accessToken };
                if (_this.feedConnection != null)
                    _this.feedConnection.qs = { Bearer: $window.sessionStorage.accessToken };
                if (_this.ioConnection != null)
                    _this.ioConnection.qs = { Bearer: $window.sessionStorage.accessToken };
                if (_this.lastActivitiesConnection != null)
                    _this.lastActivitiesConnection.qs = { Bearer: $window.sessionStorage.accessToken };
                if (_this.logConnection != null)
                    _this.logConnection.qs = { Bearer: $window.sessionStorage.accessToken };
                if (_this.speedEstimationConnection != null)
                    _this.speedEstimationConnection.qs = { Bearer: $window.sessionStorage.accessToken };
                if (_this.vehicleDetectionCalibrationConnection != null)
                    _this.vehicleDetectionCalibrationConnection.qs = { Bearer: $window.sessionStorage.accessToken };
                if (_this.recoveryConnection != null)
                    _this.recoveryConnection.qs = { Bearer: $window.sessionStorage.accessToken };
            }
            
        }).finally(function () {
            _private.refreshingToken = false;
        });
    }

    _private.checkErrorTokenExpired = function (error, $injector, $window) {
        if (error.context && (error.context.statusText === 'token expired' || error.context.status === 906))
        {
            _private.refreshTokens($injector, $window);
        }
    }

    angular
        .module("portalApp.services")
        .value("$", $)
        .service("signalRService", signalRService);

    signalRService.$inject = ["$", "$rootScope", "backendServerUrl", "backendLoggerUrl", "$log", "$q", "$window", "$injector"];

    function signalRService($, $rootScope, backendServerUrl, backendLoggerUrl, $log, $q, $window, $injector) {

        var service = {
            _private: _private, // mainly used for unit tests
            initializePerformanceHub: initializePerformanceHub,
            initializeFeedHub: initializeFeedHub,
            initializeLogHub: initializeLogHub,
            initializeIoHub: initializeIoHub,
            initializeLastActivitiesHub: initializeLastActivitiesHub,
            initializeVehicleDetectionCalibrationHub: initializeVehicleDetectionCalibrationHub,
            initializeSpeedEstimationHub: initializeSpeedEstimationHub,
            initializeRecoveryHub: initializeRecoveryHub,
            isConnected: isConnected,
            stopFeed: stopFeed,
            startFeed: startFeed,
            reduceFeed: reduceFeed,
            resetCounter: resetCounter,
            getSources: getSources,
            getLogEntries: getLogEntries,
            getXMLLogEntries: getXMLLogEntries,
            enableSource: enableSource,
            disableSource: disableSource,
            FeedRequestOptions: FeedRequestOptions,
            base64ToArrayBuffer: base64ToArrayBuffer,
            stopConnections: stopConnections,
            pauseLogs: pauseLogs,
            resumeLogs: resumeLogs,
            startTimer: startTimer,
            getTimer: getTimer
        };

        $rootScope.$on("TokenUpdated", service.updateTokens);

        return service;

        function createHubConnection(url) {
            return $.hubConnection(url);
        }

        function initializePerformanceHub() {
            if (_private.perfHub != null)
                return; //already connected!!

            _private.perfConnection = createHubConnection(backendServerUrl);
            _private.perfConnection.qs = { Bearer: $window.sessionStorage.accessToken };
            _private.perfHub = _private.perfConnection.createHubProxy("PerformanceHub");

            _private.stopped = false;

            //Publishing an event when server pushes a CpuUsage message
            _private.perfHub.on("CpuUsage", function(message) {
                $rootScope.$emit("CpuUsage", message);
            });

            _private.perfHub.on("CpusUsage", function(message) {
                $rootScope.$emit("CpusUsage", message);
            });

            _private.perfHub.on("DriveCSpace", function(message) {
                $rootScope.$emit("DriveCSpace", message);
            });

            _private.perfHub.on("DriveDSpace", function(message) {
                $rootScope.$emit("DriveDSpace", message);
            });

            _private.perfHub.on("DriveESpace", function(message) {
                $rootScope.$emit("DriveESpace", message);
            });

            _private.perfHub.on("RamUsage", function(message) {
                $rootScope.$emit("RamUsage", message);
            });

            _private.perfHub.on("StorageInfo", function(message) {
                $rootScope.$emit("StorageInfo", message);
            });

            _private.perfConnection.error(function(error) {
                _private.checkErrorTokenExpired(error, $injector, $window);
            });

            _private.perfConnection.reconnecting(function() {
                $rootScope.$emit("Reconnecting", {});
            });

            _private.perfConnection.reconnected(function() {
                _private.perfConnection.start({ jsonp: true, transport: generalTransport })
                    .done(function () {
                        $rootScope.$emit("Connected", {});
                    });
            });

            _private.perfConnection.disconnected(function() {
                if (!_private.stopped && !_private.stopping) {
                    $rootScope.$emit("Reconnecting", {});
                    setTimeout(function () {
                        if (_private.perfConnection) {                            
                            _private.perfConnection.start({ jsonp: true, transport: generalTransport })
                                .done(function() {
                                    $rootScope.$emit("Connected", {});
                                }).fail(function (error) {
                                    _private.checkErrorTokenExpired(error, $injector, $window);
                                });
                        } // keep connected when timeout occurs
                    }, 5000); // Restart connection after 5 seconds
                }
            });

            //Starting connection
            _private.perfConnection.start({ jsonp: true, transport: generalTransport })
                .fail(function (error) {
                    _private.checkErrorTokenExpired(error, $injector, $window);
                });
        }

        function initializeFeedHub(feedRequest) {
            if (_private.feedHub != null) {
                if (feedRequest.feedTypes > 0)
                    invokeStartFeed(feedRequest); // restart the feed if called again

                // Register again since feedRequest may have changed
                _private.feedRequest = feedRequest;
                _private.feedConnection.disconnected(function () {
                    $rootScope.$emit("LostConnection", {});
                    if (!_private.stopped && !_private.stopping) {
                        setTimeout(function () {
                            startConnection(_private.feedConnection); // keep connected when timeout occurs
                        }, 5000);
                    }
                });

                return; //already connected!!
            }

            _private.feedConnection = createHubConnection(backendServerUrl);
            _private.feedConnection.qs = { Bearer: $window.sessionStorage.accessToken };
            _private.feedHub = _private.feedConnection.createHubProxy("ImageFeedHub");
            
            _private.stopped = false;

            //Publishing an event to our views when receiving a NewImageFeedInfo message from server
            _private.feedHub.on("NewImageFeedInfo", function (message) {
                $rootScope.$emit("NewImageFeedInfo", message);
            });

            _private.feedHub.on("NewImageChunk", function (message) {
                $rootScope.$emit("NewImageChunk", message);
            });

            _private.feedHub.on("ImageFeedInfoDone", function (message) {
                $rootScope.$emit("ImageFeedInfoDone", message);
            });

            //Publishing an event to our views when receiving a DownloadProgression message from server
            _private.feedHub.on("DownloadProgression", function (token, count, completing, completed) {
                $rootScope.$emit("DownloadProgression", token, count, completing, completed);
            });

            _private.feedConnection.error(function (error) {
                _private.checkErrorTokenExpired(error, $injector, $window);
            });

            _private.feedConnection.reconnecting(function () {
                $rootScope.$emit("Reconnecting", {});
            });

            _private.feedConnection.reconnected(function () {
                // need to wait a second otherwise the invoke will fail silently...
                setTimeout(function () {
                    $rootScope.$emit("Connected", {});
                    invokeStartFeed(_private.feedRequest);
                }, 1000);
            });

            _private.feedRequest = feedRequest; // to be used in startConnection
            _private.feedConnection.disconnected(function () {
                if (!_private.stopped && !_private.stopping) {
                    $rootScope.$emit("Reconnecting", {});
                    $rootScope.$emit("LostFeedConnection", {});
                    setTimeout(function() {
                        startConnection(_private.feedConnection); // keep connected when timeout occurs
                    }, 5000);
                }
            });

            startConnection(_private.feedConnection);
        }

        function initializeIoHub() {
            if (_private.ioHub != null)
                return; //already connected!!

            _private.ioConnection = createHubConnection(backendServerUrl);
            _private.ioConnection.qs = { Bearer: $window.sessionStorage.accessToken };
            _private.ioHub = _private.ioConnection.createHubProxy("IoHub");

            _private.stopped = false;

            _private.ioHub.on("IoStates", function (message) {
                $rootScope.$emit("IoStates", message);
            });

            _private.ioConnection.error(function (error) {
                _private.checkErrorTokenExpired(error, $injector, $window);
            });

            _private.ioConnection.reconnecting(function () {
                $rootScope.$emit("Reconnecting", {});
            });

            _private.ioConnection.reconnected(function () {
                _private.ioConnection.start({ jsonp: true, transport: generalTransport })
                    .done(function () {
                        $rootScope.$emit("Connected", {});
                    })
                    .fail(function (error) {
                        _private.checkErrorTokenExpired(error, $injector, $window);
                    });
            });

            _private.ioConnection.disconnected(function () {
                if (!_private.stopped && !_private.stopping) {
                    setTimeout(function () {
                        if (_private.ioConnection)
                            _private.ioConnection.start({ jsonp: true, transport: generalTransport })
                                .fail(function (error) {
                                    _private.checkErrorTokenExpired(error, $injector, $window);
                                });
                    }, 5000); // keep connected when timeout occurs
                }
            });

            //Starting connection
            _private.ioConnection.start({ jsonp: true, transport: generalTransport })
                .fail(function (error) {
                    _private.checkErrorTokenExpired(error, $injector, $window);
                });
        }

        function initializeLastActivitiesHub() {
            if (_private.lastActivitiesHub != null)
                return; //already connected!!

            _private.lastActivitiesConnection = createHubConnection(backendServerUrl);
            _private.lastActivitiesConnection.qs = { Bearer: $window.sessionStorage.accessToken };
            _private.lastActivitiesHub = _private.lastActivitiesConnection.createHubProxy("LastActivitiesHub");

            _private.stopped = false;

            _private.lastActivitiesHub.on("LastActivities", function (message) {
                $rootScope.$emit("LastActivities", message);
            });

            _private.lastActivitiesHub.on("UpdateInternetConnectivityStatus", function(message) {
                $rootScope.$emit("UpdateInternetConnectivityStatus", message);
            });

            _private.lastActivitiesConnection.error(function (error) {
                _private.checkErrorTokenExpired(error, $injector, $window);
            });

            _private.lastActivitiesConnection.reconnecting(function () {
                $rootScope.$emit("Reconnecting", {});
            });

            _private.lastActivitiesConnection.reconnected(function () {
                _private.lastActivitiesConnection.start({ jsonp: true, transport: generalTransport })
                    .done(function () {
                        $rootScope.$emit("Connected", {});
                    });
            });

            _private.lastActivitiesConnection.disconnected(function () {
                if (!_private.stopped && !_private.stopping) {
                    setTimeout(function () {
                        if (_private.lastActivitiesConnection)
                            _private.lastActivitiesConnection.start({ jsonp: true, transport: generalTransport })
                                .fail(function (error) {
                                    _private.checkErrorTokenExpired(error, $injector, $window);
                                    
                                }); // keep connected when timeout occurs
                    }, 5000);
                }
            });

            //Starting connection
            _private.lastActivitiesConnection.start({ jsonp: true, transport: generalTransport })
                .fail(function (error) {
                    _private.checkErrorTokenExpired(error, $injector, $window);
                });
        }

        function initializeLogHub() {
            if (_private.logHub != null){
               var deferred = $q.defer();
               deferred.resolve();
               return deferred.promise;
            }
                
            _private.logConnection = createHubConnection(backendLoggerUrl);
            _private.logConnection.qs = { Bearer: $window.sessionStorage.accessToken };
            _private.logHub = _private.logConnection.createHubProxy("LogHub");

            _private.stopped = false;

            //Publishing an event to our views when receiving a addEntry message from server
            _private.logHub.on("addEntry", function (message) {
                message.Timestamp = moment(message.Timestamp).format(timeFormat);
                $rootScope.$emit("addEntry", message);
            });

            _private.logConnection.disconnected(function () {
                if (!_private.stopped && !_private.stopping) {
                    $rootScope.$emit("LoggerServiceConnectionFailure", {});                    
                    setTimeout(function () {
                        if (_private.logConnection) {
                            _private.logConnection.start()
                                .done(function() {
                                    $rootScope.$emit("LoggerServiceConnectionSuccess", {});
                                })
                                .fail(function (error) {
                                    _private.checkErrorTokenExpired(error, $injector, $window);
                                });
                        }
                    }, 5000);                    
                }
            });

            //Starting connection
            return _private.logConnection.start()
                .done(function () { $rootScope.$emit("LoggerServiceConnectionSuccess", {}); })
                .fail(function (error) {
                    _private.checkErrorTokenExpired(error, $injector, $window);
                });
        }

        function initializeSpeedEstimationHub() {
            if (_private.speedEstimationHub != null)
                return; //already connected!!

            _private.speedEstimationConnection = createHubConnection(backendServerUrl);
            _private.speedEstimationConnection.qs = { Bearer: $window.sessionStorage.accessToken };
            _private.speedEstimationHub = _private.speedEstimationConnection.createHubProxy("SpeedEstimationHub");

            _private.stopped = false;

            _private.speedEstimationHub.on("CalibrationReceived", function (message) {
                $rootScope.$emit("CalibrationReceived", message);
            });

            _private.speedEstimationConnection.error(function (error) {
                _private.checkErrorTokenExpired(error, $injector, $window);
            });

            _private.speedEstimationConnection.reconnecting(function () {
                $rootScope.$emit("Reconnecting", {});
            });

            _private.speedEstimationConnection.reconnected(function () {
                _private.speedEstimationConnection.start({ jsonp: true, transport: generalTransport })
                    .done(function () {
                        $rootScope.$emit("Connected", {});
                    })
                    .fail(function (error) {
                        _private.checkErrorTokenExpired(error, $injector, $window);
                    });
            });

            _private.speedEstimationConnection.disconnected(function () {
                if (!_private.stopped && !_private.stopping) {
                    setTimeout(function () {
                        if (_private.speedEstimationConnection)
                            _private.speedEstimationConnection.start({ jsonp: true, transport: generalTransport })
                                .fail(function (error) {
                                    _private.checkErrorTokenExpired(error, $injector, $window);
                                });
                    }, 5000); // keep connected when timeout occurs
                }
            });

            //Starting connection
            _private.speedEstimationConnection.start({ jsonp: true, transport: generalTransport })
                .done(function () {
                    $rootScope.$emit("Connected", {});
                })
                .fail(function (error) {
                    _private.checkErrorTokenExpired(error, $injector, $window);
                });
        }
	
	    function initializeVehicleDetectionCalibrationHub() {
	        if (_private.vehicleDetectionCalibrationHub != null)
                return; //already connected!!

            _private.vehicleDetectionCalibrationHubConnection = createHubConnection(backendServerUrl);
            _private.vehicleDetectionCalibrationHubConnection.qs = { Bearer: $window.sessionStorage.accessToken };
            _private.vehicleDetectionCalibrationHub = _private.vehicleDetectionCalibrationHubConnection.createHubProxy("VehicleDetectionCalibrationHub");

            _private.stopped = false;

            _private.vehicleDetectionCalibrationHub.on("Status", function (message) {
                $rootScope.$emit("Status", message);
            });

            _private.vehicleDetectionCalibrationHubConnection.error(function (error) {
                _private.checkErrorTokenExpired(error, $injector, $window);
            });

            _private.vehicleDetectionCalibrationHubConnection.reconnecting(function () {
                $rootScope.$emit("Reconnecting", {});
            });

            _private.vehicleDetectionCalibrationHubConnection.reconnected(function () {
                _private.vehicleDetectionCalibrationHubConnection.start({ jsonp: true, transport: generalTransport })
                    .done(function () {
                        $rootScope.$emit("Connected", {});
                    })
                    .fail(function (error) {
                        _private.checkErrorTokenExpired(error, $injector, $window);
                    });
            });

            _private.vehicleDetectionCalibrationHubConnection.disconnected(function () {
                if (!_private.stopped && !_private.stopping) {
                    setTimeout(function () {
                        if (_private.vehicleDetectionCalibrationHubConnection)
                            _private.vehicleDetectionCalibrationHubConnection.start({ jsonp: true, transport: generalTransport })
                                .fail(function (error) {
                                    _private.checkErrorTokenExpired(error, $injector, $window);
                                });
                    }, 5000); // keep connected when timeout occurs
                }
            });

            //Starting connection
            _private.vehicleDetectionCalibrationHubConnection.start({ jsonp: true, transport: generalTransport })
                .fail(function (error) {
                    _private.checkErrorTokenExpired(error, $injector, $window);
                });
	    }

	    function initializeRecoveryHub() {
	        if (_private.recoveryHub != null) {
	            var deferred = $q.defer();
	            deferred.resolve();
	            return deferred.promise;
	        }

            _private.recoveryHubConnection = createHubConnection(backendServerUrl);
	        _private.recoveryHubConnection.qs = { Bearer: $window.sessionStorage.recoveryToken };
	        _private.recoveryHub = _private.recoveryHubConnection.createHubProxy("RecoveryHub");

	        _private.stopped = false;

	        _private.recoveryHub.on("RecoveryBestPlateReceived", function (message) {
	            $rootScope.$emit("RecoveryBestPlateReceived", message);
	        });

	        _private.recoveryHub.on("RecoveryResetButtonTriggered", function (message) {
	            $rootScope.$emit("RecoveryResetButtonTriggered", message);
	        });

	        _private.recoveryHub.on("WaitingTimeRemaining", function (message) {
	            $rootScope.$emit("WaitingTimeRemaining", message);
	        });

	        _private.recoveryHub.on("TimerExpired", function (message) {
	            $rootScope.$emit("TimerExpired", message);
	        });

	        _private.recoveryHubConnection.error(function (error) {
                _private.checkErrorTokenExpired(error, $injector, $window);
	        });

	        _private.recoveryHubConnection.reconnecting(function () {
	            $rootScope.$emit("Reconnecting", {});
	        });

	        _private.recoveryHubConnection.reconnected(function () {
	            _private.recoveryHubConnection.start({ jsonp: true, transport: generalTransport })
                    .done(function () {
                        $rootScope.$emit("Connected", {});
                    })
                    .fail(function (error) {
                        _private.checkErrorTokenExpired(error, $injector, $window);
                    });
	        });

	        _private.recoveryHubConnection.disconnected(function () {
	            if (!_private.stopped && !_private.stopping) {
	                setTimeout(function () {
	                    if (_private.recoveryHubConnection)
                            _private.recoveryHubConnection.start({ jsonp: true, transport: generalTransport })
                                .fail(function (error) {
                                    _private.checkErrorTokenExpired(error, $injector, $window);
                                });
                    }, 5000); // keep connected when timeout occurs
	            }
	        });

	        //Starting connection
	        return _private.recoveryHubConnection.start({ jsonp: true, transport: generalTransport })
                .done(function () {
                    $rootScope.$emit("RecoveryHubConnected", {});
                })
                .fail(function (error) {
                    _private.checkErrorTokenExpired(error, $injector, $window);
                });
	    }

        function startConnection(connection) {
            if (!connection)
                return;

            $rootScope.$emit("Reconnecting", {});

            //Starting connection
            connection.start({ jsonp: true, transport: generalTransport })
                .done(function () {
                    if (connection) {
                        $rootScope.$emit("Connected", {});
                        //console.log("Connected, transport = " + connection.transport.name);
                    }
                    if (_private.feedRequest.feedTypes > 0)
                        invokeStartFeed(_private.feedRequest);
                })
                .fail(function (error) {
                    _private.checkErrorTokenExpired(error, $injector, $window);
                });
        }

        function stopFeed() {
            if (_private.feedHub == null)
                return; //not connected!!

            var infos = {};
            _private.feedHub.invoke('Subscribe', infos, false);
        }

        function startFeed(hasContext) {
            var request = new FeedRequestOptions(3, true, hasContext);
            invokeStartFeed(request);
        }

        function reduceFeed(hasContext) {
            var request = new FeedRequestOptions(2, true, hasContext);
            invokeStartFeed(request);
        }
        
        function invokeStartFeed(feedRequest) {

            // If connection was closed, reconnect
            if (_private.feedHub == null)
                initializeFeedHub(feedRequest);

            var infos = {};
            if (feedRequest.isLpr)
                infos["Lpr Camera"] = feedRequest.feedTypes; // 1 means read images, 2 means best read images

            if (feedRequest.isContext)
                infos["Context Camera"] = feedRequest.feedTypes; // 1 means live images, 2 means best read images
            _private.feedHub.invoke('Subscribe', infos, false);
        }

        function isConnected() {
            return _private.feedHub !== null;
        }

        function resetCounter() {
            _private.feedHub.invoke('ResetCounter');
        }

        function getSources() {
            return _private.logHub.invoke('GetSources');            
        }

        function getLogEntries(searchCriteria) {
            return _private.logHub.invoke('GetLogEntries', searchCriteria);
        }

        function getXMLLogEntries(searchCriteria) {
            return _private.logHub.invoke('GetXMLLogEntries', searchCriteria);
        }

        function enableSource(sourceName) {
            return _private.logHub.invoke('EnableSource', sourceName);
        }

        function disableSource(sourceName) {
            return _private.logHub.invoke('DisableSource', sourceName);
        }

        // Constructor of object FeedRequestOptions
        function FeedRequestOptions(feedtypes, isLpr, isContext) {
            this.feedTypes = feedtypes;
            this.isLpr = isLpr;
            this.isContext = isContext;
        }

        function base64ToArrayBuffer(base64) {
            var binaryString = window.atob(base64);
            var len = binaryString.length;
            var bytes = new Uint8Array(len);
            for (var i = 0; i < len; i++) {
                bytes[i] = binaryString.charCodeAt(i);
            }
            return bytes.buffer;
        }

        function stopConnections() {

            if (_private.stopped || _private.stopping)
                return;

            _private.stopping = true;

            // stops all signalr connections
            if (_private.perfConnection != null) {
                _private.perfConnection.stop(false, true);
                _private.perfConnection = null;
                _private.perfHub = null;
            }
            if (_private.ioConnection != null) {
                _private.ioConnection.stop(false, true);
                _private.ioConnection = null;
                _private.ioHub = null;
            }
            if (_private.lastActivitiesConnection != null) {
                _private.lastActivitiesConnection.stop(false, true);
                _private.lastActivitiesConnection = null;
                _private.lastActivitiesHub = null;
            }
            if (_private.feedConnection != null) {
                _private.feedConnection.stop(false, true);
                _private.feedConnection = null;
                _private.feedHub = null;
            }
            if (_private.logConnection != null) {
                _private.logConnection.stop(false, true);
                _private.logConnection = null;
                _private.logHub = null;
            }
            if (_private.speedEstimationConnection != null) {
                _private.speedEstimationConnection.stop(false, true);
                _private.speedEstimationConnection = null;
                _private.speedEstimationHub = null;
            }
	        if (_private.vehicleDetectionCalibrationHubConnection != null) {
	            _private.vehicleDetectionCalibrationHubConnection.stop(false, true);
	            _private.vehicleDetectionCalibrationHubConnection = null;
                _private.vehicleDetectionCalibrationHub = null;
	        }

	        if (_private.recoveryHubConnection != null) {
	            _private.recoveryHubConnection.stop(false, true);
	            _private.recoveryHubConnection = null;
	            _private.recoveryHub = null;
	        }

            _private.stopped = true;
            _private.stopping = false;
        }

        function pauseLogs() {
            return _private.logHub.invoke('Pause');
        }
        
        function resumeLogs() {
           return _private.logHub.invoke('Resume');
        }

        function startTimer() {
            return _private.recoveryHub.invoke('StartTimer');
        }

        function getTimer() {
            return _private.recoveryHub.invoke('GetTimer');
        }
    }
})();