﻿"use strict";

describe("Analytics controller", function () {
    var controller, portalService, $rootScope, $timeout, scope, $state, stateSpy, checkChangeStateFactory, $uibModal, $translate, sandbox;
    var backendServerUrl = "http://localhost:80";
    var userConfig = { data: {
            ReadModeConfiguration: {
                SingleReadTriggers: []
            },
            EstimateVehicleSpeed: true,
            CameraOrientation: 0
        }
    };
    var lprContexts = { data: "asdasdsad" };
    var mockProperties = { data: false };

    beforeEach(function () {        
        module("portalApp.services", function ($provide) {
            $provide.value("backendServerUrl", backendServerUrl);
        });
        module("portalApp.controllers");
        module("ui.bootstrap"); // for $uibModal
        module("pascalprecht.translate"); // for $translate

        sandbox = sinon.sandbox.create();

        inject(function ($injector, _$translate_) {
            portalService = $injector.get("portalService");
            checkChangeStateFactory = $injector.get("checkChangeStateFactory");
            $rootScope = $injector.get("$rootScope");
            scope = $rootScope.$new;
            scope.$on = function() {};
            $timeout = $injector.get("$timeout");            
            $state = $injector.get("$state");
            $uibModal = $injector.get("$uibModal");
            $translate = _$translate_;
            stateSpy = sinon.stub($state, "go");
        });

    });

    afterEach(function () {
        sandbox.restore();
    });


    it("should be created successfully", function () {
        inject(function($injector, $rootScope, $controller) {
            sandbox.stub(checkChangeStateFactory, "checkFormOnStateChange");
            var deferred = $injector.get("$q").defer();
            deferred.resolve(userConfig);
            var d = $injector.get("$q").defer();
            d.resolve(lprContexts);
            sandbox.stub(portalService, "getUserConfig").returns(deferred.promise);
            sandbox.stub(portalService, "getLprContexts").returns(d.promise);
            var defer = $injector.get("$q").defer();
            defer.resolve(mockProperties);
            sandbox.stub(portalService, "getProperties").returns(defer.promise);
            controller = $controller("AnalyticsController", { portalService: portalService,$rootScope: $rootScope, $scope: scope, $timeout: $timeout });
            $rootScope.$apply();
        });

        expect(controller).to.be.an('object');
        expect(controller.title).to.equal("AnalyticsController");
    });

    it("closeAlert should set show flag to false", function () {
        inject(function ($injector, $rootScope, $controller) {
            sandbox.stub(checkChangeStateFactory, "checkFormOnStateChange");
            var deferred = $injector.get("$q").defer();
            deferred.resolve(userConfig);
            var d = $injector.get("$q").defer();
            d.resolve(lprContexts);
            sandbox.stub(portalService, "getUserConfig").returns(deferred.promise);
            sandbox.stub(portalService, "getLprContexts").returns(d.promise);
            var defer = $injector.get("$q").defer();
            defer.resolve(mockProperties);
            sandbox.stub(portalService, "getProperties").returns(defer.promise);
            controller = $controller("AnalyticsController", { portalService: portalService, $scope: scope, $timeout: $timeout});
            $rootScope.$apply();
        });

        controller.closeAlert();
        expect(controller.show).to.be.false;
    });

    it("cancel should set userConfig back to its original content and call form.$setPristine", function () {
        var spy;
        inject(function ($injector, $rootScope, $controller) {
            sandbox.stub(checkChangeStateFactory, "checkFormOnStateChange");
            var deferred = $injector.get("$q").defer();
            deferred.resolve(userConfig);
            var d = $injector.get("$q").defer();
            d.resolve(lprContexts);
            sandbox.stub(portalService, "getUserConfig").returns(deferred.promise);
            sandbox.stub(portalService, "getLprContexts").returns(d.promise);
            var defer = $injector.get("$q").defer();
            defer.resolve(mockProperties);
            sandbox.stub(portalService, "getProperties").returns(defer.promise);
            controller = $controller("AnalyticsController", { portalService: portalService, $scope: scope, $timeout: $timeout });
            $rootScope.$apply();
        });

        controller.userConfig = { fake: "asdhasdhjashdjkhas", ReadModeConfiguration: { SingleReadTriggers: [] } };
        controller.form = { $valid: true, $setPristine: function () { } };
        spy = sandbox.stub(controller.form, "$setPristine");
        controller.cancel();
        expect(angular.equals(controller.userConfig, userConfig));
        sinon.assert.calledOnce(spy);
    });

    it("updateContext should set lprContextName to new value and call form.$setDirty", function () {
        var spy;
        inject(function ($injector, $rootScope, $controller) {
            userConfig = { data: { LprContextName: "Patate", ReadModeConfiguration: { SingleReadTriggers: [] } } };
            sandbox.stub(checkChangeStateFactory, "checkFormOnStateChange");
            var deferred = $injector.get("$q").defer();
            deferred.resolve(userConfig);
            var d = $injector.get("$q").defer();
            d.resolve(lprContexts);
            sandbox.stub(portalService, "getUserConfig").returns(deferred.promise);
            sandbox.stub(portalService, "getLprContexts").returns(d.promise);
            var defer = $injector.get("$q").defer();
            defer.resolve(mockProperties);
            sandbox.stub(portalService, "getProperties").returns(defer.promise);
            controller = $controller("AnalyticsController", { portalService: portalService, $scope: scope, $timeout: $timeout });
            $rootScope.$apply();
        });

        expect(controller).to.be.an('object');
        expect(controller.userConfig).to.be.an('object');
        expect(controller.userConfig.LprContextName).to.equal("Patate");

        controller.form = { $valid: true, $setDirty: function () { } };
        spy = sandbox.stub(controller.form, "$setDirty");

        controller.updateContext("Poubelle");
        expect(controller.userConfig.LprContextName).to.equal("Poubelle");
        sinon.assert.calledOnce(spy);
    });

    it("save should call portalService.setUserConfig then display a sticky alert and reset the form when save is a success", function () {
        var spy;
        inject(function ($injector, $rootScope, $controller) {
            sandbox.stub(checkChangeStateFactory, "checkFormOnStateChange");
            var deferred = $injector.get("$q").defer();
            deferred.resolve();
            sandbox.stub(portalService, "setUserConfig").returns(deferred.promise);
            var def = $injector.get("$q").defer();
            def.resolve(userConfig);
            var d = $injector.get("$q").defer();
            d.resolve(lprContexts);
            sandbox.stub(portalService, "getUserConfig").returns(def.promise);
            sandbox.stub(portalService, "getLprContexts").returns(d.promise);
            var defer = $injector.get("$q").defer();
            defer.resolve(mockProperties);
            sandbox.stub(portalService, "getProperties").returns(defer.promise);
            controller = $controller("AnalyticsController", { portalService: portalService, $scope: scope, $timeout: $timeout});
            $rootScope.$apply();
        });

        controller.form = { $valid: true, $setPristine: function () { } };
        spy = sandbox.stub(controller.form, "$setPristine");

        controller.save();
        $rootScope.$digest();
        expect(controller.show).to.be.true;
        $timeout.flush();
        expect(controller.show).to.be.false;
        expect(controller.alert.msg).to.equal('STE_SENTENCE_ConfigurationSavedSuccessfully');
        sinon.assert.calledOnce(spy);
    });

    it("save should call portalService.setUserConfig then display a sticky alert when save is a failure", function () {
        var spy;
        inject(function ($injector, $rootScope, $controller) {
            sandbox.stub(checkChangeStateFactory, "checkFormOnStateChange");
            var deferred = $injector.get("$q").defer();
            deferred.reject();
            sandbox.stub(portalService, "setUserConfig").returns(deferred.promise);
            var def = $injector.get("$q").defer();
            deferred.resolve(userConfig);
            var d = $injector.get("$q").defer();
            d.resolve(lprContexts);
            sandbox.stub(portalService, "getUserConfig").returns(def.promise);
            sandbox.stub(portalService, "getLprContexts").returns(d.promise);
            var defer = $injector.get("$q").defer();
            defer.resolve(mockProperties);
            sandbox.stub(portalService, "getProperties").returns(defer.promise);
            controller = $controller("AnalyticsController", { portalService: portalService, $scope: scope, $timeout: $timeout });
            $rootScope.$apply();
        });

        controller.form = { $valid: true, $setPristine: function () { } };
        spy = sandbox.stub(controller.form, "$setPristine");

        controller.save();
        $rootScope.$digest();
        expect(controller.show).to.be.true;
        $timeout.flush();
        expect(controller.show).to.be.false;
        expect(controller.alert.msg).to.equal('STE_SENTENCE_CouldNotSaveConfiguration');
        sinon.assert.notCalled(spy);
    });

    it("getCameraOrientationApproachingFlow called defined index 1-3 should return approaching flow direction label", function () {
        inject(function ($injector, $rootScope, $controller) {
            //userConfig = { data: { LprContextName: "Patate", ReadModeConfiguration: { SingleReadTriggers: [] } } };
            sandbox.stub(checkChangeStateFactory, "checkFormOnStateChange");
            var deferred = $injector.get("$q").defer();
            deferred.resolve(userConfig);
            var d = $injector.get("$q").defer();
            d.resolve(lprContexts);
            sandbox.stub(portalService, "getUserConfig").returns(deferred.promise);
            sandbox.stub(portalService, "getLprContexts").returns(d.promise);
            var defer = $injector.get("$q").defer();
            defer.resolve(mockProperties);
            sandbox.stub(portalService, "getProperties").returns(defer.promise);
            controller = $controller("AnalyticsController", { portalService: portalService, $scope: scope, $timeout: $timeout });
            $rootScope.$apply();
        });

        expect(controller).to.be.an('object');
        expect(controller.title).to.equal("AnalyticsController");
        var name = controller.getCameraOrientationApproachingFlow(1);
        expect(name).to.equal('STE_LABEL_CameraOrientation_FlowRightToLeft');
        name = controller.getCameraOrientationApproachingFlow(2);
        expect(name).to.equal('STE_LABEL_CameraOrientation_FlowTopToBottom');
        name = controller.getCameraOrientationApproachingFlow(3);
        expect(name).to.equal('STE_LABEL_CameraOrientation_FlowLeftToRight');
    });

    it("getCameraOrientationMovingAwayFlow called defined index 1-3 should return moving away flow direction label", function () {
        inject(function ($injector, $rootScope, $controller) {
            //userConfig = { data: { LprContextName: "Patate", ReadModeConfiguration: { SingleReadTriggers: [] } } };
            sandbox.stub(checkChangeStateFactory, "checkFormOnStateChange");
            var deferred = $injector.get("$q").defer();
            deferred.resolve(userConfig);
            var d = $injector.get("$q").defer();
            d.resolve(lprContexts);
            sandbox.stub(portalService, "getUserConfig").returns(deferred.promise);
            sandbox.stub(portalService, "getLprContexts").returns(d.promise);
            var defer = $injector.get("$q").defer();
            defer.resolve(mockProperties);
            sandbox.stub(portalService, "getProperties").returns(defer.promise);
            controller = $controller("AnalyticsController", { portalService: portalService, $scope: scope, $timeout: $timeout });
            $rootScope.$apply();
        });

        expect(controller).to.be.an('object');
        expect(controller.title).to.equal("AnalyticsController");
        var name = controller.getCameraOrientationMovingAwayFlow(1);
        expect(name).to.equal('STE_LABEL_CameraOrientation_FlowLeftToRight');
        name = controller.getCameraOrientationMovingAwayFlow(2);
        expect(name).to.equal('STE_LABEL_CameraOrientation_FlowBottomToTop');
        name = controller.getCameraOrientationMovingAwayFlow(3);
        expect(name).to.equal('STE_LABEL_CameraOrientation_FlowRightToLeft');
    });

    it("getCameraOrientationApproaching or getCameraOrientationMovingAway called defined index 0 or 3+ should return camera orientation unknown", function () {
        inject(function ($injector, $rootScope, $controller) {
            //userConfig = { data: { LprContextName: "Patate", ReadModeConfiguration: { SingleReadTriggers: [] } } };
            sandbox.stub(checkChangeStateFactory, "checkFormOnStateChange");
            var deferred = $injector.get("$q").defer();
            deferred.resolve(userConfig);
            var d = $injector.get("$q").defer();
            d.resolve(lprContexts);
            sandbox.stub(portalService, "getUserConfig").returns(deferred.promise);
            sandbox.stub(portalService, "getLprContexts").returns(d.promise);
            var defer = $injector.get("$q").defer();
            defer.resolve(mockProperties);
            sandbox.stub(portalService, "getProperties").returns(defer.promise);
            controller = $controller("AnalyticsController", { portalService: portalService, $scope: scope, $timeout: $timeout });
            $rootScope.$apply();
        });

        //TODO drouin update test
        expect(controller).to.be.an("object");
        expect(controller.title).to.equal("AnalyticsController");
        var name = controller.getCameraOrientationApproachingFlow(0);
        expect(name).to.equal("STE_LABEL_CameraOrientation_Unknown");
        name = controller.getCameraOrientationApproachingFlow(4);
        expect(name).to.equal("STE_LABEL_CameraOrientation_Unknown");
        name = controller.getCameraOrientationApproachingFlow(-1);
        expect(name).to.equal("STE_LABEL_CameraOrientation_Unknown");
        name = controller.getCameraOrientationApproachingFlow();
        expect(name).to.equal("STE_LABEL_CameraOrientation_Unknown");
        name = controller.getCameraOrientationApproachingFlow(undefined);
        expect(name).to.equal("STE_LABEL_CameraOrientation_Unknown");
        name = controller.getCameraOrientationApproachingFlow(null);
        expect(name).to.equal("STE_LABEL_CameraOrientation_Unknown");

        name = controller.getCameraOrientationMovingAwayFlow(0);
        expect(name).to.equal("");
        name = controller.getCameraOrientationMovingAwayFlow(4);
        expect(name).to.equal("");
        name = controller.getCameraOrientationMovingAwayFlow(-1);
        expect(name).to.equal("");
        name = controller.getCameraOrientationMovingAwayFlow();
        expect(name).to.equal("");
        name = controller.getCameraOrientationMovingAwayFlow(undefined);
        expect(name).to.equal("");
        name = controller.getCameraOrientationMovingAwayFlow(null);
        expect(name).to.equal("");
    });

    it("updateCameraOrientation should set UserConfig.CameraOrientation to new value and call form.$setDirty", function () {
        var spy;
        inject(function ($injector, $rootScope, $controller) {
            userConfig = { data: { LprContextName: "Patate2", ReadModeConfiguration: { SingleReadTriggers: [] }, CameraOrientation: 0} };
            sandbox.stub(checkChangeStateFactory, "checkFormOnStateChange");
            var deferred = $injector.get("$q").defer();
            deferred.resolve(userConfig);
            var d = $injector.get("$q").defer();
            d.resolve(lprContexts);
            sandbox.stub(portalService, "getUserConfig").returns(deferred.promise);
            sandbox.stub(portalService, "getLprContexts").returns(d.promise);
            var defer = $injector.get("$q").defer();
            defer.resolve(mockProperties);
            sandbox.stub(portalService, "getProperties").returns(defer.promise);
            controller = $controller("AnalyticsController", { portalService: portalService, $scope: scope, $timeout: $timeout });
            $rootScope.$apply();
        });

        expect(controller).to.be.an('object');
        expect(controller.title).to.equal("AnalyticsController");
        expect(controller.userConfig.CameraOrientation).to.equal(0);

        controller.form = { $valid: true, $setDirty: function () { } };
        spy = sandbox.stub(controller.form, "$setDirty");

        controller.updateCameraOrientation({id:1});
        expect(controller.userConfig.CameraOrientation).to.equal(1);
        sinon.assert.calledOnce(spy);

        controller.updateCameraOrientation({ id: 3 });
        expect(controller.userConfig.CameraOrientation).to.equal(3);
        sinon.assert.calledTwice(spy);

        // Test an unexisting id this shouldn't update
        var before = controller.userConfig.CameraOrientation;
        controller.updateCameraOrientation({ id: 5 });
        expect(controller.userConfig.CameraOrientation).to.equal(before);
        sinon.assert.calledTwice(spy);
    });
});