﻿"use strict";

describe("Extension controller", function () {
    var controller, portalService, $rootScope, $timeout, scope, $state, sandbox, stateSpy, checkChangeStateFactory, $translate;
    var backendServerUrl = "http://localhost:8080";
    var backendLoggerUrl = "http://localhost:88";
    var userConfig = { data: 51353541541 };
    var extensions = { data: "asdasdsad" };
    var saveSuccess = 'STE_SENTENCE_ConfigurationSavedSuccessfully';
    var saveFailure = 'STE_SENTENCE_CouldNotSaveConfiguration';
    var testFailure = 'STE_SENTENCE_ConnectionTestFailed';

    beforeEach(function () {        
        module("portalApp.services", function ($provide) {
            $provide.value("backendServerUrl", backendServerUrl);
            $provide.value("backendLoggerUrl", backendLoggerUrl);
        });
        module("portalApp.controllers");
        module("pascalprecht.translate"); // for $translate

        sandbox = sinon.sandbox.create();

        inject(function ($injector, _$translate_) {
            portalService = $injector.get("portalService");
            checkChangeStateFactory = $injector.get("checkChangeStateFactory");
            $rootScope = $injector.get("$rootScope");
            scope = $rootScope.$new;
            scope.$on = function () { };
            $timeout = $injector.get("$timeout");            
            $state = $injector.get("$state");
            stateSpy = sinon.stub($state, "go");
            $translate = _$translate_;
        });

    });

    afterEach(function () {
        sandbox.restore();
    });

    it("should be created successfully", function () {
        inject(function($injector, $rootScope, $controller) {
            sandbox.stub(checkChangeStateFactory, "checkFormOnStateChange");
            var deferred = $injector.get("$q").defer();
            deferred.resolve(userConfig);
            sandbox.stub(portalService, "getUserConfig").returns(deferred.promise);
            var d = $injector.get("$q").defer();
            d.resolve(extensions);
            sandbox.stub(portalService, "getExtensions").returns(d.promise);
            controller = $controller("ExtensionController", { portalService: portalService, $scope: scope, $timeout: $timeout, $state: $state });
            $rootScope.$apply();
        });

        expect(controller).to.be.an('object');
        expect(controller.title).to.equal("ExtensionController");
    });

    it("testFtp should call portalService.testFtp then display a sticky alert when test is a success", function () {
        inject(function ($injector, $rootScope, $controller) {
            sandbox.stub(checkChangeStateFactory, "checkFormOnStateChange");
            var deferred = $injector.get("$q").defer();
            var response =  { data: true }
            deferred.resolve(response);
            sandbox.stub(portalService, "testFtp").returns(deferred.promise);
            var defer = $injector.get("$q").defer();
            defer.resolve(userConfig);
            sandbox.stub(portalService, "getUserConfig").returns(defer.promise);
            var d = $injector.get("$q").defer();
            d.resolve(extensions);
            sandbox.stub(portalService, "getExtensions").returns(d.promise);
            controller = $controller("ExtensionController", { portalService: portalService, $scope: scope, $timeout: $timeout, $state: $state });
            $rootScope.$apply();
        });

        controller.testFtp();
        $rootScope.$digest();
        expect(controller.show).to.be.true;
        $timeout.flush();
        expect(controller.show).to.be.false;
        expect(controller.alert.msg).to.equal('STE_SENTENCE_ConnectionTestSucceeded');
    });

    it("testFtp should call portalService.testFtp then display a sticky alert when test is a failure", function () {
        inject(function ($injector, $rootScope, $controller) {
            sandbox.stub(checkChangeStateFactory, "checkFormOnStateChange");
            var deferred = $injector.get("$q").defer();
            var response = { data: false }
            deferred.resolve(response);
            sandbox.stub(portalService, "testFtp").returns(deferred.promise);
            var defer = $injector.get("$q").defer();
            defer.resolve(userConfig);
            sandbox.stub(portalService, "getUserConfig").returns(defer.promise);
            var d = $injector.get("$q").defer();
            d.resolve(extensions);
            sandbox.stub(portalService, "getExtensions").returns(d.promise);
            controller = $controller("ExtensionController", { portalService: portalService, $scope: scope, $timeout: $timeout, $state: $state });
            $rootScope.$apply();
        });

        controller.testFtp();
        $rootScope.$digest();
        expect(controller.show).to.be.true;
        $timeout.flush();
        expect(controller.show).to.be.false;
        expect(controller.alert.msg).to.equal(testFailure);
    });    

    it("updateExtension should set selectedExtension to None, then call form.$setDirty and switch extension", function () {
        var spyDirty, spySwitch;
        inject(function ($injector, $rootScope, $controller) {
            userConfig = { data: { dummy: 51353541541, CurrentExtension: "nothing" } };
            extensions = { data: [{Name: "nothing", Guid: "nothing"}] };
            sandbox.stub(checkChangeStateFactory, "checkFormOnStateChange");
            var defer = $injector.get("$q").defer();
            defer.resolve(userConfig);
            sandbox.stub(portalService, "getUserConfig").returns(defer.promise);
            var d = $injector.get("$q").defer();
            d.resolve(extensions);
            sandbox.stub(portalService, "getExtensions").returns(d.promise);
            controller = $controller("ExtensionController", { portalService: portalService, $scope: scope, $timeout: $timeout, $state: $state });
            $rootScope.$apply();

            spySwitch = sandbox.stub(controller, "switchStateOnExtension");
        });

        controller.form = { $valid: true, $setDirty: function () { } };
        spyDirty = sinon.stub(controller.form, "$setDirty");
        expect(controller.selectedExtension.Name).to.equal("nothing");
        controller.updateExtension({ Name: "None" });
        expect(controller.selectedExtension.Name).to.equal("None");
        sinon.assert.calledOnce(spyDirty);
        sinon.assert.calledOnce(spySwitch);
    });

    it("switchStateOnExtension should call portalService.getFtpExtensionSettings then set proper state when selectedExtension is ftp", function () {
        inject(function ($injector, $rootScope, $controller) {
            userConfig = { data: { dummy: 51353541541, CurrentExtension: "123456789" } };
            extensions = { data: [{ Name: "None", Guid: "123456789" }] };
            sandbox.stub(checkChangeStateFactory, "checkFormOnStateChange");
            var deferred = $injector.get("$q").defer();
            var response = { data: "It worked!" }
            deferred.resolve(response);
            sandbox.stub(portalService, "getFtpExtensionSettings").returns(deferred.promise);
            var defer = $injector.get("$q").defer();
            defer.resolve(userConfig);
            sandbox.stub(portalService, "getUserConfig").returns(defer.promise);
            var d = $injector.get("$q").defer();
            d.resolve(extensions);
            sandbox.stub(portalService, "getExtensions").returns(d.promise);
            controller = $controller("ExtensionController", { portalService: portalService, $scope: scope, $timeout: $timeout, $state: $state });
            $rootScope.$apply();
        });

        controller.selectedExtension = { Name: "Ftp" };
        controller.switchStateOnExtension();
        $rootScope.$digest();
        expect(controller.ftp).to.equal("It worked!");
        assert(stateSpy.calledWith("main.config.extension.ftp"));
    });

    it("switchStateOnExtension should call portalService.getHttpExtensionSettings then set proper state when selectedExtension is http", function () {
        inject(function ($injector, $rootScope, $controller) {
            userConfig = { data: { dummy: 51353541541, CurrentExtension: "123456789" } };
            extensions = { data: [{ Name: "None", Guid: "123456789" }] };
            sandbox.stub(checkChangeStateFactory, "checkFormOnStateChange");
            var deferred = $injector.get("$q").defer();
            var response = { data: "It worked!" }
            deferred.resolve(response);
            sandbox.stub(portalService, "getHttpExtensionSettings").returns(deferred.promise);
            var defer = $injector.get("$q").defer();
            defer.resolve(userConfig);
            sandbox.stub(portalService, "getUserConfig").returns(defer.promise);
            var d = $injector.get("$q").defer();
            d.resolve(extensions);
            sandbox.stub(portalService, "getExtensions").returns(d.promise);
            controller = $controller("ExtensionController", { portalService: portalService, $scope: scope, $timeout: $timeout, $state: $state });
            $rootScope.$apply();
        });

        controller.selectedExtension = { Name: "Http" };
        controller.switchStateOnExtension();
        $rootScope.$digest();
        expect(controller.http).to.equal("It worked!");
        assert(stateSpy.calledWith("main.config.extension.http"));
    });

    it("switchStateOnExtension should call portalService.getPatrollerExtensionSettings then set proper state when selectedExtension is patroller", function () {
        inject(function ($injector, $rootScope, $controller) {
            userConfig = { data: { dummy: 51353541541, CurrentExtension: "123456789" } };
            extensions = { data: [{ Name: "None", Guid: "123456789" }] };
            sandbox.stub(checkChangeStateFactory, "checkFormOnStateChange");
            var deferred = $injector.get("$q").defer();
            var response = { data: "It worked!" }
            deferred.resolve(response);
            sandbox.stub(portalService, "getPatrollerExtensionSettings").returns(deferred.promise);
            var defer = $injector.get("$q").defer();
            defer.resolve(userConfig);
            sandbox.stub(portalService, "getUserConfig").returns(defer.promise);
            var d = $injector.get("$q").defer();
            d.resolve(extensions);
            sandbox.stub(portalService, "getExtensions").returns(d.promise);
            controller = $controller("ExtensionController", { portalService: portalService, $scope: scope, $timeout: $timeout, $state: $state });
            $rootScope.$apply();
        });

        controller.selectedExtension = { Name: "Patroller" };
        controller.switchStateOnExtension();
        $rootScope.$digest();
        expect(controller.patroller).to.equal("It worked!");
        assert(stateSpy.calledWith("main.config.extension.patroller"));
    });

    it("switchStateOnExtension should call portalService.getSecurityCenterExtensionSettings then set proper state when selectedExtension is patroller", function () {
        var response;
        inject(function ($injector, $rootScope, $controller) {
            userConfig = { data: { dummy: 51353541541, CurrentExtension: "123456789" } };
            extensions = { data: [{ Name: "None", Guid: "123456789" }] };
            sandbox.stub(checkChangeStateFactory, "checkFormOnStateChange");
            var deferred = $injector.get("$q").defer();
            response = { data: { blabla: "It worked!", VideoPort:80 } }
            deferred.resolve(response);
            sandbox.stub(portalService, "getSecurityCenterExtensionSettings").returns(deferred.promise);
            var defer = $injector.get("$q").defer();
            defer.resolve(userConfig);
            sandbox.stub(portalService, "getUserConfig").returns(defer.promise);
            var d = $injector.get("$q").defer();
            d.resolve(extensions);
            sandbox.stub(portalService, "getExtensions").returns(d.promise);
            controller = $controller("ExtensionController", { portalService: portalService, $scope: scope, $timeout: $timeout, $state: $state });
            $rootScope.$apply();
        });

        controller.selectedExtension = { Name: "SecurityCenter" };
        controller.switchStateOnExtension();
        $rootScope.$digest();
        expect(controller.sc).to.equal(response.data);
        assert(stateSpy.calledWith("main.config.extension.securitycenter"));
    });

    it("switchStateOnExtension should set state to main.config.extension when selectedExtension is none", function () {
        inject(function ($injector, $rootScope, $controller) {
            userConfig = { data: { dummy: 51353541541, CurrentExtension: "123456789" } };
            extensions = { data: [{ Name: "nothing", Guid: "123456789" }, { Name: "None", Guid: "6666" }] };
            sandbox.stub(checkChangeStateFactory, "checkFormOnStateChange");
            var defer = $injector.get("$q").defer();
            defer.resolve(userConfig);
            sandbox.stub(portalService, "getUserConfig").returns(defer.promise);
            var d = $injector.get("$q").defer();
            d.resolve(extensions);
            sandbox.stub(portalService, "getExtensions").returns(d.promise);
            controller = $controller("ExtensionController", { portalService: portalService, $scope: scope, $timeout: $timeout, $state: $state });
            $rootScope.$apply();
        });

        controller.selectedExtension = { Name: "None" };
        controller.switchStateOnExtension();
        $rootScope.$digest();
        assert(stateSpy.calledWith("main.config.extension"));
    });

    it("showStickyAlert should show a sticky alert", function () {
        inject(function ($injector, $rootScope, $controller) {
            extensions = { data: [{ Name: "nothing", Guid: "123456789" }, { Name: "None", Guid: "6666" }] };
            sandbox.stub(checkChangeStateFactory, "checkFormOnStateChange");
            var defer = $injector.get("$q").defer();
            defer.resolve(userConfig);
            sandbox.stub(portalService, "getUserConfig").returns(defer.promise);
            var d = $injector.get("$q").defer();
            d.resolve(extensions);
            sandbox.stub(portalService, "getExtensions").returns(d.promise);
            controller = $controller("ExtensionController", { portalService: portalService, $scope: scope, $timeout: $timeout, $state: $state });
            $rootScope.$apply();
        });

        controller.isSaving = true;
        controller.showStickyAlert('My Type', 'My Message');
        expect(controller.show).to.be.true;
        $timeout.flush();
        expect(controller.show).to.be.false;
    });

    it("resetForm reset the form to pristine then get the userConfig", function () {
        var spy;
        inject(function ($injector, $rootScope, $controller) {
            extensions = { data: [{ Name: "nothing", Guid: "123456789" }, { Name: "None", Guid: "6666" }] };
            sandbox.stub(checkChangeStateFactory, "checkFormOnStateChange");
            var defer = $injector.get("$q").defer();
            defer.resolve(userConfig);
            sandbox.stub(portalService, "getUserConfig").returns(defer.promise);
            var d = $injector.get("$q").defer();
            d.resolve(extensions);
            sandbox.stub(portalService, "getExtensions").returns(d.promise);
            controller = $controller("ExtensionController", { portalService: portalService, $scope: scope, $timeout: $timeout, $state: $state });
            $rootScope.$apply();
        });

        controller.form = { $valid: true, $setPristine: function () { } };
        spy = sandbox.stub(controller.form, "$setPristine");
        controller.resetForm();
        $rootScope.$digest();
        expect(controller.userConfig.CurrentExtension).to.equal("123456789");
        sinon.assert.calledOnce(spy);
    });

    it("cancel should set userConfig back to its original content and call form.$setPristine", function () {
        var spy, spySelect, spyUpdate;
        inject(function ($injector, $rootScope, $controller) {            
            sandbox.stub(checkChangeStateFactory, "checkFormOnStateChange");
            var defer = $injector.get("$q").defer();
            defer.resolve(userConfig);
            sandbox.stub(portalService, "getUserConfig").returns(defer.promise);
            var d = $injector.get("$q").defer();
            d.resolve(extensions);
            sandbox.stub(portalService, "getExtensions").returns(d.promise);
            controller = $controller("ExtensionController", { portalService: portalService, $scope: scope, $timeout: $timeout, $state: $state });
            $rootScope.$apply();

            spySelect = sandbox.stub(controller, "setSelectedExtension");
            spyUpdate = sandbox.stub(controller, "updateExtension");
        });

        controller.userConfig = { fake: "asdhasdhjashdjkhas" };
        controller.form = { $valid: true, $setPristine: function () { } };
        spy = sinon.stub(controller.form, "$setPristine");
        controller.cancel();
        expect(angular.equals(controller.userConfig, userConfig));
        sinon.assert.calledOnce(spy);
        sinon.assert.calledOnce(spySelect);
        sinon.assert.calledOnce(spyUpdate);
    });

    it("save with None as selectedExtension should call portalService.setUserConfig then display a sticky alert and reset the form when save is a success", function () {
        var spyPristine, spySticky;
        inject(function ($injector, $rootScope, $controller) {
            extensions = { data: [{ Name: "nothing", Guid: "123456789" }, { Name: "None", Guid: "6666" }] };
            userConfig = { data: { dummy: 51353541541, CurrentExtension: "123456789" } };
            var deferred = $injector.get("$q").defer();
            deferred.resolve();
            sandbox.stub(portalService, "setUserConfig").returns(deferred.promise);
            sandbox.stub(checkChangeStateFactory, "checkFormOnStateChange");
            var defer = $injector.get("$q").defer();
            defer.resolve(userConfig);
            sandbox.stub(portalService, "getUserConfig").returns(defer.promise);
            var d = $injector.get("$q").defer();
            d.resolve(extensions);
            sandbox.stub(portalService, "getExtensions").returns(d.promise);
            controller = $controller("ExtensionController", { portalService: portalService, $scope: scope, $timeout: $timeout, $state: $state });
            $rootScope.$apply();

            spySticky = sandbox.stub(controller, "showStickyAlert");
        });

        controller.form = { $valid: true, $setPristine: function () { } };
        spyPristine = sandbox.stub(controller.form, "$setPristine");
        controller.selectedExtension = { Name: "None", Guid: 123456789}
        controller.save();
        $rootScope.$digest();
        expect(controller.userConfig.CurrentExtension).to.equal(123456789);
        sinon.assert.calledOnce(spyPristine);
        sinon.assert.calledOnce(spySticky);
        assert(spySticky.calledWith('success', saveSuccess));
    });

    it("save with None as selectedExtension should call portalService.setUserConfig then display a sticky alert and reset the form when save is a failure", function () {
        var spyPristine, spySticky;
        inject(function ($injector, $rootScope, $controller) {
            extensions = { data: [{ Name: "nothing", Guid: "123456789" }, { Name: "None", Guid: "6666" }] };
            userConfig = { data: { dummy: 51353541541, CurrentExtension: "123456789" } };
            sandbox.stub(checkChangeStateFactory, "checkFormOnStateChange");
            var deferred = $injector.get("$q").defer();
            deferred.reject();
            sandbox.stub(portalService, "setUserConfig").returns(deferred.promise);
            scope.form = { $valid: true, $setPristine: function () { } };
            spyPristine = sandbox.stub(scope.form, "$setPristine");
            var defer = $injector.get("$q").defer();
            defer.resolve(userConfig);
            sandbox.stub(portalService, "getUserConfig").returns(defer.promise);
            var d = $injector.get("$q").defer();
            d.resolve(extensions);
            sandbox.stub(portalService, "getExtensions").returns(d.promise);
            controller = $controller("ExtensionController", { portalService: portalService, $scope: scope, $timeout: $timeout, $state: $state });
            $rootScope.$apply();

            spySticky = sandbox.stub(controller, "showStickyAlert");
        });

        controller.selectedExtension = { Name: "None", Guid: 123456789 }
        controller.save();
        $rootScope.$digest();
        expect(controller.userConfig.CurrentExtension).to.equal(123456789);
        sinon.assert.notCalled(spyPristine);
        sinon.assert.calledOnce(spySticky);
        assert(spySticky.calledWith('danger', saveFailure));
    });

    it("save with Ftp as selectedExtension should call portalService.setFtpExtensionSettings then display a sticky alert and reset the form when save is a success", function () {
        var spyPristine, spySticky, spyResetForm;
        inject(function ($injector, $rootScope, $controller) {
            extensions = { data: [{ Name: "nothing", Guid: "123456789" }, { Name: "None", Guid: "6666" }] };
            userConfig = { data: { dummy: 51353541541, CurrentExtension: "123456789" } };
            sandbox.stub(checkChangeStateFactory, "checkFormOnStateChange");
            var deferred = $injector.get("$q").defer();
            deferred.resolve();
            sandbox.stub(portalService, "setFtpExtensionSettings").returns(deferred.promise);
            scope.form = { $valid: true, $setPristine: function () { } };
            spyPristine = sandbox.stub(scope.form, "$setPristine");
            var defer = $injector.get("$q").defer();
            defer.resolve(userConfig);
            sandbox.stub(portalService, "getUserConfig").returns(defer.promise);
            var d = $injector.get("$q").defer();
            d.resolve(extensions);
            sandbox.stub(portalService, "getExtensions").returns(d.promise);
            controller = $controller("ExtensionController", { portalService: portalService, $scope: scope, $timeout: $timeout, $state: $state });
            $rootScope.$apply();

            spySticky = sandbox.stub(controller, "showStickyAlert");
            spyResetForm = sandbox.stub(controller, "resetForm");
        });

        controller.selectedExtension = { Name: "Ftp", Guid: 123456789 }
        controller.ftp = {};
        controller.save();
        $rootScope.$digest();
        sinon.assert.notCalled(spyPristine);
        sinon.assert.calledOnce(spySticky);
        sinon.assert.calledOnce(spyResetForm);
        assert(spySticky.calledWith('success', 'STE_SENTENCE_ConfigurationSavedSuccessfully'));
    });

    it("save with Ftp as selectedExtension should call portalService.setFtpExtensionSettings then display a sticky alert and reset the form when save is a failure", function () {
        var spyPristine, spySticky, spyResetForm;
        inject(function ($injector, $rootScope, $controller) {
            extensions = { data: [{ Name: "nothing", Guid: "123456789" }, { Name: "None", Guid: "6666" }] };
            userConfig = { data: { dummy: 51353541541, CurrentExtension: "123456789" } };
            sandbox.stub(checkChangeStateFactory, "checkFormOnStateChange");
            var deferred = $injector.get("$q").defer();
            deferred.reject();
            sandbox.stub(portalService, "setFtpExtensionSettings").returns(deferred.promise);
            scope.form = { $valid: true, $setPristine: function () { } };
            spyPristine = sandbox.stub(scope.form, "$setPristine");
            var defer = $injector.get("$q").defer();
            defer.resolve(userConfig);
            sandbox.stub(portalService, "getUserConfig").returns(defer.promise);
            var d = $injector.get("$q").defer();
            d.resolve(extensions);
            sandbox.stub(portalService, "getExtensions").returns(d.promise);
            controller = $controller("ExtensionController", { portalService: portalService, $scope: scope, $timeout: $timeout, $state: $state });
            $rootScope.$apply();

            spySticky = sandbox.stub(controller, "showStickyAlert");
            spyResetForm = sandbox.stub(controller, "resetForm");
        });

        controller.selectedExtension = { Name: "Ftp", Guid: 123456789 }
        controller.ftp = {};
        controller.save();
        $rootScope.$digest();
        sinon.assert.notCalled(spyPristine);
        sinon.assert.calledOnce(spySticky);
        sinon.assert.notCalled(spyResetForm);
        assert(spySticky.calledWith('danger', saveFailure));
    });

    it("save with Http as selectedExtension should call portalService.setHttpExtensionSettings then display a sticky alert and reset the form when save is a success", function () {
        var spyPristine, spySticky, spyResetForm;
        inject(function ($injector, $rootScope, $controller) {
            extensions = { data: [{ Name: "nothing", Guid: "123456789" }, { Name: "None", Guid: "6666" }] };
            userConfig = { data: { dummy: 51353541541, CurrentExtension: "123456789" } };
            sandbox.stub(checkChangeStateFactory, "checkFormOnStateChange");
            var deferred = $injector.get("$q").defer();
            deferred.resolve();
            sandbox.stub(portalService, "setHttpExtensionSettings").returns(deferred.promise);
            scope.form = { $valid: true, $setPristine: function () { } };
            spyPristine = sandbox.stub(scope.form, "$setPristine");
            var defer = $injector.get("$q").defer();
            defer.resolve(userConfig);
            sandbox.stub(portalService, "getUserConfig").returns(defer.promise);
            var d = $injector.get("$q").defer();
            d.resolve(extensions);
            sandbox.stub(portalService, "getExtensions").returns(d.promise);
            controller = $controller("ExtensionController", { portalService: portalService, $scope: scope, $timeout: $timeout, $state: $state });
            $rootScope.$apply();

            spySticky = sandbox.stub(controller, "showStickyAlert");
            spyResetForm = sandbox.stub(controller, "resetForm");
        });

        controller.selectedExtension = { Name: "Http", Guid: 123456789 }
        controller.ftp = {};
        controller.save();
        $rootScope.$digest();
        sinon.assert.notCalled(spyPristine);
        sinon.assert.calledOnce(spySticky);
        sinon.assert.calledOnce(spyResetForm);
        assert(spySticky.calledWith('success', saveSuccess));
    });

    it("save with Http as selectedExtension should call portalService.setHttpExtensionSettings then display a sticky alert and reset the form when save is a failure", function () {
        var spyPristine, spySticky, spyResetForm;
        inject(function ($injector, $rootScope, $controller) {
            extensions = { data: [{ Name: "nothing", Guid: "123456789" }, { Name: "None", Guid: "6666" }] };
            userConfig = { data: { dummy: 51353541541, CurrentExtension: "123456789" } };
            sandbox.stub(checkChangeStateFactory, "checkFormOnStateChange");
            var deferred = $injector.get("$q").defer();
            deferred.reject();
            sandbox.stub(portalService, "setHttpExtensionSettings").returns(deferred.promise);
            scope.form = { $valid: true, $setPristine: function () { } };
            spyPristine = sandbox.stub(scope.form, "$setPristine");
            var defer = $injector.get("$q").defer();
            defer.resolve(userConfig);
            sandbox.stub(portalService, "getUserConfig").returns(defer.promise);
            var d = $injector.get("$q").defer();
            d.resolve(extensions);
            sandbox.stub(portalService, "getExtensions").returns(d.promise);
            controller = $controller("ExtensionController", { portalService: portalService, $scope: scope, $timeout: $timeout, $state: $state });
            $rootScope.$apply();

            spySticky = sandbox.stub(controller, "showStickyAlert");
            spyResetForm = sandbox.stub(controller, "resetForm");
        });

        controller.selectedExtension = { Name: "Http", Guid: 123456789 }
        controller.ftp = {};
        controller.save();
        $rootScope.$digest();
        sinon.assert.notCalled(spyPristine);
        sinon.assert.calledOnce(spySticky);
        sinon.assert.notCalled(spyResetForm);
        assert(spySticky.calledWith('danger', saveFailure));
    });

    it("save with Patroller as selectedExtension should call portalService.setPatrollerExtensionSettings then display a sticky alert and reset the form when save is a success", function () {
        var spyPristine, spySticky, spyResetForm;
        inject(function ($injector, $rootScope, $controller) {
            extensions = { data: [{ Name: "nothing", Guid: "123456789" }, { Name: "None", Guid: "6666" }] };
            userConfig = { data: { dummy: 51353541541, CurrentExtension: "123456789" } };
            sandbox.stub(checkChangeStateFactory, "checkFormOnStateChange");
            var deferred = $injector.get("$q").defer();
            deferred.resolve();
            sandbox.stub(portalService, "setPatrollerExtensionSettings").returns(deferred.promise);
            scope.form = { $valid: true, $setPristine: function () { } };
            spyPristine = sandbox.stub(scope.form, "$setPristine");
            var defer = $injector.get("$q").defer();
            defer.resolve(userConfig);
            sandbox.stub(portalService, "getUserConfig").returns(defer.promise);
            var d = $injector.get("$q").defer();
            d.resolve(extensions);
            sandbox.stub(portalService, "getExtensions").returns(d.promise);
            controller = $controller("ExtensionController", { portalService: portalService, $scope: scope, $timeout: $timeout, $state: $state });
            $rootScope.$apply();

            spySticky = sandbox.stub(controller, "showStickyAlert");
            spyResetForm = sandbox.stub(controller, "resetForm");
        });

        controller.selectedExtension = { Name: "Patroller", Guid: 123456789 }
        controller.ftp = {};
        controller.save();
        $rootScope.$digest();
        sinon.assert.notCalled(spyPristine);
        sinon.assert.calledOnce(spySticky);
        sinon.assert.calledOnce(spyResetForm);
        assert(spySticky.calledWith('success', saveSuccess));
    });

    it("save with Patroller as selectedExtension should call portalService.setPatrollerExtensionSettings then display a sticky alert and reset the form when save is a failure", function () {
        var spyPristine, spySticky, spyResetForm;
        inject(function ($injector, $rootScope, $controller) {
            extensions = { data: [{ Name: "nothing", Guid: "123456789" }, { Name: "None", Guid: "6666" }] };
            userConfig = { data: { dummy: 51353541541, CurrentExtension: "123456789" } };
            sandbox.stub(checkChangeStateFactory, "checkFormOnStateChange");
            var deferred = $injector.get("$q").defer();
            deferred.reject();
            sandbox.stub(portalService, "setPatrollerExtensionSettings").returns(deferred.promise);
            scope.form = { $valid: true, $setPristine: function () { } };
            spyPristine = sandbox.stub(scope.form, "$setPristine");
            var defer = $injector.get("$q").defer();
            defer.resolve(userConfig);
            sandbox.stub(portalService, "getUserConfig").returns(defer.promise);
            var d = $injector.get("$q").defer();
            d.resolve(extensions);
            sandbox.stub(portalService, "getExtensions").returns(d.promise);
            controller = $controller("ExtensionController", { portalService: portalService, $scope: scope, $timeout: $timeout, $state: $state });
            $rootScope.$apply();

            spySticky = sandbox.stub(controller, "showStickyAlert");
            spyResetForm = sandbox.stub(controller, "resetForm");
        });

        controller.selectedExtension = { Name: "Patroller", Guid: 123456789 }
        controller.ftp = {};
        controller.save();
        $rootScope.$digest();
        sinon.assert.notCalled(spyPristine);
        sinon.assert.calledOnce(spySticky);
        sinon.assert.notCalled(spyResetForm);
        assert(spySticky.calledWith('danger', saveFailure));
    });

    it("save with SecurityCenter as selectedExtension should call portalService.setSecurityCenterExtensionSettings then display a sticky alert and reset the form when save is a success", function () {
        var spyPristine, spySticky, spyResetForm;
        inject(function ($injector, $rootScope, $controller) {
            extensions = { data: [{ Name: "nothing", Guid: "123456789" }, { Name: "None", Guid: "6666" }] };
            userConfig = { data: { dummy: 51353541541, CurrentExtension: "123456789" } };
            sandbox.stub(checkChangeStateFactory, "checkFormOnStateChange");
            var deferred = $injector.get("$q").defer();
            deferred.resolve();
            sandbox.stub(portalService, "setSecurityCenterExtensionSettings").returns(deferred.promise);
            scope.form = { $valid: true, $setPristine: function () { } };
            spyPristine = sandbox.stub(scope.form, "$setPristine");
            var defer = $injector.get("$q").defer();
            defer.resolve(userConfig);
            sandbox.stub(portalService, "getUserConfig").returns(defer.promise);
            var d = $injector.get("$q").defer();
            d.resolve(extensions);
            sandbox.stub(portalService, "getExtensions").returns(d.promise);
            controller = $controller("ExtensionController", { portalService: portalService, $scope: scope, $timeout: $timeout, $state: $state });
            $rootScope.$apply();

            spySticky = sandbox.stub(controller, "showStickyAlert");
            spyResetForm = sandbox.stub(controller, "resetForm");
        });

        controller.selectedExtension = { Name: "SecurityCenter", Guid: 123456789 }
        controller.ftp = {};
        controller.save();
        $rootScope.$digest();
        sinon.assert.notCalled(spyPristine);
        sinon.assert.calledOnce(spySticky);
        sinon.assert.calledOnce(spyResetForm);
        assert(spySticky.calledWith('success', saveSuccess));
    });

    it("save with SecurityCenter as selectedExtension should call portalService.setSecurityCenterExtensionSettings then display a sticky alert and reset the form when save is a failure", function () {
        var spyPristine, spySticky, spyResetForm;
        inject(function ($injector, $rootScope, $controller) {
            extensions = { data: [{ Name: "nothing", Guid: "123456789" }, { Name: "None", Guid: "6666" }] };
            userConfig = { data: { dummy: 51353541541, CurrentExtension: "123456789" } };
            sandbox.stub(checkChangeStateFactory, "checkFormOnStateChange");
            var deferred = $injector.get("$q").defer();
            deferred.reject();
            sandbox.stub(portalService, "setSecurityCenterExtensionSettings").returns(deferred.promise);
            scope.form = { $valid: true, $setPristine: function () { } };
            spyPristine = sandbox.stub(scope.form, "$setPristine");
            var defer = $injector.get("$q").defer();
            defer.resolve(userConfig);
            sandbox.stub(portalService, "getUserConfig").returns(defer.promise);
            var d = $injector.get("$q").defer();
            d.resolve(extensions);
            sandbox.stub(portalService, "getExtensions").returns(d.promise);
            controller = $controller("ExtensionController", { portalService: portalService, $scope: scope, $timeout: $timeout, $state: $state });
            $rootScope.$apply();

            spySticky = sandbox.stub(controller, "showStickyAlert");
            spyResetForm = sandbox.stub(controller, "resetForm");
        });

        controller.selectedExtension = { Name: "SecurityCenter", Guid: 123456789 }
        controller.ftp = {};
        controller.save();
        $rootScope.$digest();
        sinon.assert.notCalled(spyPristine);
        sinon.assert.calledOnce(spySticky);
        sinon.assert.notCalled(spyResetForm);
        assert(spySticky.calledWith('danger', saveFailure));
    });
});