﻿"use strict";

describe("LanguageSwitch Controller", function () {

    var scope, rootScope, controller, localizationFactory;

    var mockNavigator = {
        language: "en-US"
        // don't care about userLanguage
    };

    var mockMoment = {
        locale: function (langKey) { }
    };

    beforeEach(function () {
        module("portalApp.localization", function ($provide) {
            $provide.value("navigator", mockNavigator);
            $provide.value("moment", mockMoment)
        });
        module("portalApp.controllers");

        inject(function ($injector, $controller, $translate) {
            rootScope = $injector.get("$rootScope");
            scope = rootScope.$new();
            localizationFactory = $injector.get("localizationFactory");
            rootScope.$apply(); // make the $rootScope available
            controller = $controller("LanguageSwitchController", { $scope: scope, $rootScope: rootScope, $translate: $translate, localizationFactory: localizationFactory });
        });
    });

    it("should be created successfully", function () {
        expect(controller).to.be.an('object');
    });

    it("changeLanguage works for all supported languages", function () {

        scope.changeLanguage("fr");
        expect(scope.currentLanguageText()).to.equal("Français");

        scope.changeLanguage("en");
        expect(scope.currentLanguageText()).to.equal("English");
    });

    it("changeLanguage works with unsupported languages", function () {

        scope.changeLanguage("zz"); // invalid (or unsupported) language
        expect(scope.currentLanguageText()).to.equal("zz");
    });
});