﻿"use strict";

describe("NetworkController controller", function () {
    var controller, portalService, $rootScope, $timeout, scope, sandbox, checkChangeStateFactory, $translate;
    var backendServerUrl = "http://localhost:80";
    var userConfig = {
        data: {
            ReadModeConfiguration: {
                ReadingMode: '' //!!!
            },
            NetworkConfiguration: {
                NetworkType: 0 // DHCP
            }
        }
    };

    beforeEach(function () {        
        module("portalApp.services", function ($provide) {
            $provide.value("backendServerUrl", backendServerUrl);
        });
        module("portalApp.controllers");
        module("pascalprecht.translate"); // for $translate

        sandbox = sinon.sandbox.create();

        inject(function ($injector, _$translate_) {
            portalService = $injector.get("portalService");
            checkChangeStateFactory = $injector.get("checkChangeStateFactory");
            $rootScope = $injector.get("$rootScope");
            scope = $rootScope.$new;
            scope.$on = function() {};
            $timeout = $injector.get("$timeout");
            $translate = _$translate_;
        });
    });

    afterEach(function () {
        sandbox.restore();
    });

    it("should be created successfully", function () {
        inject(function($injector, $rootScope, $controller) {
            sandbox.stub(checkChangeStateFactory, "checkFormOnStateChange");
            var d = $injector.get("$q").defer();
            d.resolve(userConfig);
            sandbox.stub(portalService, "getUserConfig").returns(d.promise);
            controller = $controller("NetworkController", { portalService: portalService, $scope: scope, $timeout: $timeout });
            $rootScope.$apply();
        });

        expect(controller).to.be.an('object');
        expect(controller.title).to.equal("NetworkController");
    });

    it("cancel should set userConfig back to its original content and call form.$setPristine", function () {
        var spy;
        inject(function ($injector, $rootScope, $controller) {
            sandbox.stub(checkChangeStateFactory, "checkFormOnStateChange");
            var d = $injector.get("$q").defer();
            d.resolve(userConfig);
            sandbox.stub(portalService, "getUserConfig").returns(d.promise);
            controller = $controller("NetworkController", { portalService: portalService, $scope: scope, $timeout: $timeout });
            $rootScope.$apply();
        });

        controller.form = { $valid: true, $setPristine: function () { } };
        spy = sandbox.stub(controller.form, "$setPristine");
        controller.cancel();
        expect(angular.equals(controller.userConfig, userConfig));
        sinon.assert.calledOnce(spy);
    });

    it("save should call portalService.setUserConfig then display a sticky alert and reset the form when save is a success", function () {
        var spyPristine, spyConfig;
        inject(function ($injector, $rootScope, $controller) {
            sandbox.stub(checkChangeStateFactory, "checkFormOnStateChange");
            var d = $injector.get("$q").defer();
            userConfig = { data: { NetworkConfiguration: { NetworkType: 1 } } };
            d.resolve(userConfig);
            sandbox.stub(portalService, "getUserConfig").returns(d.promise);
            var deferred = $injector.get("$q").defer();
            deferred.resolve();
            spyConfig = sandbox.stub(portalService, "setUserConfig").returns(deferred.promise);                        
            controller = $controller("NetworkController", { portalService: portalService, $scope: scope, $timeout: $timeout });
            $rootScope.$apply();
        });

        controller.userConfig.NetworkConfiguration.NetworkType = 2;
        controller.form = { $valid: true, $setPristine: function () { } };
        spyPristine = sandbox.stub(controller.form, "$setPristine");
        controller.save();
        $rootScope.$digest();
        sinon.assert.calledOnce(spyPristine);
        expect(controller.show).to.be.true;
        $timeout.flush();
        expect(controller.show).to.be.false;
        expect(spyConfig.args[0][0].NetworkConfiguration.NetworkType).to.equal(2);
        expect(controller.alert.type).to.equal('success');
        expect(controller.alert.msg).to.equal('STE_SENTENCE_ConfigurationSavedSuccessfully');
    });

    it("save should call portalService.setUserConfig then display a sticky alert and doesn't reset the form when save is a failure", function () {
        var spyPristine, spyConfig;
        inject(function ($injector, $rootScope, $controller) {
            userConfig = { data: { NetworkConfiguration: { NetworkType: 1 } } };
            sandbox.stub(checkChangeStateFactory, "checkFormOnStateChange");
            var d = $injector.get("$q").defer();
            d.resolve(userConfig);
            sandbox.stub(portalService, "getUserConfig").returns(d.promise);
            var deferred = $injector.get("$q").defer();
            deferred.reject();
            spyConfig = sandbox.stub(portalService, "setUserConfig").returns(deferred.promise);                        
            controller = $controller("NetworkController", { portalService: portalService, $scope: scope, $timeout: $timeout });
            $rootScope.$apply();
        });
        controller.form = { $valid: true, $setPristine: function () { } };
        spyPristine = sandbox.stub(controller.form, "$setPristine");
        controller.userConfig.NetworkConfiguration.NetworkType = 2;
        controller.save();
        $rootScope.$digest();
        sinon.assert.notCalled(spyPristine);
        expect(controller.show).to.be.true;
        expect(controller.alert.type).to.equal('danger');
        expect(controller.alert.msg).to.equal('STE_SENTENCE_CouldNotSaveConfiguration');
        $timeout.flush();
        expect(controller.show).to.be.false;
        expect(spyConfig.args[0][0].NetworkConfiguration.NetworkType).to.equal(2);
    });
});