﻿"use strict";

describe("Overview controller", function () {
    var controller, portalService, scope, $state, filter, signalRService, signalRServiceSpy, $uibModal, $translate, sandbox, $ocLazyLoad, mockFactory;
    var backendServerUrl = "http://localhost:8080";
    var backendLoggerUrl = "http://localhost:88";

    // mockfilter that just returns the value
    var mockFilter = function (value) {
        return value;
    };

    var fakeModal = {
        result: {
            then: function (confirmCallback, cancelCallback) {
                //Store the callbacks for later when the user clicks on the OK or Cancel button of the dialog
                this.confirmCallBack = confirmCallback;
                this.cancelCallback = cancelCallback;
            }
        },
        close: function (item) {
            //The user clicked OK on the modal dialog, call the stored confirm callback with the selected item
            this.result.confirmCallBack(item);
        },
        dismiss: function (type) {
            //The user clicked cancel on the modal dialog, call the stored cancel callback
            this.result.cancelCallback(type);
        }
    };

    var mockProperties = { data: false };

    beforeEach(function () {
        module("portalApp.controllers", function ($provide) {
            $provide.value("backendServerUrl", backendServerUrl);
            $provide.value("backendLoggerUrl", backendLoggerUrl);
            $provide.value("setDecimalFilter", mockFilter);
            $provide.value("bytesFilter", mockFilter);
        });
        module("portalApp.services");
        module("ui.bootstrap"); // for $uibModal
        module("uiGmapgoogle-maps");
        module("pascalprecht.translate"); // for $translate
        module("oc.lazyLoad"); // for $ocLazyLoad
        module("portalApp.testing"); // load the module, otherwise: [$injector:unpr] Unknown provider: mockFactoryProvider <- mockFactory

        sandbox = sinon.sandbox.create();

        inject(function ($injector, _$translate_) {
            signalRService = $injector.get("signalRService");
            portalService = $injector.get("portalService");
            filter = $injector.get("$filter");
            $state = $injector.get("$state");
            $uibModal = $injector.get("$uibModal");
            $ocLazyLoad = $injector.get("$ocLazyLoad");
            $translate = _$translate_;
            mockFactory = $injector.get("mockFactory");
        });

        mockFactory.setSandbox(sandbox);
        mockFactory.stubGetProperties(mockProperties);
    });

    afterEach(function () {
        sandbox.restore();
    });

    
    it("should be created successfully", function () {
        inject(function ($injector, $rootScope, $controller) {         
            signalRServiceSpy = sandbox.stub(signalRService, "initializePerformanceHub");
            sandbox.stub(signalRService, "initializeIoHub");
            sandbox.stub(signalRService, "initializeLastActivitiesHub");
            scope = $rootScope.$new();
            controller = $controller("OverviewController", { $scope: scope, portalService: portalService, signalRService: signalRService });

            $rootScope.$apply();
        });

        expect(controller).to.be.an('object');
        expect(controller.title).to.equal("OverviewController");
        expect(controller.error).to.equal("");
        expect($state.current.name).to.equal(""); // no state should be loaded
        sinon.assert.calledOnce(signalRServiceSpy);
    });

    it("should update totalCpuUsage when rootscope emits CpuUsage event with value", function () {
        var rootScope;
        inject(function ($injector, $rootScope, $controller) {                
            signalRServiceSpy = sandbox.stub(signalRService, "initializePerformanceHub");
            sandbox.stub(signalRService, "initializeIoHub");
            sandbox.stub(signalRService, "initializeLastActivitiesHub");
            rootScope = $rootScope;
            scope = $rootScope.$new();
            controller = $controller("OverviewController", { $scope: scope, portalService: portalService, signalRService: signalRService, $filter: filter });

            $rootScope.$apply();
        });

        var message = 69;
        rootScope.$emit("CpuUsage", message);

        sinon.assert.calledOnce(signalRServiceSpy);
        expect(controller.totalCpuUsage).to.equal(message);
    });

    it("should update cpusUsage when rootscope emits CpusUsage event with value", function () {
        var rootScope;
        inject(function ($injector, $rootScope, $controller) {          
            signalRServiceSpy = sandbox.stub(signalRService, "initializePerformanceHub");
            sandbox.stub(signalRService, "initializeIoHub");
            sandbox.stub(signalRService, "initializeLastActivitiesHub");
            rootScope = $rootScope;
            scope = $rootScope.$new();
            controller = $controller("OverviewController", { $scope: scope, portalService: portalService, signalRService: signalRService });

            $rootScope.$apply();
        });

        var message = [
        { CpuUsage: 17 },
        { CpuUsage: 24 }
        ];
        rootScope.$emit("CpusUsage", message);

        sinon.assert.calledOnce(signalRServiceSpy);
        expect(controller.cpusUsage).to.equal(message);
    });

    it("should update driveC when rootscope emits DriveCSpace event with value", function () {
        var rootScope;
        inject(function ($injector, $rootScope, $controller) {
            signalRServiceSpy = sandbox.stub(signalRService, "initializePerformanceHub");
            sandbox.stub(signalRService, "initializeIoHub");
            sandbox.stub(signalRService, "initializeLastActivitiesHub");
            rootScope = $rootScope;
            scope = $rootScope.$new();
            controller = $controller("OverviewController", { $scope: scope, portalService: portalService, signalRService: signalRService });

            $rootScope.$apply();
        });

        var message = {
            TotalSpace: 100,
            FreeSpace:0,
            RatioUsed:70
        };
        rootScope.$emit("DriveCSpace", message);

        sinon.assert.calledOnce(signalRServiceSpy);
        expect(controller.driveCUsage).to.equal(message.TotalSpace);
        expect(controller.driveCTotal).to.equal(message.TotalSpace);
        expect(controller.driveCRatio).to.equal(message.RatioUsed);
    });

    it("should update driveD when rootscope emits DriveDSpace event with value", function () {
        var rootScope;
        inject(function ($injector, $rootScope, $controller) {
            signalRServiceSpy = sandbox.stub(signalRService, "initializePerformanceHub");
            sandbox.stub(signalRService, "initializeIoHub");
            sandbox.stub(signalRService, "initializeLastActivitiesHub");
            rootScope = $rootScope;
            scope = $rootScope.$new();
            controller = $controller("OverviewController", { $scope: scope, portalService: portalService, signalRService: signalRService });

            $rootScope.$apply();
        });

        var message = {
            TotalSpace: 100,
            FreeSpace: 0,
            RatioUsed: 54
        };
        rootScope.$emit("DriveDSpace", message);

        sinon.assert.calledOnce(signalRServiceSpy);
        expect(controller.driveDUsage).to.equal(message.TotalSpace);
        expect(controller.driveDTotal).to.equal(message.TotalSpace);
        expect(controller.driveDRatio).to.equal(message.RatioUsed);
    });

    it("should update driveE when rootscope emits DriveESpace event with value", function () {
        var rootScope;
        inject(function ($injector, $rootScope, $controller) {
            signalRServiceSpy = sandbox.stub(signalRService, "initializePerformanceHub");
            sandbox.stub(signalRService, "initializeIoHub");
            sandbox.stub(signalRService, "initializeLastActivitiesHub");
            rootScope = $rootScope;
            scope = $rootScope.$new();
            controller = $controller("OverviewController", { $scope: scope, portalService: portalService, signalRService: signalRService });

            $rootScope.$apply();
        });

        var message = {
            TotalSpace: 100,
            FreeSpace: 0,
            RatioUsed: 44
        };
        rootScope.$emit("DriveESpace", message);

        sinon.assert.calledOnce(signalRServiceSpy);
        expect(controller.driveEUsage).to.equal(message.TotalSpace);
        expect(controller.driveETotal).to.equal(message.TotalSpace);
        expect(controller.driveERatio).to.equal(message.RatioUsed);
    });

    it("should update ramUsage when rootscope emits RamUsage event with value", function () {
        var rootScope;
        inject(function ($injector, $rootScope, $controller) {
            signalRServiceSpy = sandbox.stub(signalRService, "initializePerformanceHub");
            sandbox.stub(signalRService, "initializeIoHub");
            sandbox.stub(signalRService, "initializeLastActivitiesHub");
            rootScope = $rootScope;
            scope = $rootScope.$new();
            controller = $controller("OverviewController", { $scope: scope, portalService: portalService, signalRService: signalRService });

            $rootScope.$apply();
        });

        var message = {
            TotalSpace: 100,
            FreeSpace: 0,
            RatioUsed: 44
        };
        rootScope.$emit("RamUsage", message);

        sinon.assert.calledOnce(signalRServiceSpy);
        expect(controller.ramUsage).to.equal(message.TotalSpace);
        expect(controller.ramTotal).to.equal(message.TotalSpace);
        expect(controller.ramRatio).to.equal(message.RatioUsed);
    });

    it("should update storageInfo when rootscope emits StorageInfo event with value", function () {
        var rootScope;
        inject(function ($injector, $rootScope, $controller) {
            signalRServiceSpy = sandbox.stub(signalRService, "initializePerformanceHub");
            sandbox.stub(signalRService, "initializeIoHub");
            sandbox.stub(signalRService, "initializeLastActivitiesHub");
            rootScope = $rootScope;
            scope = $rootScope.$new();
            controller = $controller("OverviewController", { $scope: scope, portalService: portalService, signalRService: signalRService });
            $state = $injector.get("$state");

            $rootScope.$apply();
        });

        var message = {
            NbOfReads:2500,
            TotalSpace: 100,
            DbSize: 250,
            MaxDbSize: 500
        };
        rootScope.$emit("StorageInfo", message);

        sinon.assert.calledOnce(signalRServiceSpy);
        expect(controller.storageNbOfReads).to.equal(message.NbOfReads);
        expect(controller.storageRatio).to.equal(50);
        expect(controller.storageDbSize).to.equal(message.DbSize);
        expect(controller.storageMaxDbSize).to.equal(message.MaxDbSize);
    });

    it("getProgressBarClassOnPercentage should return progress-bar-success when value is lower than 50", function () {
        var rootScope;
        inject(function ($injector, $rootScope, $controller) {
            signalRServiceSpy = sandbox.stub(signalRService, "initializePerformanceHub");
            sandbox.stub(signalRService, "initializeIoHub");
            sandbox.stub(signalRService, "initializeLastActivitiesHub");
            rootScope = $rootScope;
            scope = $rootScope.$new();
            controller = $controller("OverviewController", { $scope: scope, portalService: portalService, signalRService: signalRService });
            scope.vm = controller;

            $rootScope.$apply();
        });

        var message = 4;
        var cssClass = controller.getProgressBarClassOnPercentage(message, 50, 85);

        sinon.assert.calledOnce(signalRServiceSpy);
        expect(cssClass).to.equal("progress-bar-success");
    });

    it("getProgressBarClassOnPercentage should return progress-bar-warning when value is lower than 85 and higher than 50", function () {
        var rootScope;
        inject(function ($injector, $rootScope, $controller) {
            signalRServiceSpy = sandbox.stub(signalRService, "initializePerformanceHub");
            sandbox.stub(signalRService, "initializeIoHub");
            sandbox.stub(signalRService, "initializeLastActivitiesHub");
            rootScope = $rootScope;
            scope = $rootScope.$new();
            controller = $controller("OverviewController", { $scope: scope, portalService: portalService, signalRService: signalRService });
            scope.vm = controller;

            $rootScope.$apply();
        });

        var message = 50;
        var cssClass = controller.getProgressBarClassOnPercentage(message, 50, 85);

        sinon.assert.calledOnce(signalRServiceSpy);
        expect(cssClass).to.equal("progress-bar-warning");
    });

    it("getProgressBarClassOnPercentage should return progress-bar-danger when value is higher than 85", function () {
        var rootScope;
        inject(function ($injector, $rootScope, $controller) {
            signalRServiceSpy = sandbox.stub(signalRService, "initializePerformanceHub");
            sandbox.stub(signalRService, "initializeIoHub");
            sandbox.stub(signalRService, "initializeLastActivitiesHub");
            rootScope = $rootScope;
            scope = $rootScope.$new();
            controller = $controller("OverviewController", { $scope: scope, portalService: portalService, signalRService: signalRService });
            scope.vm = controller;

            $rootScope.$apply();
        });

        var message = 85;
        var cssClass = controller.getProgressBarClassOnPercentage(message, 50, 85);

        sinon.assert.calledOnce(signalRServiceSpy);
        expect(cssClass).to.equal("progress-bar-danger");
    });

    it("getTextClassOnBoolean should return text-success when value is true", function () {
        var rootScope;
        inject(function ($injector, $rootScope, $controller) {
            signalRServiceSpy = sandbox.stub(signalRService, "initializePerformanceHub");
            sandbox.stub(signalRService, "initializeIoHub");
            sandbox.stub(signalRService, "initializeLastActivitiesHub");
            rootScope = $rootScope;
            scope = $rootScope.$new();
            controller = $controller("OverviewController", { $scope: scope, portalService: portalService, signalRService: signalRService });
            scope.vm = controller;

            $rootScope.$apply();
        });

        var cssClass = controller.getTextClassOnBoolean(true);

        sinon.assert.calledOnce(signalRServiceSpy);
        expect(cssClass).to.equal("text-success");
    });

    it("getTextClassOnBoolean should return text-danger when value is false", function () {
        var rootScope;
        inject(function ($injector, $rootScope, $controller) {
            signalRServiceSpy = sandbox.stub(signalRService, "initializePerformanceHub");
            sandbox.stub(signalRService, "initializeIoHub");
            sandbox.stub(signalRService, "initializeLastActivitiesHub");
            rootScope = $rootScope;
            scope = $rootScope.$new();
            controller = $controller("OverviewController", { $scope: scope, portalService: portalService, signalRService: signalRService });
            scope.vm = controller;

            $rootScope.$apply();
        });

        var cssClass = controller.getTextClassOnBoolean(false);

        sinon.assert.calledOnce(signalRServiceSpy);
        expect(cssClass).to.equal("text-danger");
    });

    it("textClassOnLicense should return text-success when value is Valid", function () {
        var rootScope;
        inject(function ($injector, $rootScope, $controller) {
            signalRServiceSpy = sandbox.stub(signalRService, "initializePerformanceHub");
            sandbox.stub(signalRService, "initializeIoHub");
            sandbox.stub(signalRService, "initializeLastActivitiesHub");
            mockProperties = { data: { License: "Valid" } }
            mockFactory.stubGetProperties(mockProperties); // recreate a specific stub here
            rootScope = $rootScope;
            scope = $rootScope.$new();
            controller = $controller("OverviewController", { $scope: scope, portalService: portalService, signalRService: signalRService });
            scope.vm = controller;

            $rootScope.$apply();
        });

        var cssClass = controller.textClassOnLicense;

        expect(cssClass).to.equal("text-success");
    });

    it("textClassOnLicense should return text-info when value is Unknown", function () {
        var rootScope;
        inject(function ($injector, $rootScope, $controller) {
            signalRServiceSpy = sandbox.stub(signalRService, "initializePerformanceHub");
            sandbox.stub(signalRService, "initializeIoHub");
            sandbox.stub(signalRService, "initializeLastActivitiesHub");
            mockProperties = { data: { License: "Unknown" } }
            mockFactory.stubGetProperties(mockProperties); // recreate a specific stub here
            rootScope = $rootScope;
            scope = $rootScope.$new();
            controller = $controller("OverviewController", { $scope: scope, portalService: portalService, signalRService: signalRService });
            scope.vm = controller;

            $rootScope.$apply();
        });

        var cssClass = controller.textClassOnLicense;

        expect(cssClass).to.equal("text-info");
    });

    it("textClassOnLicense should return text-danger when value is neither Valid or Unknown", function () {
        var rootScope;
        inject(function ($injector, $rootScope, $controller) {
            signalRServiceSpy = sandbox.stub(signalRService, "initializePerformanceHub");
            sandbox.stub(signalRService, "initializeIoHub");
            sandbox.stub(signalRService, "initializeLastActivitiesHub");
            mockProperties = { data: { License: "Patate" } }
            mockFactory.stubGetProperties(mockProperties); // recreate a specific stub here
            rootScope = $rootScope;
            scope = $rootScope.$new();
            controller = $controller("OverviewController", { $scope: scope, portalService: portalService, signalRService: signalRService });
            scope.vm = controller;

            $rootScope.$apply();
        });

        var cssClass = controller.textClassOnLicense;

        expect(cssClass).to.equal("text-danger");
    });

    it("getStateFromBoolean should return StateOn when value is true", function () {
        var rootScope;
        inject(function ($injector, $rootScope, $controller) {
            signalRServiceSpy = sandbox.stub(signalRService, "initializePerformanceHub");
            sandbox.stub(signalRService, "initializeIoHub");
            sandbox.stub(signalRService, "initializeLastActivitiesHub");
            rootScope = $rootScope;
            scope = $rootScope.$new();
            controller = $controller("OverviewController", { $scope: scope, portalService: portalService, signalRService: signalRService });
            scope.vm = controller;

            $rootScope.$apply();
        });

        var message = true;
        var state = controller.getStateFromBoolean(message);

        expect(state).to.equal('STE_LABEL_InputStateOn'); // token returned because no translation is available
    });

    it("getStateFromBoolean should return StateOff when value is false", function () {
        var rootScope;
        inject(function ($injector, $rootScope, $controller) {
            signalRServiceSpy = sandbox.stub(signalRService, "initializePerformanceHub");
            sandbox.stub(signalRService, "initializeIoHub");
            sandbox.stub(signalRService, "initializeLastActivitiesHub");
            rootScope = $rootScope;
            scope = $rootScope.$new();
            controller = $controller("OverviewController", { $scope: scope, portalService: portalService, signalRService: signalRService });
            scope.vm = controller;

            $rootScope.$apply();
        });

        var message = false;
        var state = controller.getStateFromBoolean(message);

        expect(state).to.equal("STE_LABEL_InputStateOff"); // token returned because no translation is available
    });

    it("BlinkLed should call portalservice.startIdentify", function () {
        var rootScope, scope, spy;
        inject(function ($injector, $rootScope, $controller) {
            spy = sandbox.spy(portalService, "startIdentify");
            signalRServiceSpy = sandbox.stub(signalRService, "initializePerformanceHub");
            sandbox.stub(signalRService, "initializeIoHub");
            sandbox.stub(signalRService, "initializeLastActivitiesHub");
            rootScope = $rootScope;
            scope = $rootScope.$new();
            controller = $controller("OverviewController", { $scope: scope, portalService: portalService, signalRService: signalRService });
            scope.vm = controller;

            $rootScope.$apply();
        });
        
        controller.blinkLed();

        sinon.assert.calledOnce(spy);
    });

    it("OpenVersionsModal should report an error", function () {
        var rootScope, scope, error;
        inject(function ($injector, $rootScope, $controller) {
            error = "this is an error";
            var deferred = $injector.get("$q").defer();
            deferred.reject(error);
            sandbox.stub(portalService, "getComponents").returns(deferred.promise);
            sandbox.stub(portalService, "getInstalledFiles").returns(deferred.promise);
            signalRServiceSpy = sandbox.stub(signalRService, "initializePerformanceHub");
            sandbox.stub(signalRService, "initializeIoHub");
            sandbox.stub(signalRService, "initializeLastActivitiesHub");

            var $httpBackend = $injector.get("$httpBackend");
            $httpBackend.expectGET("versions.html");
            $httpBackend
                .whenGET("versions.html")
                .respond(200, false, { }, 'OK');

            rootScope = $rootScope;
            scope = $rootScope.$new();
            controller = $controller("OverviewController", { $scope: scope, portalService: portalService, signalRService: signalRService });
            scope.vm = controller;

            $rootScope.$apply();
        });

        controller.openVersionsModal();
        expect(scope.modalInstance).to.be.an('object');
        rootScope.$digest();
        expect(controller.error).to.equal(error);
    });

    it("OpenVersionsModal should setup components properly with an error getting installed files", function () {
        var rootScope, scope, components, error;
        inject(function ($injector, $rootScope, $controller) {
            components = { data: [{ "IsVersionAcceptable": false, "MinimumVersion": "1.7.2457", "IsKnownComponent": true, "ComponentId": "1", "ComponentType": "FpgaMon_Fpga_FirmVersion", "Version": "1.5.2305", "IpAddress": null, "MacAddress": null }, { "IsVersionAcceptable": true, "MinimumVersion": "0.1", "IsKnownComponent": true, "ComponentId": "1", "ComponentType": "FpgaMon_Fpga_PcbVersion", "Version": "3.7", "IpAddress": null, "MacAddress": null }, { "IsVersionAcceptable": false, "MinimumVersion": "1.5.2461.0", "IsKnownComponent": true, "ComponentId": "1", "ComponentType": "Cmui_Cmu_FirmVersion", "Version": "0.6.2353.0", "IpAddress": null, "MacAddress": null }, { "IsVersionAcceptable": false, "MinimumVersion": "1.2.2448.0", "IsKnownComponent": true, "ComponentId": "1", "ComponentType": "Cmui_Pmu_RunningFirmVersion", "Version": "0.25.2350", "IpAddress": null, "MacAddress": null }, { "IsVersionAcceptable": false, "MinimumVersion": "12.2.123.0", "IsKnownComponent": true, "ComponentId": "1", "ComponentType": "Hal_Hal_FirmVersion", "Version": "12.0.0.0", "IpAddress": null, "MacAddress": null }, { "IsVersionAcceptable": true, "MinimumVersion": "0.0", "IsKnownComponent": true, "ComponentId": "1", "ComponentType": "Cmui_Cpu_PcbVersion", "Version": "3.7", "IpAddress": null, "MacAddress": null }, { "IsVersionAcceptable": false, "MinimumVersion": "12.2.0.19", "IsKnownComponent": true, "ComponentId": "1", "ComponentType": "System_EvoCamDrvr_FirmVersion", "Version": "9.20.27.277", "IpAddress": null, "MacAddress": null }, { "IsVersionAcceptable": true, "MinimumVersion": "", "IsKnownComponent": false, "ComponentId": "1", "ComponentType": "System_CpmuDrvr_FirmVersion", "Version": "16.55.17.534", "IpAddress": null, "MacAddress": null }, { "IsVersionAcceptable": true, "MinimumVersion": "12.0.0.25", "IsKnownComponent": true, "ComponentId": "1", "ComponentType": "System_FpgaFlashDrvr_FirmVersion", "Version": "14.23.10.31", "IpAddress": null, "MacAddress": null }, { "IsVersionAcceptable": true, "MinimumVersion": "1.1", "IsKnownComponent": true, "ComponentId": "1", "ComponentType": "Evoi_LprCam_PcbVersion", "Version": "1.7", "IpAddress": null, "MacAddress": null }, { "IsVersionAcceptable": true, "MinimumVersion": "1.0", "IsKnownComponent": true, "ComponentId": "1", "ComponentType": "Evoi_CtxCam_PcbVersion", "Version": "1.7", "IpAddress": null, "MacAddress": null }, { "IsVersionAcceptable": false, "MinimumVersion": "1876.8704", "IsKnownComponent": true, "ComponentId": "1", "ComponentType": "Evoi_LprCam_CmosVersion", "Version": "3.1.0", "IpAddress": null, "MacAddress": null }, { "IsVersionAcceptable": true, "MinimumVersion": "9222.4864", "IsKnownComponent": true, "ComponentId": "1", "ComponentType": "Evoi_CtxCam_CmosVersion", "Version": "9222.4864", "IpAddress": null, "MacAddress": null }] };
            error = "this is an error";

            var deferredComponents = $injector.get("$q").defer();
            deferredComponents.resolve(components);
            sandbox.stub(portalService, "getComponents").returns(deferredComponents.promise);
            var deferredFiles = $injector.get("$q").defer();
            deferredFiles.reject(error);
            sandbox.stub(portalService, "getInstalledFiles").returns(deferredFiles.promise);
            signalRServiceSpy = sandbox.stub(signalRService, "initializePerformanceHub");
            sandbox.stub(signalRService, "initializeIoHub");
            sandbox.stub(signalRService, "initializeLastActivitiesHub");

            var $httpBackend = $injector.get("$httpBackend");
            $httpBackend.expectGET("versions.html");
            $httpBackend
            .whenGET("versions.html")
            .respond(200, false, {}, 'OK');

            rootScope = $rootScope;
            scope = $rootScope.$new();
            controller = $controller("OverviewController", { $scope: scope, portalService: portalService, signalRService: signalRService });
            scope.vm = controller;

            $rootScope.$apply();
        });

        controller.openVersionsModal();
        expect(scope.modalInstance).to.be.an('object');
        rootScope.$digest();
        expect(scope.vm.components).to.equal(components.data);
        expect(controller.error).to.equal(error);
    });

    it("OpenVersionsModal should setup components and installed files properly", function () {
        var rootScope, scope, components, files;
        inject(function ($injector, $rootScope, $controller) {
            components = { data: [{ "IsVersionAcceptable": false, "MinimumVersion": "1.7.2457", "IsKnownComponent": true, "ComponentId": "1", "ComponentType": "FpgaMon_Fpga_FirmVersion", "Version": "1.5.2305", "IpAddress": null, "MacAddress": null }, { "IsVersionAcceptable": true, "MinimumVersion": "0.1", "IsKnownComponent": true, "ComponentId": "1", "ComponentType": "FpgaMon_Fpga_PcbVersion", "Version": "3.7", "IpAddress": null, "MacAddress": null }, { "IsVersionAcceptable": false, "MinimumVersion": "1.5.2461.0", "IsKnownComponent": true, "ComponentId": "1", "ComponentType": "Cmui_Cmu_FirmVersion", "Version": "0.6.2353.0", "IpAddress": null, "MacAddress": null }, { "IsVersionAcceptable": false, "MinimumVersion": "1.2.2448.0", "IsKnownComponent": true, "ComponentId": "1", "ComponentType": "Cmui_Pmu_RunningFirmVersion", "Version": "0.25.2350", "IpAddress": null, "MacAddress": null }, { "IsVersionAcceptable": false, "MinimumVersion": "12.2.123.0", "IsKnownComponent": true, "ComponentId": "1", "ComponentType": "Hal_Hal_FirmVersion", "Version": "12.0.0.0", "IpAddress": null, "MacAddress": null }, { "IsVersionAcceptable": true, "MinimumVersion": "0.0", "IsKnownComponent": true, "ComponentId": "1", "ComponentType": "Cmui_Cpu_PcbVersion", "Version": "3.7", "IpAddress": null, "MacAddress": null }, { "IsVersionAcceptable": false, "MinimumVersion": "12.2.0.19", "IsKnownComponent": true, "ComponentId": "1", "ComponentType": "System_EvoCamDrvr_FirmVersion", "Version": "9.20.27.277", "IpAddress": null, "MacAddress": null }, { "IsVersionAcceptable": true, "MinimumVersion": "", "IsKnownComponent": false, "ComponentId": "1", "ComponentType": "System_CpmuDrvr_FirmVersion", "Version": "16.55.17.534", "IpAddress": null, "MacAddress": null }, { "IsVersionAcceptable": true, "MinimumVersion": "12.0.0.25", "IsKnownComponent": true, "ComponentId": "1", "ComponentType": "System_FpgaFlashDrvr_FirmVersion", "Version": "14.23.10.31", "IpAddress": null, "MacAddress": null }, { "IsVersionAcceptable": true, "MinimumVersion": "1.1", "IsKnownComponent": true, "ComponentId": "1", "ComponentType": "Evoi_LprCam_PcbVersion", "Version": "1.7", "IpAddress": null, "MacAddress": null }, { "IsVersionAcceptable": true, "MinimumVersion": "1.0", "IsKnownComponent": true, "ComponentId": "1", "ComponentType": "Evoi_CtxCam_PcbVersion", "Version": "1.7", "IpAddress": null, "MacAddress": null }, { "IsVersionAcceptable": false, "MinimumVersion": "1876.8704", "IsKnownComponent": true, "ComponentId": "1", "ComponentType": "Evoi_LprCam_CmosVersion", "Version": "3.1.0", "IpAddress": null, "MacAddress": null }, { "IsVersionAcceptable": true, "MinimumVersion": "9222.4864", "IsKnownComponent": true, "ComponentId": "1", "ComponentType": "Evoi_CtxCam_CmosVersion", "Version": "9222.4864", "IpAddress": null, "MacAddress": null }] };
            files = { data: ["a", "b", "c", "d", "e", "f", "g", "h", "i", "j"] };

            var deferredComponents = $injector.get("$q").defer();
            deferredComponents.resolve(components);
            sandbox.stub(portalService, "getComponents").returns(deferredComponents.promise);
            var deferredFiles = $injector.get("$q").defer();
            deferredFiles.resolve(files);
            sandbox.stub(portalService, "getInstalledFiles").returns(deferredFiles.promise);
            signalRServiceSpy = sandbox.stub(signalRService, "initializePerformanceHub");
            sandbox.stub(signalRService, "initializeIoHub");
            sandbox.stub(signalRService, "initializeLastActivitiesHub");

            var $httpBackend = $injector.get("$httpBackend");
            $httpBackend.expectGET("versions.html");
            $httpBackend
            .whenGET("versions.html")
            .respond(200, false, {}, 'OK');

            rootScope = $rootScope;
            scope = $rootScope.$new();
            controller = $controller("OverviewController", { $scope: scope, portalService: portalService, signalRService: signalRService });
            scope.vm = controller;

            $rootScope.$apply();
        });

        controller.openVersionsModal();
        expect(scope.modalInstance).to.be.an('object');
        rootScope.$digest();
        expect(scope.vm.components).to.equal(components.data);
        expect(angular.equals(scope.files, files.data));
        expect(angular.equals(controller.files, files.data));
        expect(controller.error).to.equal("");
    });

    it("$scope.closeModal should call $scope.modalInstance.close", function () {
        var scope, spy;
        inject(function ($injector, $rootScope, $controller) {            
       
            signalRServiceSpy = sandbox.stub(signalRService, "initializePerformanceHub");
            sandbox.stub(signalRService, "initializeIoHub");
            sandbox.stub(signalRService, "initializeLastActivitiesHub");

            scope = $rootScope.$new();
            controller = $controller("OverviewController", { $scope: scope, portalService: portalService, signalRService: signalRService });
            scope.vm = controller;

            scope.modalInstance = fakeModal;
            spy = sandbox.stub(scope.modalInstance, "close");

            $rootScope.$apply();
        });

        // This is what is called in modal windows to close them
        scope.closeModal();
        sinon.assert.calledOnce(spy);
    });

    it("OpenMapModal should setup modal properly", function () {
        var rootScope, scope;
        inject(function ($injector, $rootScope, $controller) {

            signalRServiceSpy = sandbox.stub(signalRService, "initializePerformanceHub");
            sandbox.stub(signalRService, "initializeIoHub");
            sandbox.stub(signalRService, "initializeLastActivitiesHub");
            var deferred = $injector.get("$q").defer();
            deferred.resolve("");
            sandbox.stub($ocLazyLoad, "load").returns(deferred.promise);

            var $httpBackend = $injector.get("$httpBackend");
            $httpBackend.expectGET("map.html");
            $httpBackend
            .whenGET("map.html")
            .respond(200, false, {}, 'OK');

            rootScope = $rootScope;
            scope = $rootScope.$new();
            controller = $controller("OverviewController", { $scope: scope, portalService: portalService, signalRService: signalRService, $filter: filter, $ocLazyLoad: $ocLazyLoad });
            scope.vm = controller;

            $rootScope.$apply();
        });

        controller.openMapModal();
        rootScope.$apply();
        expect(scope.modalInstance).to.be.an('object');
        rootScope.$digest();
    });

    it("Overview reflect SharpV type (alone)", function () {
        var rootScope;
        var props =
            {
                data: { // dto
                    Type: "SharpV",
                    IsIts: false
                }
            };
        inject(function ($injector, $rootScope, $controller) {
            signalRServiceSpy = sandbox.stub(signalRService, "initializePerformanceHub");
            sandbox.stub(signalRService, "initializeIoHub");
            sandbox.stub(signalRService, "initializeLastActivitiesHub");

            mockFactory.stubGetProperties(props); // recreate a specific stub here
            rootScope = $rootScope;
            scope = $rootScope.$new();
            controller = $controller("OverviewController", { $scope: scope, portalService: portalService, signalRService: signalRService });

            $rootScope.$apply();
        });

        sinon.assert.calledOnce(signalRServiceSpy);
        expect(controller.displayType).to.equal("SharpV");
    });

    it("Overview reflect SharpV type (ITS)", function () {
        var rootScope;
        var props =
            {
                data: { // dto
                    Type: "SharpV",
                    IsIts: true
                }
            };
        inject(function ($injector, $rootScope, $controller) {
            signalRServiceSpy = sandbox.stub(signalRService, "initializePerformanceHub");
            sandbox.stub(signalRService, "initializeIoHub");
            sandbox.stub(signalRService, "initializeLastActivitiesHub");

            mockFactory.stubGetProperties(props); // recreate a specific stub here
            rootScope = $rootScope;
            scope = $rootScope.$new();
            controller = $controller("OverviewController", { $scope: scope, portalService: portalService, signalRService: signalRService });

            $rootScope.$apply();
        });

        sinon.assert.calledOnce(signalRServiceSpy);
        expect(controller.displayType).to.equal("SharpV" + " ITS");
    });
});