﻿"use strict";

describe("SecurityController controller", function () {
    var controller, portalService, $rootScope, $timeout, scope, sandbox, $uibModal, checkChangeStateFactory, $translate;
    var backendServerUrl = "http://localhost:8080";
    var backendLoggerUrl = "http://localhost:88";
    var userConfig = { data: { PortalSecurityConfiguration: { CertificateType: 0} } };
    var certificateInfos = { data: 4854857483975 };
    var remoteAssistanceConfig = { data: { enabled: false, enabledUntilISO8601Formatted: '2016-10-05T08:15:30-05:00' } };
    var certificates = {
        data: [{
            Active: false,
            Algorithm: "",
            FriendlyName: "Test certificate",
            IsRequest: false,
            Issuer: "Genetec",
            Pem: "",
            Subject: "C=CA",
            Thumbprint: "DFD$RXDFDF#F0",
            ValidFrom: "2017-01-01",
            ValidTo: "2019-01-01"}]
    };
    var properties = { data: false };

    beforeEach(function () {        
        module("portalApp.services", function ($provide) {
            $provide.value("backendServerUrl", backendServerUrl);
            $provide.value("backendLoggerUrl", backendLoggerUrl);
        });

        module("portalApp.controllers");
        module("ui.bootstrap");
        module("pascalprecht.translate"); // for $translate
        module("portalApp.factories"); // for ngClipboard

        sandbox = sinon.sandbox.create();

        inject(function ($injector, _$translate_) {
            portalService = $injector.get("portalService");
            checkChangeStateFactory = $injector.get("checkChangeStateFactory");
            $rootScope = $injector.get("$rootScope");
            scope = $rootScope.$new();
            $timeout = $injector.get("$timeout");
            $uibModal = $injector.get("$uibModal");
            $translate = _$translate_;
        });

    });

    afterEach(function () {
        sandbox.restore();
    });
    
    it("should be created successfully", function () {
        inject(function($injector, $rootScope, $controller) {
            sandbox.stub(checkChangeStateFactory, "checkFormOnStateChange");
            var deferred = $injector.get("$q").defer();
            deferred.resolve(userConfig);
            var d = $injector.get("$q").defer();
            d.resolve(certificateInfos);
            var defer = $injector.get("$q").defer();
            defer.resolve(certificates);
            var d1 = $injector.get("$q").defer();
            d1.resolve(remoteAssistanceConfig);
            var dto = $injector.get("$q").defer();
            dto.resolve(properties);
            sandbox.stub(portalService, "getUserConfig").returns(deferred.promise);
            sandbox.stub(portalService, "getInstalledCertificateInfos").returns(d.promise);
            sandbox.stub(portalService, "getCertificates").returns(defer.promise);
            sandbox.stub(portalService, "getRemoteAssistanceConfig").returns(d1.promise);
            sandbox.stub(portalService, "getProperties").returns(dto.promise);
            controller = $controller("SecurityController", { portalService: portalService, $scope: scope, $timeout: $timeout });
            $rootScope.$apply();
        });

        expect(controller).to.be.an('object');
        expect(controller.title).to.equal("SecurityController");
    });

    it("cancel should set userConfig back to its original content and call form.$setPristine", function () {
        inject(function ($injector, $rootScope, $controller) {
            sandbox.stub(checkChangeStateFactory, "checkFormOnStateChange");
            var deferred = $injector.get("$q").defer();
            deferred.resolve(userConfig);
            var d = $injector.get("$q").defer();
            d.resolve(certificateInfos);
            var defer = $injector.get("$q").defer();
            defer.resolve(certificates);
            var d1 = $injector.get("$q").defer();
            d1.resolve(remoteAssistanceConfig);
            var dto = $injector.get("$q").defer();
            dto.resolve(properties);
            sandbox.stub(portalService, "getUserConfig").returns(deferred.promise);
            sandbox.stub(portalService, "getInstalledCertificateInfos").returns(d.promise);
            sandbox.stub(portalService, "getCertificates").returns(defer.promise);
            sandbox.stub(portalService, "getRemoteAssistanceConfig").returns(d1.promise);
            sandbox.stub(portalService, "getProperties").returns(dto.promise);
            controller = $controller("SecurityController", { portalService: portalService, $scope: scope, $timeout: $timeout });
            $rootScope.$apply();
        });

        controller.userConfig = { fake: "asdhasdhjashdjkhas" };
        controller.form = { $valid: true, $setPristine: function () { } };
        var spy = sandbox.stub(controller.form, "$setPristine");
        controller.cancel();
        expect(angular.equals(controller.userConfig, userConfig));
        sinon.assert.calledOnce(spy);
    });

    it("save should call portalService.setUserConfig then display a sticky alert and reset the form when save is a success", function () {
        var spyConfig;
        inject(function ($injector, $rootScope, $controller) {
            userConfig = { data: { dummy: 51353541541, PortalSecurityConfiguration: { UseHttps: true }, DummyValue: true } };
            sandbox.stub(checkChangeStateFactory, "checkFormOnStateChange");
            var deferred = $injector.get("$q").defer();
            deferred.resolve(userConfig);
            var d = $injector.get("$q").defer();
            d.resolve(certificateInfos);
            var defer = $injector.get("$q").defer();
            defer.resolve(certificates);
            var d1 = $injector.get("$q").defer();
            d1.resolve(remoteAssistanceConfig);
            var dto = $injector.get("$q").defer();
            dto.resolve(properties);
            sandbox.stub(portalService, "getUserConfig").returns(deferred.promise);
            sandbox.stub(portalService, "getInstalledCertificateInfos").returns(d.promise);
            sandbox.stub(portalService, "getCertificates").returns(defer.promise);
            sandbox.stub(portalService, "getRemoteAssistanceConfig").returns(d1.promise);
            sandbox.stub(portalService, "getProperties").returns(dto.promise);
            var ds = $injector.get("$q").defer();
            ds.resolve(); // will accept the save
            spyConfig = sandbox.stub(portalService, "setUserConfig").returns(ds.promise);
            controller = $controller("SecurityController", { portalService: portalService, $scope: scope, $timeout: $timeout });
            $rootScope.$apply();
        });

        controller.form = { $valid: true, $setPristine: function () { } };
        var spyPristine = sandbox.stub(controller.form, "$setPristine");
        controller.userConfig.DummyValue = false; // any change to original user config
        controller.save();
        expect(controller.isSaving).to.be.true;
        expect(controller.needReboot).to.be.false;
        $rootScope.$digest();
        sinon.assert.calledOnce(spyPristine);
        expect(controller.show).to.be.true;
        $timeout.flush();
        expect(controller.show).to.be.false;
        expect(spyConfig.args[0][0].DummyValue).to.be.false;
        expect(controller.alert.type).to.equal('success');
        expect(controller.alert.msg).to.equal('STE_SENTENCE_ConfigurationSavedSuccessfully');
        expect(controller.isSaving).to.be.false;
    });

    it("save should call portalService.setUserConfig then display a sticky alert and reset the form when save is a failure", function () {
        var spyConfig;
        inject(function ($injector, $rootScope, $controller) {
            userConfig = { data: { dummy: 51353541541, PortalSecurityConfiguration: { UseHttps: true }, DummyValue: true } };
            sandbox.stub(checkChangeStateFactory, "checkFormOnStateChange");
            var deferred = $injector.get("$q").defer();
            deferred.resolve(userConfig);
            var d = $injector.get("$q").defer();
            d.resolve(certificateInfos);
            var defer = $injector.get("$q").defer();
            defer.resolve(certificates);
            var d1 = $injector.get("$q").defer();
            d1.resolve(remoteAssistanceConfig);
            var dto = $injector.get("$q").defer();
            dto.resolve(properties);
            sandbox.stub(portalService, "getUserConfig").returns(deferred.promise);
            sandbox.stub(portalService, "getInstalledCertificateInfos").returns(d.promise);
            sandbox.stub(portalService, "getCertificates").returns(defer.promise);
            sandbox.stub(portalService, "getRemoteAssistanceConfig").returns(d1.promise);
            sandbox.stub(portalService, "getProperties").returns(dto.promise);
            var ds = $injector.get("$q").defer();
            ds.reject(); // will reject the save
            spyConfig = sandbox.stub(portalService, "setUserConfig").returns(ds.promise);
            scope.form = { $valid: true, $setPristine: function () { } };
            spyPristine = sandbox.stub(scope.form, "$setPristine");
            controller = $controller("SecurityController", { portalService: portalService, $scope: scope, $timeout: $timeout, userConfig: userConfig });
            $rootScope.$apply();
        });
        controller.form = { $valid: true, $setPristine: function () { } };
        var spyPristine = sandbox.stub(controller.form, "$setPristine");
        controller.userConfig.DummyValue = false; // any change to original user config
        controller.save();
        expect(controller.isSaving).to.be.true;
        expect(controller.needReboot).to.be.false;
        $rootScope.$digest();
        sinon.assert.notCalled(spyPristine);
        expect(controller.show).to.be.true;        
        $timeout.flush();
        expect(controller.show).to.be.false;
        expect(spyConfig.args[0][0].DummyValue).to.be.false;
        expect(controller.alert.type).to.equal('danger');
        expect(controller.alert.msg).to.equal('STE_SENTENCE_CouldNotSaveConfiguration');
        expect(controller.isSaving).to.be.false;
    });
});