﻿"use strict";

describe("SpeedEstimationController controller", function () {
    var controller, portalService, signalRService, $rootScope, $timeout, $uibModal, scope, sandbox, checkChangeStateFactory, $translate, spySignalrHub, deferred, $state;
    var backendServerUrl = "http://localhost:80";
    var backendLoggerUrl = "http://localhost:88";
    var userConfig = { data: 51353541541 };
    var summary = { data: 'blablabla' };
    var calibrationData = {
        data: {
            AwaitedPlates : [],
            OtherPlates: []
        }
    };

    beforeEach(function () {        
        module("portalApp.services", function ($provide) {
            $provide.value("backendServerUrl", backendServerUrl);
            $provide.value("backendLoggerUrl", backendLoggerUrl);
        });
        module("portalApp.controllers");
        module("ui.bootstrap");
        module("pascalprecht.translate"); // for $translate

        sandbox = sinon.sandbox.create();

        inject(function ($injector, _$translate_) {
            portalService = $injector.get("portalService");
            signalRService = $injector.get("signalRService");
            checkChangeStateFactory = $injector.get("checkChangeStateFactory");
            $rootScope = $injector.get("$rootScope");
            $state = $injector.get("$state");
            scope = $rootScope.$new;
            scope.$on = function() {};
            $timeout = $injector.get("$timeout");
            $uibModal = $injector.get("$uibModal");
            $translate = _$translate_;

            deferred = $injector.get("$q").defer();

            deferred.resolve({ data: true });

            spySignalrHub = sandbox.stub(signalRService, "initializeSpeedEstimationHub").returns(deferred.promise);
            sandbox.stub(checkChangeStateFactory, "checkFormOnStateChange");
        });

    });

    afterEach(function () {
        sandbox.restore();
    });

    it("should be created successfully", function () {
        inject(function($injector, $rootScope, $controller) {
            
            var defer = $injector.get("$q").defer();
            defer.resolve(userConfig);
            sandbox.stub(portalService, "getUserConfig").returns(defer.promise);
            defer = $injector.get("$q").defer();
            defer.resolve(summary);
            sandbox.stub(portalService, "getSpeedEstimationSummary").returns(defer.promise);
            defer = $injector.get("$q").defer();
            defer.resolve(calibrationData);
            sandbox.stub(portalService, "getSpeedEstimationCalibrationData").returns(defer.promise);

            controller = $controller("SpeedEstimationController", { portalService: portalService, $scope: scope, $timeout: $timeout });
            $rootScope.$apply();
        });

        expect(controller).to.be.an('object');
        expect(controller.title).to.equal("SpeedEstimationController");
    });

    it("should indicate connectionLost when service call fails on activation", function () {
        inject(function ($injector, $rootScope, $controller) {

            var defer = $injector.get("$q").defer();
            defer.resolve(userConfig);
            sandbox.stub(portalService, "getUserConfig").returns(defer.promise);
            defer = $injector.get("$q").defer();
            defer.resolve(summary);
            sandbox.stub(portalService, "getSpeedEstimationSummary").returns(defer.promise);
            defer = $injector.get("$q").defer();
            defer.resolve(calibrationData);
            sandbox.stub(portalService, "getSpeedEstimationCalibrationData").returns(defer.promise);

            controller = $controller("SpeedEstimationController", { portalService: portalService, $scope: scope, $timeout: $timeout });
            $rootScope.$apply();
        });

        $rootScope.reconnecting = false;
        $rootScope.connectionLost = true;
        $rootScope.isLoading = false;
        $timeout.flush();
        $rootScope.reconnecting = true;
    });

    it("addPlate should call portalService.postSpeedEstimationAddPlate with a plate to add with success then call doneEdit", function () {
        inject(function ($injector, $rootScope, $controller) {

            var defer = $injector.get("$q").defer();
            defer.resolve(userConfig);
            sandbox.stub(portalService, "getUserConfig").returns(defer.promise);
            defer = $injector.get("$q").defer();
            defer.resolve(summary);
            sandbox.stub(portalService, "getSpeedEstimationSummary").returns(defer.promise);
            defer = $injector.get("$q").defer();
            defer.resolve(calibrationData);
            sandbox.stub(portalService, "getSpeedEstimationCalibrationData").returns(defer.promise);

            defer = $injector.get("$q").defer();
            defer.resolve(summary);
            sandbox.stub(portalService, "postSpeedEstimationAddPlate").returns(defer.promise);

            controller = $controller("SpeedEstimationController", { portalService: portalService, $scope: scope, $timeout: $timeout });
            $rootScope.$apply();
        });

        
        var spy = sandbox.stub(controller, "doneEdit");

        controller.plateToAdd = "allo";

        controller.form = { $valid: true, $setPristine: function () { } }; // Mock the form

        controller.addPlate();
        $rootScope.$digest();
        sinon.assert.calledOnce(spy);
        expect(controller.plateToAdd).to.equal('');
    });

    it("addPlate should call portalService.postSpeedEstimationAddPlate with a plate to add with failure then not call doneEdit", function () {
        inject(function ($injector, $rootScope, $controller) {

            var defer = $injector.get("$q").defer();
            defer.resolve(userConfig);
            sandbox.stub(portalService, "getUserConfig").returns(defer.promise);
            defer = $injector.get("$q").defer();
            defer.resolve(summary);
            sandbox.stub(portalService, "getSpeedEstimationSummary").returns(defer.promise);
            defer = $injector.get("$q").defer();
            defer.resolve(calibrationData);
            sandbox.stub(portalService, "getSpeedEstimationCalibrationData").returns(defer.promise);

            defer = $injector.get("$q").defer();
            defer.reject();
            sandbox.stub(portalService, "postSpeedEstimationAddPlate").returns(defer.promise);

            controller = $controller("SpeedEstimationController", { portalService: portalService, $scope: scope, $timeout: $timeout });
            $rootScope.$apply();
        });


        var spy = sandbox.stub(controller, "doneEdit");
        var spyAlert = sandbox.stub(controller, "showStickyAlert");

        controller.plateToAdd = "allo";
        controller.addPlate();
        $rootScope.$digest();
        sinon.assert.notCalled(spy);
        expect(controller.plateToAdd).to.equal("allo");
        sinon.assert.calledOnce(spyAlert);
    });

    it("updateSummary should call portalService.getSpeedEstimationSummary with success then update calibrationStateSummary with null value and not call updateCalibrationStateClass", function () {
        var d, nullSummary, summaryStub;
        inject(function ($injector, $rootScope, $controller) {
            nullSummary = $injector.get("$q").defer();
            var defer = $injector.get("$q").defer();
            defer.resolve(userConfig);
            sandbox.stub(portalService, "getUserConfig").returns(defer.promise);
            d = $injector.get("$q").defer();
            d.resolve(summary);
            summaryStub = sandbox.stub(portalService, "getSpeedEstimationSummary").returns(d.promise);
            defer = $injector.get("$q").defer();
            defer.resolve(calibrationData);
            sandbox.stub(portalService, "getSpeedEstimationCalibrationData").returns(defer.promise);

            controller = $controller("SpeedEstimationController", { portalService: portalService, $scope: scope, $timeout: $timeout });
            $rootScope.$apply();
        });


        var spy = sandbox.stub(controller, "updateCalibrationStateClass");
        summaryStub.restore();
        nullSummary.resolve({ data: null });
        sandbox.stub(portalService, "getSpeedEstimationSummary").returns(nullSummary.promise);

        controller.updateSummary();
        $rootScope.$digest();
        sinon.assert.notCalled(spy);
        expect(controller.calibrationStateSummary).to.be.null;
    });

    it("updateSummary should call portalService.getSpeedEstimationSummary with success then update calibrationStateSummary with new value and call updateCalibrationStateClass", function () {
        var d, nullSummary, summaryStub;
        inject(function ($injector, $rootScope, $controller) {
            nullSummary = $injector.get("$q").defer();
            var defer = $injector.get("$q").defer();
            defer.resolve(userConfig);
            sandbox.stub(portalService, "getUserConfig").returns(defer.promise);
            d = $injector.get("$q").defer();
            d.resolve(summary);
            summaryStub = sandbox.stub(portalService, "getSpeedEstimationSummary").returns(d.promise);
            defer = $injector.get("$q").defer();
            defer.resolve(calibrationData);
            sandbox.stub(portalService, "getSpeedEstimationCalibrationData").returns(defer.promise);

            controller = $controller("SpeedEstimationController", { portalService: portalService, $scope: scope, $timeout: $timeout });
            $rootScope.$apply();
        });


        var spy = sandbox.stub(controller, "updateCalibrationStateClass");
        summaryStub.restore();
        nullSummary.resolve({ data: "12345" });
        sandbox.stub(portalService, "getSpeedEstimationSummary").returns(nullSummary.promise);

        controller.updateSummary();
        $rootScope.$digest();
        sinon.assert.calledOnce(spy);
        expect(controller.calibrationStateSummary).to.equal("12345");
    });

    it("cancelEdit should updateSummary with success then change state to main.config.lpr.speedEstimation", function () {
        var stateSpy, d;
        inject(function ($injector, $rootScope, $controller) {
            d = $injector.get("$q").defer();
            var defer = $injector.get("$q").defer();
            defer.resolve(userConfig);
            sandbox.stub(portalService, "getUserConfig").returns(defer.promise);
            defer = $injector.get("$q").defer();
            defer.resolve(summary);
            sandbox.stub(portalService, "getSpeedEstimationSummary").returns(defer.promise);
            defer = $injector.get("$q").defer();
            defer.resolve(calibrationData);
            sandbox.stub(portalService, "getSpeedEstimationCalibrationData").returns(defer.promise);

            stateSpy = sinon.stub($state, "go");

            controller = $controller("SpeedEstimationController", { portalService: portalService, $scope: scope, $timeout: $timeout, $state: $state });
            $rootScope.$apply();
        });

        d.resolve();
        var spy = sandbox.stub(controller, "updateSummary").returns(d.promise);

        controller.cancelEdit();
        $rootScope.$digest();
        sinon.assert.calledOnce(spy);
        assert(stateSpy.calledWith("main.config.lpr.speedEstimation"));
    });

    it("edit should change state to main.config.lpr.speedEstimationEdit", function () {
        var stateSpy, d;
        inject(function ($injector, $rootScope, $controller) {
            d = $injector.get("$q").defer();
            var defer = $injector.get("$q").defer();
            defer.resolve(userConfig);
            sandbox.stub(portalService, "getUserConfig").returns(defer.promise);
            defer = $injector.get("$q").defer();
            defer.resolve(summary);
            sandbox.stub(portalService, "getSpeedEstimationSummary").returns(defer.promise);
            defer = $injector.get("$q").defer();
            defer.resolve(calibrationData);
            sandbox.stub(portalService, "getSpeedEstimationCalibrationData").returns(defer.promise);

            stateSpy = sinon.stub($state, "go");

            controller = $controller("SpeedEstimationController", { portalService: portalService, $scope: scope, $timeout: $timeout, $state: $state });
            $rootScope.$apply();
        });

        controller.edit();
        sinon.assert.calledOnce(stateSpy);
        assert(stateSpy.calledWith("main.config.lpr.speedEstimationEdit"));
    });

    it("restart should open a modal window to ask for confirmation", function () {
        inject(function ($injector, $rootScope, $controller, $uibModal) {
            var defer = $injector.get("$q").defer();
            defer.resolve(userConfig);
            sandbox.stub(portalService, "getUserConfig").returns(defer.promise);
            defer = $injector.get("$q").defer();
            defer.resolve(summary);
            sandbox.stub(portalService, "getSpeedEstimationSummary").returns(defer.promise);
            defer = $injector.get("$q").defer();
            defer.resolve(calibrationData);
            sandbox.stub(portalService, "getSpeedEstimationCalibrationData").returns(defer.promise);

            sandbox.stub($uibModal, "open").returns({ close: function () { } });

            controller = $controller("SpeedEstimationController", { portalService: portalService, $scope: scope, $timeout: $timeout, $state: $state, $uibModal: $uibModal });
            $rootScope.$apply();
        });

        controller.cancelCalibration();
    });

    it("deletePlateToWaitFor should call portalService.deleteSpeedEstimationPlateToWaitFor with success then call doneEdit with success", function () {
        var d, spyDelete;

        inject(function ($injector, $rootScope, $controller) {
            d = $injector.get("$q").defer();
            var defer = $injector.get("$q").defer();
            defer.resolve(userConfig);
            sandbox.stub(portalService, "getUserConfig").returns(defer.promise);
            defer = $injector.get("$q").defer();
            defer.resolve(summary);
            sandbox.stub(portalService, "getSpeedEstimationSummary").returns(defer.promise);
            defer = $injector.get("$q").defer();
            defer.resolve(calibrationData);
            sandbox.stub(portalService, "getSpeedEstimationCalibrationData").returns(defer.promise);

            defer = $injector.get("$q").defer();
            defer.resolve();
            spyDelete = sandbox.stub(portalService, "deleteSpeedEstimationPlateToWaitFor").returns(defer.promise);
            
            controller = $controller("SpeedEstimationController", { portalService: portalService, $scope: scope, $timeout: $timeout, $state: $state });
            $rootScope.$apply();
        });

        d.resolve();
        var spy = sandbox.stub(controller, "doneEdit").returns(d.promise);

        var id = 1;
        var resolved = false;
        controller.deletePlateToWaitFor(id).then(function () {
            resolved = true;
        });
        $rootScope.$digest();
        sinon.assert.calledOnce(spyDelete);
        sinon.assert.calledWith(spyDelete, id);
        sinon.assert.calledOnce(spy);
        expect(resolved).to.be.true;
    });

    it("deletePlateToWaitFor should call portalService.deleteSpeedEstimationPlateToWaitFor with failure then call not doneEdit", function () {
        var d, spyDelete;

        inject(function ($injector, $rootScope, $controller) {
            d = $injector.get("$q").defer();
            var defer = $injector.get("$q").defer();
            defer.resolve(userConfig);
            sandbox.stub(portalService, "getUserConfig").returns(defer.promise);
            defer = $injector.get("$q").defer();
            defer.resolve(summary);
            sandbox.stub(portalService, "getSpeedEstimationSummary").returns(defer.promise);
            defer = $injector.get("$q").defer();
            defer.resolve(calibrationData);
            sandbox.stub(portalService, "getSpeedEstimationCalibrationData").returns(defer.promise);

            defer = $injector.get("$q").defer();
            defer.reject();
            spyDelete = sandbox.stub(portalService, "deleteSpeedEstimationPlateToWaitFor").returns(defer.promise);

            controller = $controller("SpeedEstimationController", { portalService: portalService, $scope: scope, $timeout: $timeout, $state: $state });
            $rootScope.$apply();
        });

        d.resolve();
        var spyDone = sandbox.stub(controller, "doneEdit").returns(d.promise);

        var id = 1;
        var rejected = false;
        controller.deletePlateToWaitFor(id).then(function () {
            assert();
        }, function () {
            rejected = true;
        });
        $rootScope.$digest();
        sinon.assert.calledOnce(spyDelete);
        sinon.assert.calledWith(spyDelete, id);
        sinon.assert.notCalled(spyDone);
        expect(rejected).to.be.true;
    });

    it("deletePlateToWaitFor should call portalService.deleteSpeedEstimationPlateToWaitFor with success then call doneEdit with failure", function () {
        var d, spyDelete;

        inject(function ($injector, $rootScope, $controller) {
            d = $injector.get("$q").defer();
            var defer = $injector.get("$q").defer();
            defer.resolve(userConfig);
            sandbox.stub(portalService, "getUserConfig").returns(defer.promise);
            defer = $injector.get("$q").defer();
            defer.resolve(summary);
            sandbox.stub(portalService, "getSpeedEstimationSummary").returns(defer.promise);
            defer = $injector.get("$q").defer();
            defer.resolve(calibrationData);
            sandbox.stub(portalService, "getSpeedEstimationCalibrationData").returns(defer.promise);

            defer = $injector.get("$q").defer();
            defer.resolve();
            spyDelete = sandbox.stub(portalService, "deleteSpeedEstimationPlateToWaitFor").returns(defer.promise);

            controller = $controller("SpeedEstimationController", { portalService: portalService, $scope: scope, $timeout: $timeout, $state: $state });
            $rootScope.$apply();
        });

        d.reject();
        var spy = sandbox.stub(controller, "doneEdit").returns(d.promise);

        var id = 1;
        var rejected = false;
        controller.deletePlateToWaitFor(id).then(function () {
            assert();
        }, function () {
            rejected = true;
        });
        $rootScope.$digest();
        sinon.assert.calledOnce(spyDelete);
        sinon.assert.calledWith(spyDelete, id);
        sinon.assert.calledOnce(spy);       
        expect(rejected).to.be.true;
    });
});