﻿"use strict";

describe("Vehicle Detection controller", function () {

    var controller, portalService, signalRService, $rootScope, $timeout, scope, $state, checkChangeStateFactory, $uibModal, $translate, sandbox, stringFormatFilter, $window, mockFactory;
    var backendServerUrl = "http://localhost:8080";
    var backendLoggerUrl = "http://localhost:88";
    var userConfig = {
         data: {
              ReadModeConfiguration: {
                   ReadingMode: 0
              },
              VehicleDetectionConfiguration: {
                   ValidationRegion: {
                       ValidationRegionType: 0,
                       VertexList: []
                   },
                   ContextImagePointA: {
                        X: 0, Y: 0
                   },
                   ContextImagePointB: {
                        X: 0, Y: 0
                   },
                   LprImagePointA: {
                        X: 0, Y: 0
                   },
                   LprImagePointB: {
                        X: 0, Y: 0
                   },
                   ContextImageSize: {},
                   CamerasSnapshot: []
              }
         }
    };
    var lprContexts = { data: "asdasdsad" };
    var mockProperties = { data: false };

    beforeEach(function () {
        module("portalApp.services", function ($provide) {
            $provide.value("backendServerUrl", backendServerUrl);
            $provide.value("backendLoggerUrl", backendLoggerUrl);
        });
        module("portalApp.controllers");
        module("portalApp.filters"); // stringFormatFilter
        module("ui.bootstrap"); // for $uibModal
        module("pascalprecht.translate"); // for $translate
        module("portalApp.testing"); // load the module, otherwise: [$injector:unpr] Unknown provider: mockFactoryProvider <- mockFactory

        sandbox = sinon.sandbox.create();

        $window = {
            sessionStorage: { accessToken: "Bonjour", refreshToken: "Allo" }, location: { href: "Mourial", reload: function (value) { } },
            document: {
                getElementById: function (id) {
                    return {
                        getContext: null,
                    }
                }
            }
        };

        inject(function ($injector, _$translate_) {
            stringFormatFilter = $injector.get("stringFormatFilter");
            portalService = $injector.get("portalService");
            signalRService = $injector.get("signalRService");
            checkChangeStateFactory = $injector.get("checkChangeStateFactory");
            $rootScope = $injector.get("$rootScope");
            scope = $rootScope.$new;
            scope.$on = function () { };
            $timeout = $injector.get("$timeout");
            $state = $injector.get("$state");
            $uibModal = $injector.get("$uibModal");
            $translate = _$translate_;
            $translate.refresh = function() {};
            sandbox.stub(signalRService, "initializeVehicleDetectionCalibrationHub");
            mockFactory = $injector.get("mockFactory");
        });

        mockFactory.setSandbox(sandbox);
        mockFactory.stubGetUserConfig(userConfig);
        mockFactory.stubGetProperties(mockProperties);
    });

    afterEach(function () {
        sandbox.restore();
    });

    it("should be created successfully", function () {
        inject(function ($injector, $rootScope, $controller) {
            sandbox.stub(checkChangeStateFactory, "checkFormOnStateChange");
            var d = $injector.get("$q").defer();
            d.resolve(lprContexts);
            sandbox.stub(portalService, "getLprContexts").returns(d.promise);
            controller = $controller("VehicleDetectionController", { portalService: portalService, signalRService: signalRService, $scope: scope, $rootScope: $rootScope, stringFormatFilter: stringFormatFilter, $window: $window });
            $rootScope.$apply();
        });

        expect(controller).to.be.an('object');
        expect(controller.title).to.equal("VehicleDetectionController");
    });

    it("should detect when controller initialization is a failure", function () {
        inject(function ($injector, $rootScope, $controller) {
            //userConfig = { data: { NetworkConfiguration: { NetworkType: 1 }, ReadModeConfiguration: { ReadingMode: 0 }, VehicleDetectionConfiguration: { ValidationRegion: { ValidationRegionType: 0, VertexList: [] }, ContextImagePointA: { X: 0, Y: 0 }, ContextImagePointB: { X: 0, Y: 0 }, LprImagePointA: { X: 0, Y: 0 }, LprImagePointB: { X: 0, Y: 0 }, ContextImageSize: {}, CamerasSnapshot: [] } } };
            mockFactory.stubGetUserConfig(null, false); // recreate a specific stub here, and also reject
            sandbox.stub(checkChangeStateFactory, "checkFormOnStateChange");
            controller = $controller("VehicleDetectionController", { portalService: portalService, signalRService: signalRService, $scope: scope, $rootScope: $rootScope, $timeout: $timeout, stringFormatFilter: stringFormatFilter, $window: $window });
            $rootScope.$apply();
        });

        $rootScope.reconnecting = false;
        $rootScope.connectionLost = true;
        $rootScope.isLoading = false;
        $timeout.flush();
        $rootScope.reconnecting = true;
    });

    it("cancel should set userConfig back to its original content and call form.$setPristine", function () {
        var spy;
        inject(function ($injector, $rootScope, $controller) {
            controller = $controller("VehicleDetectionController", { portalService: portalService, signalRService: signalRService, $scope: scope, $rootScope: $rootScope, $timeout: $timeout, stringFormatFilter: stringFormatFilter, $window: $window });
            $rootScope.$apply();
        });

        controller.userConfig = { data: { ReadModeConfiguration: { ReadingMode: 0 }, VehicleDetectionConfiguration: { ValidationRegion: { ValidationRegionType: 0, VertexList: [] }, ContextImagePointA: { X: 0, Y: 0 }, ContextImagePointB: { X: 0, Y: 0 }, LprImagePointA: { X: 0, Y: 0 }, LprImagePointB: { X: 0, Y: 0 }, ContextImageSize: {}, CamerasSnapshot: [] } }, fake: "asdhasdhjashdjkhas" };
        var original = controller.virtualLoopEnabled;
        controller.virtualLoopEnabled = !original;
        controller.form = { $valid: true, $setPristine: function () { } };
        spy = sandbox.stub(controller.form, "$setPristine");
        controller.cancel();
        $rootScope.$apply();
        expect(angular.equals(controller.userConfig, userConfig)); // user config was restored to original value
        expect(controller.virtualLoopEnabled).equals(original);
        sinon.assert.calledOnce(spy);
    });

    it("save should call portalService.setUserConfig then display a sticky alert and reset the form when save is a success", function () {
        var spyPristine, spyConfig;
        inject(function ($injector, $rootScope, $controller) {
            sandbox.stub(checkChangeStateFactory, "checkFormOnStateChange");
            userConfig = { data: { NetworkConfiguration: { NetworkType: 1 }, ReadModeConfiguration: { ReadingMode: 0 }, VehicleDetectionConfiguration: { ValidationRegion: { ValidationRegionType: 0, VertexList: [] }, ContextImagePointA: { X: 0, Y: 0 }, ContextImagePointB: { X: 0, Y: 0 }, LprImagePointA: { X: 0, Y: 0 }, LprImagePointB: { X: 0, Y: 0 }, ContextImageSize: {}, CamerasSnapshot: [] } } }; // this is the default value for comparison reference
            mockFactory.stubGetUserConfig(userConfig); // recreate a specific stub here
            spyConfig = mockFactory.stubSetUserConfig();
            controller = $controller("VehicleDetectionController", { portalService: portalService, signalRService: signalRService, $scope: scope, $rootScope: $rootScope, $timeout: $timeout, stringFormatFilter: stringFormatFilter, $window: $window });
            $rootScope.$apply();
        });

        controller.userConfig.NetworkConfiguration.NetworkType = 2; // override the default value
        controller.form = { $valid: true, $setPristine: function () { } };
        spyPristine = sandbox.stub(controller.form, "$setPristine");
        controller.save();
        $rootScope.$digest();
        sinon.assert.calledOnce(spyPristine);// form is reset
        expect(controller.show).to.be.true;
        expect(controller.alert.type).to.equal('success');
        expect(controller.alert.msg).to.equal('STE_SENTENCE_ConfigurationSavedSuccessfully');
        $timeout.flush();
        expect(controller.show).to.be.false;
        expect(spyConfig.args[0][0].NetworkConfiguration.NetworkType).to.equal(2); // should reflect the saved value
    });

    it("save should call portalService.setUserConfig then display a sticky alert and doesn't reset the form when save is a failure", function () {
        var spyPristine, spyConfig;
        inject(function ($injector, $rootScope, $controller) {
            userConfig = { data: { NetworkConfiguration: { NetworkType: 1 }, ReadModeConfiguration: { ReadingMode: 0 }, VehicleDetectionConfiguration: { ValidationRegion: { ValidationRegionType: 0, VertexList: [] }, ContextImagePointA: { X: 0, Y: 0 }, ContextImagePointB: { X: 0, Y: 0 }, LprImagePointA: { X: 0, Y: 0 }, LprImagePointB: { X: 0, Y: 0 }, ContextImageSize: {}, CamerasSnapshot: [] } } }; // this is the default value for comparison reference
            mockFactory.stubGetUserConfig(userConfig); // recreate a specific stub here
            sandbox.stub(checkChangeStateFactory, "checkFormOnStateChange");
            spyConfig = mockFactory.stubSetUserConfig(null, false); // reject
            controller = $controller("VehicleDetectionController", { portalService: portalService, signalRService: signalRService, $scope: scope, $rootScope: $rootScope, $timeout: $timeout, stringFormatFilter: stringFormatFilter, $window: $window });
            $rootScope.$apply();
        });
        controller.form = { $valid: true, $setPristine: function () { } };
        spyPristine = sandbox.stub(controller.form, "$setPristine");
        controller.userConfig.NetworkConfiguration.NetworkType = 2; // don't care the data as long as it differs from the default value
        controller.save();
        $rootScope.$digest();
        sinon.assert.notCalled(spyPristine); // form is not reset
        expect(controller.show).to.be.true;
        expect(controller.alert.type).to.equal('danger');
        expect(controller.alert.msg).to.equal('STE_SENTENCE_CouldNotSaveConfiguration');
        $timeout.flush();
        expect(controller.show).to.be.false;
        expect(spyConfig.args[0][0].NetworkConfiguration.NetworkType).to.equal(2); // modified form value is preserved
    });

    it("configure button brings to edit page", function () {
        inject(function ($injector, $controller, $rootScope) {
            userConfig = { data: { NetworkConfiguration: { NetworkType: 1 }, ReadModeConfiguration: { ReadingMode: 0 }, VehicleDetectionConfiguration: { ValidationRegion: { ValidationRegionType: 0, VertexList: [] }, ContextImagePointA: { X: 0, Y: 0 }, ContextImagePointB: { X: 0, Y: 0 }, LprImagePointA: { X: 0, Y: 0 }, LprImagePointB: { X: 0, Y: 0 }, ContextImageSize: {}, CamerasSnapshot: [] } } }; // this is the default value for comparison reference
            mockFactory.stubGetUserConfig(userConfig); // recreate a specific stub here
            sandbox.stub(checkChangeStateFactory, "checkFormOnStateChange");
            controller = $controller("VehicleDetectionController", { portalService: portalService, signalRService: signalRService, $scope: scope, userConfig: userConfig, $window: $window });
            $rootScope.$apply();
        });

        var stateSpy = sandbox.stub($state, "go");
        $state.current.name = "main.config.lpr.vehicleDetection";
        controller.configureFovMatch();
        sinon.assert.calledWith(stateSpy, "main.config.lpr.vehicleDetectionEdit");
    });

    it("should display warning if no zone defined", function () {
        inject(function ($injector, $controller, $rootScope) {
            userConfig = { data: { NetworkConfiguration: { NetworkType: 1 }, ReadModeConfiguration: { ReadingMode: 0 }, VehicleDetectionConfiguration: { ValidationRegion: null, ContextImagePointA: { X: 0, Y: 0 }, ContextImagePointB: { X: 0, Y: 0 }, LprImagePointA: { X: 0, Y: 0 }, LprImagePointB: { X: 0, Y: 0 }, ContextImageSize: {}, CamerasSnapshot: [] } } }; // this is the default value for comparison reference
            mockFactory.stubGetUserConfig(userConfig); // recreate a specific stub here
            controller = $controller("VehicleDetectionController", { portalService: portalService, signalRService: signalRService, $scope: scope, userConfig: userConfig, $window: $window, $translate: $translate });

            $rootScope.$apply();
        });

        $rootScope.$emit('$translateChangeSuccess', {});
        $timeout.flush(); // trigger the timer now

        expect(controller).to.be.an('object');
        expect(controller.zoneDefined).to.be.false;
        expect(controller.CurrentZoneUsageName).to.be.equal("STE_LABEL_DetectionZone_EnteringOrExiting"); // default value set by the controller
    });

    it("should display vehicle path summary if zone defined", function () {
        inject(function ($injector, $controller, $rootScope) {
            userConfig = { data: { NetworkConfiguration: { NetworkType: 1 }, ReadModeConfiguration: { ReadingMode: 0 }, VehicleDetectionConfiguration: { ValidationRegion: { ValidationRegionType: 1, VertexList: [{ X: 10, Y: 10 }, { X: 20, Y: 10 }, { X: 20, Y: 20 }, { X: 10, Y: 20 }] }, ContextImagePointA: { X: 0, Y: 0 }, ContextImagePointB: { X: 0, Y: 0 }, LprImagePointA: { X: 0, Y: 0 }, LprImagePointB: { X: 0, Y: 0 }, ContextImageSize: {}, CamerasSnapshot: [] } } }; // this is the default value for comparison reference
            mockFactory.stubGetUserConfig(userConfig); // recreate a specific stub here

            controller = $controller("VehicleDetectionController", { portalService: portalService, signalRService: signalRService, $scope: scope, userConfig: userConfig, $window: $window, $translate: $translate });

            $rootScope.$apply();
        });

        $rootScope.$emit('$translateChangeSuccess', {});
        $timeout.flush(); // trigger the timer now

        expect(controller).to.be.an('object');
        expect(controller.zoneDefined).to.be.true;
        expect(controller.CurrentZoneUsageName).to.be.equal("STE_LABEL_DetectionZone_Exiting");
    });

});