﻿"use strict";

describe("Vehicle Detection Edit controller", function () {
    var controller, portalService, signalRService, $rootScope, $timeout, scope, $state, checkChangeStateFactory, $uibModal, $translate, sandbox, stringFormatFilter, $window;
    var backendServerUrl = "http://localhost:8080";
    var backendLoggerUrl = "http://localhost:88";
    var userConfig = { data: { ReadModeConfiguration: { ReadingMode: 0 }, VehicleDetectionConfiguration: { ValidationRegion: { ValidationRegionType: 0, VertexList: [] }, ContextImagePointA: { X: 0, Y: 0 }, ContextImagePointB: { X: 0, Y: 0 }, LprImagePointA: { X: 0, Y: 0 }, LprImagePointB: { X: 0, Y: 0 }, ContextImageSize: {} } } };

    beforeEach(function () {
        module("portalApp.services", function ($provide) {
            $provide.value("backendServerUrl", backendServerUrl);
            $provide.value("backendLoggerUrl", backendLoggerUrl);
        });
        module("portalApp.controllers");
        module("portalApp.filters"); // stringFormatFilter
        module("ui.bootstrap"); // for $uibModal
        module("pascalprecht.translate"); // for $translate

        sandbox = sinon.sandbox.create();

        $window = {
            document: {
                getElementById: function (id) {
                    return {
                        getContext: null,
                    }
                }
            }
        };

        inject(function ($injector, _$translate_) {
            stringFormatFilter = $injector.get("stringFormatFilter");
            portalService = $injector.get("portalService");
            signalRService = $injector.get("signalRService");
            checkChangeStateFactory = $injector.get("checkChangeStateFactory");
            $rootScope = $injector.get("$rootScope");
            scope = $rootScope.$new;
            scope.$on = function () { };
            scope.$watch = function() {};
            $timeout = $injector.get("$timeout");
            $state = $injector.get("$state");
            $uibModal = $injector.get("$uibModal");
            $translate = _$translate_;
            sandbox.stub(signalRService, "initializeFeedHub");
            sandbox.stub(signalRService, "initializeVehicleDetectionCalibrationHub");
            sandbox.stub(checkChangeStateFactory, "checkFormOnStateChange");
            sandbox.stub($translate, "refresh");

            // Mock the getUserConfig service
            var d = $injector.get("$q").defer();
            d.resolve(userConfig);
            sandbox.stub(portalService, "getUserConfig").returns(d.promise);
        });
    });

    afterEach(function () {
        sandbox.restore();
    });

    it("should be created successfully", function () {
        inject(function ($injector, $rootScope, $controller) {
            controller = $controller("VehicleDetectionEditController", { portalService: portalService, signalRService: signalRService, $scope: scope, $rootScope: $rootScope, stringFormatFilter: stringFormatFilter });
            $rootScope.$apply();
        });

        expect(controller).to.be.an('object');
        expect(controller.title).to.equal("VehicleDetectionEditController");
    });

    it("should detect when controller initialization is a failure", function () {
        inject(function ($injector, $rootScope, $controller) {
            userConfig = { data: { NetworkConfiguration: { NetworkType: 1 }, ReadModeConfiguration: { ReadingMode: 0 }, VehicleDetectionConfiguration: { ValidationRegion: { ValidationRegionType: 0, VertexList: [] }, ContextImagePointA: { X: 0, Y: 0 }, ContextImagePointB: { X: 0, Y: 0 }, LprImagePointA: { X: 0, Y: 0 }, LprImagePointB: { X: 0, Y: 0 }, ContextImageSize: {} } } };

            // Mock the getUserConfig to return a failure
            var d = $injector.get("$q").defer();
            d.reject(); // failure
            portalService.getUserConfig.restore(); // original functional restore (restub the mock)
            sandbox.stub(portalService, "getUserConfig").returns(d.promise);
            controller = $controller("VehicleDetectionEditController", { portalService: portalService, signalRService: signalRService, $scope: scope, $rootScope: $rootScope, $timeout: $timeout, stringFormatFilter: stringFormatFilter });
            $rootScope.$apply();
        });

        $rootScope.reconnecting = false;
        $rootScope.connectionLost = true;
        $rootScope.isLoading = false;

        $timeout.flush();
        $rootScope.reconnecting = true;
    });

    it("select all zone link selects all the zone polygon", function () {
        inject(function ($injector, $rootScope, $controller) {
            controller = $controller("VehicleDetectionEditController", { portalService: portalService, signalRService: signalRService, $scope: scope, $rootScope: $rootScope, $timeout: $timeout, stringFormatFilter: stringFormatFilter });
            $rootScope.$apply();
        });

        var imageSize = { width: 1280, height: 960 };
        var alertSpy = sandbox.stub(controller, "showStickyAlert");

        controller.form = { $valid: false, $invalid: true, $dirty: false, $setDirty: function () { this.$dirty = true } };
        controller.isFovValid = true;
        controller.isZoneValid = false;

        controller.translate(); // Fill zoneUsages
        controller.selectZoneAll();

        // All the zone should be selected
        expect(JSON.stringify(controller.userConfig.VehicleDetectionConfiguration.ValidationRegion.VertexList)).to.equal(JSON.stringify([{ "X": 0, "Y": 0 }, { "X": imageSize.width - 1, "Y": 0 }, { "X": imageSize.width - 1, "Y": imageSize.height - 1 }, { "X": 0, "Y": imageSize.height - 1 }]));

        expect(controller.form.$dirty).to.be.true;
        expect(controller.form.$valid).to.be.true;
        expect(controller.isZoneValid).to.be.true;

        expect(controller.userConfig.VehicleDetectionConfiguration.ValidationRegion.ValidationRegionType).to.equal(2); // Entering or exiting
        sinon.assert.calledOnce(alertSpy);
    });

    it("clear zone link clears the zone polygon", function () {
        inject(function ($injector, $rootScope, $controller) {
            controller = $controller("VehicleDetectionEditController", { portalService: portalService, signalRService: signalRService, $scope: scope, $rootScope: $rootScope, $timeout: $timeout, stringFormatFilter: stringFormatFilter });
            $rootScope.$apply();
        });

        controller.form = { $valid: true, $invalid: false, $dirty: false, $setDirty: function() { this.$dirty = true} };
        controller.isFovValid = true;
        controller.isZoneValid = true;
        controller.userConfig.VehicleDetectionConfiguration.ValidationRegion.VertexList = [{ X: 1, Y: 1 }, { X: 3, Y: 3 }, { X: 5, Y: 5 }];

        controller.clearZone(false);

        expect(controller.userConfig.VehicleDetectionConfiguration.ValidationRegion.VertexList.length).to.equal(0);
        expect(controller.form.$dirty).to.be.true;
        expect(controller.form.$valid).to.be.false; // zone is now mandatory
        expect(controller.isZoneValid).to.be.false; // zone is now mandatory
    });

    it("cancel button brings to summary page", function () {
        inject(function ($injector, $rootScope, $controller) {
            controller = $controller("VehicleDetectionEditController", { portalService: portalService, signalRService: signalRService, $scope: scope, $rootScope: $rootScope, stringFormatFilter: stringFormatFilter });
            $rootScope.$apply();
            sandbox.stub(signalRService, "startFeed");
            sandbox.stub(signalRService, "stopFeed");
        });

        var stateSpy = sandbox.stub($state, "go");
        $state.current.name = "main.config.lpr.vehicleDetectionEdit";
        controller.form = { $valid: true, $setPristine: function () { } };

        controller.cancel();

        sinon.assert.calledWith(stateSpy, "main.config.lpr.vehicleDetection");
    });

    it("next button brings from page 1 to page 2", function () {
        inject(function ($injector, $rootScope, $controller) {
            controller = $controller("VehicleDetectionEditController", { portalService: portalService, signalRService: signalRService, $scope: scope, $rootScope: $rootScope, stringFormatFilter: stringFormatFilter });
            $rootScope.$apply();
            sandbox.stub(signalRService, "startFeed");
            sandbox.stub(signalRService, "stopFeed");
        });

        controller.editFovMatch = true;
        controller.editDetectionZone = undefined;

        controller.gotoDetectionZone();

        expect(controller.editFovMatch).to.be.false;
        expect(controller.editDetectionZone).to.be.true;
    });

    it("previous button brings from page 2 to page 1", function () {
        inject(function ($injector, $rootScope, $controller) {
            controller = $controller("VehicleDetectionEditController", { portalService: portalService, signalRService: signalRService, $scope: scope, $rootScope: $rootScope, stringFormatFilter: stringFormatFilter });
            $rootScope.$apply();
            sandbox.stub(signalRService, "startFeed");
            sandbox.stub(signalRService, "stopFeed");
        });

        controller.editFovMatch = undefined;
        controller.editDetectionZone = true;

        controller.gotoFovMatch();

        expect(controller.editFovMatch).to.be.true;
        expect(controller.editDetectionZone).to.be.false;
    });

    it("done button saves (successfully) and brings to summary page", function () {
        var defer;
        inject(function ($injector, $controller, $rootScope) {
            userConfig = {
                data: {
                    NetworkConfiguration: { NetworkType: 1 },
                    ReadModeConfiguration: { ReadingMode: 0 },
                    VehicleDetectionConfiguration: {
                        ValidationRegion: null,
                        ContextImagePointA: { X: 0, Y: 0 },
                        ContextImagePointB: { X: 0, Y: 0 },
                        LprImagePointA: { X: 0, Y: 0 },
                        LprImagePointB: { X: 0, Y: 0 },
                        ContextImageSize: {}
                    }
                }
            }; // this is the default value for comparison reference

            var d2 = $injector.get("$q").defer();
            d2.resolve(); // call to setVehDetectionConfig succeeds
            sandbox.stub(portalService, "setVehDetectionConfig").returns(d2.promise);

            defer = $injector.get("$q").defer();
            defer.resolve();
            
            controller = $controller("VehicleDetectionEditController", { portalService: portalService, signalRService: signalRService, $scope: scope, userConfig: userConfig, $window: $window });
            $rootScope.$apply();
        });

        var stateSpy = sandbox.stub($state, "go");
        $state.current.name = "main.config.lpr.vehicleDetectionEdit";
        controller.form = { $valid: true, $setPristine: function () { } };
        var spy = sandbox.stub(controller.form, "$setPristine");
        var spyEndZoomAndFocus = sandbox.stub(controller, "endZoomAndFocus").returns(defer.promise);
        controller.userConfig.VehicleDetectionConfiguration = {
            ContextImagePointA: { X: 10, Y: 11 },
            ContextImagePointB: { X: 12, Y: 13 },
            LprImagePointA: { X: 20, Y: 21 },
            LprImagePointB: { X: 22, Y: 23 },
            ValidationRegion: { VertexList: [{ X: 1, Y: 1 }, { X: 3, Y: 3 }, { X: 5, Y: 5 }] }
        };

        controller.done();
        $rootScope.$digest(); // make sure all promises are terminated

        expect(controller.userConfig.VehicleDetectionConfiguration.ContextImagePointA.X).equal(10);
        expect(controller.userConfig.VehicleDetectionConfiguration.ContextImagePointA.Y).equal(11);
        expect(controller.userConfig.VehicleDetectionConfiguration.ContextImagePointB.X).equal(12);
        expect(controller.userConfig.VehicleDetectionConfiguration.ContextImagePointB.Y).equal(13);
        expect(controller.userConfig.VehicleDetectionConfiguration.LprImagePointA.X).equal(20);
        expect(controller.userConfig.VehicleDetectionConfiguration.LprImagePointA.Y).equal(21);
        expect(controller.userConfig.VehicleDetectionConfiguration.LprImagePointB.X).equal(22);
        expect(controller.userConfig.VehicleDetectionConfiguration.LprImagePointB.Y).equal(23);
        sinon.assert.calledWith(stateSpy, "main.config.lpr.vehicleDetection");
        sinon.assert.calledOnce(spy);
        sinon.assert.calledOnce(spyEndZoomAndFocus);
    });

    it("done button fails and stays in page 2", function () {
        var defer;
        inject(function ($injector, $controller, $rootScope) {
            userConfig = {
                data: {
                    NetworkConfiguration: { NetworkType: 1 },
                    ReadModeConfiguration: { ReadingMode: 0 },
                    VehicleDetectionConfiguration: {
                        ValidationRegion: null,
                        ContextImagePointA: { X: 0, Y: 0 },
                        ContextImagePointB: { X: 0, Y: 0 },
                        LprImagePointA: { X: 0, Y: 0 },
                        LprImagePointB: { X: 0, Y: 0 },
                        ContextImageSize: {}
                    }
                }
            }; // this is the default value for comparison reference

            var d2 = $injector.get("$q").defer();
            d2.reject({ status: 400, data: { Message: "HasVertexOutsideOfContextFov" } }); // call to setVehDetectionConfig fails
            sandbox.stub(portalService, "setVehDetectionConfig").returns(d2.promise);

            defer = $injector.get("$q").defer();
            defer.resolve();

            controller = $controller("VehicleDetectionEditController", { portalService: portalService, signalRService: signalRService, $scope: scope, userConfig: userConfig, $window: $window });
            $rootScope.$apply();
        });

        var alertSpy = sandbox.stub(controller, "showStickyAlert");
        var stateSpy = sandbox.stub($state, "go");
        $state.current.name = "main.config.lpr.vehicleDetectionEdit";
        controller.form = { $valid: true, $setPristine: function () { } };
        var spy = sandbox.stub(controller.form, "$setPristine");
        var spyEndZoomAndFocus = sandbox.stub(controller, "endZoomAndFocus").returns(defer.promise);
        controller.contextCameraWidget = { pointA: { x: 10, y: 11 }, pointB: { x: 12, y: 13 }, width: 1260, height: 980 };
        controller.lprCameraWidget = { pointA: { x: 20, y: 21 }, pointB: { x: 22, y: 23 } };
        controller.userConfig.VehicleDetectionConfiguration.ValidationRegion = { VertexList: [{ X: 1, Y: 1 }, { X: 3, Y: 3 }, { X: 5, Y: 5 }] };

        controller.done();
        $rootScope.$digest(); // make sure all promises are terminated

        sinon.assert.calledOnce(spyEndZoomAndFocus);
        sinon.assert.notCalled(stateSpy);
        sinon.assert.notCalled(spy);
        sinon.assert.calledOnce(alertSpy);
    });
});