﻿"use strict";

describe("Focus Factory", function () {
    var focusFactory, window, timeout, sandbox;

    beforeEach(function () {
        module("portalApp.factories");

        sandbox = sinon.sandbox.create();

        inject(function ($injector) {
            focusFactory = $injector.get("focusFactory");
            window = $injector.get("$window");
            timeout = $injector.get("$timeout");
        });
    });

    afterEach(function () {
        sandbox.restore();
    });
   
    it("should be created successfully", function() {
        expect(focusFactory).to.exist;
        expect(focusFactory).to.be.an('object');
    });

    it("doFocus method should focus on an valid element", function () {
        expect(focusFactory).to.exist;
        expect(focusFactory).to.be.an('object');

        var element = window.document.createElement("div");
        element.id = "password";

        var focusSpy = sinon.spy(element, "focus");

        var stub = sandbox.stub(window.document, "getElementById");
        stub.withArgs("password").returns(element);

        focusFactory.doFocus("password");

        timeout.flush();

        focusSpy.restore();
        stub.restore();
        assert(focusSpy.calledOnce);
    });

    it("doFocus method should throw an exception when passed an invalid element id", function () {
        expect(focusFactory).to.exist;
        expect(focusFactory).to.be.an('object');

        var stub = sandbox.stub(window.document, "getElementById");
        stub.withArgs("password").returns(null);

        focusFactory.doFocus("password");

        assert.throws(timeout.flush);
    });
});