﻿"use strict";

describe("LocalizationFactory Factory", function () {
    var $window;
    var mockNavigator = {
        language: "en-US"
        // don't care about userLanguage
    };

    var mockMoment = {
        locale: function (langKey) { }
    };

    beforeEach(function () {
        module("portalApp.factories", function($provide) {
            $provide.value("navigator", mockNavigator);
            $provide.value("moment", mockMoment);
        });
        module("portalApp.localization");

        inject(function (_$window_) {
            $window = _$window_;
            delete $window.sessionStorage.currentLanguage; // make sure it's deleted so that factory behaves normally
        });
    });

    it("should be created successfully", function () {
        var localizationFactory;
        inject(function ($injector) {
            localizationFactory = $injector.get("localizationFactory");
        });
        expect(localizationFactory).to.exist;
        expect(localizationFactory).to.be.an('object');
    });

    it("localizationFactory should be initialized in English", function () {
        var localizationFactory;
        inject(function ($injector) {
            mockNavigator.language = "en-US";
            localizationFactory = $injector.get("localizationFactory");
        });
        var value = localizationFactory.getData();
        expect(value.langKey).to.equal("en");
        expect(value.text).to.equal("English");
    });

    it("localizationFactory should be initialized in French", function () {
        var localizationFactory;
        inject(function ($injector) {
            mockNavigator.language = "fr-CA";
            localizationFactory = $injector.get("localizationFactory");
        });
        var value = localizationFactory.getData();
        expect(value.langKey).to.equal("fr");
        expect(value.text).to.equal("Français");
    });

    it("localizationFactory should be initialized in English (unsupported language)", function () {
        var localizationFactory;
        inject(function ($injector) {
            mockNavigator.language = "zz-XX"; // unsupported language
            localizationFactory = $injector.get("localizationFactory");
        });
        var value = localizationFactory.getData();
        expect(value.langKey).to.equal("en");
        expect(value.text).to.equal("English");
    });

    it("updateData method with unsupported language should save values properly", function () {
        var localizationFactory;
        inject(function($injector) {
            localizationFactory = $injector.get("localizationFactory");
        });
        var testLangKey = "xx";                
        localizationFactory.updateData(testLangKey);
        var value = localizationFactory.getData();
        expect(value.langKey).to.equal(testLangKey);
        expect(value.text).to.equal(testLangKey); // description same as key when unknown
    });

    it("updateData method with french language should save values properly", function () {
        var localizationFactory;
        inject(function ($injector) {
            localizationFactory = $injector.get("localizationFactory");
        });
        var testLangKey = "fr";
        localizationFactory.updateData(testLangKey);
        var value = localizationFactory.getData();
        expect(value.langKey).to.equal(testLangKey);
        expect(value.text).to.equal("Français");
    });
});