﻿//This is a mock for a small part of SignalR's javascript client.
//This code does not mock autogenerated SignalR hub scripts as the
//signalRService uses the connection.createHubProxy('HubName') method,
//followed by .on or .invoke to setup the receive and send methods respectively

// taken from here: http://www.thereformedprogrammer.net/mocking-signalr-client-for-better-unit-testing/

var mockSignalRClient = (function ($) {

        var mock = {};

        //first the items used by unit testing to see what has happened
        mock.callLog = null;
        mock.onFunctionDict = function () { };
        mock.doneFunc = null;
        mock.failFunc = null;
        mock.errorFunc = null;
        mock.createHubProxy = false;
        mock.channelOn = false;
        mock.channelInvoke = null;
        mock.startDone = false;
        mock.startFail = false;
        mock.connectionStart = false;
        mock.connectionError = false;
        mock.connectionStop = false;
        mock.connectionDisconnected = false;
        
        mock.reset = function () {
            mock.callLog = [];
            mock.onFunctionDict = function() { };
            mock.doneFunc = null;
            mock.failFunc = null;
            mock.errorFunc = null;
            mock.createHubProxy = false;
            mock.channelOn = false;
            mock.channelInvoke = null;
            mock.startDone = false;
            mock.startFail = false;
            mock.connectionStart = false;
            mock.connectionError = false;
            mock.connectionStop = false;
            mock.connectionDisconnected = false;
            mock.connectionReconnected = false;
            mock.connectionReconnecting = false;
        };

        //doneFail is the object returned by connection.start()
        var doneFail = {};
        doneFail.done = function (startFunc) {
            mock.startDone = true;
            mock.doneFunc = startFunc;
            return doneFail;
        };
        doneFail.fail = function (failFunc) {
            mock.startFail = true;
            mock.failFunc = failFunc;
            return doneFail;
        };

        //Channel is the object returned by connection.createHubProxy
        mock.channel = {};
        mock.channel.on = function (namedMessage, functionToCall) {
            mock.channelOn = true;
            mock.onFunctionDict[namedMessage] = functionToCall;
        };
        mock.channel.invoke = function (actionName, actionGuid) {
            mock.channelInvoke = actionName;
        };

        //connection is the object returned by $.hubConnection
        var connection = {};
        connection.createHubProxy = function (hubName) {
            mock.createHubProxy = true;
            return mock.channel;
        };
        connection.error = function (errorFunc) {
            mock.errorFunc = errorFunc;
        };
        connection.start = function () {
            mock.connectionStart = true;
            return doneFail;
        };
        connection.stop = function () {
            mock.connectionStop = true;
            return doneFail;
        };

        connection.reconnecting = function (reconnectingFunc) {
            mock.connectionReconnecting = true;
            return reconnectingFunc();
        };

        connection.reconnected = function (reconnectedFunc) {
            mock.connectionReconnected = true;
            return reconnectedFunc();
        };

        connection.disconnected = function(disconnectedFunc) {
            mock.connectionDisconnected = true;
            return disconnectedFunc();
        }

        connection.transport = { name : "pizza pocket" };

        //now we run once the method to add the hubConnection function to jQuery
        $.hubConnection = function () {
            return connection;
        };

        //Return the mock base which has all the error feedback information in it
        return mock;

    }(window.jQuery));