﻿"use strict";

describe("Portal Filter Module", function() {

    beforeEach(function() {
    
        module("portalApp.filters");
        module("pascalprecht.translate"); // for $translate
    });

    it("should define and use 'format' string extension successfully", function () {
        
        // String extension (format) handling numbers, strings and bool
        expect("{0}-{1}-{2}".format(1, "2", false)).to.be.equal("1-2-false");
    });

    it("should define and use 'bytes' filter", inject(function (bytesFilter) { // automatically inject "bytes" filter

        expect(bytesFilter).to.not.be.null;

        expect(bytesFilter(1023, 0)).to.be.equal("1023 STE_LABEL_BytesLowerCase");
        expect(bytesFilter(1024 * 1024 - 1, 0)).to.be.equal("1024 KiSTE_LABEL_BytesAbbreviation");
        expect(bytesFilter(1024 * 1024 * 1024 - 1, 0)).to.be.equal("1024 MiSTE_LABEL_BytesAbbreviation");
        expect(bytesFilter(1024 * 1024 * 1024 * 1024 - 1, 0)).to.be.equal("1024 GiSTE_LABEL_BytesAbbreviation");
        expect(bytesFilter(1024 * 1024 * 1024 * 1024 * 1024 - 1, 0)).to.be.equal("1 PiSTE_LABEL_BytesAbbreviation"); //!!! 1024 TiB ?
        expect(bytesFilter(1024 * 1024 * 1024 * 1024 * 1024 * 1024 - 1, 0)).to.be.equal("1 undefined"); //!!! 1024 PiB ?
        expect(bytesFilter(1111.1111 * 2000, 2)).to.be.equal("2.12 MiSTE_LABEL_BytesAbbreviation");
        expect(bytesFilter(1111 * 2000, 2)).to.be.equal("2.12 MiSTE_LABEL_BytesAbbreviation");

        expect(bytesFilter(0, 0)).to.be.equal("STE_LABEL_ZeroBytes"); // zero
        expect(bytesFilter(14)).to.be.equal("14 STE_LABEL_BytesLowerCase"); // no precision specified, integer (defaults to 0)
        expect(bytesFilter(27.83)).to.be.equal("27.8 STE_LABEL_BytesLowerCase"); // no precision specified, float (defaults to 1)
        expect(bytesFilter("abc", 0)).to.be.equal("-"); // invalid entry
    }));

    it("should define and use 'setDecimal' filter properly", inject(function (setDecimalFilter) { // automatically injects "setDecimal" filter

        expect(setDecimalFilter).to.not.be.null;

        expect(setDecimalFilter(1024.897, 0)).to.be.equal(1025); // zero decimals, rounded up
        expect(setDecimalFilter(1024.4, 0)).to.be.equal(1024); // zero decimals, not rounded up
        expect(setDecimalFilter(1111.123, 2)).to.be.equal(1111.12); // 2 decimals
        expect(setDecimalFilter("abc", 0)).to.be.equal("abc"); // NaN
    }));

    it("should define and use 'stringFormat' filter properly", inject(function (stringFormatFilter) { // automatically injects "stringFormat" filter

        expect(stringFormatFilter).to.not.be.null;

        //
        // NOTE: The commented out tests are failing, but since this code is not ours and we don't use it 
        //        with invalid arguments, it's just fine.
        //

        expect(stringFormatFilter("{0}-{1}-{2}-end", [10, "2", false])).to.equal("10-2-false-end");
        expect(stringFormatFilter("{0:3}", "2")).to.equal("2");
        expect(stringFormatFilter(null)).to.equal(null);
        expect(stringFormatFilter("xyz")).to.equal("xyz");
        //expect(stringFormatFilter("abc{{123")).to.equal("abc{123");
        //expect(stringFormatFilter("abc}}123")).to.equal("abc}123");
        //expect(stringFormatFilter("xyz{{000")).to.equal("xyz{000");
        expect(stringFormatFilter("{0:}", "10")).to.equal("1");
        expect(stringFormatFilter("{0}-{1}-{2}")).to.equal("{0}-{1}-{2}"); // missing arguments

        //expect(function() { stringFormatFilter("0}erreur"); }).to.throw(Error("format stringFormatBraceMismatch"));
        //expect(function() { stringFormatFilter("{0erreur"); }).to.throw(Error("format stringFormatBraceMismatch"));
        //expect(function() { stringFormatFilter("{0:xxx}", "2"); }).to.throw(Error("format stringFormatInvalid"));
    }));

    it("should define and use 'elapsedTime' filter properly", inject(function (elapsedTimeFilter) { // automatically injects "elapsedTime" filter

        expect(elapsedTimeFilter).to.not.be.null;

        var date0 = new Date(2016, 2, 10, 13, 58, 59);
        var date1 = new Date(2016, 2, 10, 13, 59, 13); // few seconds later
        var date2 = new Date(2016, 2, 10, 14, 3, 23); // few minutes later
        var date3 = new Date(2016, 2, 10, 19, 45, 17); // few hours later
        var date4 = new Date(2016, 2, 22, 22, 22, 22); // few days later
        var date5 = new Date(2016, 12, 10, 10, 10, 10); // few months later
        var date6 = new Date(2017, 2, 10, 11, 11, 11); // a year later
        var date7 = new Date(2033, 3, 33, 33, 33, 33); // few years later

        expect(elapsedTimeFilter(date0, date1)).to.be.equal("a few seconds ago");
        expect(elapsedTimeFilter(date0, date2)).to.be.equal("4 minutes ago");
        expect(elapsedTimeFilter(date0, date3)).to.be.equal("6 hours ago");
        expect(elapsedTimeFilter(date0, date4)).to.be.equal("12 days ago");
        expect(elapsedTimeFilter(date0, date5)).to.be.equal("10 months ago");
        expect(elapsedTimeFilter(date0, date6)).to.be.equal("a year ago");
        expect(elapsedTimeFilter(date0, date7)).to.be.equal("17 years ago");

        moment.locale('fr');

        expect(elapsedTimeFilter(date0, date1)).to.be.equal("il y a quelques secondes");
        expect(elapsedTimeFilter(date0, date2)).to.be.equal("il y a 4 minutes");
        expect(elapsedTimeFilter(date0, date3)).to.be.equal("il y a 6 heures");
        expect(elapsedTimeFilter(date0, date4)).to.be.equal("il y a 12 jours");
        expect(elapsedTimeFilter(date0, date5)).to.be.equal("il y a 10 mois");
        expect(elapsedTimeFilter(date0, date6)).to.be.equal("il y a un an");
        expect(elapsedTimeFilter(date0, date7)).to.be.equal("il y a 17 ans");

        var minDate = new Date(1,0,1,0,0,0);
        expect(elapsedTimeFilter(minDate, date0)).to.be.equal("-");
        expect(elapsedTimeFilter(minDate.toString(), date0.toString())).to.be.equal("-"); // string parameters

        expect(elapsedTimeFilter(null, date0)).to.be.equal("-"); // first param null
        expect(elapsedTimeFilter(date0, null)).to.be.equal("-"); // second param null
        expect(elapsedTimeFilter(null, null)).to.be.equal("-"); // both params null

        expect(elapsedTimeFilter(24, date0)).to.be.equal("-"); // first param is not a Date nor a string
        expect(elapsedTimeFilter(date0, 56)).to.be.equal("-"); // second param is not a Date nor a string

    }));

    it("should define and use 'gnLatLong' filter properly", inject(function (gnLatLongFilter) { // automatically injects "gnLatLong" filter

        expect(gnLatLongFilter).to.not.be.null;

        var result = gnLatLongFilter({ X: 0, Y: 0 });
        expect(result).to.be.equal("0°0'0\"STE_LABEL_NorthAbbreviation 0°0'0\"STE_LABEL_EastAbbreviation");

        var result = gnLatLongFilter({ X: -45.33, Y: 160.8 });
        expect(result).to.be.equal("160°48'0\"STE_LABEL_NorthAbbreviation 45°19'48\"STE_LABEL_WestAbbreviation");

        var result = gnLatLongFilter({ X: -32.63, Y: -36.093 });
        expect(result).to.be.equal("36°5'34.8\"STE_LABEL_SouthAbbreviation 32°37'48\"STE_LABEL_WestAbbreviation");

        var result = gnLatLongFilter({ X: 117.2133, Y: 50.45372 });
        expect(result).to.be.equal("50°27'13.392\"STE_LABEL_NorthAbbreviation 117°12'47.88\"STE_LABEL_EastAbbreviation");

        var result = gnLatLongFilter({ X: 54.44, Y: -33.0 });
        expect(result).to.be.equal("33°0'0\"STE_LABEL_SouthAbbreviation 54°26'24\"STE_LABEL_EastAbbreviation");

        var result = gnLatLongFilter("This is not a lat long object");
        expect(result).to.be.equal("-");
    }));
});