﻿"use strict";

describe("Portal Service", function () {

    var $httpBackend, portalService, $state, $base64, stateSpy, $window;
    var backendServerUrl = "http://localhost:80/api/";
    var backendLoggerUrl = "http://localhost:80/logger/";

    beforeEach(function() {
        module("portalApp.services", function($provide) {
            $provide.value("backendServerUrl", backendServerUrl);
            $provide.value("backendLoggerUrl", backendLoggerUrl);
        });

        inject(function ($injector) {
            $httpBackend = $injector.get("$httpBackend");
            portalService = $injector.get("portalService");
            $state = $injector.get("$state");
            $base64 = $injector.get("$base64");
            stateSpy = sinon.stub($state, "go");
            $window = $injector.get("$window");
        });

    });
       
    it("should be created successfully", function () {
        expect(portalService).to.exist;
    });

    if ("createHash method should create valid hash", function () {
        var value = "a";
        var hashedValue = portalService.createHash(value);
        expect(hashedValue).to.equal("0A04B971B03DA607CE6C455184037B660CA89F78");

    });

    it("login method should make the http post with proper loginrequest", function () {
        var username = "bob";
        var password = "mike;"
        var hash = portalService.createHash(password);

        // expectPOST to make sure this is called once.
        var loginRequest = { 'username': username, 'password': hash };

        $httpBackend.expectPOST("/login", loginRequest).respond(200);

        // make the call.
        portalService.login(username, password);

        // flush the backend to "execute" the request to do the expectedGET assertion.
        $httpBackend.flush();
    });

    it("getProperties method should make http get with proper header", function () {
        var header = { "Accept": "application/json, text/plain, */*" };

        $httpBackend
            .expect("GET", backendServerUrl+ "/portal/properties", undefined, header)
            .respond();


        // make the call
        portalService.getProperties();

        // flush the backend to "execute" the request to do the expectedGET assertion.
        $httpBackend.flush();
    });

    it("getInstalledFiles method should make http get with proper header", function () {
        var header = { "Accept": "application/json, text/plain, */*" };

        $httpBackend
            .expect("GET", backendServerUrl + "/portal/installedfiles", undefined, header)
            .respond();


        // make the call
        portalService.getInstalledFiles();

        // flush the backend to "execute" the request to do the expectedGET assertion.
        $httpBackend.flush();
    });

    it("getComponents method should make http get with proper header", function () {
        var header = { "Accept": "application/json, text/plain, */*" };

        $httpBackend
            .expect("GET", backendServerUrl + "/portal/componentfirmwareversions", undefined, header)
            .respond();


        // make the call
        portalService.getComponents();

        // flush the backend to "execute" the request to do the expectedGET assertion.
        $httpBackend.flush();
    });

    it("setTime method should make http post with proper header", function () {

        $httpBackend.expectPOST(backendServerUrl + "/portal/time?newTime=2000-01-01", undefined).respond(200);

        // make the call
        portalService.setTime("2000-01-01");

        // flush the backend to "execute" the request to do the expectedPOST assertion.
        $httpBackend.flush();
    });

    it("getExposureCapabilities method should make http get with proper header", function () {
        var header = { "Accept": "application/json, text/plain, */*" };

        $httpBackend
            .expect("GET", backendServerUrl + "/portal/exposurecapabilities?cameraName=patate", undefined, header)
            .respond();


        // make the call
        portalService.getExposureCapabilities("patate");

        // flush the backend to "execute" the request to do the expectedGET assertion.
        $httpBackend.flush();
    });


    it("getPing method should make http get with proper header", function () {
        var header = { "Accept": "application/json, text/plain, */*" };

        $httpBackend
            .expect("GET", backendServerUrl + "/portal/ping", undefined, header)
            .respond();


        // make the call
        portalService.getPing(backendServerUrl);

        // flush the backend to "execute" the request to do the expectedGET assertion.
        $httpBackend.flush();
    });

    it("setExposureSettings method should not make http post with exposure settings Gain and ShutterTime at 0", function () {

        var exposureSettings = { Gain: 0, ShutterTime: 0 };

        // make the call
        portalService.setExposureSettings("patate", exposureSettings);

        // flush the backend to "execute" the request to do the expectedPOST assertion.
        //$httpBackend.flush();
    });

    it("setExposureSettings method should not make http post with exposure settings Gain at 0", function () {

        var exposureSettings = { Gain: 0, ShutterTime: 10 };

        // make the call
        portalService.setExposureSettings("patate", exposureSettings);
    });

    it("setExposureSettings method should not make http post with exposure settings ShutterTime at 0", function () {

        var exposureSettings = { Gain: 10, ShutterTime: 0 };

        // make the call
        portalService.setExposureSettings("patate", exposureSettings);
    });

    it("setExposureSettings method should make http post with proper header", function () {

        var exposureSettings = { Gain: 10, ShutterTime: 10 };

        $httpBackend.expectPOST(backendServerUrl + "/portal/exposureconfig?cameraName=patate", exposureSettings).respond(200);

        // make the call
        portalService.setExposureSettings("patate", exposureSettings);

        // flush the backend to "execute" the request to do the expectedPOST assertion.
        $httpBackend.flush();
    });

    it("getVideoformats method should make http get with proper header", function () {
        var header = { "Accept": "application/json, text/plain, */*" };

        $httpBackend
            .expect("GET", backendServerUrl + "/portal/videoformats", undefined, header)
            .respond();


        // make the call
        portalService.getVideoformats();

        // flush the backend to "execute" the request to do the expectedGET assertion.
        $httpBackend.flush();
    });

    it("getExposureModes method should make http get with proper header", function () {
        var header = { "Accept": "application/json, text/plain, */*" };

        $httpBackend
            .expect("GET", backendServerUrl + "/portal/exposuremodes", undefined, header)
            .respond();


        // make the call
        portalService.getExposureModes();

        // flush the backend to "execute" the request to do the expectedGET assertion.
        $httpBackend.flush();
    });

    it("getTime method should make http get with proper header", function () {
        var header = { "Accept": "application/json, text/plain, */*" };

        $httpBackend
            .expect("GET", backendServerUrl + "/portal/time", undefined, header)
            .respond();

        // make the call
        portalService.getTime();

        // flush the backend to "execute" the request to do the expectedGET assertion.
        $httpBackend.flush();
    });

    it("testNtp method should make http post with proper header", function () {

        var ntpServer = "ntp.server.com";

        $httpBackend.expectPOST(backendServerUrl + "/portal/ntptest", "'" + ntpServer + "'").respond(200);

        // make the call
        portalService.testNtp(ntpServer);

        // flush the backend to "execute" the request to do the expectedPOST assertion.
        $httpBackend.flush();
    });

    it("getRegionOfInterest method should make http get with proper header", function () {
        var header = { "Accept": "application/json, text/plain, */*" };

        $httpBackend
            .expect("GET", backendServerUrl + "/portal/regionofinterest?lprCameraName=patate", undefined, header)
            .respond();

        // make the call
        portalService.getRegionOfInterest("patate");

        // flush the backend to "execute" the request to do the expectedGET assertion.
        $httpBackend.flush();
    });

    it("postRegionOfInterest method should make http post with proper header", function () {

        var coordinates = [1, 2, 3, 4, 5];

        $httpBackend.expectPOST(backendServerUrl + "/portal/regionofinterest?lprCameraName=patate&imageWidth=50&imageHeight=50", coordinates).respond(200);

        // make the call
        portalService.postRegionOfInterest("patate", coordinates, 50, 50);

        // flush the backend to "execute" the request to do the expectedPOST assertion.
        $httpBackend.flush();
    });

    it("getLprContexts method should make http get with proper header", function () {
        var header = { "Accept": "application/json, text/plain, */*" };

        $httpBackend
            .expect("GET", backendServerUrl + "/portal/lprcontexts", undefined, header)
            .respond();

        // make the call
        portalService.getLprContexts();

        // flush the backend to "execute" the request to do the expectedGET assertion.
        $httpBackend.flush();
    });

    it("deleteCertificate method should make http delete with proper header", function () {
        var header = { "Accept": "application/json, text/plain, */*" };
        var certificate = { Thumbprint: "ABC123", IsRequest: false };
        $httpBackend
            .expect("DELETE", backendServerUrl + "/portal/certificate?thumbprint="+certificate.Thumbprint + "&isRequest=false", undefined, header)
            .respond();

        // make the call
        
        portalService.deleteCertificate(certificate);

        // flush the backend to "execute" the request to do the expectedGET assertion.
        $httpBackend.flush();
    });

    it("getCsr method should make http get with proper header", function () {
        var header = { "Accept": "application/json, text/plain, */*" };

        $httpBackend
            .expect("GET", backendServerUrl + "/portal/csr", undefined, header)
            .respond();

        // make the call
        portalService.getCsr();

        // flush the backend to "execute" the request to do the expectedGET assertion.
        $httpBackend.flush();
    });

    it("setCertificate method should make http post with proper header", function () {

        var certificate = { data: 12132 };

        $httpBackend.expectPOST(backendServerUrl + "/portal/certificate", certificate).respond(200);

        // make the call
        portalService.setCertificate(certificate);

        // flush the backend to "execute" the request to do the expectedPOST assertion.
        $httpBackend.flush();
    });

    it("setSecurityCenterExtensionSettings method should make http post with proper header", function () {

        var guid = "fake-guid-totally-not-a-guid-but-it-doesnt-matter";
        var settings = { data: 541654113 };

        $httpBackend.expectPOST(backendServerUrl + "/portal/securitycenterextensionsettings?guid=" + guid, settings).respond(200);

        // make the call
        portalService.setSecurityCenterExtensionSettings(settings, guid);

        // flush the backend to "execute" the request to do the expectedPOST assertion.
        $httpBackend.flush();
    });

    it("getSecurityCenterExtensionSettings method should make http get with proper header", function () {

        var header = { "Accept": "application/json, text/plain, */*" };

        $httpBackend
            .expect("GET", backendServerUrl + "/portal/securitycenterextensionsettings?guid=abcdef", undefined, header)
            .respond();

        // make the call
        portalService.getSecurityCenterExtensionSettings("abcdef");

        // flush the backend to "execute" the request to do the expectedGET assertion.
        $httpBackend.flush();
    });

    it("setPatrollerExtensionSettings method should make http post with proper header", function () {

        var guid = "fake-guid-totally-not-a-guid-but-it-doesnt-matter";
        var settings = { data: 541654113 };

        $httpBackend.expectPOST(backendServerUrl + "/portal/patrollerextensionsettings?guid=" + guid, settings).respond(200);

        // make the call
        portalService.setPatrollerExtensionSettings(settings, guid);

        // flush the backend to "execute" the request to do the expectedPOST assertion.
        $httpBackend.flush();
    });

    it("getPatrollerExtensionSettings method should make http get with proper header", function () {

        var header = { "Accept": "application/json, text/plain, */*" };

        $httpBackend
            .expect("GET", backendServerUrl + "/portal/patrollerextensionsettings?guid=abcdef", undefined, header)
            .respond();

        // make the call
        portalService.getPatrollerExtensionSettings("abcdef");

        // flush the backend to "execute" the request to do the expectedGET assertion.
        $httpBackend.flush();
    });

    it("setHttpExtensionSettings method should make http post with proper header", function () {

        var guid = "fake-guid-totally-not-a-guid-but-it-doesnt-matter";
        var settings = { data: 541654113 };

        $httpBackend.expectPOST(backendServerUrl + "/portal/httpextensionsettings?guid=" + guid, settings).respond(200);

        // make the call
        portalService.setHttpExtensionSettings(settings, guid);

        // flush the backend to "execute" the request to do the expectedPOST assertion.
        $httpBackend.flush();
    });

    it("getHttpExtensionSettings method should make http get with proper header", function () {

        var header = { "Accept": "application/json, text/plain, */*" };

        $httpBackend
            .expect("GET", backendServerUrl + "/portal/httpextensionsettings?guid=abcdef", undefined, header)
            .respond();

        // make the call
        portalService.getHttpExtensionSettings("abcdef");

        // flush the backend to "execute" the request to do the expectedGET assertion.
        $httpBackend.flush();
    });

    it("testFtp method should make http post with proper header", function () {

        var settings = { data: 541654113 };

        $httpBackend.expectPOST(backendServerUrl + "/portal/ftptest", settings).respond(200);

        // make the call
        portalService.testFtp(settings);

        // flush the backend to "execute" the request to do the expectedPOST assertion.
        $httpBackend.flush();
    });

    it("setFtpExtensionSettings method should make http post with proper header", function () {

        var guid = "fake-guid-totally-not-a-guid-but-it-doesnt-matter";
        var settings = { data: 541654113 };

        $httpBackend.expectPOST(backendServerUrl + "/portal/ftpextensionsettings?guid=" + guid, settings).respond(200);

        // make the call
        portalService.setFtpExtensionSettings(settings, guid);

        // flush the backend to "execute" the request to do the expectedPOST assertion.
        $httpBackend.flush();
    });

    it("getFtpExtensionSettings method should make http get with proper header", function () {

        var header = { "Accept": "application/json, text/plain, */*" };

        $httpBackend
            .expect("GET", backendServerUrl + "/portal/ftpextensionsettings?guid=abcdef", undefined, header)
            .respond();

        // make the call
        portalService.getFtpExtensionSettings("abcdef");

        // flush the backend to "execute" the request to do the expectedGET assertion.
        $httpBackend.flush();
    });

    it("getExtensions method should make http get with proper header", function () {

        var header = { "Accept": "application/json, text/plain, */*" };

        $httpBackend
            .expect("GET", backendServerUrl + "/portal/extensions", undefined, header)
            .respond();

        // make the call
        portalService.getExtensions();

        // flush the backend to "execute" the request to do the expectedGET assertion.
        $httpBackend.flush();
    });

    it("getInstalledCertificateInfos method should make http get with proper header", function () {

        var header = { "Accept": "application/json, text/plain, */*" };

        $httpBackend
            .expect("GET", backendServerUrl + "/portal/certificateinfos", undefined, header)
            .respond();

        // make the call
        portalService.getInstalledCertificateInfos();

        // flush the backend to "execute" the request to do the expectedGET assertion.
        $httpBackend.flush();
    });

    it("resetBestReadCounter method should make http put with proper header", function () {
        
        $httpBackend.expectPUT(backendServerUrl + "/portal/bestreadcount", undefined).respond(200);

        // make the call
        portalService.resetBestReadCounter("abcdef");

        // flush the backend to "execute" the request to do the expectedGET assertion.
        $httpBackend.flush();
    });

    it("startIdentify method should make http get with proper header", function () {

        var header = { "Accept": "application/json, text/plain, */*" };

        $httpBackend
            .expect("GET", backendServerUrl + "/identify.cgi", undefined, header)
            .respond();

        // make the call
        portalService.startIdentify();

        // flush the backend to "execute" the request to do the expectedGET assertion.
        $httpBackend.flush();
    });

    it("getEnvironment method should make http get without header", function () {
       
        $httpBackend
            .expect("GET", "/environment", undefined)
            .respond();

        // make the call
        portalService.getEnvironment();

        // flush the backend to "execute" the request to do the expectedGET assertion.
        $httpBackend.flush();
    });

    it("setPassword method should make http post with proper header", function () {

        var password = "fake-password";
        var hash = portalService.createHash(password);
        var md5 = portalService.createMD5(password);

        $httpBackend.expectPOST("/password", "'" + hash + "_" + md5 + "'").respond(200);

        // make the call
        portalService.setPassword(password);

        // flush the backend to "execute" the request to do the expectedPOST assertion.
        $httpBackend.flush();
    });

    it("setUserConfig method should make http post with proper header", function () {

        var userconfig = { data: 465654151 };

        $httpBackend.expectPOST(backendServerUrl + "/portal/userconfig", userconfig).respond(200);

        // make the call
        portalService.setUserConfig(userconfig);

        // flush the backend to "execute" the request to do the expectedPOST assertion.
        $httpBackend.flush();
    });

    it("getUserConfig method should make http get with proper header", function () {

        var header = { "Accept": "application/json, text/plain, */*" };

        $httpBackend
            .expect("GET", backendServerUrl +"/portal/userconfig?type=JSON", undefined, header)
            .respond();

        // make the call
        portalService.getUserConfig();

        // flush the backend to "execute" the request to do the expectedGET assertion.
        $httpBackend.flush();
    });

    it("logout method should reset the password", function () {

        // make the call.
        portalService.logout();

    });
});